Package: genomation
Type: Package
Title: Summary, annotation and visualization of genomic data
Version: 1.42.0
Author: Altuna Akalin [aut, cre], Vedran Franke [aut, cre], Katarzyna Wreczycka
    [aut], Alexander Gosdschan [ctb], Liz Ing-Simmons [ctb], Bozena Mika-Gospodorz [ctb]
Maintainer: Altuna Akalin <aakalin@gmail.com>, Vedran Franke
 <vedran.franke@gmail.com>, Katarzyna Wreczycka <katwre@gmail.com>
Description: A package for summary and annotation of genomic intervals. Users
    can visualize and quantify genomic intervals over pre-defined functional
    regions, such as promoters, exons, introns, etc. The genomic intervals
    represent regions with a defined chromosome position, which may be associated
    with a score, such as aligned reads from HT-seq experiments, TF binding sites,
    methylation scores, etc. The package can use any tabular genomic feature data
    as long as it has minimal information on the locations of genomic intervals. In
    addition, It can use BAM or BigWig files as input.
License: Artistic-2.0
LazyLoad: yes
VignetteBuilder: knitr
biocViews: Annotation, Sequencing, Visualization, CpGIsland
Encoding: latin1
URL: http://bioinformatics.mdc-berlin.de/genomation/
BugReports: https://github.com/BIMSBbioinfo/genomation/issues
Depends: R (>= 3.5.0), grid
Imports: Biostrings (>= 2.47.6), BSgenome (>= 1.47.3), data.table,
        Seqinfo, GenomicRanges (>= 1.31.8), GenomicAlignments (>=
        1.15.6), S4Vectors (>= 0.17.25), ggplot2, gridBase, impute,
        IRanges (>= 2.13.12), matrixStats, methods, parallel, plotrix,
        plyr, readr, reshape2, Rsamtools (>= 1.31.2), seqPattern,
        rtracklayer (>= 1.39.7), Rcpp (>= 0.12.14)
Suggests: BiocGenerics, genomationData, knitr, RColorBrewer, rmarkdown,
        RUnit
Collate: 'combineScoreMatrixList.R' 'documentData.R'
        'genomation-classes.R' 'getRandomEnrichment.R'
        'findFeatureComb.R' 'patternMatrix.R' 'plotMatrix.R'
        'randomizeFeature.R' 'readAnnotate.R' 'readData.R'
        'scoreMatrix.R' 'scoreMatrixBin.R' 'scoreMatrixList.R' 'Ops.R'
        'test_genomation_package.R' 'deprecated_defunct.R'
        'enrichmentMatrix.R' 'RcppExports.R'
LinkingTo: Rcpp
RoxygenNote: 6.0.1.9000
git_url: https://git.bioconductor.org/packages/genomation
git_branch: RELEASE_3_22
git_last_commit: 01d23ac
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: yes
Packaged: 2025-10-30 04:08:16 UTC; biocbuild

import(methods)
import(Biostrings)
import(BSgenome)
import(Seqinfo)
import(GenomicRanges)
import(GenomicAlignments)
import(IRanges)
import(Rsamtools)
import(S4Vectors)
import(ggplot2)
import(grid)
import(gridBase)
import(impute)
importMethodsFrom("rtracklayer", import)


#importMethodsFrom("GenomicRanges",elementMetadata,countOverlaps,findOverlaps, seqnames, seqlengths, strand,"strand<-")

#importClassesFrom("GenomicRanges",GRanges,GRangesList)

#importClassesFrom("IRanges", RangedData, RangedDataList, DataTable,
#                  Annotated, Vector, List, DataTable_OR_NULL, character_OR_NULL,
#                  RangedSelection, SimpleList, DataFrame,SimpleRleList,RleList,SimpleAtomicList,AtomicList,RleViews,RleViewsList)

#importFrom("IRanges", DataFrame, IRanges, RangedDataList, RangedData,
#           IntegerRangesList, Rle,matchMatrix,findOverlaps, RangedSelection,ranges,countOverlaps,subsetByOverlaps,elementMetadata,"elementMetadata<-",Views,viewApply,RleViewsList)

#importMethodsFrom("IRanges",nearest,as.data.frame,values,length,elementNROWS,width,start,end)

importFrom("data.table",data.table,dcast)
importFrom("data.table", rbindlist)
importFrom("graphics", "axis", "barplot", "image", "legend", "lines",
             "par", "pie", "plot", "plot.new", "plot.window", "polygon",
             "text")
importFrom("grDevices", "colorRamp", "hcl", "rainbow", "rgb")
importFrom("matrixStats",colMedians)
importFrom("matrixStats",colSds)
importFrom("matrixStats",colQuantiles)
importFrom("parallel", mclapply)
importFrom("plotrix", dispersion)
importFrom("plotrix", std.error)
importFrom("plyr",rbind.fill)
importFrom("readr", read_delim)
importFrom("readr", locale)
importFrom("reshape2", melt)
importFrom("seqPattern", motifScanScores)
importFrom("seqPattern", getPatternOccurrenceList)
importFrom("stats", "dist", "hclust", "pnorm", "runif")
importFrom("utils", "download.file", "read.table", "unzip")
importFrom(Rcpp, sourceCpp)

exportClasses(AnnotationByFeature)
exportClasses(AnnotationByGeneParts)
exportClasses(RandomEnrichment)
exportClasses(ScoreMatrix)
exportClasses(ScoreMatrixList)

exportMethods(annotateWithFeature)
exportMethods(annotateWithFeatures)
exportMethods(annotateWithFeatureFlank)
exportMethods(annotateWithGeneParts)
exportMethods(binMatrix)
exportMethods(convertBedDf)
exportMethods(convertBed2Exons)
exportMethods(convertBed2Introns)
exportMethods(enrichmentMatrix)
exportMethods(findFeatureComb)
exportMethods(getAssociationWithTSS)
exportMethods(getFeatsWithTargetsStats)
exportMethods(getFlanks)
exportMethods(getMembers)
exportMethods(getRandomEnrichment)
exportMethods(getTargetAnnotationStats)
export(heatMatrix)
export(heatMeta)
export(intersectScoreMatrixList)
export(multiHeatMatrix)
exportMethods(patternMatrix)
export(plotMeta)
export(plotGeneAnnotation)
export(heatTargetAnnotation)
export(plotTargetAnnotation)
exportMethods(orderBy)
exportMethods(randomizeFeature)
export(readGeneric)
export(readBed)
export(readBroadPeak)
exportMethods(readFeatureFlank)
export(readNarrowPeak)
exportMethods(readTranscriptFeatures)
exportMethods(ScoreMatrix)
exportMethods(ScoreMatrixBin)
exportMethods(scaleScoreMatrix)
exportMethods(scaleScoreMatrixList)
export(listSliceMean)
export(listSliceMedian)
export(listSliceMax)
export(listSliceMin)
export(listSliceSum)
export(Mean_c)
export(Median_c)
export(Max_c)
export(Min_c)
export(Sum_c)

export(gffToGRanges) 
export(ScoreMatrixList)

S3method(c,ScoreMatrixList)
S3method(c,ScoreMatrix)

useDynLib(genomation)
exportPattern("^[[:alpha:]]+")

<a name="logo"/>
<div align="center">
<img src="https://github.com/BIMSBbioinfo/genomation/blob/master/inst/genomation_logo.png" alt="hex Logo"  ></img>
</a>
</div>

# genomation: convenience functions for visualing and summarizing genomic intervals

Status [![Build Status](https://api.travis-ci.org/BIMSBbioinfo/genomation.svg?branch=master)](https://travis-ci.org/BIMSBbioinfo/genomation) [![codecov.io](https://codecov.io/github/BIMSBbioinfo/genomation/coverage.svg?branch=master)](https://codecov.io/github/BIMSBbioinfo/genomation?branch=master)    [![BioC_years](http://www.bioconductor.org/shields/years-in-bioc/genomation.svg)](http://www.bioconductor.org/packages/release/bioc/html/genomation.html) [![BioC_availability](http://www.bioconductor.org/shields/availability/release/genomation.svg)](http://www.bioconductor.org/packages/release/bioc/html/genomation.html)


This package is a collection of functions for simplfiying common tasks in genomic feature/interval
analysis. It provides functions for reading BED and GFF files as GRanges objects, summarizing genomic features over predefined windows so users can make average enrichment of features over defined regions or produce heatmaps. It can also annotate given regions
with other genomic features such as exons,introns and promoters.

# Installation

### install via Bioconductor
```R
if (!requireNamespace("BiocManager", quietly=TRUE))
    install.packages("BiocManager")
BiocManager::install("genomation")

```

### Install the latest version via install_github
```R
#' Install dependecies
install.packages( c("data.table","plyr","reshape2","ggplot2","gridBase","devtools"))
if (!requireNamespace("BiocManager", quietly=TRUE))
    install.packages("BiocManager")
BiocManager::install(c("GenomicRanges","rtracklayer","impute","Rsamtools"))

#' install the packages
library(devtools)
install_github("BIMSBbioinfo/genomation",build_vignettes=FALSE)

```

# Using the package
see the package vignette [here](http://bioconductor.org/packages/release/bioc/vignettes/genomation/inst/doc/GenomationManual.html)

(require("genomation") & require("GenomicRanges") & require("GenomicAlignments") & require("rtracklayer") & require("readr") & require("Biostrings")) || stop("unable to load genomation package")
genomation:::.test()

# documents data sets that come with the package

#' Example promoter data set.
#' 
#' promoters of hg19 assembly of human genome on chr21 and chr22. Promoter set is 
#' derived from refseq TSS.
#' @name promoters
#' @docType data
#' @format  \code{\link{GRanges}} object
#'
NULL


#' Example CpG island data set.
#' 
#' CpG islands of hg19 assembly of human genome on chr21 and chr22. Downloaded
#' from UCSC genome browser.
#' 
#' @name cpgi
#' @docType data
#' @format  \code{\link{GRanges}} object
#'
NULL



#' Example RefSeq genes data set.
#' 
#' RefSeq genes of hg19 assembly of human genome on chr21 and chr22. Downloaded
#' from UCSC genome browser.
#' 
#' @name genes
#' @docType data
#' @format  \code{\link{GRanges}} object
#'
NULL



#' Example CAGE data set.
#' 
#' Location and tag per million values for CAGE TSS clusters on chr21 and chr22
#' of human genome (hg19 assembly). The clusters are dowloaded from ENCODE project
#' downloads for NHEK cells.
#' 
#' @name cage
#' @docType data
#' @format  \code{\link{GRanges}} object
#'
NULL

# ------------------------------------------------------------------------------------------------ #
# randomizeFeature
#' function that randomizes the genomic coordinates
#'
#' This function randomly distributes the coordinates of genomic features which
#' is stored in a \code{GRanges} object. The randomization can be constrained by
#'  supplied arguments.
#' The function is still in Beta mode - the regions can overlap excluded regions, and the randomized regions are not disjoint. Please take care that the excluded and included regions are not too strict when compared to the total width of the ranges.
#'
#'
#' @param feature a GRanges object to be randomized
#' @param chrom.sizes sizes of chromosomes as a named vector (names are
#'        chromsomes names and elements of the vectors are lengths). , if not
#'        given sizes in GRanges object will be used if no sizes there the end
#'        of each chr will be the end last feature on each chr
#' @param stranded if FALSE, all of the returned features will be strandless
#'        (will have "*" in the strand slot)
#' @param keep.strand.prop If TRUE strands will have the same proportion as the
#'        features
#' @param keep.chrom If TRUE, number of features and randomized features for a
#'        chromosome will match. Currently seeting this to FALSE is not supported.
#' @param exclude A GRanges object where no randomized feature should overlap,
#'        can be gaps or unmappable regions in the genome as an example.
#' @param include A GRanges object which defines the boundaries of randomized
#'        features. If not provided the whole genome is used, as defined using 
#'        the chrom.sizes parameter.
#' @param seed random number generator seed
#' @param nrand number of randomizations (default:1)
#' 
#' @return returns a GRanges object which is randomized version of the feature, 
#' along with a "set" column in the metadata which designates to which iteration 
#' of the randomization the range belong.

#' @export
#' @docType methods
#' @rdname randomizeFeature-methods
setGeneric("randomizeFeature", 
                function(feature,chrom.sizes=NULL,stranded=TRUE,
                         keep.strand.prop=TRUE,keep.chrom=TRUE,
                         exclude=NULL,include=NULL,seed=NULL,nrand=1)
                                standardGeneric("randomizeFeature") )

#' @aliases randomizeFeature,GRanges-method
#' @rdname randomizeFeature-methods
setMethod("randomizeFeature", signature(feature = "GRanges"),
            function( feature,
                      chrom.sizes ,
                      stranded ,
                      keep.strand.prop, 
                      keep.chrom,
                      exclude,
                      include,
                      seed,
                      nrand ){

            
            
            # sets the random seed generator if it isn't set
            if(!is.null(seed) & is.numeric(seed)){set.seed(seed)}
            
            #checks whether the chromosome sizes are defined, if not uses the end coordinates of features
            if(is.null(chrom.sizes)){
                chrom.sizes=list()
                chrs=as.character(unique(seqnames(feature)))
                my.starts=c()
                my.ends=c()
                for( i in 1:length(chrs) ){
                    my.set= feature[seqnames(feature)==chrs[i], ]
                    chrom.sizes[chrs[i]]  =max( end(my.set) )
                }
                chrom.sizes=unlist(chrom.sizes)
            }

            # if the include ranges are not defined, defines the ranges across the whole chromosome
            if(is.null(include))
                include = GRanges(names(chrom.sizes), IRanges(1, chrom.sizes))
            
            # if the exclude ranges are not defined it defines them
            if(is.null(exclude))
                exclude = GRanges()
            
            # gives a warning if the restricted ranges are too small
            if(sum(width(feature))/(sum(as.numeric(width(include))) - sum(as.numeric(width(exclude)))) < 1e-3)
                warning('The width of the feature sizes is too big compared to the allowed genome')
            
            # checks whether the features are reduced
            if(sum(width(feature)) != sum(width(reduce(feature))))
                stop('Feature needs to be a disjoined set of ranges')
            
            # selects the chromosome set to be used
            chrs = intersect(as.character(seqnames(include)), names(chrom.sizes))
            chrom.sizes = chrom.sizes[chrs]
            exclude = exclude[seqnames(exclude) %in% chrs]
            
            # defines the regions that can be sampled from
            r = sapply(chrom.sizes, function(x)Rle(0, x))
            re = sapply(names(r), function(x){a=r[[x]];a[ranges(include[seqnames(include) == x])]= 1;a})
            if(length(exclude) > 0)
                re = sapply(names(r), function(x){a=re[[x]];a[ranges(exclude[seqnames(exclude) == x])]= 0;a})
            names(re) = names(chrom.sizes)
            
            # loops over the chromosome and constructs the defined set of ranges
            glist = list()
            for(chr in chrs){
            
                # gets the indices of the features on the chromosome
                ind = seqnames(feature) == chr
                
                # gets the permitted ranges
                a = ranges(re[[chr]])[runValue(re[[chr]]) == 1]
                nl = sum(ind)
                total = nrand*nl
                
                # gets the weight for the number of regions per allowed region based on the width
                w = round(total*(width(a)/sum(width(a))))
                i = sample(length(w),1)
                w[i] = w[i] + total - sum(w)
                
                # generates the random regions
                m = mapply(function(n,x,y)runif(n,x,y), w, start(a), end(a), SIMPLIFY=FALSE)
                g = GRanges(chr, IRanges(start = unlist(m), width=rep(width(feature[ind]), times=nrand)))
                values(g)$set = rep(1:nrand, each=nl)
                
                # sets the strand of the features
                if(stranded == TRUE)
                    strand(g) = sample(c('+','-'), length(g), replace=TRUE)
                if(keep.strand.prop == TRUE)
                    strand(g) = sample(as.character(strand(feature[ind])))
                glist[[chr]] = g
            }
            return(unlist(GRangesList(glist)))
})


# ------------------------------------------------------------------------------------------------ #
# calculateOverlapSignificance
#' function that calculates the significance of overlaps of two sets of features using randomization
#'
#' This function calculates the significance of overlaps of two sets of features using randomization. #' It returns a distributon of overlaps of a target set with a given randomized feature set. The 
#' randomization can be constrained by supplied arguments.
#' The function is still in Beta mode - the regions can overlap excluded regions, and the randomized regions are not disjoint. Please take care that the excluded and included regions are not too strict when compared to the total width of the ranges.
#'
#' @param target a GRanges object for which the overlap needs to be calculates
#' @param feature a GRanges object to be randomized
#' @param chrom.sizes sizes of chromosomes as a named vector (names are
#'        chromsomes names and elements of the vectors are lengths). , if not
#'        given sizes in GRanges object will be used if no sizes there the end
#'        of each chr will be the end last feature on each chr
#' @param stranded if FALSE, all of the returned features will be strandless
#'        (will have "*" in the strand slot)
#' @param keep.strand.prop If TRUE strands will have the same proportion as the
#'        features
#' @param keep.chrom If TRUE, number of features and randomized features for a
#'        chromosome will match. Currently seeting this to FALSE is not supported.
#' @param exclude A GRanges object where no randomized feature should overlap,
#'        can be gaps or unmappable regions in the genome as an example.
#' @param include A GRanges object which defines the boundaries of randomized
#'        features
#' @param seed random number generator seed
#' @param nrand number of randomizations (default:1)
#'
#'
#' @return returns a GRanges object which is randomized version of the feature
#' @export
#' @docType methods
#' @rdname calculateOverlapSignificance-methods
setGeneric("calculateOverlapSignificance", 
                function(target, feature, chrom.sizes=NULL, stranded=TRUE,
                         keep.strand.prop=TRUE, keep.chrom=TRUE,
                         exclude=NULL, include=NULL, seed=NULL, nrand=1)
                standardGeneric("calculateOverlapSignificance") )

#' @aliases calculateOverlapSignificance,GRanges,GRanges-method
#' @rdname calculateOverlapSignificance-methods
setMethod("calculateOverlapSignificance", signature(target="GRanges", feature="GRanges"),
            function( target,
                      feature,
                      chrom.sizes ,
                      stranded ,
                      keep.strand.prop, 
                      keep.chrom,
                      exclude,
                      include,
                      seed,
                      nrand ){
        
        # randomizes the ranges nrand times
        rand.ranges = randomizeFeature(feature=feature, 
                                        chrom.sizes=chrom.sizes,
                                        stranded=stranded, 
                                        keep.strand.prop=keep.strand.prop, 
                                        keep.chrom=keep.chrom, 
                                        exclude=exclude, 
                                        include=include, 
                                        seed=seed,nrand = nrand)
        
        # converts the overlaps into a data.table object and summarizes the overlaps
        message('Summarizing the overlaps...\n')
        co = data.table(as.matrix(findOverlaps(rand.ranges, target)))
        co$set = values(rand.ranges)$set[co$queryHits]
        co$queryHits = NULL
        co = unique(co)
        co = co[,length(subjectHits), by='set']
        return(co$V1/length(target))
})

# ---------------------------------------------------------------------------- #
#' Find combitations of genomic features
#' 
#' Provided a GRangesList, finds the combinations of sets of ranges. 
#' It is mostly used to look at the combinatorics of transcription factor 
#' binding. The function works by, firstly, constructing a union of all 
#' ranges in the list, which are then designated by the combinatorics of 
#' overlap with the original sets. A caveat of this approach is that the 
#' number of possible combinations increases exponentially, so we would 
#' advise you to use it with up to 6 data sets. If you wish to take a 
#' look at a greater number of factors, methods like self organizing 
#' maps or ChromHMM might be more appropriate.
#'
#' @param gl a \code{GRangesList} object, containing ranges for which 
#'                 represent regions enriched for transcription factor binding
#' @param width \code{integer} is the requested width of each enriched region. If 0 the ranges are not resized, if a positive integer, the width of all ranges is set to that number. Ranges are resized relative to the center of original ranges.
#' @param use.names a boolean which tells the function whether to return the resulting ranges with a numeric vector which designates each class (the default), or to construct the names of each class using the names from the GRangesList
#' @param collapse.char a character which will be used to separate the class names if use.names=TRUE. The default is ':'
#'
#' @usage findFeatureComb(gl, width=0, use.names=FALSE, collapse.char=':')
#' @return a GRanges object
#'
#'@examples
#' library(GenomicRanges)
#' g = GRanges(paste('chr',rep(1:2, each=3), sep=''), IRanges(rep(c(1,5,9), times=2), width=3))
#' gl = GRangesList(g1=g, g2=g[2:5], g3=g[3:4])
#' findFeatureComb(gl)
#' findFeatureComb(gl, use.names=TRUE)
#'
#' @docType methods
#' @rdname findFeatureComb-methods
#' @export
setGeneric("findFeatureComb", 
           function(gl, width=0, use.names=FALSE, collapse.char=':') 
             standardGeneric("findFeatureComb") )

#' @aliases findFeatureComb,GRangesList-method
#' @rdname findFeatureComb-methods
setMethod("findFeatureComb", signature("GRangesList"),
          function(gl, width, use.names, collapse.char){
          
              if(width < 0)
                stop('width needs to be a positive integer')
              
              # resizes the ranges before doing the overlaps
              if(width > 0)
                gl = endoapply(gl, function(x)resize(x, width=width, fix='center'))
                
              # makes a union of ranges and calculates the combinatorics
              r = reduce(unlist(gl))
              do = data.frame(lapply(gl, function(x)
                                            as.numeric(countOverlaps(r, x) > 0)))
              # finds the combinations of ranges for each interval in the reduced ranges
              do.comb = as.numeric(factor(rowSums(t(t(do)*(2^(0:(ncol(do)-1)))))))
              
              if(use.names == FALSE){
                values(r)$class = do.comb
              
              }
              if(use.names == TRUE){
                
                cnames = colnames(do)
                # selects only unique representatives of combinations
                do = do[!duplicated(do.comb),]
                # pastes names of individual sets
                cnames = apply(do, 1, function(x)paste(cnames[x == 1], 
                                                  collapse=collapse.char))
                
                # designates each reduced set by the combination of overlapping ranges
                r$class = cnames[match(do.comb, unique(do.comb))]
                
              }
              return(r)
          }
)

# test for readGeneric function
test_readGeneric = function()
{
  tab.test1=system.file('unitTests/tab.test1', package='genomation')
  
  #1. test regular bed input
  r1 = readGeneric(tab.test1)
  g1 = GRanges(c('chr1','chr1'), IRanges(c(1,5), c(10,15)))
  checkIdentical(g1, r1)
  
  #2. test strand
  r2 = readGeneric(tab.test1, strand=4)
  g2 = GRanges(c('chr1','chr1'), IRanges(c(1,5), c(10,15)), strand=c('+','-'))
  checkIdentical(g2, r2)
  
  #3. all metadata colums
  r3 = readGeneric(tab.test1, strand=4, keep.all.metadata=TRUE)
  g3 = GRanges(c('chr1','chr1'), 
               IRanges(c(1,5), c(10,15)), 
               strand=c('+','-'),
               V5=as.integer(c(15,20)),
               V6=as.integer(c(20,25)))
  checkIdentical(g3,r3)
  
  #4. all named metadata colums
  r4 = readGeneric(tab.test1, strand=4, meta.col=list(score1=5, score2=6))
  g4 =  GRanges(c('chr1','chr1'), 
                IRanges(c(1,5), c(10,15)), 
                strand=c('+','-'),
                score1=as.integer(c(15,20)),
                score2=as.integer(c(20,25)))
  checkIdentical(g4,r4)
  
  #5. selected metadata columns
  r5 = readGeneric(tab.test1, strand=4, meta.cols=list(score1=6))
  g5 =  GRanges(c('chr1','chr1'), 
                IRanges(c(1,5), c(10,15)), 
                strand=c('+','-'),
                score1=as.integer(c(20,25)))
  checkIdentical(g5,r5)
  
  #6. test whether it can read a file containing a header
  tab.test2=system.file('unitTests/tab.test2', package='genomation')
  r6 = readGeneric(tab.test2, header=TRUE)
  g6 = GRanges(c('chr1','chr1'), IRanges(c(1,5), c(10,15)))
  checkIdentical(g6,r6)
  
  #7. header=TRUE, keep.all.metadata=TRUE
  r7 = readGeneric(tab.test2,strand=4, header=TRUE, keep.all.metadata=TRUE)
  
  
  #8. test whether it can read a file with permutted columns
  tab.test3=system.file('unitTests/tab.test3', package='genomation')
  r7 = readGeneric(tab.test3, chr=5, start=3, end=4, strand=6, 
                   meta.col=c(score1=1, score2=2), header=TRUE)
  g7 =  GRanges(c('chr1','chr1'), 
                IRanges(c(1,5), c(10,15)), 
                strand=c('+','-'),
                score1=as.integer(c(15,20)),
                score2=as.integer(c(20,25)))
  checkIdentical(g7,r7)
  
  
  #9. test whether it can read a compressed file (.gz, .zip)
  tab.test3=system.file('unitTests/tab.test3', package='genomation')
  tab.test3.zip=system.file('unitTests/tab.test3.zip', package='genomation')

  tab.test3.gz <- paste(tab.test3, ".gz", sep="")
  gzf <- gzfile(tab.test3.gz, "w")
  write.table(read.table(tab.test3, header=TRUE), 
              file=gzf, 
              sep = "\t", row.names = FALSE)
  close(gzf)
  
  r7.gz = readGeneric(tab.test3.gz, chr=5, start=3, end=4, strand=6, 
                      meta.col=c(score1=1, score2=2), header=TRUE)  
  r7.zip = readGeneric(tab.test3.zip, chr=5, start=3, end=4, strand=6, 
                       meta.col=c(score1=1, score2=2), header=TRUE)        
                       
  g7 =  GRanges(c('chr1','chr1'), 
                IRanges(c(1,5), c(10,15)), 
                strand=c('+','-'),
                score1=as.integer(c(15,20)),
                score2=as.integer(c(20,25)))
  
  checkIdentical(g7, r7.gz)
  checkIdentical(g7, r7.zip)
  
  if (file.exists(tab.test3.gz)) file.remove(tab.test3.gz)
  
  #10. test a file with UCSC header
  tab.test4=system.file('unitTests/tab.test4', package='genomation')
  r8 = readGeneric(tab.test4, chr=1, start=2, end=3, strand=6, 
                   meta.col=c(score=5,
                              name=4, 
                              thickStart=7,  
                              thickEnd=8, 
                              itemRgb=9), 
                   header=FALSE, skip=3,
                   zero.based=TRUE)
  r9 = readBed(tab.test4, track.line=3)
  r10 = readBed(tab.test4, track.line="auto")
  
  g8 =  GRanges(c('chr7','chr7','chr7'), 
                IRanges(c(1,11,16), c(10,15,20)), 
                strand=c('+','+','+'),
                score=as.integer(c(0,0,0)),
                name=c("Pos1","Pos2","Pos3"),
                thickStart=as.integer(c(0,10,15)),
                thickEnd=as.integer(c(10,15,20)),
                itemRgb=c("255,0,0","255,0,0","255,0,0"))
  
  checkIdentical(g8, r8)
  checkIdentical(g8, r9)
  checkIdentical(g8, r10)
}

test_gffToGRanges = function()
{
  library(GenomicRanges)
  tab.test = system.file('unitTests/test.gtf', package='genomation')
  gff1 = gffToGRanges(tab.test)
  checkIdentical(length(gff1), 3L)
 
  gff2 = gffToGRanges(tab.test, filter='exon')
  checkIdentical(length(gff2), 1L)
  
  gff3 = gffToGRanges(tab.test, ensembl=TRUE)
  checkIdentical(as.character(seqlevels(gff3)), 'chr1')
}

# S4 classes for genomation show and accessor functions

# ---------------------------------------------------------------------------- #
# --------------------- #
# Randomization Classes #
# --------------------- #
#' An S4 class for storing \code{getRandomEnrichment} function results
#'
#' The resulting object stores the results of \code{getRandomEnrichment} function
#'
#' @section Slots:\describe{
#'                  \item{\code{orig.cnt}:}{number of features overlapping with query at \code{getRandomEnrichment} }
#'                  \item{\code{rand.olap.dist}:}{set of number of features overlapping with randomized queries at \code{getRandomEnrichment}}
#'                  \item{\code{log2fc}:}{ log2 fold change calculated by dividing \code{orig.cnt} by mean(\code{rand.olap.dist}) and taking log2 of that result}
#'                  \item{\code{p.value}:}{P-value assuming \code{rand.olap.dist} has a normal distribution and comparing \code{orig.cnt} with that distribution }
#'                  \item{\code{rand.p.value}:}{ p-value from randomization by calculation the proportion of how many times a random number of overlap exceeds the original number of overlap}
#'                 }
#'
#' @name RandomEnrichment-class
#' @rdname RandomEnrichment-class
#' @seealso \code{\link{getRandomEnrichment}}
#' @export
setClass("RandomEnrichment", 
        representation(
            orig.cnt = "numeric", 
            rand.olap.dist = "numeric",
            log2fc="numeric",
            p.value="numeric",
            rand.p.value="numeric" 
))


#' show method for some of the genomation classes
#' @param object object of class RandomEnrichment
#' @rdname show-methods
#' @aliases show,RandomEnrichment-method
setMethod("show", "RandomEnrichment", function(object) {
  message("orig.cnt:",object@orig.cnt,"\n")
  message("log2fc:",object@log2fc,"\n")
  message("p.value:",object@p.value,"\n")
  message("rand.p.value:", object@rand.p.value,"\n")
})





# ---------------------------------------------------------------------------- #
# ------------------- #
# ScoreMatrix Classes #
# ------------------- #

#' An S4 class for storing \code{ScoreMatrix} function results
#'
#' The resulting object is an extension of a \code{matrix} object, and stores values (typically genome-wide scores) for a predefined set of regions
#' Each row on the ScoreMatrix is a predefined region (Ex: CpG islands, promoters) and columns are values across those regions.
#'
#' @name ScoreMatrix-class
#' @rdname ScoreMatrix-class
#' 
#' @section Constructors:
#' see \code{\link{ScoreMatrix}}
#' 
#' @section Coercion:
#' as(from, "matrix"): Creates a matrix from \code{ScoreMatrix} object. You can also use S3Part() function to extract the matrix from \code{ScoreMatrix} object.
#' 
#' @section Subsetting:
#' In the code snippets below, x is a ScoreMatrix object.
#' \code{'x[i,j]'}: Get or set elements from row \code{i} and column \code{j} and return a subset ScoreMatrix object.
#' 
#' @seealso \code{\link{ScoreMatrix}}
#' @export
setClass("ScoreMatrix",contains = "matrix")


# ---------------------------------------------------------------------------- #
#' An S4 class for storing a set of \code{ScoreMatrixList} 
#'
#' The resulting object is an extension of a \code{list} object, where each element corresponds to a score matrix object
#' 
#' @name ScoreMatrixList-class
#' @rdname ScoreMatrixList-class
#'  
#' @section Constructors:
#' see \code{\link{ScoreMatrixList}}
#' 
#' @section Coercion:
#' as(from, "ScoreMatrixList"): Creates a \code{ScoreMatrixList} object from a list containing \code{\link{ScoreMatrix}} or \code{\link{ScoreMatrixBin}} objects. 
#' 
#' @section Subsetting:
#' In the code snippets below, x is a ScoreMatrixList object.
#' 
#' \code{x[[i]]},\code{x[[i]]}: Get or set elements \code{i}, where \code{i} is a numeric or character vector of length 1.
#' 
#' \code{x$name}, \code{x$name}: value: Get or set element \code{name}, where \code{name} is a name or character vector of length 1.
#' 
#' @seealso \code{\link{ScoreMatrixList}}
#' @export
setClass("ScoreMatrixList",
            slots = c(names="character"),
            contains = "list"
#            validity=.valid.ScoreMatrixList
            )

# ---------------------------------------------------------------------------- #
# ------------------ #
# Annotation Classes #
# ------------------ #

# A set of objects that will hold statistics about feature and annotation overlap
#' An S4 class that information on overlap of target features with annotation features  
#'
#' This object is desgined to hold statistics and information about genomic feature overlaps
#'          
#' @section Slots:\describe{
#'                  \item{\code{members}}{a matrix showing overlap of target features with annotation genomic features}
#'
#'                  \item{\code{annotation}}{a named vector of percentages}
#'
#'                  \item{\code{precedence}}{a named vector of percentages}
#'
#'                  \item{\code{num.annotation}}{vector}

#'                  \item{\code{num.precedence}}{vector}
#'
#'                  \item{\code{no.of.OlapFeat}}{vector}
#'
#'                  \item{\code{perc.of.OlapFeat}}{vector}
#' }
#' @name AnnotationByFeature-class
#' @rdname AnnotationByFeature-class
#' @export
setClass("AnnotationByFeature", 
         representation(members  = "matrix",
                        annotation = "numeric",
                        precedence = "numeric",
                        num.annotation = "numeric",
                        num.precedence = "numeric",
                        no.of.OlapFeat = "numeric",
                        perc.of.OlapFeat = "numeric"))



#' An S4 class that information on overlap of target features with annotation features  
#'
#' This object is desgined to hold statistics and information about genomic feature overlaps
#'          
#' @section Slots:\describe{
#'                  \item{\code{members}}{a matrix showing overlap of target features with annotation genomic features}
#'
#'                  \item{\code{annotation}}{a named vector of percentages}
#'
#'                  \item{\code{precedence}}{a named vector of percentages}
#'
#'                  \item{\code{num.annotation}}{vector}

#'                  \item{\code{num.precedence}}{vector}
#'
#'                  \item{\code{no.of.OlapFeat}}{vector}
#'
#'                  \item{\code{perc.of.OlapFeat}}{vector}
#'
#'                  \item{dist.to.TSS}{a data frame showing distances to TSS and gene/TSS names and strand}
#' }
#' @name AnnotationByGeneParts-class
#' @rdname AnnotationByGeneParts-class
#' @export
setClass("AnnotationByGeneParts", 
         representation(dist.to.TSS = "data.frame"), contains = "AnnotationByFeature")

.onUnload <- function (libpath) {
  library.dynam.unload("genomation", libpath)
}

#######################################
# S4 functions
#######################################

# ---------------------------------------------------------------------------- #
#' Get bin score for bins on each window
#'
#' The function first bins each window to equal number of bins, and calculates
#' the a summary matrix for scores of each bin (currently, mean, max and min supported)
#' A scoreMatrix object can be used to draw average profiles or heatmap of read coverage
#' or wig track-like data. \code{windows} can be a predefined region such as CpG islands,
#' gene bodies, transcripts or CDS (coding sequences) that are not necessarily equi-width. 
#' Each window will be chopped to equal number of bins based on \code{bin.num} option.
#'
#' @param target  \code{RleList}, \code{GRanges}, a BAM file or a bigWig file 
#'                object to be overlapped with ranges in \code{windows}
#' @param windows \code{GRanges} or \code{GRangesList} object that contains
#'                the windows of interest. It could be promoters, CpG islands, 
#'                exons, introns as GRanges object or GrangesList object representing
#'                exons of each transcript. Exons must be ordered by ascending rank
#'                by their position in transcript. The sizes of windows 
#'                does NOT have to be equal.
#' @param bin.num single \code{integer} value denoting how many bins there 
#'                should be for each window
#' @param bin.op bin operation that is either one of the following strings: 
#'                "max","min","mean","median","sum". The operation is applied on the 
#'                values in the bin. Defaults to "mean"
#' @param strand.aware If TRUE (default: FALSE), the strands of the windows will 
#'                     be taken into account in the resulting \code{scoreMatrix}. 
#'                     If the strand of a window is -, the values of the bins 
#'                     for that window will be reversed
#' @param weight.col if the object is \code{GRanges} object a numeric column
#'                in meta data part can be used as weights. This is particularly
#'                useful when genomic regions have scores other than their
#'                coverage values, such as percent methylation, conservation
#'                scores, GC content, etc. 
#' @param is.noCovNA (Default:FALSE)
#'                  if TRUE,and if 'target' is a GRanges object with 'weight.col'
#'                   provided, the bases that are uncovered will be preserved as
#'                   NA in the returned object. This useful for situations where
#'                   you can not have coverage all over the genome, such as CpG
#'                   methylation values.
#' @param type   (Default:"auto")
#'               if target is a character vector of file paths, then type designates 
#'               the type of the corresponding files (bam or bigWig)
#' @param rpm boolean telling whether to normalize the coverage to per milion reads. 
#'            FALSE by default. See \code{library.size}.
#' @param unique boolean which tells the function to remove duplicated reads 
#'               based on chr, start, end and strand
#' @param extend numeric which tells the function to extend the reads to width=extend
#' @param param ScanBamParam object 
#' @param bam.paired.end boolean indicating whether given BAM file contains paired-end 
#'                       reads (default:FALSE). Paired-reads will be treated as 
#'                       fragments.             
#' @param library.size numeric indicating total number of mapped reads in a BAM file
#'                            (\code{rpm} has to be set to TRUE).
#'                            If is not given (default: NULL) then library size 
#'                            is calculated using the Rsamtools idxstatsBam function:
#'                            sum(idxstatsBam(target)$mapped).
#'                                          
#' @return returns a \code{scoreMatrix} object
#' 
#' @examples
#' data(cage)
#' data(cpgi)
#' data(promoters)
#' myMat=ScoreMatrixBin(target=cage,
#'                       windows=cpgi,bin.num=10,bin.op="mean",weight.col="tpm")
#' \donttest{
#' plot(colMeans(myMat,na.rm=TRUE),type="l")
#' }
#'   
#' myMat2=ScoreMatrixBin(target=cage,
#'                        windows=promoters,bin.num=10,bin.op="mean",
#'                        weight.col="tpm",strand.aware=TRUE)
#' \donttest{
#' plot(colMeans(myMat2,na.rm=TRUE),type="l")
#' }
#' 
#' # Compute transcript coverage of a set of exons.
#' library(GenomicRanges)
#' bed.file = system.file("extdata/chr21.refseq.hg19.bed", 
#'                        package = "genomation")
#' gene.parts = readTranscriptFeatures(bed.file)
#' transcripts = split(gene.parts$exons, gene.parts$exons$name)
#' transcripts = transcripts[]
#' myMat3 = ScoreMatrixBin(target=cage, windows=transcripts[1:250], 
#'                     bin.num=10)
#' myMat3                     
#' @seealso \code{\link{ScoreMatrix}}
#' @docType methods
#' @useDynLib genomation
#' @importFrom Rcpp sourceCpp
#' @rdname ScoreMatrixBin-methods           
#' @export
setGeneric("ScoreMatrixBin",
           function(target,windows,
                    bin.num=10,bin.op="mean",
                    strand.aware=FALSE,
                    weight.col=NULL,is.noCovNA=FALSE,
                    type='auto',
                    rpm=FALSE,
                    unique=FALSE,
                    extend=0,
                    param=NULL,
                    bam.paired.end=FALSE,
                    library.size=NULL
           ) 
           standardGeneric("ScoreMatrixBin") )

# ---------------------------------------------------------------------------- #
#' @aliases ScoreMatrixBin,RleList,GRanges-method
#' @rdname ScoreMatrixBin-methods
#' @usage \\S4method{ScoreMatrixBin}{RleList,GRanges}(target, windows, bin.num, bin.op,
#'                                                    strand.aware)
setMethod("ScoreMatrixBin",signature("RleList","GRanges"),
          function(target, windows, bin.num, bin.op, strand.aware){

            mcols(windows)$X_rank <- 1:length(windows)
            
            # check if windows have width > 1
            if( any(width(windows)==1) ){
              stop("provide 'windows' with widths greater than 1")
            } 
            
            # removes windows that fall of the chromosomes - window id is in values(windows)$X_rank 
            windows.len <- length(windows)
            windows <- constrainRanges(target, windows)
            if(length(windows)!=windows.len){
              warning(paste0(windows.len-length(windows),
                             " windows fall off the target"))
            }
            
            # checks whether some windows are shorter than the wanted window size
            wi <- IRanges::width(windows) < bin.num
            if(any(wi)){
              windows <- windows[!wi]
              if(length(windows) == 0)
                stop('all supplied windows have width < number of bins')
              warning('supplied GRanges object contains ranges of width < number of bins')
            }
            

            # fetches the windows and the scores
            chrs <- sort(intersect(names(target), as.character(unique(seqnames(windows)))))
            myViews <- Views(target[chrs],as(windows,"IntegerRangesList")[chrs]); # get subsets of coverage

 
            mat <- lapply(myViews,function(x) as.list((viewApply(x,as.vector,simplify=FALSE))))
            mat <- do.call("c",mat)
            
            functs = c("min",'mean','max','median','sum')
            if(!bin.op %in% functs)
              stop(paste(c('Supported binning functions are', functs,'\n')))
            if(bin.op =="mean"){
              mat_res <- listSliceMean(mat, bin.num)
            }else if(bin.op =="median"){
              mat_res <- listSliceMedian(mat, bin.num)
            }else if(bin.op =="sum"){
              mat_res <- listSliceSum(mat, bin.num)
            }else if(bin.op =="max"){
              mat_res <- listSliceMax(mat, bin.num)
            }else if(bin.op =="min"){
              mat_res <- listSliceMin(mat, bin.num)
            }
            
            # copied from scoreMatrix()
            # get the ranks of windows, when things are reorganized by as(...,"IntegerRangesList")
            r.list <- split(mcols(windows)[,"X_rank"], as.vector(seqnames(windows))  )
            r.list <- r.list[order(names(r.list))]
            ranks <- do.call("c",r.list)    
            rownames(mat_res) <- ranks
             
            # if strand aware is TRUE, we need to flip the windows on the minus strand
                  if(strand.aware == TRUE){
                    orig.rows=windows[strand(windows) == '-',]$X_rank
                    mat_res[rownames(mat_res) %in% orig.rows,] = mat_res[rownames(mat_res) %in% 
                                                                 orig.rows, ncol(mat_res):1]                                                                                                             
                  }
            # reorder matrix
            mat_res = mat_res[order(as.numeric(rownames(mat_res))),,drop=FALSE]

       new("ScoreMatrix", mat_res)
          })


# ---------------------------------------------------------------------------- #
#' @aliases  ScoreMatrixBin,GRanges,GRanges-method
#' @rdname ScoreMatrixBin-methods
#' @usage \\S4method{ScoreMatrixBin}{GRanges,GRanges}(target,windows,
#'                                                    bin.num,bin.op,
#'                                                    strand.aware,weight.col,
#'                                                    is.noCovNA)
setMethod("ScoreMatrixBin",signature("GRanges","GRanges"),
          function(target,windows,bin.num,bin.op,
                   strand.aware,weight.col,is.noCovNA){
            
            #make coverage vector  from target
            
            if(is.null(weight.col)){
              target.rle=coverage(target)
            }else{
              if(! weight.col %in% names(mcols(target)) ){
                stop("provided column 'weight.col' does not exist in target\n")
              }
              if(is.noCovNA)
              {  
                # adding 1 to figure out NA columns later
                target.rle=coverage(target,weight=(mcols(target)[weight.col][,1]+1) )
                
                # figure out which ones are real 0 score
                # which ones has no coverage
                # put NAs for no coverage bases
                runValue(target.rle)=runValue(target.rle)-1
                runValue(target.rle)[runValue(target.rle)<0]=NA
                
              }else{
                # get coverage with weights
                target.rle=coverage(target,weight= weight.col ) 
              }
            }
            
            # call scoreMatrix function
            ScoreMatrixBin(target.rle,windows,bin.num,
                           bin.op,strand.aware)
            
          })

# ---------------------------------------------------------------------------- #
#' @aliases ScoreMatrixBin,character,GRanges-method
#' @rdname ScoreMatrixBin-methods
#' @usage \\S4method{ScoreMatrixBin}{character,GRanges}(target, windows, bin.num=10,
#'                                                      bin.op='mean',strand.aware, 
#'                                                      is.noCovNA=FALSE, type='auto',
#'                                                      rpm, unique, extend, param,
#'                                                      bam.paired.end=FALSE, 
#'                                                      library.size=NULL)
setMethod("ScoreMatrixBin",signature("character","GRanges"),
          function(target, windows, bin.num=10, 
                   bin.op='mean', strand.aware, 
                   is.noCovNA=FALSE,
                   type='auto', rpm, unique, extend, param,
                   bam.paired.end=FALSE, library.size=NULL){
            
            if(!file.exists(target)){
              stop("Indicated 'target' file does not exist\n")
            }
            
            type = target.type(target, type)
            
            if( type=="bigWig" & rpm==TRUE)
              warning("rpm=TRUE is not supported for type='bigWig'")

            if(type == 'bam'){
              covs = readBam(target, windows, rpm=rpm, unique=unique, 
                             extend=extend, param=param,
                             paired.end=bam.paired.end, library.size=library.size)
              return(ScoreMatrixBin(covs,
                                    windows,
                                    bin.num=bin.num,
                                    bin.op=bin.op,
                                    strand.aware=strand.aware))
            }
            
            if(type == 'bigWig'){
              if(is.noCovNA==FALSE){
                covs = readBigWig(target=target, windows=windows)
                # get coverage vectors
                return(ScoreMatrixBin(covs,
                               windows,
                               bin.num=bin.num,
                               bin.op=bin.op,
                               strand.aware=strand.aware))
                
              }else{
                if(is.null(windows)){
                  target.gr = import(target)
                }else{
                  target.gr = import(target, which=windows)
                }
                if(length(target.gr) == 0)
                  stop('There are no ranges selected')
                
                # bigwig contains only one column for score
                weight.col=names(mcols(target.gr)) # name of a score vector
                
                return(ScoreMatrixBin(target.gr,
                               windows,
                               bin.num=bin.num,
                               bin.op=bin.op,
                               strand.aware=strand.aware,
                               weight.col=weight.col,
                               is.noCovNA=is.noCovNA))
              }
            }
          })

# ---------------------------------------------------------------------------- #
#' @aliases ScoreMatrixBin,RleList,GRangesList-method
#' @rdname ScoreMatrixBin-methods
#' @usage  \\S4method{ScoreMatrixBin}{RleList,GRangesList}(target,windows,
#'                                                         bin.num, bin.op,
#'                                                         strand.aware)
setMethod("ScoreMatrixBin",signature("RleList","GRangesList"),
          function(target, windows, bin.num, bin.op, strand.aware){
            
            seqinfo(target) <- merge(seqinfo(target), seqinfo(windows))
            if (!isTRUEorFALSE(strand.aware))
              stop("'ignore.strand' must be TRUE or FALSE")
            
            if( length(names(windows)) != length(unique(names(windows))) )
              stop("Windows don't have unique names")
            
            #when the windows are unnamed, give the names
            if(is.null(names(windows)))
               names(windows) <- seq(1:length(windows))
            
            if(any(sum(width(windows))< bin.num))
              stop('Please remove transcripts that are shorter than bin.num')
            
            # Remove windows that fall of the chromosomes
            unlisted = unlist(windows)
            unlisted$'.trname' = rep(names(windows), elementNROWS(windows))
            if(is.unsorted(unlisted))
              unlisted = sort(unlisted)
            ex = genomation:::constrainRanges(target, 
                                              unlisted)
            # extracts the coverage vectors for individual exons and
            # concatenates them together
            ex_cvg = as(target[ex],'NumericList')
            dex.cvg = data.table(cvg = as.list(ex_cvg), id=ex$'.trname')
            dex.cvg = dex.cvg[,list(cvg=list(unlist(cvg, use.names=FALSE))),by=id]
            
            # reorders the coverage vectors to correspond to the original GRList
            dex.cvg = dex.cvg[match(unique(ex$'.trname'),dex.cvg$id),]
            
            if (strand.aware){
              dex.cvg$strand = unlist(runValue(strand(windows))[dex.cvg$id])
              dex.cvg[dex.cvg$strand == '-',cvg := list(list(rev(unlist(cvg)))), by=list(id)]
            }
            
            # constructs the bins for each transcript
            seqlen = dex.cvg[,length(unlist(cvg, use.names=TRUE)), by=id]$V1
            bins = IRangesList(lapply(seqlen,
                                       function(x)
                                         IRanges(breakInChunks(x, 
                                                               nchunk=bin.num))))
            names(bins) = dex.cvg$id
            rle = RleList(dex.cvg$cvg)
            names(rle) = dex.cvg$id
            
            # bins the signal
            my.vList = RleViewsList(
              lapply(names(rle),
                     function(seqname)
                       Views(rle[[seqname]], bins[[seqname]])))
            # Copied from genomation:::summarizeViewsRle
            # Calculate min/mean/max/median in each bin
            functs = c("min",'mean','max','median')
            if(!bin.op %in% functs)
              stop(paste(c('Supported binning functions are', functs,'\n')))
            if(bin.op=="min")
              sum.bins=unlist(viewMins(my.vList,na.rm=TRUE), use.names=FALSE)     
            if(bin.op=="max")
              sum.bins=unlist(viewMaxs(my.vList,na.rm=TRUE), use.names=FALSE)
            if(bin.op=="mean")
              sum.bins=unlist(viewMeans(my.vList,na.rm=TRUE), use.names=FALSE)
            if(bin.op=="median")
              sum.bins=unlist(viewApply(my.vList, 
                                        function(x) median(x, na.rm=TRUE), 
                                        simplify = TRUE), use.names=FALSE)
            
            mat = matrix( sum.bins, ncol=bin.num, byrow=TRUE )
            mat[is.nan(mat)] = NA
            new("ScoreMatrix",mat)
          })

# ---------------------------------------------------------------------------- #
#' @aliases  ScoreMatrixBin,GRanges,GRangesList-method
#' @rdname ScoreMatrixBin-methods
#' @usage \\S4method{ScoreMatrixBin}{GRanges,GRangesList}(target,windows,
#'                                                    bin.num,bin.op,
#'                                                    strand.aware,weight.col,
#'                                                    is.noCovNA)
setMethod("ScoreMatrixBin",signature("GRanges","GRangesList"),
          function(target,windows,bin.num,bin.op,
                   strand.aware,weight.col,is.noCovNA){
            
            #make coverage vector  from target
            if(is.null(weight.col)){
              target.rle=coverage(target)
            }else{
              if(! weight.col %in% names(mcols(target)) ){
                stop("provided column 'weight.col' does not exist in target\n")
              }
              if(is.noCovNA)
              {
                # adding 1 to figure out NA columns later
                target.rle=coverage(target,weight=(mcols(target)[weight.col][,1]+1) )

                # figure out which ones are real 0 score
                # which ones has no coverage
                # put NAs for no coverage bases
                runValue(target.rle)=runValue(target.rle)-1
                runValue(target.rle)[runValue(target.rle)<0]=NA

              }else{
                # get coverage with weights
                target.rle=coverage(target,weight=weight.col)
              }
            }
            # call scoreMatrix function
            ScoreMatrixBin(target.rle,windows,bin.num,
                           bin.op,strand.aware)
          })

# ---------------------------------------------------------------------------- #
#' @aliases ScoreMatrixBin,character,GRangesList-method
#' @rdname ScoreMatrixBin-methods
#' @usage  \\S4method{ScoreMatrixBin}{character,GRangesList}(target, windows, bin.num=10,
#'                                                      bin.op='mean',strand.aware, 
#'                                                      weight.col=NULL,
#'                                                      is.noCovNA=FALSE, type='auto',
#'                                                      rpm, unique, extend, param,
#'                                                      bam.paired.end=FALSE, 
#'                                                      library.size=NULL)
setMethod("ScoreMatrixBin",signature("character","GRangesList"),
          function(target, windows, bin.num=10, 
                   bin.op='mean', strand.aware, 
                   weight.col=NULL, is.noCovNA=FALSE, type='auto', 
                   rpm, unique, extend, param,
                   bam.paired.end=FALSE, library.size=NULL){
            if(!file.exists(target)){
              stop("Indicated 'target' file does not exist\n")
            }
            type = target.type(target, type)
            if( type=="bigWig" & rpm==TRUE)
              warning("rpm=TRUE is not supported for type='bigWig'")
            
            if(type == 'bam'){
              covs = readBam(target, unlist(windows), 
                             rpm=rpm, unique=unique, 
                             extend=extend, param=param,
                             paired.end=bam.paired.end, library.size=library.size)
              return(ScoreMatrixBin(covs,
                                  windows,
                                  bin.num=bin.num,
                                  bin.op=bin.op,
                                  strand.aware=strand.aware))
            }
            if(type == 'bigWig'){
              if(is.noCovNA==FALSE){
                covs = readBigWig(target=target, windows=unlist(windows))
                # get coverage vectors
                return(ScoreMatrixBin(covs,
                                      windows,
                                      bin.num=bin.num,
                                      bin.op=bin.op,
                                      strand.aware=strand.aware))
                
              }else{
                if(is.null(windows)){
                  target.gr = import(target)
                }else{
                  target.gr = import(target, which=unlist(windows))
                }
                if(length(target.gr) == 0)
                  stop('There are no ranges selected')
                return(ScoreMatrixBin(target.gr,
                                      windows,
                                      bin.num=bin.num,
                                      bin.op=bin.op,
                                      strand.aware=strand.aware,
                                      weight.col=weight.col,
                                      is.noCovNA=is.noCovNA))
              }
            }
          })

# ---------------------------------------------------------------------------- #
#' Arithmetic method for ScoreMatrix and ScoreMatrixList classes
#' @title Ops method for a ScoreMatrix object. It enables to use arithmetic, indicator and logic operations on ScoreMatrix objects.
#' @param e1 the \code{\link{ScoreMatrix}} object or numeric value
#' @param e2 the \code{\link{ScoreMatrix}} object or numeric value
#' @return \code{ScoreMatrix} 
#' @aliases Ops,ScoreMatrix,ScoreMatrix-method
setMethod("Ops", signature(e1="ScoreMatrix", e2="ScoreMatrix"),
          function(e1, e2) {
            e1@.Data=callGeneric(e1@.Data, e2@.Data)
            validObject(e1)
            return(e1)
          }
)

#' Extract method for a ScoreMatrix object. 
#' 
#' @aliases [,ScoreMatrix-method
#' @param x the \code{\link{ScoreMatrix}} object
#' @param i numeric value
#' @param j numeric value
#' @aliases extract,ScoreMatrix,ANY-method
setMethod("[", signature(x="ScoreMatrix", i = "ANY", j="ANY"),  
          function(x,i,j){
            if(missing(j)){
              res=new("ScoreMatrix",x@.Data[i,])
            }
            else if(missing(i)){
              res=new("ScoreMatrix",x@.Data[,j])  
            }
            else{
              res=new("ScoreMatrix",x@.Data[i,j])  
            }
            
            res
          }
)

# ---------------------------------------------------------------------------- #
#' Arithmetic methods for ScoreMatrixList
#' @title Ops method for a ScoreMatrixList object. It enables to use arithmetic, indicator and logic operations on ScoreMatrixList objects.
#' @param e1 the \code{\link{ScoreMatrixList}} object 
#' @param e2 the \code{\link{ScoreMatrixList}} object
#' @return \code{ScoreMatrixList} 
#' @aliases Ops,ScoreMatrixList,ScoreMatrixList-method
setMethod("Ops", signature(e1="ScoreMatrixList", e2="ScoreMatrixList"),
          function(e1, e2) {
            e1d = e1@.Data
            e2d = e2@.Data
            if( length(e1d) != length(e2d) ){
              stop("ScoreMatrixList objects must have the same length")
            }
            #sml = lapply(1:length(e1d), function(i) callGeneric(e1d[[i]],e2d[[i]]))
            # it doesnt work: Error in get(as.character(call[[1L]]), envir = methodEnv) : 
            # object 'FUN' not found 
            smllist = list()
            for(i in 1:length(e1d)){
              smllist[[i]] <- callGeneric(e1d[[i]], e2d[[i]])
            }
            sml = as(smllist, "ScoreMatrixList")
            validObject(sml)
            return(sml)
          }
)

# ---------------------------------------------------------------------------- #
#' Arithmetic method for ScoreMatrixList
#' @title Ops method for a ScoreMatrixList object. It enables to use arithmetic, indicator and logic operations on ScoreMatrixList objects.
#' @param e1 the \code{\link{ScoreMatrixList}} object 
#' @param e2 the numeric value
#' @return \code{ScoreMatrixList} 
#' @aliases Ops,ScoreMatrixList,numeric-method
setMethod("Ops", signature(e1="ScoreMatrixList", e2="numeric"),
          function(e1, e2) {
            e1d <- e1@.Data #list
            smllist = list()
            for(i in 1:length(e1d)){
              smllist[[i]] <- callGeneric(e1d[[i]], e2)
            }
            sml = as(smllist, "ScoreMatrixList")
            validObject(sml)
            return(sml)
          }
)

# ---------------------------------------------------------------------------- #
#' Arithmetic method for ScoreMatrixList
#' @title Ops method for a ScoreMatrixList object. It enables to use arithmetic, indicator and logic operations on ScoreMatrixList objects.
#' @param e1 the numeric value
#' @param e2 the \code{\link{ScoreMatrixList}} object
#' @return \code{ScoreMatrixList} 
#' @aliases Ops,numeric,ScoreMatrixList-method
setMethod("Ops", signature(e1="numeric", e2="ScoreMatrixList"),
          function(e1, e2) {
            e2d <- e2@.Data #list
            smllist = list()
            for(i in 1:length(e2d)){
              smllist[[i]] <- callGeneric(e1, e2d[[i]])
            }
            sml = as(smllist, "ScoreMatrixList")
            validObject(sml)
            return(sml)
          }
)

#' Extract method for a ScoreMatrixList object. 
#' 
#' @aliases [,ScoreMatrixList-method
#' @param x the \code{\link{ScoreMatrixList}} object
#' @param i numeric value
#' @aliases extract,ScoreMatrixList,ANY-method
setMethod("[",signature(x="ScoreMatrixList", i = "ANY"), 
          function(x,i){
            tmp=x@.Data[i]
            names(tmp)=names(x)[i]
            ScoreMatrixList( tmp)
          }
)

#######################
# tests and development scripts to be deleted before release
# ONLY for developer use!!!!!!!!
#######################
library(roxygen2)
roxygenise("~/Dropbox/PAPERS/R-devel/genomation",roclets = c("rd"))

load("~/Dropbox/PAPERS/R-devel/genomation/data/feature.RData")
# cage: cage TCs hg18
# feature: CpG islands hg18
# chr.len: chromosome lengths hg18
# chr.gaps: assembly gaps on hg18



library(devtools)
load_all("~/Dropbox/PAPERS/R-devel/genomation")




randomize.feature(feature)
randomize.feature(feature,chrom.sizes=chr.len)
randomize.feature(feature,chrom.sizes=chr.len,exclude=chr.gaps)


trace("randomize.feature", browser, exit=browser, signature = c("GRanges"))


trace("getRandomEnrichment", browser, exit=browser, signature = c("GRanges","GRanges"))
getRandomEnrichment(target=cage,query=feature,randomizations=10)
system.time(getRandomEnrichment(target=cage,query=feature,randomizations=10)) # 11.2 seconds for my macbook

x=modCoverage(cage,col.name="score",multiply=1000,add=1)
is(x,"RleList") # TRUE
is(x,"modRleList") # TRUE 


# make equi-width windows for scoreMatrix function
wins=feature
end(wins)=start(wins)+500

# work/test on scoreMatrix function
mat=scoreMatrix(target=x,windows=wins);class(mat);dim(mat)
head(mat)
#trace("scoreMatrix", browser, exit=browser, signature = c("RleList","GRanges"))

mat=scoreMatrixBin(target=x,windows=feature,bin.num=20);class(mat);dim(mat)

# 
library(GenomicRanges)


# GET CAGE DATA
cage=read.table("~/Dropbox/annotation_data/wgEncodeRikenCageNhekNucleusPapTssHmmV3.bedRnaElements.txt",
          header=FALSE,sep="\t",stringsAsFactors=FALSE)
cage=cage[cage[,1] %in% c("chr22","chr21"),]
cage=GRanges(seqnames=cage[,1],ranges=IRanges(cage[,2],cage[,3]),
                  strand=cage[,6],tpm=cage[,7] )

# GET REFSEQ GENES and produce GRanges
ref=read.table("~/Dropbox/annotation_data/hg19.refseq.bed",
           header=FALSE,sep="\t",stringsAsFactors=FALSE)
ref=ref[ref[,1] %in% c("chr22","chr21"),]

# get promoters
prom=ref
prom[prom[,6]=="+",3]=prom[prom[,6]=="+",2]
prom[prom[,6]=="-",2]=prom[prom[,6]=="-",3]
prom[,2]=prom[,2]-1000
prom[,3]=prom[,3]+1000
promoters=GRanges(seqnames=prom[,1],ranges=IRanges(prom[,2],prom[,3]),
                  strand=prom[,6] )
promoters=unique(promoters)

# get genes
genes=GRanges(seqnames=ref[,1],ranges=IRanges(ref[,2],ref[,3]),
                  strand=ref[,6],name=ref[,4])

# GET CPG ISLANDS and produce GRanges
cpg=read.table("~/Dropbox/annotation_data/hg19.CpGi.bed",
               header=FALSE,sep="\t",stringsAsFactors=FALSE)
cpg=cpg[cpg[,1] %in% c("chr22","chr21"),]
cpgi=GRanges(seqnames=cpg[,1],ranges=IRanges(cpg[,2],cpg[,3]) )

# GET CHROMOSOME SIZES
chr.sizes=read.table("~/Dropbox/annotation_data/hg19.chrom.sizes",
                    header=FALSE,sep="\t",stringsAsFactors=FALSE)


save(cage     ,file="data/cage.RData")
save(cpgi     ,file="data/cpgi.RData")
save(genes    ,file="data/genes.RData")
save(promoters,file="data/promoters.RData")

#' Compute an enrichment of IP over control both stored in ScoreMatrix objects
#' 
#' This is an \code{enrichmentMatrix} function for \code{ScoreMatrix} objects, 
#' that enables to normalize ChIP-seq signals with respect to IgG or input DNA control.
#' @param IP \code{\link{ScoreMatrix}} object storing an IP sample
#' @param control \code{\link{ScoreMatrix}} object storing a control sample
#' @note 
#' The function computes an enrichment of IP over control as follow:
#' Suppose both IP and control are ScoreMatrix objects that have same dimensions. 
#' Then, the enrichment is calculated usign a formula: log2((IP + 1) / (control + 1)).
#' @return \code{ScoreMatrix} object
#' @usage  \\S4method{enrichmentMatrix}{ScoreMatrix,ScoreMatrix}(IP, control)
#' @aliases enrichmentMatrix,ScoreMatrix,ScoreMatrix-method
#' @rdname enrichmentMatrix-ScoreMatrix-method
#' @seealso \code{\link{ScoreMatrix}}
#' @docType methods
#' @examples  
#' #load IP and control BAM files and create ScoreMatrix objects
#' library('genomationData')
#' bam.file_IP <- system.file("extdata", 
#' "wgEncodeBroadHistoneH1hescSuz12051317AlnRep1.chr21.bam", package = "genomationData")
#' bam.file_c <- system.file("extdata", 
#' "wgEncodeBroadHistoneH1hescCtcfStdAlnRep1.chr21.bam", package = "genomationData")
#' data(promoters)
#' IP <- ScoreMatrix(target = bam.file_IP, windows = promoters, type = 'bam')
#' control <- ScoreMatrix(target = bam.file_c, windows = promoters, type = 'bam')
#' 
#' # compute an enrichment of IP over control
#' enrichmentMatrix(IP, control)
#' 
setGeneric("enrichmentMatrix",
           function(IP, control)
             standardGeneric("enrichmentMatrix") )

setMethod("enrichmentMatrix", signature("ScoreMatrix", "ScoreMatrix"),
          function(IP, control) {
            enrichment <- matrix()
            enrichment <- log2((IP + 1) / (control + 1))
            enrichment <- as(enrichment, "ScoreMatrix")
            validObject(enrichment)
            return(enrichment) 
          }
)

# ---------------------------------------------------------------------------- #
#' Compute an enrichment of IP over control both stored in ScoreMatrixList objects  
#' 
#' This is an \code{enrichmentMatrix} function for \code{ScoreMatrixList} objects, 
#' that enables to normalize ChIP-seq signals with respect to IgG or input DNA control.
#' 
#' @param IP \code{\link{ScoreMatrixList}} object storing IP samples
#' @param control \code{\link{ScoreMatrixList}} storing control samples
#' @note 
#' The function computes an enrichment of IP over control as follow:
#' Suppose both IP and control are ScoreMatrix objects that have same dimensions. 
#' Then, the enrichment is calculated usign a formula: log2((IP + 1) / (control + 1)).
#' @return \code{ScoreMatrixList} object
#' @usage  \\S4method{enrichmentMatrix}{ScoreMatrixList,ScoreMatrixList}(IP, control)
#' @aliases enrichmentMatrix,ScoreMatrixList,ScoreMatrixList-method
#' @rdname enrichmentMatrix-ScoreMatrixList-method
#' @seealso \code{\link{ScoreMatrixList}}
#' @docType methods
#' @examples
#' #load IP and control BAM files and create ScoreMatrix objects
#' library('genomationData')
#' data(promoters)
#' bam.file_IP_1 <- system.file("extdata", 
#' "wgEncodeSydhTfbsH1hescZnf143IggrabAlnRep1.chr21.bam", package = "genomationData")
#' IP_1 <- ScoreMatrix(target = bam.file_IP_1, windows = promoters, type = 'bam')
#'
#' bam.file_IP_2 <- system.file("extdata", 
#' "wgEncodeBroadHistoneH1hescSuz12051317AlnRep1.chr21.bam", package = "genomationData")
#' IP_2 <- ScoreMatrix(target=bam.file_IP_2, windows = promoters, type = 'bam')
#' 
#'bam.file_c <- system.file("extdata", 
#'"wgEncodeBroadHistoneH1hescCtcfStdAlnRep1.chr21.bam", package = "genomationData")
#' control <- ScoreMatrix(target = bam.file_c, windows = promoters, type = 'bam')
#' 
#' # create a ScoreMatrixList object storing IP ScoreMatrix objects
#' sml_IP <- ScoreMatrixList(list(IP1 = IP_1, IP2 = IP_2))
#' 
#' # create a ScoreMatrixList object storing control ScoreMatrix objects
#' sml_control <- ScoreMatrixList(list(c1 = control, c2 = control))
#' 
#' # compute an enrichment of IP over control
#' enrichmentMatrix(sml_IP, sml_control)
#' 
setMethod("enrichmentMatrix", signature("ScoreMatrixList","ScoreMatrixList"),
          function(IP, control) {
            
            # checks whether the arguments consist of the same number of elements 
            if(length(IP) != length(control))
              stop('IP argument and control argument should have the same length')
            
            # checks whether all corresponding elements of ScoreMatrixList objects have the same size 
            check.size <- function (i, IP, control){
              if(dim(IP[[i]])[1] != dim(control[[i]])[1] || dim(IP[[i]])[2] != dim(control[[i]])[2])
                stop( "element:", i, "\t IP argument and control argument do not have the same size")
            }
            
            n <- mclapply(1:length(IP), check.size, IP = IP, control = control)
            
            calc.enrichment<- function(i, IP, control){
              enrichment <- matrix()
              enrichment <- log2((IP[[i]] + 1) / (control[[i]] + 1))
              enrichment <- as(enrichment, "ScoreMatrix")
            }
            
            smlc <- list()
            smlc <- mclapply(1:length(IP), calc.enrichment, IP=IP, control=control)
            smlc <- as(smlc, "ScoreMatrixList")
            names(smlc) <- names(IP)
            
            validObject(smlc)
            return(smlc) 
          }
)

# -------------------------------------------------------------------------------------------------------- #
#' Compute an enrichment of IP (stored in ScoreMatrixList object) over control (stored in ScoreMatrix object)   
#' 
#' This is an \code{enrichmentMatrix} function for IP \code{ScoreMatrixList} object and control \code{ScoreMatrix} object, 
#' that enables to normalize ChIP-seq signals with respect to IgG or input DNA control.
#' 
#' @param IP \code{\link{ScoreMatrixList}} object storing IP samples
#' @param control \code{\link{ScoreMatrix}} storing control sample
#' @note 
#' The function computes an enrichment of IP over control as follow:
#' Suppose both IP and control are ScoreMatrix objects that have same dimensions. 
#' Then, the enrichment is calculated usign a formula: log2((IP + 1) / (control + 1)).
#' @return \code{ScoreMatrixList} object
#' @usage  \\S4method{enrichmentMatrix}{ScoreMatrixList,ScoreMatrix}(IP, control)
#' @aliases enrichmentMatrix,ScoreMatrixList,ScoreMatrix-method
#' @rdname enrichmentMatrix-ScoreMatrixList-ScoreMatrix-method
#' @seealso \code{\link{ScoreMatrixList}}, \code{\link{ScoreMatrix}}
#' @docType methods
#' @examples
#' #load IP and control BAM files and create ScoreMatrix objects
#' library('genomationData')
#' data(promoters)
#' bam.file_IP_1 <- system.file("extdata", 
#' "wgEncodeSydhTfbsH1hescZnf143IggrabAlnRep1.chr21.bam", package = "genomationData")
#' IP_1 <- ScoreMatrix(target = bam.file_IP_1, windows = promoters, type = 'bam')
#'
#' bam.file_IP_2 <- system.file("extdata", 
#' "wgEncodeBroadHistoneH1hescSuz12051317AlnRep1.chr21.bam", package = "genomationData")
#' IP_2 <- ScoreMatrix(target=bam.file_IP_2, windows = promoters, type = 'bam')
#' 
#' bam.file_c <- system.file("extdata", 
#' "wgEncodeBroadHistoneH1hescCtcfStdAlnRep1.chr21.bam", package = "genomationData")
#' control <- ScoreMatrix(target = bam.file_c, windows = promoters, type = 'bam')
#' 
#' # create a ScoreMatrixList object storing IP ScoreMatrix objects
#' sml_IP <- ScoreMatrixList(list(IP1 = IP_1, IP2 = IP_2))
#'  
#' # compute an enrichment of IP over control
#' enrichmentMatrix(sml_IP, control)
#' 
setMethod("enrichmentMatrix", signature("ScoreMatrixList","ScoreMatrix"),
          function(IP, control) {
            
            # checks whether each element of IP ScoreMatrixList has the same size like control ScoreMatrix 
            check.size <- function (i, IP, control){
              if(dim(IP[[i]])[1] != dim(control)[1] || dim(IP[[i]])[2] != dim(control)[2])
                stop( "element:",i,"\t IP argument and control argument do not have the same size")
            }
            
            n <- mclapply(1:length(IP), check.size, IP = IP, control = control)
            
            calc.enrichment<- function(i, IP, control){
              enrichment <- matrix()
              enrichment <- log2((IP[[i]] + 1) / (control + 1))
              enrichment <- as(enrichment, "ScoreMatrix")
            }
            
            smlc <- list()
            smlc <- mclapply(1:length(IP), calc.enrichment, IP = IP, control = control)
            smlc <- as(smlc, "ScoreMatrixList")
            names(smlc) <- names(IP)
            
            validObject(smlc)
            return(smlc) 
          }
)

#######################################
# S3 functions
#######################################

# ---------------------------------------------------------------------------- #
#' c.ScoreMatrixList
#' 
#' Combine a scoreMatrix into a scoreMatrixList object - when a ScoreMatrixList is a first argument
#'
#' @param ... contais scoreMatrix and scoreMatrixList objects
#' @param recursive logical
#' @param use.names logical
#' @return returns a \code{scoreMatrixList} object 
c.ScoreMatrixList<-function(..., recursive = FALSE, use.names = TRUE) {
  y <- list(...)
  n <- length(y)
  y3 <- list()
  Ln <- vector()
  
  for (i in 1:n){
    #combine the scoreMatrixList object
    if (is(y[[i]], "ScoreMatrixList")){
      y2 <- as(y[[i]], "list")
      y3 <- append(y3, y2)
      #combine its label name
      Ln <- append(Ln, names(y[[i]]))
    #combine the scoreMatrix object
    }else if(is(y[[i]], "ScoreMatrix")){
      y3 <- append(y3, list(y[[i]]))
      #combine its label name
      if (identical(c(""), names(y[i])) || is.null(names(y[i]))) {
        Ln <- append(Ln, c(""))
      }else{
        Ln <- append(Ln, names(y[i]))
      } 
      
    }
  }  
  #create the ScoreMatrixList
  SML <- as(y3, "ScoreMatrixList")
  names(SML) <- Ln
  
  validObject(SML)
  return(SML)
}

# ---------------------------------------------------------------------------- #
#' c.ScoreMatrix
#' 
#' Combine a scoreMatrix into a scoreMatrixList object - when a ScoreMatrix is a first argument
#'
#' @param ... contais scoreMatrix and scoreMatrixList objects
#' @param recursive logical
#' @param use.names logical
#' @return returns a \code{scoreMatrixList} object 

c.ScoreMatrix<-function(..., recursive = FALSE, use.names = TRUE) {
  y <- list(...)
  n <- length(y)
  y3 <- list()
  Ln <- vector()
  
  for (i in 1:n){
    #combine the scoreMatrixList object
    if (is(y[[i]], "ScoreMatrixList")){
      y2 <- as(y[[i]], "list")
      y3 <- append(y3, y2)
      #combine its label name
      Ln <- append(Ln, names(y[[i]]))
   #combine the scoreMatrix object
    }else if(is(y[[i]], "ScoreMatrix")){
      y3 <- append(y3, list(y[[i]]))
      #combine its label name
      if (identical(c(""), names(y[i])) || is.null(names(y[i]))) {
        Ln <- append(Ln, c(""))
      }else{
        Ln <- append(Ln, names(y[i]))
      } 
      
    }
  }  
  #create the ScoreMatrixList
  SML <- as(y3, "ScoreMatrixList")
  names(SML) <- Ln
  
  validObject(SML)
  return(SML)
}

#######################################
# S3 functions
#######################################

# It finds positions of sequence motif hits above
# a specified threshold in a list of sequences of the same length.
# Adapted from seqPattern::motifScanHits - 
# it returns vector of 0s and 1s instead of start positions
# of pattern occurrences
# Args:
# pwm: matrix, seq: character
.scan.sequence.with.pwm <- function(seq, pwm, minScore){
  
  if(is.null(minScore)){
    # calculate scores themselves
    
    # suppressWarnings is done because matchPWM function gives warnings 
    # if you there is "N" nucleotide within sequence (can be only [A,T,C,G])
    # It assigns weight 0 to them.
    pwm.match <- suppressWarnings(matchPWM(pwm = pwm, subject = seq, with.score=TRUE))
    return(mcols(pwm.match)$score)
    
  }else{
    # calculate 0 and 1s
    
    # if minScore is a character then convert it to a number 
    # (e.g. 80% to 0.8)
    nc <- nchar(minScore)
    if (substr(minScore, nc, nc) == "%"){
      perc.threshold <- substr(minScore, 1L, nc-1L)
      min.score <- minScore(pwm)
      max.score <- maxScore(pwm)
      score.threshold = min.score + as.double(perc.threshold)/100 *
        (max.score-min.score)
    }else{
      score.threshold <- minScore
    }
  
    # make all PWM scores positive to avoid matching the Ns which are assigned score 0
    pwm.corr <- pwm + abs(min(pwm))
    score.threshold <- score.threshold + ncol(pwm) * abs(min(pwm))
  
    # suppressWarnings is done because matchPWM function gives warnings 
    # if you there is "N" nucleotide within sequence (can be only [A,T,C,G]).
    # It assigns weight 0 to them.
    pwm.match <- suppressWarnings(matchPWM(pwm = pwm.corr, subject = seq, min.score = score.threshold))
  
    # if PWM does not match any position in the sequence then return only 0s
    if(length(pwm.match)==0){
      return(rep(0,length(seq)))
    }
  
    # gets the indeces where hits occur
    indeces=unlist(mapply(function(x,y) x:y, start(pwm.match),end(pwm.match), SIMPLIFY=FALSE))
  
    # create output vector
    res=rep(0,length(seq))
  
    # count how many times a PWM occurs in a given index
    ind.rle=rle(indeces)
  
    # get the indeces and assign the number of occurrence for that index
    res[ind.rle$values]=ind.rle$lengths
  
    return(res)
  }
}

# Find positions of specified PWM and calculate score matrix
# Args:
# pwm: matrix, windows: character
.patternPWM_windowsDNAStringSet2matrix = function(pattern, windows, 
                                                  min.score, asPercentage,
                                                  cores){
  if(is.null(min.score)){
    # for each window calculate vector of scores themselves
    
    # suppressWarnings is done because motifScanScores function gives warnings 
    # if you there is "N" nucleotide within sequence (can be only [A,T,C,G])
    # It assigns weight 0 to them.
    # motifScanScores is fast
    if(cores==1){
      mat <- suppressWarnings(motifScanScores(regionsSeq = windows,
                                             motifPWM = pattern,
                                             asPercentage = asPercentage))
    }else{
      mat <- mclapply(windows, 
                      motifScanScores,
                      motifPWM = pattern,
                      asPercentage = asPercentage,
                      mc.cores=cores)
      mat <- matrix(unlist(mat), ncol = length(mat[[1]]), byrow = TRUE)
    }

  # min.score!=NULL
  }else{
    pwm.match <- mclapply(windows, 
                          .scan.sequence.with.pwm,
                          pwm=pattern,
                          minScore=min.score,
                          mc.cores=cores
                          )
    # convert list to matrix
    mat <- matrix(unlist(pwm.match), ncol = length(pwm.match[[1]]), byrow = TRUE)
  }
  return(mat)
}

.getPatternOccurrenceList.wrapper = function(i, windows, pattern){
  # [i] instead of [[i]]
  pat <- getPatternOccurrenceList(windows[i], pattern)
  pat <- pat[[1]]
  pat$sequence <- rep(i, nrow(pat))
  pat
}

# Find positions of specified sequence patterns 
# and calculate score matrices (list of matrices)
# Args:
# pattern: character, windows: DNAStringSet
.patterncharacter_windowsDNAStringSet2matrix = function(pattern, windows, cores){

  # getPatternOccurrenceList has nrCores and useMulticore args for paralelization,
  # it uses mclapply for every pattern
  # and before mclappling there is only one such line:
  # regionsSeq <- DNAStringSet(gsub("N", "+", regionsSeq)), 
  # where regionsSeq=windows 
  if(cores==1){
    pat = getPatternOccurrenceList(windows, pattern)
    pat = pat[[1]]
  }else{
    pat = mclapply(1:length(windows),
                   .getPatternOccurrenceList.wrapper,
                   windows=windows,
                   pattern=pattern,
                   mc.cores=cores)
    
    # data.table:rbindlist
    # Same as do.call("rbind", l) on data.frames, but much faster
    pat = rbindlist(pat)
  }
  
  positions_of_01 = function(pat, pattern, windows){
    # gets the indeces where hits occur
    indeces=unlist(mapply(function(x,y) x:y, 
                          pat$position, pat$position+width(pattern[1])-1, SIMPLIFY=FALSE))
  
    # create output vector
    res=rep(0,length(windows[[1]]))
  
    # count how many times a PWM occurs in a given index
    ind.rle=rle(indeces)
  
    # get the indeces and assign the number of occurrence for that index
    res[ind.rle$values]=ind.rle$lengths
  
    return(res)
  }

  # number windows from result of getPatternOccurrenceList
  pat.windows = unique(pat$sequence)
  
  # for each window calculate vector of 1 and 0s
  pwm.match <- lapply(1:length(windows), function(i){
    if(i %in% pat.windows){
      positions_of_01(pat[pat$sequence==i,], pattern, windows)
    }else{
      # if pattern did not match to window
      rep(0,length(windows[[1]]))
    }
  })
  # convert list to matrix
  mat <- matrix(unlist(pwm.match), ncol = length(pwm.match[[1]]), byrow = TRUE)
}


#######################################
# S4 functions
#######################################

# ---------------------------------------------------------------------------- #
#' Get scores that correspond to k-mer or PWM matrix occurrence for bases in each window
#'
#' The function produces a base-pair resolution matrix or matrices of scores that correspond to 
#' k-mer or PWM matrix occurrence over predefined windows that  have equal width.
#' It finds either positions of pattern hits above a specified threshold and 
#' creates score matrix filled with 1 (presence of pattern) and 0 (its absence) or
#' matrix with scores themselves.
#' If pattern is a character of length 1 or PWM matrix then the function returns 
#' a ScoreMatrix object, if character of length more than 1 or list of PWMs 
#' then ScoreMatrixList.
#'
#' @param pattern matrix (a PWM matrix), list of matrices or a character vector of length 1 or more.
#'                A matrix is a PWM matrix that needs to have one row for each nucleotide 
#'                ("A","C","G" and "T" respectively).
#'                IUPAC ambiguity codes can be used and it will match any letter 
#'                in the subject that is associated with the code.
#' @param windows \code{\link{GRanges}} object or \code{\link{DNAStringSet}} object 
#'                that have equal width of ranges or sequences.
#' @param genome \code{\link{BSgenome}} object
#' @param min.score numeric or character indicating minimum score to count a match. 
#'                  It can be given as a character string containing
#'                  a percentage of the highest possible score or a single number
#'                  (by default "80\%" or 0.8). If min.score is set to NULL
#'                  then \code{patternMatrix} returns scores themselves (default).
#' @param asPercentage boolean telling whether scores represent percentage of the maximal 
#'                     motif PWM score (default: TRUE) or raw scores (FALSE).             
#' @param cores the number of cores to use (default: 1). It is supported only on Unix-like platforms.
#'              
#' @details
#' \code{patternMatrix} is based on functions from the seqPattern package:
#' getPatternOccurrenceList function to find position of pattern that is a character vector in 
#' a list of sequences (a DNAStringSet object)
#' and adapted function motifScanHits to find pattern that is a PWM matrix
#' in sequences (a DNAStringSet object).
#' 
#' If cores > 1 is provided then for every window occurrence of pattern is counted in paralallel.
#' 
#' @examples
#' library(Biostrings)
#' 
#' # consensus sequence of the ctcf motif
#' motif = "CCGCGNGGNGGCAG"
#' # Creates 10 random DNA sequences
#' seqs = sapply(1:10, 
#'        function(x) paste(sample(c("A","T","G","C"), 180, replace=TRUE), collapse=""))
#' windows = DNAStringSet(seqs)
#' p = patternMatrix(pattern=motif, windows=windows, min.score=0.8)
#' p
#' 
#' @return returns a \code{scoreMatrix} object or a \code{scoreMatrixList} object
#' 
#' @seealso \code{\link{ScoreMatrix}}, \code{\link{ScoreMatrixList}}
#' @docType methods
#' @rdname patternMatrix-methods           
#' @export
setGeneric(
  name="patternMatrix",
  def=function(pattern, windows, 
               genome=NULL, min.score = 0.8, 
               asPercentage=FALSE, cores=1){
    standardGeneric("patternMatrix")
  }
)

#' @aliases patternMatrix,character,DNAStringSet-method
#' @rdname patternMatrix-methods
#' @usage  \\S4method{patternMatrix}{character,DNAStringSet}(pattern, windows,
#'                                                           asPercentage, cores)
setMethod("patternMatrix",
          signature(pattern = "character", windows = "DNAStringSet"),
          function(pattern, windows, cores = 1){
            
            if(!(length(unique(width(windows))) == 1)){
              stop("All sequences in windows must have the same 
                   length!")
            }
            if(length(pattern)==1 & cores>1){
              warning("There is only one pattern provided. Setting number of cores to 1..")
              cores=1
            }

           if(length(nchar(pattern))==1){
                # if there is only one pattern then create ScoreMatrix
                mat <- .patterncharacter_windowsDNAStringSet2matrix(pattern, windows, cores=cores)
                return(new("ScoreMatrix",mat))
              
            }else{
                lmat <- lapply(1:length(pattern), 
                             function(i) .patterncharacter_windowsDNAStringSet2matrix(pattern[i],
                                                                                      windows,
                                                                                      cores=cores))
                return(new("ScoreMatrixList", lmat))
            }
})

#' @aliases patternMatrix,character,GRanges,BSgenome-method
#' @rdname patternMatrix-methods
#' @usage  \\S4method{patternMatrix}{character,GRanges,BSgenome}(pattern, windows, genome,
#'                                                               cores)
setMethod("patternMatrix",
          signature(pattern = "character", windows = "GRanges", genome="BSgenome"),
          function(pattern, windows, genome, cores=1){
            
            if(!(length(unique(width(windows))) == 1)){
              stop("All sequences in the input DNAStringSet must have the same 
                   length!")
            }
            
            #Error in .starfreeStrand(strand(names)) : 
            #cannot mix "*" with other strand values
            strand(windows) <- c("*")
            
            seqinfo(windows) <- seqinfo(genome)[seqnames(seqinfo(windows))]
            
            windows = getSeq(genome, windows)
            
            # call patternMatrix function
            # pattern: character, windows: DNAStringSet
            patternMatrix(pattern=pattern, windows=windows,
                          cores=cores)
})

#' @aliases patternMatrix,matrix,DNAStringSet-method
#' @rdname patternMatrix-methods
#' @usage  \\S4method{patternMatrix}{matrix,DNAStringSet}(pattern, windows,
#'                                                        min.score, asPercentage, 
#'                                                        cores)
setMethod("patternMatrix",
          signature(pattern = "matrix", windows = "DNAStringSet"),
          function(pattern, windows, min.score = 0.8, asPercentage=FALSE, cores=1){
                        
            if(!(length(unique(width(windows))) == 1)){
              stop("All sequences in the input DNAStringSet must have the same 
                   length!")
            }
            
            mat <- .patternPWM_windowsDNAStringSet2matrix(pattern, windows, 
                                                          min.score, asPercentage,
                                                          cores=cores)
            return(new("ScoreMatrix",mat))
})

#' @aliases patternMatrix,matrix,GRanges,BSgenome-method
#' @rdname patternMatrix-methods
#' @usage  \\S4method{patternMatrix}{matrix,GRanges,BSgenome}(pattern, windows, genome,
#'                                                            min.score, asPercentage, 
#'                                                            cores)
setMethod("patternMatrix",
          signature(pattern = "matrix", windows = "GRanges", genome="BSgenome"),
          function(pattern, windows, genome, 
                   min.score = 0.8, asPercentage=FALSE, cores=1){
            
            if(!(length(unique(width(windows))) == 1)){
              stop("All sequences in the input DNAStringSet must have the same 
                   length!")
            }
            
            strand(windows) <- c("*")
            
            seqinfo(windows) <- seqinfo(genome)[seqnames(seqinfo(windows))]
            # after it can show info 
            #....
            #Note that only ranges located on a non-circular sequence whose length
            #is not NA can be considered out-of-bound (use seqlengths() and
            #isCircular() to get the lengths and circularity flags of the underlying
            #sequences). You can use trim() to trim these ranges. See
            #?`trim,GenomicRanges-method` for more information.
            #windows = trim(windows)
            
            windows = getSeq(genome, windows)
            
            # call patternMatrix function
            # pattern: matrix, windows: DNAStringSet
            patternMatrix(pattern=pattern, windows=windows, 
                          min.score=min.score, asPercentage=asPercentage,
                          cores=1)
})   

#' @aliases patternMatrix,list,DNAStringSet-method
#' @rdname patternMatrix-methods
#' @usage  \\S4method{patternMatrix}{list,DNAStringSet}(pattern, windows,
#'                                                      min.score, asPercentage, 
#'                                                      cores)
setMethod("patternMatrix",
          signature(pattern = "list", windows = "DNAStringSet"),
          function(pattern, windows, min.score = 0.8, 
                   asPercentage=FALSE, cores=1){
            
            if(inherits(pattern[[1]], "matrix")){
              lmat <- lapply(1:length(pattern), 
                            function(i) patternMatrix(pattern=pattern[[i]], 
                                                      windows=windows, 
                                                      min.score=min.score, 
                                                      asPercentage=asPercentage,
                                                      cores=cores))
              return(new("ScoreMatrixList",lmat))
            }
            
            if(is.character(pattern[[1]])){
              pattern = unlist(pattern)
              patternMatrix(pattern, windows, min.score)
            }
})

#' @aliases patternMatrix,list,GRanges,BSgenome-method
#' @rdname patternMatrix-methods
#' @usage  \\S4method{patternMatrix}{list,GRanges,BSgenome}(pattern, windows, genome, 
#'                                                          min.score, asPercentage, 
#'                                                          cores)
setMethod("patternMatrix",
          signature(pattern = "list", windows = "GRanges", genome="BSgenome"),
          function(pattern, windows, genome, min.score = 0.8, 
                   asPercentage=FALSE, cores=1){
                   
            if(inherits(pattern[[1]],"matrix")){
              lmat <- lapply(1:length(pattern), 
                            function(i) patternMatrix(pattern=pattern[i], 
                                                      windows=windows, 
                                                      genome=genome,
                                                      min.score=min.score, 
                                                      asPercentage=asPercentage,
                                                      cores=cores))
              return(new("ScoreMatrixList",lmat))
            }
            
            if(is.character(pattern[[1]])){
              pattern = unlist(pattern)
              patternMatrix(pattern, windows, genome=genome, 
                            min.score=min.score, asPercentage=asPercentage)
            }
})

#######################
#         S3          #
#######################

.plotXYlab = function(lab, pos){
  
  if(!pos %in% c('x','y'))
    stop('pos can only be x or y')
  
  if(pos == 'x'){
    .plotLab(lab, srt=0)
  }
  
  if(pos == 'y'){
    .plotLab(lab, srt=90)
  }
}

.plotLab = function(lab, srt, xlim, ylim, x, y){
  plot.new()
  plot.window(xlim=c(1,5), ylim=c(1,5))
  text(3, 3, labels=lab, cex=2, srt=srt)
}

# ---------------------------------------------------------------------------- #
#' Make ScoreMatrixList from multiple targets
#' 
#' The function constructs a list of \code{ScoreMatrix} objects in the form
#' of \code{ScoreMatrixList} object. This object can be visualized using 
#' \code{multiHeatMatrix}, \code{heatMeta} or \code{plotMeta}
#'
#' @param targets can be a list of \code{scoreMatrix} objects, that are coerced 
#'        to the \code{ScoreMatrixList}, a list of \code{RleList} objects, or a 
#'        character vector specifying the locations of mulitple bam files  or
#'        bigWig files that 
#'        are used to construct the \code{scoreMatrixList}. If it is either a 
#'        RleList object or a character vector of files, it is obligatory to 
#'        give a windows argument.
#' @param windows \code{GenomicRanges} containing viewpoints for the scoreMatrix 
#'        or ScoreMatrixList functions
#' @param bin.num an integer telling the number of bins to bin the score matrix
#' @param bin.op an name of the function that will be used for smoothing windows of ranges
#' @param strand.aware a boolean telling the function whether to reverse the 
#'        coverage of ranges that come from - strand (e.g. when plotting 
#'        enrichment around transcription start sites)
#' @param weight.col if the object is \code{GRanges} object a numeric column
#'                 in meta data part can be used as weights. This is particularly
#'                useful when genomic regions have scores other than their
#'                coverage values, such as percent methylation, conservation
#'                scores, GC content, etc. 
#' @param is.noCovNA (Default:FALSE)
#'                  if TRUE,and if 'targets' is a GRanges object with 'weight.col'
#'                   provided, the bases that are uncovered will be preserved as
#'                   NA in the returned object. This useful for situations where
#'                   you can not have coverage all over the genome, such as CpG
#'                    methylation values.
#'                    
#' @param type (Default:"auto")
#'             if \code{targets} is a character vector of file paths, then type 
#'             designates the type of the corresponding files (bam or bigWig)
#' @param rpm boolean telling whether to normalize the coverage to per milion reads. 
#'            FALSE by default. See \code{library.size}.
#' @param unique boolean which tells the function to remove duplicated reads 
#'                       based on chr, start, end and strand
#' @param extend numeric which tells the function to extend the features
#'               ( i.e aligned reads) to total
#'               length ofwidth+extend
#' @param param ScanBamParam object  
#' @param library.size a numeric vector of the same length as \code{targets} 
#'                     indicating total number of mapped reads in BAM files (\code{targets}).
#'                     If is not given (default: NULL) then library sizes for every target
#'                     is calculated using the Rsamtools idxstatsBam function:
#'                     sum(idxstatsBam(target)$mapped).
#'                     \code{rpm} argument has to be set to TRUE.
#' @param cores the number of cores to use (default: 1)
#'
#' @return returns a \code{ScoreMatrixList} object
#' 
#' @examples
#' 
#' # visualize the distribution of cage clusters and cpg islands around promoters
#' library(GenomicRanges)
#' data(cage)
#' data(cpgi)
#' data(promoters)
#'  
#' cage$tpm = NULL
#' targets = GRangesList(cage=cage, cpgi=cpgi)
#' sml = ScoreMatrixList(targets, promoters, bin.num=10, strand.aware=TRUE)
#' sml
#' \donttest{
#' multiHeatMatrix(sml)
#' }
#' @export
#' @docType methods
#' @rdname ScoreMatrixList-methods
ScoreMatrixList = function(targets, windows=NULL, bin.num=NULL, 
                           bin.op='mean', strand.aware=FALSE, weight.col=NULL, 
                           is.noCovNA=FALSE, type='auto', rpm=FALSE, unique=FALSE, 
                           extend=0, param=NULL, library.size=NULL, cores=1){
  len = length(targets)
  if(len == 0L)
    stop('targets argument is empty')
  
  # this checks whether we can work with the corresponding targets object class set
  list.ind = grepl('list', class(targets)) | grepl('List', class(targets))
  if(len > 1L & !list.ind){
    if(all(is.character(targets)) && is.null(type) && all(file.exists(targets)))
      stop('targets argument is neither a list like object (e.g. GRangesList),
           nor a set of files') 
  }
  
  
  # ----------------------------------------------------------------- #
  # checks whether the list argument contains only scoreMatrix objects
  if(all(unlist(lapply(targets, class)) == 'ScoreMatrix'))
    return(new("ScoreMatrixList",targets))
  
  # ----------------------------------------------------------------- #
  if(is.null(windows))
    stop("windows of class GRanges must be defined")
  
  # Given a list of RleList objects and a granges object, returns the scoreMatrix list Object
  if(list.ind & !all(unlist(lapply(targets, class)) %in% c('SimpleRleList', 'RleList','GRanges'))){
    stop('targets should be one of the following: an RleList, list of Rle, 
         GRangesList, a list of GRanges objects')
  }  
  
  if(!list.ind && all(file.exists(targets)) && is.null(type))
    stop('When providing a file, it is necessary to give the type of the file')
  
  # gets the names for the resulting list
  if(all(is.character(targets))){
    names = basename(targets)
  }else{
    names = names(targets)
  }
  
  sml = list()
  calc.ScoreMatrices <- function(i, targets=targets, windows=windows,
                                 strand.aware=strand.aware, weight.col=weight.col,
                                 is.noCovNA=is.noCovNA, type=type, rpm=rpm,
                                 unique=unique, extend=extend, param=param,
                                 bin.num,library.size=library.size){
    
    if(is.null(bin.num) && all(width(windows) == unique(width(windows)))){
      
      ScoreMatrix(targets[[i]], windows=windows, 
                  strand.aware=strand.aware,
                  weight.col=weight.col, 
                  is.noCovNA=is.noCovNA,
                  type=type,
                  rpm=rpm,
                  unique=unique,
                  extend=extend,
                  param=param,
                  library.size=library.size[i])
    }else{
      if(is.null(bin.num))
        bin.num = 10
      
      ScoreMatrixBin(targets[[i]], windows=windows, 
                     bin.num=bin.num, 
                     bin.op=bin.op,
                     strand.aware=strand.aware,
                     weight.col=weight.col, 
                     is.noCovNA = is.noCovNA,
                     type=type,
                     rpm=rpm,
                     unique=unique,
                     extend=extend,
                     param=param,
                     library.size=library.size[i])
    }
  }    
  sml <- mclapply(1:length(targets), 
                  calc.ScoreMatrices,
                  targets=targets,
                  windows=windows,strand.aware=strand.aware,
                  weight.col=weight.col,is.noCovNA=is.noCovNA,type=type,
                  rpm=rpm,unique=unique,extend=extend,param=param,bin.num,
                  library.size=library.size,
                  mc.cores=cores)
  names(sml) = names
  return(new("ScoreMatrixList",sml))
  }

# ---------------------------------------------------------------------------- #
# Validator
.valid.ScoreMatrixList = function(l){
  errors = character()
  # checks whether all matrices are of class scoreMatrix
  if(!all(unlist(lapply(l, function(x)class(x) == 'scoreMatrix'))))
    errors = paste(errors, 
                   'All elements for ScoreMatrixList 
                   need to be of class scoreMatrix', 
                   sep='\n')
  
  # checks whether all matrices are numeric
  if(!all(unlist(lapply(l, function(x)all(is.integer(x) | is.numeric(x))))))
    errors = paste(errors, '
                   Not all matrices are of type integer or numeric', 
                   sep='\n')
  
  if(length(errors) == 0) TRUE else errors 
}


# ---------------------------------------------------------------------------- #
# show Methods
#' @rdname show-methods
#' @return Shows the number of matrices and their sizes
setMethod("show", "ScoreMatrixList",
          function(object){
            dims = lapply(object, dim)
            len = length(object)
            widths = apply(do.call(rbind, dims),2, function(x)max(nchar(x)))
            message('scoreMatrixlist of length:', len, '\n')
            for(i in 1:len){
              s=sprintf(paste('%d%s ','%',widths[1],'d %',widths[2],'d', sep=''), 
                        i, '. scoreMatrix with dims:', dims[[i]][1], dims[[i]][2])
              message(s)
            }
          }
)


# ---------------------------------------------------------------------------- #
#' Scale the ScoreMatrixList
#' 
#' Scales each ScoreMatrix in the ScoreMatrixList object, by rows and/or columns
#' 
#' @param sml a \code{ScoreMatrixList} object
#' @param columns a \code{columns} whether to scale the matrix by columns. 
#'               Set by default to FALSE
#' @param rows  a \code{rows} Whether to scale the matrix by rows. Set by default 
#'            to TRUE
#' @param scalefun a function object that takes as input a matrix and returns a 
#'        matrix.
#'  By default  the argument is set to the R scale function with center=TRUE and 
#'  scale=TRUE
#'
#' @usage scaleScoreMatrixList(sml, columns, rows, scalefun)
#' @return \code{ScoreMatrixList} object
#' @examples 
#' library(GenomicRanges)
#' data(cage)
#' data(cpgi)
#' data(promoters)
#'  
#' cage$tpm = NULL
#' targets = GRangesList(cage=cage, cpgi=cpgi)
#' sml = ScoreMatrixList(targets, promoters, bin.num=10, strand.aware=TRUE)
#' sml.scaled = scaleScoreMatrixList(sml, rows=TRUE)
#' sml.scaled
#' \donttest{
#' multiHeatMatrix(sml) 
#' }
#'
#' @docType methods
#' @rdname scaleScoreMatrixList
#' @export

setGeneric("scaleScoreMatrixList", 
           function(sml, 
                    columns=FALSE, rows=TRUE, 
                    scalefun=NULL) 
             standardGeneric("scaleScoreMatrixList") )

#' @aliases scaleScoreMatrixList,ScoreMatrixList-method
#' @rdname scaleScoreMatrixList
setMethod("scaleScoreMatrixList", signature("ScoreMatrixList"),
          function(sml, columns, rows, scalefun){
            
            
            sml = lapply(sml, function(x)
              scaleScoreMatrix(x, 
                               columns=columns, 
                               rows=rows, 
                               scalefun=scalefun))
            sml = as(sml,'ScoreMatrixList')
            return (sml)
          }
)

# ---------------------------------------------------------------------------- #
#' Get common rows from all matrices in a ScoreMatrixList object
#' 
#' Returns a intersection of rows for each matrix in a ScoreMatrixList object. 
#' This is done using the rownames of each element in the list.
#'
#' @param sml a \code{ScoreMatrixList} object
#' @param reorder if TRUE \code{ScoreMatrix} objects in the list are sorted
#'                based on their common row ids.
#'
#' @return \code{ScoreMatrixList} object
#' @examples
#' library(GenomicRanges)
#' target = GRanges(rep(c(1,2),each=7), 
#'                   IRanges(rep(c(1,1,2,3,7,8,9), times=2), width=5), 
#'                   weight = rep(c(1,2),each=7))
#'                  
#' windows1 = GRanges(rep(c(1,2),each=2), 
#'                     IRanges(rep(c(1,2), times=2), width=5), 
#'                     strand=c('-','+','-','+'))
#' windows2 = windows1[c(1,3)]
#' sml = as(list(ScoreMatrix(target, windows1),
#'                ScoreMatrix(target, windows2)), 'ScoreMatrixList')
#' sml
#' intersectScoreMatrixList(sml)
#'
#' @docType methods
#' @rdname intersectScoreMatrixList-methods
#' @export
setGeneric("intersectScoreMatrixList", 
           function(sml,reorder=FALSE)
             standardGeneric("intersectScoreMatrixList") )

#' @aliases intersectScoreMatrixList,ScoreMatrixList-method
#' @rdname intersectScoreMatrixList-methods
setMethod("intersectScoreMatrixList", signature("ScoreMatrixList"),
          function(sml,reorder){
            
            rnames = Reduce('intersect' ,lapply(sml, rownames))
            if(reorder){
              sml = as(lapply(sml, function(x){ 
                x=x[rownames(x) %in% rnames,]
                x[order(rownames(x)),]
              }), 
              'ScoreMatrixList')
            }else{
              sml = as(lapply(sml, function(x)x[rownames(x) %in% rnames,]), 
                       'ScoreMatrixList')              
            }
            return (sml)
          }
)

# ---------------------------------------------------------------------------- #
#' Reorder all elements of a ScoreMatrixList to a given ordering vector

#' @param sml \code{ScoreMatrixList} object
#' @param ord.vec an integer vector
#' @return \code{ScoreMatrixList} object
#' 
#' @examples
#' library(GenomicRanges)
#' data(cage)
#' data(cpgi)
#' data(promoters)
#' 
#' cage$tpm = NULL
#' targets = GRangesList(cage=cage, cpgi=cpgi)
#' sml = ScoreMatrixList(targets, promoters, bin.num=10)
#' kmeans.clust = kmeans(sml$cage,3)
#'  
#' sml.ordered = orderBy(sml, kmeans.clust$cluster)
#' \donttest{
#' multiHeatMatrix(sml.ordered)
#' }
#' @docType methods
#' @rdname orderBy-methods
#' @export
setGeneric("orderBy", 
           function(sml,ord.vec)
             standardGeneric("orderBy") )

#' @aliases orderBy,ScoreMatrixList-method
#' @rdname orderBy-methods
setMethod("orderBy", signature("ScoreMatrixList"),
          function(sml, ord.vec){
            
            if(is.null(ord.vec))
              return(sml)
            
            if(!is.integer(ord.vec))
              stop('ord.vec needs to be of an integer vector')  
            
            sml = lapply(sml, function(x)x[ord.vec,])
            return (as(sml,'ScoreMatrixList'))
          }
)


# ---------------------------------------------------------------------------- #
#' @aliases binMatrix,ScoreMatrixList-method
#' @rdname binMatrix-methods
setMethod("binMatrix", signature("ScoreMatrixList"),
          function(x, bin.num=NULL, fun='mean'){
            
            if(is.null(bin.num))
              return(x)
            
            return(new("ScoreMatrixList", 
                       lapply(x, function(y)binMatrix(y, bin.num=bin.num, fun=fun))))
          }
)

# ---------------------------------------------------------------------------- #
# enrichmentMatrix()
# ---------------------------------------------------------------------------- #
bam.file <- system.file('unitTests/test.bam', package='genomation')
windows <- GRanges(rep(c(1,2), each=2), IRanges(rep(c(1, 2), times=2), width=5))
IP <- ScoreMatrix(target=bam.file,windows=windows, type='bam')

bam.file <- system.file('unitTests/test.bam', package='genomation')
control <- ScoreMatrix(target=bam.file, windows=windows, type='bam') * 0.002

# IP - ScoreMatrix, control - ScoreMatrix
esm_1 <- enrichmentMatrix(IP, control)
esm_2 <-log2((IP + 1) / (control + 1))
checkEquals(esm_1, esm_2)

# IP - ScoreMatrixList, control - ScoreMatrixList
sml_IP <- ScoreMatrixList(list(IP1 = IP, IP2 = IP))
sml_control <- ScoreMatrixList(list(c1 = control, c2 = control))
esml_1 <- enrichmentMatrix(sml_IP, sml_control)
es1 <- log2((IP + 1) / (control + 1))
esml_2 <- ScoreMatrixList(list(IP1 = es1, IP2 = es1))
checkEquals(esml_1, esml_2)

# IP - ScoreMatrixList, control - ScoreMatrix
sml_IP <- ScoreMatrixList(list(IP1 = IP, IP2 = IP))
esml_3 <- enrichmentMatrix(sml_IP, control)
es2 <- log2((IP + 1) / (control + 1))
esml_4 <- ScoreMatrixList(list(IP1 = es1, IP2 = es1))
checkEquals(esml_3, esml_4)

# error for unequal list lengths
checkException(enrichmentMatrix(ScoreMatrixList(list(IP, IP, IP)), sml_control), silent=TRUE)

# ---------------------------------------------------------------------------- #
# scoreMatrixList: standard
test_scoreMatrixList = function()
{

  target = GRanges(rep(c(1,2),each=7), 
                   IRanges(rep(c(1,1,2,3,7,8,9), times=2), width=5),
                   weight = rep(c(1,2),each=7), 
                   strand=c('-', '-', '-', '-', '+', '-', '+', '-', 
                            '-', '-', '-', '-', '-', '+'))
  tar.list = list(tar1=target, tar2=target)
  tar.glist = GRangesList(tar1=target, tar2=target)
  windows = GRanges(rep(c(1,2),each=2), IRanges(rep(c(1,2), times=2), width=5), 
                    strand=c('-','+','-','+'))
  
  # -----------------------------------------------#
  # usage
  #1. return sml
  m = as(matrix(0, ncol=5, nrow=4), 'ScoreMatrix')
  ml1 = list(m1=m, m2=m)
  sml1 = ScoreMatrixList(ml1)
  tml1 = as(ml1, 'ScoreMatrixList')
  checkEquals(sml1, tml1)
  
  #2. ScoreMatrixList: list, GRanges
  sml2 = ScoreMatrixList(tar.list, target)
  tml2 = as(list(tar1=ScoreMatrix(tar.list$tar1, target), 
                 tar2=ScoreMatrix(tar.list$tar1, target)),
            'ScoreMatrixList')
  checkEquals(sml2, tml2)
  
  #3. ScoreMatrixList: GRangesList, GRanges
  sml3 = ScoreMatrixList(tar.glist, target)
  tml3 = as(list(tar1=ScoreMatrix(tar.list$tar1, target), 
                 tar2=ScoreMatrix(tar.list$tar1, target)),
            'ScoreMatrixList')
  checkEquals(sml3, tml3)
  
  #4. ScoreMatrixList: list, GRanges, weight.col
  sml4 = ScoreMatrixList(tar.list, target, weight.col='weight')
  tml4 = as(list(tar1=ScoreMatrix(tar.list$tar1, target, weight.col='weight'), 
                 tar2=ScoreMatrix(tar.list$tar1, target, weight.col='weight')),
            'ScoreMatrixList')
  checkEquals(sml4, tml4)
  
  #5. ScoreMatrixList: list, GRanges, strand.aware
  sml5 = ScoreMatrixList(tar.list, target, strand.aware=TRUE)
  tml5 = as(list(tar1=ScoreMatrix(tar.list$tar1, target, strand.aware=TRUE), 
                 tar2=ScoreMatrix(tar.list$tar1, target, strand.aware=TRUE)),
            'ScoreMatrixList')
  checkEquals(sml5, tml5)
  
  #6. ScoreMatrixList: list, GRanges, Bin
  sml6 = ScoreMatrixList(tar.list, target, bin.num=2)
  tml6 = as(list(tar1=ScoreMatrixBin(tar.list$tar1, target, bin.num=2), 
                 tar2=ScoreMatrixBin(tar.list$tar1, target, bin.num=2)),
            'ScoreMatrixList')
  checkEquals(sml6, tml6)
  
  #7. ScoreMatrixList: list, GRanges, Bin, bin.op='min'
  sml7 = ScoreMatrixList(tar.list, target, bin.num=2, bin.op='min')
  tml7 = as(list(tar1=ScoreMatrixBin(tar.list$tar1, target, bin.num=2, bin.op='min'), 
                 tar2=ScoreMatrixBin(tar.list$tar1, target, bin.num=2, bin.op='min')),
            'ScoreMatrixList')
  checkEquals(sml7, tml7)
  
  
  
  # -----------------------------------------------#
  #1. error given a single granges object
  checkException(ScoreMatrixList(target, windows), silent=TRUE)          
  
  #2. error for empty target
  checkException(ScoreMatrixList(NULL, windows), silent=TRUE)
  
  #3. error for empty windows
  checkException(ScoreMatrixList(tar.list, NULL), silent=TRUE)
  
  #4. error for character not a file
  checkException(ScoreMatrixList(c('',''), windows), silent=TRUE) 
}

# ---------------------------------------------------------------------------- #
# scoreMatrixList: character, bigWig
test_ScoreMatrixList_BigWig = function()
{
  if (.Platform$OS.type == "windows")
    return()
  test_bw <- system.file("unitTests/test.bw", package = "genomation")
          
  st = seq(200, 300, 20)
  g = GRanges(rep('chr2', length(st)), IRanges(st, width=10))
  s = ScoreMatrixList(c(test_bw, test_bw), g, type='bigWig')
          
  m = matrix(-1, ncol=10, nrow=6)
  m[6,-1] = -0.75
  rownames(m) = 1:6
  m = as(m, 'ScoreMatrix')
  l = as(list(test.bw=m, test.bw=m), 'ScoreMatrixList')
  checkEquals(s, l)
}

# ---------------------------------------------------------------------------- #
# scoreMatrixList: character, bigWig, bin
test_ScoreMatrixListBin_bigWig = function()
{
  if (.Platform$OS.type == "windows")
    return()
  test_bw <- system.file("unitTests/test.bw", package = "genomation")
          
  st = seq(200, 300, 20)
  g = GRanges(rep('chr2', length(st)), IRanges(st, width=10))
#   b = import(test_bw, which=g)
#   covs = coverage(b, weight=b$score)        
#   s = ScoreMatrixBin(covs, g, bin.num=5)
          
  s = ScoreMatrixList(c(test_bw, test_bw), g,bin.num=5, type='bigWig')
          
  m = matrix(-1, ncol=10, nrow=6)
  m[6,-1] = -0.75
  rownames(m) = 1:6
  m = as(m, 'ScoreMatrix')
  m = binMatrix(m, bin.num=5)
  l = as(list(test.bw=m, test.bw=m), 'ScoreMatrixList')
  checkEquals(s, l)
}

# ---------------------------------------------------------------------------- #
test_ScoreMatrixList_Bam = function()
{
  target = GRanges(rep(c(1,2),each=7), 
                   IRanges(rep(c(1,1,2,3,7,8,9), times=2), width=5),
                           weight = rep(c(1,2),each=7), 
                           strand=c('-', '-', '-', '-', '+', '-', '+', '-', 
                                    '-', '-', '-', '-', '-', '+'))
  tar.list = list(tar1=target, tar2=target)
  tar.glist = GRangesList(tar1=target, tar2=target)
  windows = GRanges(rep(c(1,2),each=2), IRanges(rep(c(1,2), times=2), width=5), 
                    strand=c('-','+','-','+'))
          
  # -----------------------------------------------#
  # usage
  # bam file
  bam.file = system.file('unitTests/test.bam', package='genomation')
  bam.files = c(bam.file, bam.file)
  sml1 = ScoreMatrixList(bam.files, windows, type='bam')
  tml1 = ScoreMatrixList(tar.list, windows)
  names(tml1) = basename(bam.files)
  checkEquals(sml1,tml1)
          
  # bam file, rpm=TRUE
  s2 = ScoreMatrixList(bam.files, windows, type='bam', rpm=TRUE)
  tot = 1e6/unlist(lapply(bam.files,function(x) {sum(idxstatsBam(normalizePath(x))$mapped)} ))
  m2 = ScoreMatrixList(lapply(sml1, function(x)x*tot))
  checkEquals(s2, m2)
          
  #bam file, rpm=FALSE, unique=TRUE
  s3 = ScoreMatrix(bam.file, windows, type='bam', unique=T)
  m3 = ScoreMatrix(unique(target), windows)
  checkEquals(s3,m3)
          
  #bam file, rpm=FALSE, unique=TRUE, extend=1
  s4 = ScoreMatrix(bam.file, windows, type='bam', unique=T, extend=1)
  m4 = ScoreMatrix(resize(unique(target), width=1), windows)
  checkEquals(s4,m4)    
  
  # bam file, rpm=TRUE, library.size
  s3 = ScoreMatrixList(bam.files, windows, type='bam', rpm=TRUE)
  tot = unlist(lapply(bam.files,function(x) {sum(idxstatsBam(normalizePath(x))$mapped)} ))
  s4 = ScoreMatrixList(bam.files, windows, type='bam', rpm=TRUE,
                       library.size=c(tot,tot))
  checkEquals(s3, s4)
        
}

# ---------------------------------------------------------------------------- #
# test arithmetic on ScoreMatrixList
test_ScoreMatrixList_Ops = function()
{
  target = GRanges(rep(c(1,2),each=7), 
                   IRanges(rep(c(1,1,2,3,7,8,9), times=2), width=5),
                   weight = rep(c(1,2),each=7), 
                   strand=c('-', '-', '-', '-', '+', '-', '+', '-', 
                            '-', '-', '-', '-', '-', '+'))
  windows = GRanges(rep(c(1,2),each=2), IRanges(rep(c(1,2), times=2), width=5), 
                    strand=c('-','+','-','+'))
  
  tar.list = list(tar1=target, tar2=target)
  sml = ScoreMatrixList(tar.list, windows)
  sml1 = sml*10
  sml2 = as(lapply(sml, function(x) x*10), "ScoreMatrixList")
  checkEquals(sml1, sml2, checkNames=FALSE)
}
# ---------------------------------------------------------------------------- #
# Combine scoreMatrix objects into a scoreMatrixList
test_ScoreMatrixList_combine = function()
{
  target = GRanges(rep(c(1,2),each=7), 
                   IRanges(rep(c(1,1,2,3,7,8,9), times=2), width=5),
                   weight = rep(c(1,2),each=7), 
                   strand=c('-', '-', '-', '-', '+', '-', '+', '-', 
                            '-', '-', '-', '-', '-', '+'))
  windows = GRanges(rep(c(1,2),each=2), IRanges(rep(c(1,2), times=2), width=5), 
                    strand=c('-','+','-','+'))
  
  tar.list1 = list(tar1=target, tar2=target)
  sml = ScoreMatrixList(tar.list1, windows)
  s1 = ScoreMatrix(target=target, windows=windows)
  s2 = ScoreMatrix(target=target, windows=windows)
  
  # label names are given by a user
  sml1 = c(sml,s1=s1,s2=s2)
  tar.list2 = list(tar1=target, tar2=target, s1=target, s2=target)
  sml2 = ScoreMatrixList(tar.list2, windows)
  checkEquals(sml1, sml2)
  
  # label names are not given by a user
  sml3 = c(sml,s1,s2)
  tar.list4 = list(tar1=target, tar2=target,target,target)
  sml4 = ScoreMatrixList(tar.list4, windows)
  checkEquals(sml3, sml4)
  
  # lack of the label name of one of the labels
  sml5 = c(sml,s1,s2=s2)
  tar.list6 = list(tar1=target, tar2=target,target, s2=target)
  sml6 = ScoreMatrixList(tar.list6, windows)
  checkEquals(sml5, sml6)
  
  # combine two ScoreMatrixList objects
  sml7 = c(sml,sml)
  tar.list8 = list(tar1=target, tar2=target, tar1=target, tar2=target)
  sml8 = ScoreMatrixList(tar.list8, windows)
  checkEquals(sml7, sml8)
  
  # combine a ScoreMatrix into a ScoreMatrixList object
  sml9 = c(s1, sml)
  tar.list10 = list(target, tar1=target, tar2=target)
  sml10 = ScoreMatrixList(tar.list10, windows)
  checkEquals(sml9, sml10)
  
  # combine a ScoreMatrix into a ScoreMatrixList object
  sml11 = c(s1, sml, s2)
  tar.list12 = list(target, tar1=target, tar2=target, target)
  sml12 = ScoreMatrixList(tar.list12, windows)
  checkEquals(sml11, sml12)
  
  # combine a ScoreMatrix into a ScoreMatrixList object
  sml13 = c(s1=s1, sml,s2=s2)
  tar.list14 = list(s1=target, tar1=target, tar2=target, s2=target)
  sml14 = ScoreMatrixList(tar.list14, windows)
  checkEquals(sml13, sml14)
}

# ---------------------------------------------------------------------------- #
#' get enrichment based on randomized feature overlap
#'
#' This function measures the association between two genomic features by randomizing one feature and counting the overlaps in randomized sets.
#' That is to say, \code{query} feature will be randomly distributed over the genome (constrained by provided options), and the overlap of \code{target} with these randomized features will be measured.
#'
#' @param target a \code{GRanges} object to be overlapped with \code{query}
#' @param query a \code{GRanges} object that will be randomly placed across the genome and overlap of these random regions with \code{target} will be the background distribution of association between \code{target} and \code{query}.
#' @param rand.set instead of randomly placing features in \code{query} one can supply an already shuffled set of \code{query} genomic features.
#' @param randomizations  number of times the features to be shuffled
#' @param ... other parameters to be passed to \code{randomizeFeature} function. These parameters ccontrol how randomization is done.
#' @return returns a \code{RandomEnrichment} object
#' @seealso \code{\link{randomizeFeature}}
#' @examples
#' data(cage)
#' data(cpgi)
#' \donttest{ 
#' enr = getRandomEnrichment(cage, cpgi, randomizations=50)
#' }
#' @export
#' @docType methods
#' @rdname getRandomEnrichment-methods
setGeneric("getRandomEnrichment", 
           function(target, query, randomizations=1000, rand.set=NULL,...) 
             standardGeneric("getRandomEnrichment") )

#' @aliases getRandomEnrichment,GRanges,GRanges-method
#' @rdname getRandomEnrichment-methods
setMethod("getRandomEnrichment",
            signature(target = "GRanges",query="GRanges"),
            function( target, query, randomizations, rand.set,... ){

                orig.cnt = sum(countOverlaps(target,query) > 0)

                if( is.null(rand.set) ){
                
                    rand.olap.dist=numeric(randomizations)
                    for(i in 1:randomizations){
#                         message("Iteration number:",i,"\r")
                        my.rand = randomizeFeature(query,...)
                        my.cnts = sum(countOverlaps(target,my.rand)>0)
                        rand.olap.dist[i] = my.cnts
                    }
#                     message("\n")
                    
                }else if(is(rand.set,"GRangesList")){
                    
                    randomizations = length(rand.set)
                    rand.olap.dist = numeric(randomizations)
                    for(i in 1:randomizations){
                        message("Iteration number:",i,"\r")
                        my.cnts = sum(countOverlaps(target,rand.set[[i]])>0)
                        rand.olap.dist[i] = my.cnts
                    }
                    message("\n")
                }else{
                    stop("Wrong 'rand.set' argument supplied, it should be a 'GRangesList'")
                }

                # calculate p-value assuming normal distribution
                p.value=pnorm(orig.cnt, 
                      mean=mean(rand.olap.dist), 
                      sd=sd(rand.olap.dist),
                      lower.tail=FALSE )
                #if(p1>0.5){p.value=1-p1}else{p.value=p1}

                # calculate randomized raw p-value
                rand.p.value = sum(rand.olap.dist>orig.cnt)/length(rand.olap.dist)

                n = new('RandomEnrichment', 
                orig.cnt = orig.cnt,
                          rand.olap.dist = rand.olap.dist,
                        log2fc = log2(orig.cnt/mean(rand.olap.dist)),
                          p.value = p.value,
                          rand.p.value = rand.p.value)
        return(n)
})

#######################################
# S3 functions
#######################################

#' binner function
#'
#'given a vector and length smooths the vector to a given size
# the function is not safe - check for the window length before
#'
#' @param start start position
#' @param end end position
#' @param nbins number of bins
#' @keywords internal
binner=function(start,end,nbins){
  
  if(! is.numeric(start))
    stop('start needs to be class numeric')
  if(! is.numeric(end))
    stop('end needs to be class numeric')
  if(! is.numeric(nbins))
    stop('nbins needs to be class numeric')
  
  x = unique(seq(from = start, to = end,length.out=nbins + 1 ) )
  my.start = ceiling(x)[-length(x)]
  my.end = floor(x)[-1]
  
  return( t(cbind(my.start, my.end) )  )
}

# ---------------------------------------------------------------------------- #
#' getColors function
#'
#'gets colors for a factor variable
#'
#' @param n n
#' @keywords internal
getColors = function(n) {
  
  black = "#000000"
  c(black,hcl(h=seq(0,(n-2)/(n-1),length=n-1)*360,c=100,l=65,fixup=TRUE))
}

#' file.ext function
#'
#' Extract file extension from file path
#'
#' @param x file 
#' @keywords internal
file.ext = function(x) {
  pos <- regexpr("\\.([[:alnum:]]+)$", x)
  ifelse(pos > -1L, substring(x, pos + 1L), "")
}

#' target.type function
#'
#'Check if a target file is in bam and bigWig formats
#'by looking at the file extension
#'
#' @param target target file
#' @param type type name
#' @keywords internal
target.type = function(target, type=""){
  
  if(length(target)!=1){
    stop("target has to be a single character")
  }
  bw.exts = c("bw","bigWig","bigwig","BigWig", "BIGWIG", "BW")
  bam.exts = c("bam", "BAM", "Bam")
  if(type=="" | length(type)!=1 | nchar(type)==0 | is.null(type))
      stop(paste0('set argument type to "auto", "bam" or "bigWig"\n'))
  
  if(type=="auto"){
    # automatically recognize type of target files
    # by looking at the file extensions
    exts = file.ext(target)  
    if( any(!(exts %in% c(bam.exts, bw.exts))) )
      stop(paste0('currently supported formats are bam and bigWig\n'))
    if( exts %in% bw.exts)  return("bigWig") 
    if( exts %in% bam.exts) return("bam")
    
  }else if(type %in% c(bam.exts, bw.exts)){
    # if type is BAM or bigWig
    bam.grepl = paste0((paste0(bam.exts, collapse="$|")), "$")
    bw.grepl = paste0((paste0(bw.exts, collapse="$|")), "$")
    if(type %in% bam.exts){
      if(!grepl(bam.grepl, target)){
        warning(paste0('you have set type="',type,
                       '", but the designated file does not have ',
                       'an extension of BAM file (.bam)'))}
      return("bam")
    }
    if(type %in% bw.exts){
      if(!grepl(bw.grepl, target)){
        warning(paste0('you have set type="',type,
                       '", but the designated file does not have',
                       ' an extension of biWig file (.bw or .bigWig)')) }
      return("bigWig")
    }
  }
}

# ---------------------------------------------------------------------------- #

#' constrainRanges function
#' 
#' removes ranges that fell of the rle object
#' does not check for the correspondence of the chromosome names - always 
#' check before using this function
#'
#' @param target target file
#' @param windows windows
#' @keywords internal
constrainRanges = function(target, windows){
  
  checkClass(target, c('SimpleRleList','RleList','CompressedRleList'))
  checkClass(windows, 'GRanges')
  
  mcols(windows)$X_rank = 1:length(windows)
  r.chr.len = elementNROWS(target)
  constraint = GRanges(seqnames=names(r.chr.len),
                       IRanges(start=rep(1,length(r.chr.len)),
                               end=as.numeric(r.chr.len)))
  # suppressWarnings is done becuause GenomicRanges function give warnings 
  #if you don't have the same seqnames in both objects
  win.list.chr = suppressWarnings(subsetByOverlaps(windows, 
                                                   constraint,
                                                   type = "within",
                                                   ignore.strand = TRUE))
  
  if(length(win.list.chr) == 0)
    stop('All windows fell have coordinates outside windows boundaries')
  return(win.list.chr)
}


# ---------------------------------------------------------------------------- #

#' checkClass function
#' 
#' check whether the x object corresponds to the given class
#'
#' @param x object
#' @param class.name class name
#' @param var.name uses x object
#' @keywords internal
checkClass = function(x, class.name, var.name = deparse(substitute(x))){
  
  fun.name = match.call(call=sys.call(sys.parent(n=1)))[[1]]
  if(!class(x) %in% class.name)
    stop(paste(fun.name,': ', 
               var.name, 
               ' is not of class: ', 
               paste(class.name, collapse=' '), 
               '\n', sep=''))
}

#' galpTo2Ranges function
#'
#' @param x object
#' @keywords internal
galpTo2Ranges <- function(x)
{
  gr1 <- granges(first(x))
  gr2 <- granges(invertStrand(last(x)))
  ans <- split(c(gr1, gr2), rep.int(seq_along(x), 2L))
  names(ans) <- names(x)
  ans
}

# ---------------------------------------------------------------------------- #
#' readBam function
#' 
#' given a big bam path reads the big wig file into a RleList
#' to be used by ScoreMatrix:char,GRanges
#'
#' @param target target object
#' @param windows windows
#' @param rpm logical
#' @param unique logical
#' @param extend numeric
#' @param param ScanBamParam object 
#' @param paired.end llogical
#' @param library.size numeric
#' @param ... additional parameters
#' @keywords internal

readBam = function(target, windows, rpm=FALSE,
                   unique=FALSE, extend=0, param=NULL, 
                   paired.end=FALSE, library.size=NULL, ...){

  # check the ScanBamParam object
  if(!is.null(param) & class(param)!='ScanBamParam')
    stop('param needs to be an object of class ScanBamParam')
  
  if(paired.end){
    
    flag <- scanBamFlag(isPaired=TRUE, isProperPair=TRUE, 
                        isUnmappedQuery=FALSE, hasUnmappedMate=FALSE)
    # reads that map into window which stretches from start of the first window
    # to end of the last window
    if(is.null(param)){
      param <- ScanBamParam(which=reduce(windows, ignore.strand=TRUE), flag=flag)
    }else{
      bamWhich(param) <- reduce(windows, ignore.strand=TRUE)
    }
    alnp <- readGAlignmentPairs(target, param=param, use.names=TRUE)
    
    # remove rows that are duplicated when mates of pair map into two different windows
    alnp = alnp[!duplicated(names(alnp))]
    alns <- granges(alnp)
    
  }else{
    
    if(is.null(param)){
      param <- ScanBamParam(which=reduce(windows, ignore.strand=TRUE))
    }else{
      bamWhich(param) <- reduce(windows, ignore.strand=TRUE)
    }
    alns <- granges(readGAlignments(target, param=param, use.names=TRUE))
  }
  
  if(unique)
    alns = unique(alns)
  
  if(extend > 0)
    alns = resize(alns, width=extend, fix='start')
  if(extend < 0)
    stop('extend needs to be a positive integer')
  
  # get the coverage vector for given locations
  covs=coverage(alns)
  
  if(rpm){
    message('Normalizing to rpm ...')
    if(is.null(library.size)){
      total = 1e6/sum(idxstatsBam(normalizePath(target))[3])
    }else{
      total = 1e6/library.size
    }
    covs = covs * total
  }
  
  return(covs)
  
}

# ---------------------------------------------------------------------------- #
#' readBigWig function
#' 
#' given a big wig path reads the big wig file into a RleList
#' to be used by ScoreMatrix:char,GRanges
#'
#' @param target target object
#' @param windows windows
#' @param ... additional parameters
#' @keywords internal
readBigWig = function(target, windows=NULL, ...){
  
  
  if(class(windows) != 'GRanges')
    stop('windows argument needs to be a GRanges object')
  
  
  if(is.null(windows)){
    bw = import(target)
  }else{
    bw = import(target, which=windows)
  }
  if(length(bw) == 0)
    stop('There are no ranges selected')
  
  covs = coverage(bw, weight=bw$score)
  return(covs)
}



#######################################
# S4 functions
#######################################


# ---------------------------------------------------------------------------- #
#' Get base-pair score for bases in each window
#'
#' The funcion produces a base-pair resolution matrix of scores for given equal
#' width windows of interest. The returned matrix  can be used to 
#' draw meta profiles or heatmap of read coverage or wig track-like data.
#' The \code{windows} argument can be a predefined region around transcription start sites 
#' or other regions of interest that have equal lengths
#' The function removes all window that fall off the Rle object - 
#' have the start coordinate < 1 or end coordinate > length(Rle)
#' The function takes the intersection of names in the Rle and GRanges objects.
#' On Windows OS the function will give an error if the target is a file in .bigWig format.
#'
#' @param target \code{RleList} , \code{GRanges}, a BAM file or a BigWig
#'  to be overlapped with ranges in \code{windows}
#' @param windows \code{GRanges} object that contains the windows of interest. 
#'                It could be promoters, CpG islands, exons, introns. 
#'                However the sizes of windows have to be equal.
#' @param strand.aware If TRUE (default: FALSE), the strands of the
#'                   windows will be taken into account in the resulting
#'                    \code{ScoreMatrix}.
#'                     If the strand of a window is -, the values of the bins 
#'                     for that window will be reversed
#' @param weight.col if the object is \code{GRanges} object a numeric column
#'                 in meta data part can be used as weights. This is particularly
#'                useful when genomic regions have scores other than their
#'                coverage values, such as percent methylation, conservation
#'                scores, GC content, etc. 
#' @param is.noCovNA (Default:FALSE)
#'                  if TRUE,and if 'target' is a GRanges object with 'weight.col'
#'                   provided, the bases that are uncovered will be preserved as
#'                   NA in the returned object. This useful for situations where
#'                   you can not have coverage all over the genome, such as CpG 
#'                   methylation values.
#' @param type  (Default:"auto")
#'              if target is a character vector of file paths, then type designates
#'              the type of the corresponding files (bam or bigWig).
#' @param rpm boolean telling whether to normalize the coverage to per milion 
#'                    reads. FALSE by default. See \code{library.size}.
#' @param unique boolean which tells the function to remove duplicated reads 
#'              based on chr, start, end and strand
#' @param extend numeric which tells the function to extend the reads to width=extend
#' @param param ScanBamParam object 
#' @param bam.paired.end boolean indicating whether given BAM file contains 
#'                       paired-end reads (default:FALSE).
#'                       Paired-reads will be treated as fragments.
#' @param library.size numeric indicating total number of mapped reads in a BAM file
#'                            (\code{rpm} has to be set to TRUE).
#'                            If is not given (default: NULL) then library size 
#'                            is calculated using the Rsamtools idxstatsBam function:
#'                            sum(idxstatsBam(target)$mapped).
#' 
#' @note
#' We assume that a paired-end BAM file contains reads with unique ids and we remove 
#' both mates of reads if they are repeated. Due to the fact that \code{ScoreMatrix} 
#' uses the GenomicAlignments:readGAlignmentPairs function to read paired-end BAM files
#' a duplication of reads occurs when mates of one pair map into two different windows.
#' 
#' Strands of reads in a paired-end BAM are inferred depending on strand of 
#' first alignment from the pair. This is a default setting in the 
#' GenomicAlignments:readGAlignmentPairs function (see a strandMode argument). 
#' This mode should be used when the paired-end data was generated using 
#' one of the following stranded protocols: 
#' Directional Illumina (Ligation), Standard SOLiD.
#' 
#' @return returns a \code{ScoreMatrix} object
#' @seealso \code{\link{ScoreMatrixBin}}
#' 
#' @examples
#
#' # When target is GRanges
#' data(cage)
#' data(promoters)
#' scores1=ScoreMatrix(target=cage,windows=promoters,strand.aware=TRUE,
#'                                 weight.col="tpm")                                
#'                                  
#' 
#' # When target is RleList
#' library(GenomicRanges)
#' covs = coverage(cage)
#' scores2 = ScoreMatrix(target=covs,windows=promoters,strand.aware=TRUE)   
#' scores2 
#'
#' # When target is a bam file
#' bam.file = system.file('unitTests/test.bam', package='genomation')
#' windows = GRanges(rep(c(1,2),each=2), IRanges(rep(c(1,2), times=2), width=5))
#' scores3 = ScoreMatrix(target=bam.file,windows=windows, type='bam') 
#' scores3
#' 
#' @useDynLib genomation
#' @importFrom Rcpp sourceCpp
#' @docType methods
#' @rdname ScoreMatrix-methods           
#' @export
setGeneric("ScoreMatrix",
           function(target,windows,
                    strand.aware=FALSE,
                    weight.col=NULL,
                    is.noCovNA=FALSE,
                    type="auto",
                    rpm=FALSE, 
                    unique=FALSE, 
                    extend=0,
                    param=NULL,
                    bam.paired.end=FALSE,
                    library.size=NULL) 
             standardGeneric("ScoreMatrix") )


#' @aliases ScoreMatrix,RleList,GRanges-method
#' @rdname ScoreMatrix-methods
#' @usage  \\S4method{ScoreMatrix}{RleList,GRanges}(target,windows,strand.aware)
setMethod("ScoreMatrix",signature("RleList","GRanges"),
          function(target,windows,strand.aware){
            
            #check if all windows are equal length
            if( length(unique(width(windows))) >1 ){
              stop("width of 'windows' are not equal, provide 'windows' with equal widths")
            }  
            # check if windows have width > 1
            if( any(width(windows)==1) ){
              stop("provide 'windows' with widths greater than 1")
            } 
            
            # set a uniq id for the GRanges
            windows.len=length(windows)
            windows = constrainRanges(target, windows)
            if(length(windows)!=windows.len){
              warning(paste0(windows.len-length(windows),
                             " windows fall off the target"))
            }
            
            # fetches the windows and the scores
            chrs = sort(intersect(names(target), as.character(unique(seqnames(windows)))))
            myViews=Views(target[chrs],as(windows,"IntegerRangesList")[chrs]) # get subsets of RleList
            
            #  get a list of matrices from Views object
            #  operation below lists a matrix for each chromosome
            mat = lapply(myViews,function(x) t(viewApply(x,as.vector)) )
            
            # combine the matrices from chromosomes 
            mat = do.call("rbind",mat)   
            
            # get the ranks of windows, when things are reorganized by as(...,"IntegerRangesList")
            r.list=split(mcols(windows)[,"X_rank"], as.vector(seqnames(windows))  )
            r.list=r.list[order(names(r.list))]
            ranks=do.call("c",r.list)
            
            # put original window order as rownames
            # this is very important to keep
            # if we have multiple RleLists for the same set of windows
            # we need to know which windows are removed from which set
            # so when we are doing something comparative (clustering windows
            # based on different histone marks) we only work with windows
            # that are covered by all histone marks
            rownames(mat) = ranks  
            
            # if strand aware is TRUE, we need to flip the windows on the minus strand
            if(strand.aware == TRUE){
              orig.rows=windows[strand(windows) == '-',]$X_rank
              mat[rownames(mat) %in% orig.rows,] = mat[rownames(mat) %in% 
                                                         orig.rows, ncol(mat):1]
            }
            
            # reorder matrix
            mat = mat[order(ranks),] 
            
            return(new("ScoreMatrix",mat))
          })



# ---------------------------------------------------------------------------- #
#' @aliases ScoreMatrix,GRanges,GRanges-method
#' @rdname ScoreMatrix-methods
#' @usage \\S4method{ScoreMatrix}{GRanges,GRanges}(target, windows, strand.aware,
#'                                                 weight.col, is.noCovNA)
setMethod("ScoreMatrix",signature("GRanges","GRanges"),
          function(target, windows, strand.aware, weight.col, is.noCovNA){
            
            #make coverage vector  from target
            if(is.null(weight.col)){
              target.rle=coverage(target)
            }else{
              if(! weight.col %in% names(mcols(target)) ){
                stop("provided column 'weight.col' does not exist in target\n")
              }
              if(is.noCovNA)
              { # adding 1 to figure out NA columns later
                target.rle=coverage(target,weight=(mcols(target)[weight.col][,1]+1) )
                mat=ScoreMatrix(target.rle,windows,strand.aware)
                mat=mat-1 # substract 1
                mat[mat<0]=NA # everything that are <0 are NA
                return(mat)
              }
              target.rle=coverage(target,weight= weight.col ) 
              
            }
            
            # call ScoreMatrix function
            ScoreMatrix(target.rle,windows,strand.aware)
          })

# ---------------------------------------------------------------------------- #
#' @aliases ScoreMatrix,character,GRanges-method
#' @rdname ScoreMatrix-methods
#' @usage \\S4method{ScoreMatrix}{character,GRanges}(target, windows, strand.aware,
#'                                                   weight.col=NULL,is.noCovNA=FALSE, 
#'                                                   type='auto', rpm=FALSE,
#'                                                   unique=FALSE, extend=0, param=NULL, 
#'                                                   bam.paired.end=FALSE,
#'                                                   library.size=NULL)
setMethod("ScoreMatrix",signature("character","GRanges"),
          function(target,windows, strand.aware,
                   weight.col=NULL,is.noCovNA=FALSE,
                   type='auto',rpm=FALSE, 
                   unique=FALSE, extend=0,param=NULL,
                   bam.paired.end=FALSE,
                   library.size=NULL){
            
            if(!file.exists(target)){
              stop("Indicated 'target' file does not exist\n")
            }
            
            type = target.type(target, type)
            
            if( type=="bigWig" & rpm==TRUE)
              warning("rpm=TRUE is not supported for type='bigWig'")

            if(type == 'bam'){
              
              covs = readBam(target, windows, rpm=rpm, unique=unique, 
                             extend=extend, param=param, 
                             paired.end=bam.paired.end, library.size)
              #get coverage vectors
              return(ScoreMatrix(covs,windows,strand.aware))
            }
              
            if(type == 'bigWig'){
              if(is.noCovNA==FALSE){
                
                covs = readBigWig(target=target, windows=windows)
                ScoreMatrix(covs,windows,strand.aware)
                
              }else{
                
                if(is.null(windows)){
                  bw = import(target)
                }else{
                  bw = import(target, which=windows)
                }
                if(length(bw) == 0)
                  stop('There are no ranges selected')
                
                return(ScoreMatrix(bw, windows, strand.aware, 
                                   weight.col=weight.col, 
                                   is.noCovNA=is.noCovNA))
              }
            }           
          })



# ---------------------------------------------------------------------------- #
#' Bins the columns of a matrix using a user provided function 
#'
#' @param x \code{ScoreMatrix} or a \code{ScoreMatrixList} object
#' @param bin.num \code{integer} number of bins in the final matrix
#' @param fun \code{character} vector or an anonymous function that will be used for binning
#'
#' @return \code{ScoreMatrix} or \code{ScoreMatrixList} object
#'
#' @examples
#' 
#' # binning the columns in a ScoreMatrix object
#' library(GenomicRanges)
#' target = GRanges(rep(c(1,2),each=7), IRanges(rep(c(1,1,2,3,7,8,9), times=2), width=5), 
#' weight = rep(c(1,2),each=7), 
#' strand=c('-', '-', '-', '-', '+', '-', '+', '-', '-', '-', '-', '-', '-', '+'))
#' windows = GRanges(rep(c(1,2),each=2), IRanges(rep(c(1,2), times=2), width=5), 
#' strand=c('-','+','-','+'))
#' sm = ScoreMatrix(target, windows)
#' bin = binMatrix(sm, bin.num=2)
#'
#' @docType methods
#' @rdname binMatrix-methods
#' @export
setGeneric("binMatrix", 
           function(x, bin.num=NULL, fun='mean')
             standardGeneric("binMatrix") )

#' @aliases binMatrix,ScoreMatrix-method
#' @rdname binMatrix-methods
setMethod("binMatrix", signature("ScoreMatrix"),
          function(x, bin.num=NULL, fun='mean'){
            
            if(is.null(bin.num))
              return(x)
            
            if(bin.num > ncol(x))
              stop("number of given bins is bigger 
                   than the number of matrix columns")
            
            if(is.character(fun))
              fun = match.fun(fun)
            
            coord = binner(1, ncol(x), bin.num)
            bmat = mapply(function(a,b)apply(x[,a:b],1,fun), coord[1,], coord[2,])
            
            return(new("ScoreMatrix", bmat))
          }
)

# ---------------------------------------------------------------------------- #
# show Methods
#' @rdname show-methods
#' @return Shows the dimension of the ScoreMatrix
setMethod("show", "ScoreMatrix",
          function(object){
            dims = dim(object)
            
            s=sprintf("  %s %d %d", "scoreMatrix with dims:", dims[1], dims[2])
            message(s)  
          })

# ---------------------------------------------------------------------------- #
#' Scales the values in the matrix by rows and/or columns
#'
#' @param mat \code{ScoreMatrix} object
#' @param columns \code{columns} whether to scale the matrix by columns. Set by default to FALSE.
#' @param rows  \code{rows} Whether to scale the matrix by rows. Set by default to TRUE
#' @param scalefun function object that takes as input a matrix and returns a matrix. 
#'         By default  the argument is set to (x - mean(x))/(max(x)-min(x)+1)
#' 
#' @return \code{ScoreMatrix} object
#'
#' @examples
#' 
#' # scale the rows of a scoreMatrix object
#' library(GenomicRanges)
#' target = GRanges(rep(c(1,2),each=7), IRanges(rep(c(1,1,2,3,7,8,9), times=2), width=5), 
#'          weight = rep(c(1,2),each=7), 
#'          strand=c('-', '-', '-', '-', '+', '-', '+', '-', '-', '-', '-', '-', '-', '+'))
#' windows = GRanges(rep(c(1,2),each=2), IRanges(rep(c(1,2), times=2), width=5), 
#'            strand=c('-','+','-','+'))
#' sm = ScoreMatrix(target, windows)
#' ssm = scaleScoreMatrix(sm, rows=TRUE)
#'
#' @docType methods
#' @rdname scaleScoreMatrix-methods
#' @export
setGeneric("scaleScoreMatrix", 
           function(mat, 
                    columns=FALSE, rows=TRUE, 
                    scalefun=NULL) 
             standardGeneric("scaleScoreMatrix") )

#' @aliases scaleScoreMatrix,ScoreMatrix-method
#' @rdname scaleScoreMatrix-methods
setMethod("scaleScoreMatrix", signature("ScoreMatrix"),
          function(mat, columns, rows, scalefun){
            
            if(is.null(scalefun))
              scalefun = function(x)(x-mean(x))/(max(x)-min(x)+1)
            
            if(columns)
              mat = apply(mat, 2, scalefun)
            
            if(rows)
              mat = t(apply(mat,1,scalefun))
            
            return(new("ScoreMatrix", mat))
          }
)

# ---------------------------------------------------------------------------- #
# test for ScoreMatrix function
test_ScoreMatrix_RleList_GRanges = function()
	{

		# input RleList
		rl = RleList(chr1 = Rle(rep(c(1,2,3), each=3)), chr2=Rle(rep(c(4,5,6), each=3)))
		
		#1. test for proper workings
		gr1 = GRanges(rep(c('chr1','chr2'), each=2), IRanges(c(1,5,1,5),c(3,7,3,7)), strand=c('+','-','+','-'))
		m1 = as(matrix(c(1,1,1,2,2,3,4,4,4,5,5,6), ncol=3, byrow=T), 'ScoreMatrix')
		rownames(m1) = 1:4
		checkIdentical(ScoreMatrix(rl, gr1, strand.aware = FALSE), m1)
		
		m2 = as(matrix(c(1,1,1,3,2,2,4,4,4,6,5,5), ncol=3, byrow=T), 'ScoreMatrix')
		rownames(m2) = 1:4
		checkIdentical(ScoreMatrix(rl, gr1, strand.aware = TRUE), m2)
		
		#2. test for different lengths
		gr2 = GRanges(rep(c('chr1','chr2'), each=2), IRanges(c(1,5,1,5),c(3,7,3,9)))
		checkException(ScoreMatrix(rl, gr2), silent=TRUE)
    #"width of 'windows' are not equal, provide 'windows' with equal widths")
		
		#3. test for removing outliers
		gr3 = GRanges(rep(c('chr1','chr2'), each=2), IRanges(c(1,5,8,1),c(3,7,10,3)))
		m3 = as(matrix(c(1,1,1,2,2,3,4,4,4), ncol=3, byrow=T), 'ScoreMatrix')
		rownames(m3) = c(1,2,4)
		checkIdentical(ScoreMatrix(rl, gr3), m3)
		
		#4. test for different chromosomes
		gr4 = GRanges(rep(c('chr1','chr3'), each=2), IRanges(c(1,5,1,5),c(3,7,3,7)))
		m4 = as(matrix(c(1,1,1,2,2,3), ncol=3, byrow=T), 'ScoreMatrix')
		rownames(m4) = 1:2
		checkIdentical(ScoreMatrix(rl, gr4, strand.aware = FALSE), m4)
		
		#5. no overlapping chromosomes
		gr5 = GRanges(rep(c('chr3','chr4'), each=2), IRanges(c(1,5,1,5),c(3,7,3,7)))
		checkException(ScoreMatrix(rl, gr5), silent=TRUE)
    # "All windows fell have coordinates outside chromosome boundaries"
		
		#6. only windows that fall off chromosomes
		gr6 = GRanges(rep(c('chr1','chr4'), each=2), IRanges(c(-1,8,1,5),c(1,10,3,7)))
		checkException(ScoreMatrix(rl, gr6), silent=TRUE)
    # "All windows fell have coordinates outside chromosome boundaries")
		
	}

# ---------------------------------------------------------------------------- #
test_ScoreMatrix_GRanges_GRanges = function()
{
  target = GRanges(rep(c(1,2),each=6), 
                   IRanges(rep(c(1,2,3,7,8,9), times=2), width=5),
                   weight = rep(c(1,2),each=6))
  windows = GRanges(rep(c(1,2),each=2), IRanges(rep(c(1,2), times=2), width=5), 
                    strand=c('-','+','-','+'))
          
  # -----------------------------------------------#
  # usage
  # normal function works
  s1 = ScoreMatrix(target=target, windows=windows)
  m1 = matrix(rep(c(1,2,3,3,3,2,3,3,3,2),times=2), ncol=5, byrow=T)
  rownames(m1) = 1:4
  m1 = as(m1, 'ScoreMatrix')
  checkEquals(s1,m1)
          
  # function with weight col works
  s2 = ScoreMatrix(target=target, windows=windows, weight.col='weight')
  m2 = matrix(rep(c(1,2,3,3,3,2,3,3,3,2),times=2), ncol=5, byrow=T)
  m2[3:4,] = m2[3:4,]*2
  rownames(m2) = 1:4
  m2 = as(m2, 'ScoreMatrix')
  checkEquals(s2,m2)  
          
  #strand aware
  s3 = ScoreMatrix(target=target, windows=windows, strand.aware=T)
  m3 = matrix(rep(c(1,2,3,3,3,2,3,3,3,2),times=2), ncol=5, byrow=T)
  rownames(m3) = 1:4
  m3[c(1,3),] = rev(m3[c(1,3),])
  m3 = as(m3, 'ScoreMatrix')
  checkEquals(s3,m3)
          
  # -----------------------------------------------#
  # errors
  checkException(ScoreMatrix(target, windows, weight.col=''), silent=TRUE)
          
}


# ---------------------------------------------------------------------------- #
test_ScoreMatrix_character_GRanges = function()
{
  target = GRanges(rep(c(1,2),each=7), 
                   IRanges(rep(c(1,1,2,3,7,8,9), times=2), width=5),
                   weight = rep(c(1,2),each=7), 
                   strand=c('-', '-', '-', '-', '+', '-', '+', '-', '-', '-', '-', '-', '-', '+'))  
  windows = GRanges(rep(c(1,2),each=2), IRanges(rep(c(1,2), times=2), width=5), 
                    strand=c('-','+','-','+'))
  
  target.paired.end = GRanges(rep(1,each=12), 
                              IRanges(start=c(1,1,2,2,3,3,7,7,8,8,9,9), end=c(19, 19, 20, 20, 21, 21,22,22,23,23,24,24)),
                              strand=rep("+", times=12))
  windows.paired.end = GRanges(rep(c(1),each=4), IRanges(c(7,8,20, 21), width=10), 
                               strand=c('+','+','+','+'))
    

  # -----------------------------------------------#
  # usage
  # bam file
  bam.file = system.file('unitTests/test.bam', package='genomation')
  s1 = ScoreMatrix(bam.file, windows, type='bam')
  m1 = ScoreMatrix(target, windows)
  checkEquals(s1,m1)
  
  # bam file, rpm=TRUE
  s2 = ScoreMatrix(bam.file, windows, type='bam', rpm=TRUE)
  tot = 1e6/sum(idxstatsBam(normalizePath(bam.file))$mapped)
  m2 = m1*tot
  checkEquals(s2, m2)
  
  #bam file, rpm=FALSE, unique=TRUE
  s3 = ScoreMatrix(bam.file, windows, type='bam', unique=TRUE)
  m3 = ScoreMatrix(unique(target), windows)
  checkEquals(s3,m3)
  
  #bam file, rpm=FALSE, unique=TRUE, extend=1
  s4 = ScoreMatrix(bam.file, windows, type='bam', unique=TRUE, extend=1)
  m4 = ScoreMatrix(resize(unique(target), width=1), windows)
  checkEquals(s4,m4)
  
  # bam file with paired-end reads
  bam.pe.file = system.file('unitTests/test_pairedend.bam', package='genomation')
  s5 = ScoreMatrix(bam.pe.file, windows.paired.end, type='bam', bam.paired.end=TRUE)
  names(target.paired.end) <-c("1:1-19","1:1-19.1","1:2-20","1:2-20","1:3-21","1:3-21",
                               "1:7-22","1:7-22","1:8-23","1:8-23","1:9-24","1:9-24")
  target.paired.end = target.paired.end[!duplicated(names(target.paired.end))]
  m5 = ScoreMatrix(target.paired.end, windows.paired.end, bam.paired.end=TRUE)
  checkEquals(s5,m5)
  
  # test library.size argument
  s6 = ScoreMatrix(bam.pe.file, windows.paired.end, type='bam', bam.paired.end=TRUE,
                   rpm=TRUE, library.size=14)
  s7 = ScoreMatrix(bam.pe.file, windows.paired.end, type='bam', bam.paired.end=TRUE,
                   rpm=TRUE)
  checkEquals(s6,s7)

  # -----------------------------------------------#
  # errors
  # error upon not specifying the file
  checkException(ScoreMatrix('',windows), silent=TRUE)
  
}

# ---------------------------------------------------------------------------- #
test_ScoreMatrix_character_GRanges_bigWig = function()
{
  if (.Platform$OS.type == "windows")
    return()
  test_bw <- system.file("unitTests/test.bw", package = "genomation")
          
  st = seq(200, 300, 20)
  g = GRanges(rep('chr2', length(st)), IRanges(st, width=10))
  s = ScoreMatrix(test_bw, g, type='bigWig')
          
  m = matrix(-1, ncol=10, nrow=6)
  m[6,-1] = -0.75
  rownames(m) = 1:6
  m = as(m, 'ScoreMatrix')
  checkEquals(s, m)
  
  # test is.noCovNA
  test_bw <- system.file("unitTests/test.bw", package = "genomation")
  g = GRanges(seqnames=c("chr2","chr19", "chr19", "chr19"),
              IRanges(start=c(1201, 1501, 2401, 2800), width=6),
              strand='*')
  s1 = ScoreMatrix(test_bw, g, strand.aware=FALSE, 
                   weight.col="score", 
                   is.noCovNA=TRUE,  type="bigWig")
  m1 = matrix(c(rep(0, 6), rep(0.25,6), rep(1,6), rep(NA,6)), 
             nrow=4,  byrow = TRUE)
  rownames(m1) = 1:4
  m1 = as(m1, 'ScoreMatrix')
  checkEquals(s1, m1)
}


# ---------------------------------------------------------------------------- #
# test arithmetic and logic operations on ScoreMatrix
test_ScoreMatrix_Ops = function()
{
  target1 = GRanges(rep(c(1,2),times=6), 
                   IRanges(rep(c(1,2,3,7,8,9), times=2), width=5),
                   weight = rep(c(1,2),each=6))
  target2 = GRanges(rep(c(1,2),times=12), 
                   IRanges(rep(c(1,2,3,7,8,9), times=4), width=5),
                   weight = rep(c(1,2),each=12))
  windows = GRanges(rep(c(1,2),each=2), IRanges(rep(c(1,2), times=2), width=5), 
                    strand=c('-','+','-','+'))
  
  s1 = ScoreMatrix(target=target1, windows=windows)
  s2 = ScoreMatrix(target=target2, windows=windows)
  checkEquals(s1*2,s2)

  checkEquals(sum(s1==s1), nrow(s1)*ncol(s1))
}
	
# ---------------------------------------------------------------------------- #
# test for constrainRanges
# test_constrainRanges = function()
# {
# 	target = RleList(chr1 = Rle(rep(c(1,2,3), each=3)), chr2=Rle(rep(c(4,5,6), each=3)))
# 	gr1 = GRanges(rep(c('chr1','chr2'), each=2), IRanges(c(1,5,1,8),c(3,7,3,10)))
# 	gt1 = GRanges(c('chr1','chr1','chr2'), IRanges(c(1,5,1),c(3,7,3)))
# 	expect_identical(genomation:::constrainRanges(target, gr1), gt1)
# }


# ---------------------------------------------------------------------------- #
# test for binMatrix
test_binMatrix = function()
{
	m1 = as(matrix(rep(1:4, 4), ncol=4), 'ScoreMatrix')
	
	# no binning
	checkEquals(binMatrix(m1), m1)
	
	# nbins 2, default function
	mb1 = as(matrix(c(1,1,2,2,3,3,4,4), ncol=2, byrow=T), 'ScoreMatrix')
	checkEquals(binMatrix(m1, bin.num=2), mb1)
	
	# nbins 5
	checkException(binMatrix(m1, bin.num=5), silent=TRUE)
	
	# nbins 2, not default function
	m2 = as(matrix(rep(1:4, 4), ncol=4, byrow=T), 'ScoreMatrix')
	mb2 = as(matrix(rep(c(2,4), 4), ncol=2, byrow=T), 'ScoreMatrix')
	checkEquals(binMatrix(m2, bin.num=2, max), mb2)	
}


# ---------------------------------------------------------------------------- #
test_scaleScoreMatrix = function()
{
  s = as(matrix(1:9, ncol=3),'ScoreMatrix')
          
  s1=scaleScoreMatrix(s, rows=T, columns=F)
  m1 = as(matrix(c(rep(-0.4285714,3),rep(0,3), rep(0.4285714,3)), 
                 nrow=3),'ScoreMatrix')
  checkEquals(s1,m1, tolerance=1e-5)
          
  s2=scaleScoreMatrix(s, rows=F, columns=T)
  m2 = as(matrix(c(rep(-0.3333333,3),rep(0,3), rep(0.3333333,3)), 
                 nrow=3, byrow=T),'ScoreMatrix')
  checkEquals(s2,m2, tolerance=1e-5)
          
  s3=scaleScoreMatrix(s, rows=T, columns=T)
  m3 = as(matrix(0,nrow=3,ncol=3),'ScoreMatrix')
  checkEquals(s3,m3, tolerance=1e-5)
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' Function that computes a mean value
#'
#' @param x NumericVector
#' @keywords internal
Mean_c <- function(x) {
    .Call('_genomation_Mean_c', PACKAGE = 'genomation', x)
}

#' Function that computes a mean value for each bin
#'
#' @param x NumericVector
#' @param n intiger - number of bins
#' @keywords internal
binMean <- function(x, n) {
    .Call('_genomation_binMean', PACKAGE = 'genomation', x, n)
}

#' Function that computes a median value
#'
#' @param x NumericVector
#' @keywords internal
Median_c <- function(x) {
    .Call('_genomation_Median_c', PACKAGE = 'genomation', x)
}

#' Function that computes a median value for each bin
#'
#' @param x NumericVector
#' @param n intiger - number of bins
#' @keywords internal
binMedian <- function(x, n) {
    .Call('_genomation_binMedian', PACKAGE = 'genomation', x, n)
}

#' Function that computes a max value
#'
#' @param x NumericVector
#' @keywords internal
Max_c <- function(x) {
    .Call('_genomation_Max_c', PACKAGE = 'genomation', x)
}

#' Function that computes a maximum value for each bin
#'
#' @param x NumericVector
#' @param n intiger - number of bins
#' @keywords internal
binMax <- function(x, n) {
    .Call('_genomation_binMax', PACKAGE = 'genomation', x, n)
}

#' Function that computes a min value
#'
#' @param x NumericVector
#' @keywords internal
Min_c <- function(x) {
    .Call('_genomation_Min_c', PACKAGE = 'genomation', x)
}

#' Function that computes a minimum value for each bin
#'
#' @param x NumericVector
#' @param n intiger - number of bins
#' @keywords internal
binMin <- function(x, n) {
    .Call('_genomation_binMin', PACKAGE = 'genomation', x, n)
}

#' Function that computes a sum value
#'
#' @param x NumericVector
#' @keywords internal
Sum_c <- function(x) {
    .Call('_genomation_Sum_c', PACKAGE = 'genomation', x)
}

#' Function that computes a sum of values in a bin
#'
#' @param x NumericVector - vector of values of a bin
#' @param n intiger - number of bins
#' @keywords internal
binSum <- function(x, n) {
    .Call('_genomation_binSum', PACKAGE = 'genomation', x, n)
}

#' Function creates a matrix storing data with desirable number of bins for each window 
#'
#' listSliceMean() function calls the binMean() function
#'  
#' @param xlist List of vectors storing values of a bin
#' @param n intiger - number of bins
#' @keywords internal
#' @export
listSliceMean <- function(xlist, n) {
    .Call('_genomation_listSliceMean', PACKAGE = 'genomation', xlist, n)
}

#' Function creates a matrix storing data with desirable number of bins for each window 
#'
#' listSliceMean() function calls the binMedian() function 
#' 
#' @param xlist List of vectors storing values of a bin
#' @param n intiger - number of bins
#' @keywords internal
#' @export
listSliceMedian <- function(xlist, n) {
    .Call('_genomation_listSliceMedian', PACKAGE = 'genomation', xlist, n)
}

#' Function creates a matrix storing data with desirable number of bins for each window 
#'
#' listSliceMax() function calls the binMax() function 
#' 
#' @param xlist List of vectors storing values of a bin
#' @param n intiger - number of bins
#' @keywords internal
#' @export
listSliceMax <- function(xlist, n) {
    .Call('_genomation_listSliceMax', PACKAGE = 'genomation', xlist, n)
}

#' Function creates a matrix storing data with desirable number of bins for each window 
#'
#' listSliceMin() function calls the binMin() function 
#' 
#' @param xlist List of vectors storing values of a bin
#' @param n intiger - number of bins
#' @keywords internal
#' @export
listSliceMin <- function(xlist, n) {
    .Call('_genomation_listSliceMin', PACKAGE = 'genomation', xlist, n)
}

#' Function creates a matrix storing data with desirable number of bins for each window 
#'
#' listSliceSum() function calls the binSum() function 
#' @param xlist List of vectors storing values of a bin
#' @param n intiger - number of bins
#' @keywords internal
#' @export
listSliceSum <- function(xlist, n) {
    .Call('_genomation_listSliceSum', PACKAGE = 'genomation', xlist, n)
}

## ----setup2, include=FALSE, cache=FALSE, eval=TRUE----------------------------
library(knitr)
opts_chunk$set(root.dir=getwd(),fig.align='center',
               fig.path='Figures', 
               dev='png',
               fig.show='hold', 
               cache=FALSE)
library(genomation)
library(genomationData)
library(GenomicRanges)
set.seed=10

## ----echo=FALSE,warnings=FALSE,out.width=500----------------------------------

knitr::include_graphics("https://raw.githubusercontent.com/BIMSBbioinfo/genomation/master/vignettes/Figures/genomationFlowChart1.png")

## ----eval=FALSE---------------------------------------------------------------
# BiocManager::install('genomationData')
# BiocManager::install('genomation')

## ----listgenomationData, eval=TRUE--------------------------------------------
list.files(system.file('extdata',package='genomationData'))

## ----genomationDataInfo, eval=TRUE, tidy=TRUE---------------------------------
sampleInfo = read.table(system.file('extdata/SamplesInfo.txt',
                                    package='genomationData'),header=TRUE, sep='\t')
sampleInfo[1:5,1:5]

## ----genomationInfo, eval=FALSE, tidy=TRUE------------------------------------
# library(genomation)
# data(cage)
# data(cpgi)
# 
# list.files(system.file('extdata', package='genomation'))

## ----readGeneric1, eval=TRUE, tidy=TRUE---------------------------------------
library(genomation)
tab.file1=system.file("extdata/tab1.bed", package = "genomation")
readGeneric(tab.file1, header=TRUE)

## ----readGeneric2, eval=TRUE, tidy=TRUE---------------------------------------
readGeneric(tab.file1, header=TRUE, keep.all.metadata=TRUE)

readGeneric(tab.file1, header=TRUE, meta.col = list(CpGnum=4))

## ----readGeneric3, eval=TRUE, tidy=TRUE---------------------------------------
readGeneric(tab.file1, header=TRUE, keep.all.metadata=TRUE)

## ----readGeneric4, eval=TRUE, tidy=TRUE---------------------------------------
tab.file2=system.file("extdata/tab2.bed", package = "genomation")
readGeneric(tab.file2, chr=3, start=2, end=1)

## ----readGff1, eval=TRUE, tidy=TRUE-------------------------------------------
gff.file=system.file("extdata/chr21.refseq.hg19.gtf", package = "genomation")
gff = gffToGRanges(gff.file)
head(gff)

## ----readGff2, eval=FALSE, tidy=TRUE------------------------------------------
# gff = gffToGRanges(gff.file, split.group=TRUE)
# head(gff)

## ----readFeatureFlank, eval=TRUE, tidy=TRUE-----------------------------------
# reading genes stored as a BED files
cpgi.file = system.file("extdata/chr21.CpGi.hg19.bed", package = "genomation")
cpgi.flanks = readFeatureFlank(cpgi.file)
head(cpgi.flanks$flanks)

## ----ScoreMatrixExample, eval=TRUE, tidy=TRUE---------------------------------
data(cage)
data(promoters)
sm = ScoreMatrix(target=cage, windows=promoters)
sm

## ----ScoreMatrixBinExample, eval=FALSE, tidy=TRUE-----------------------------
# data(cage)
# gff.file = system.file('extdata/chr21.refseq.hg19.gtf', package="genomation")
# exons = gffToGRanges(gff.file, filter='exon')
# sm = ScoreMatrixBin(target=cage, windows=exons, bin.num=50)
# sm

## ----ScoreMatrixBinExample_GRangesList, eval=FALSE, tidy=TRUE-----------------
# data(cage)
# library(GenomicRanges)
# gff.file = system.file('extdata/chr21.refseq.hg19.gtf', package="genomation")
# exons = gffToGRanges(gff.file, filter="exon")
# transcripts = split(exons, exons$transcript_id)
# sm = ScoreMatrixBin(target=cage, windows=transcripts, bin.num=10)
# sm

## ----ScoreMatrixList, eval=TRUE, tidy=TRUE------------------------------------
data(promoters)
data(cpgi)
data(cage)

cage$tpm = NULL
targets = list(cage=cage, cpgi=cpgi)
sm = ScoreMatrixList(targets=targets, windows=promoters, bin.num=50)
sm

## ----heatMatrix1, eval=FALSE, tidy=TRUE,  fig.keep='all'----------------------
# data(cage)
# data(promoters)
# sm = ScoreMatrix(target=cage, windows=promoters)
# 
# heatMatrix(sm, xcoords=c(-1000, 1000))
# plotMeta(sm, xcoords=c(-1000, 1000))

## ----echo=FALSE,out.width="300px",fig.align='default'-------------------------
knitr::include_graphics(c("https://raw.githubusercontent.com/BIMSBbioinfo/genomation/master/vignettes/Figures/heatMatrix1-1.png",
                          "https://raw.githubusercontent.com/BIMSBbioinfo/genomation/master/vignettes/Figures/heatMatrix1-2.png"),dpi=30)

## ----heatMatrix2, eval=FALSE, tidy=TRUE, fig.width=4, fig.height=3------------
# library(GenomicRanges)
# data(cage)
# data(promoters)
# data(cpgi)
# 
# sm = ScoreMatrix(target=cage, windows=promoters, strand.aware=TRUE)
# cpg.ind = which(countOverlaps(promoters, cpgi)>0)
# nocpg.ind = which(countOverlaps(promoters, cpgi)==0)
# heatMatrix(sm, xcoords=c(-1000, 1000), group=list(CpGi=cpg.ind, noCpGi=nocpg.ind))

## ----echo=FALSE,out.width="400px"---------------------------------------------
knitr::include_graphics("https://raw.githubusercontent.com/BIMSBbioinfo/genomation/master/vignettes/Figures/heatMatrix2-1.png")

## ----heatMatrixScales, eval=FALSE, tidy=TRUE, echo=TRUE, fig.width=4, fig.height=3, dpi=300----
# sm.scaled = scaleScoreMatrix(sm)
# heatMatrix(sm.scaled, xcoords=c(-1000, 1000), group=list(CpGi=cpg.ind, noCpGi=nocpg.ind))

## ----echo=FALSE,out.width="400px"---------------------------------------------
knitr::include_graphics("https://raw.githubusercontent.com/BIMSBbioinfo/genomation/master/vignettes/Figures/heatMatrixScales-1.png")

## ----multiHeatMatrix1, eval=FALSE, tidy=TRUE, fig.width=4.5, fig.height=3, dpi=300----
# cage$tpm = NULL
# targets = list(cage=cage, cpgi=cpgi)
# sml = ScoreMatrixList(targets=targets, windows=promoters, bin.num=50, strand.aware=TRUE)
# multiHeatMatrix(sml, xcoords=c(-1000, 1000))

## ----echo=FALSE,out.width="400px"---------------------------------------------
knitr::include_graphics("https://raw.githubusercontent.com/BIMSBbioinfo/genomation/master/vignettes/Figures/multiHeatMatrix1-1.png")

## ----multiHeatMatrix2, eval=FALSE, tidy=TRUE, fig.width=4.5, fig.height=3, dpi=300----
# multiHeatMatrix(sml, xcoords=c(-1000, 1000), clustfun=function(x) kmeans(x, centers=2)$cluster)

## ----echo=FALSE,out.width="400px"---------------------------------------------
knitr::include_graphics("https://raw.githubusercontent.com/BIMSBbioinfo/genomation/master/vignettes/Figures/multiHeatMatrix2-1.png")

## ----getFiles_annotation, eval=TRUE, tidy=TRUE--------------------------------
library(genomationData)
genomationDataPath = system.file('extdata',package='genomationData')
sampleInfo = read.table(file.path(genomationDataPath, 'SamplesInfo.txt'), 
                        header=TRUE, sep='\t', stringsAsFactors=FALSE)

peak.files=list.files(genomationDataPath, full.names=TRUE, pattern='broadPeak')
names(peak.files) = sampleInfo$sampleName[match(basename(peak.files),sampleInfo$fileName)]

ctcf.peaks = readBroadPeak(peak.files['Ctcf'])

## ----readAllPeaks_annotation, eval=TRUE, tidy=TRUE----------------------------
data(cpgi)
peak.annot = annotateWithFeature(ctcf.peaks, cpgi, intersect.chr=TRUE)
peak.annot

## ----readTranscriptFeatures, eval=TRUE, tidy=TRUE-----------------------------
bed.file=system.file("extdata/chr21.refseq.hg19.bed", package = "genomation")
gene.parts = readTranscriptFeatures(bed.file)

## ----annotateCtcf, eval=TRUE, tidy=TRUE---------------------------------------
ctcf.annot=annotateWithGeneParts(ctcf.peaks, gene.parts, intersect.chr=TRUE)
ctcf.annot

## ----annotateTargetList, eval=TRUE, tidy=TRUE---------------------------------
peaks = GRangesList(lapply(peak.files, readGeneric))
names(peaks) = names(peak.files)
annot.list = annotateWithGeneParts(peaks, gene.parts, intersect.chr=TRUE)

## ----plotGeneAnnotation, eval=FALSE, tidy=TRUE, dpi=300-----------------------
# plotGeneAnnotation(annot.list)
# plotGeneAnnotation(annot.list, cluster=TRUE)

## ----echo=FALSE,out.width="300px",fig.align='default'-------------------------
knitr::include_graphics(c("https://raw.githubusercontent.com/BIMSBbioinfo/genomation/master/vignettes/Figures/plotGeneAnnotation-1.png",
                     "https://raw.githubusercontent.com/BIMSBbioinfo/genomation/master/vignettes/Figures/plotGeneAnnotation-2.png"     
                          ))

## ----selectBamChipseq, eval=TRUE, tidy=TRUE-----------------------------------
genomationDataPath = system.file('extdata',package='genomationData')
bam.files = list.files(genomationDataPath, full.names=TRUE, pattern='bam$')
bam.files = bam.files[!grepl('Cage', bam.files)]

## ----readCtcfPeaks, eval=TRUE, tidy=TRUE--------------------------------------
ctcf.peaks = readBroadPeak(file.path(genomationDataPath, 
                                     'wgEncodeBroadHistoneH1hescCtcfStdPk.broadPeak.gz'))
ctcf.peaks = ctcf.peaks[seqnames(ctcf.peaks) == 'chr21']
ctcf.peaks = ctcf.peaks[order(-ctcf.peaks$signalValue)]
ctcf.peaks = resize(ctcf.peaks, width=1000, fix='center')

## ----ctcfScoreMatrixList, eval=FALSE, tidy=TRUE, fig.width=5, fig.height=3.5, dpi=300----
# sml = ScoreMatrixList(bam.files, ctcf.peaks, bin.num=50, type='bam')
# sampleInfo = read.table(system.file('extdata/SamplesInfo.txt',
#                                     package='genomationData'),header=TRUE, sep='\t')
# names(sml) = sampleInfo$sampleName[match(names(sml),sampleInfo$fileName)]
# multiHeatMatrix(sml, xcoords=c(-500, 500), col=c('lightgray', 'blue'))

## ----echo=FALSE,out.width="350px",fig.cap="Heatmap profile of unscaled coverage shows a slight colocalization of Ctcf, Rad21 and Znf143."----
knitr::include_graphics(c("https://raw.githubusercontent.com/BIMSBbioinfo/genomation/master/vignettes/Figures/ctcfScoreMatrixList-1.png"     
                          ))

## ----plotScaledProfile, eval=FALSE, tidy=TRUE, fig.width=5, fig.height=3.5, dpi=300----
# sml.scaled = scaleScoreMatrixList(sml)
# multiHeatMatrix(sml.scaled, xcoords=c(-500, 500), col=c('lightgray', 'blue'))

## ----echo=FALSE,out.width="350px",fig.cap="Heatmap profile of scaled coverage shows much stronger colocalization   of the transcription factors; nevertheless, it is evident that some of the CTCF   peaks have a very weak enrichment"----
knitr::include_graphics(c("https://raw.githubusercontent.com/BIMSBbioinfo/genomation/master/vignettes/Figures/plotScaledProfile-1.png"     
                          ))

## ----eadAllPeaks, eval=TRUE, tidy=TRUE----------------------------------------
genomationDataPath = system.file('extdata',package='genomationData')
sampleInfo = read.table(file.path(genomationDataPath, 'SamplesInfo.txt'), 
                        header=TRUE, sep='\t', stringsAsFactors=FALSE)


peak.files = list.files(genomationDataPath, full.names=TRUE, pattern='Peak.gz$')
peaks = list()
for(i in 1:length(peak.files)){
  file = peak.files[i]
  name=sampleInfo$sampleName[match(basename(file),sampleInfo$fileName)]
  message(name)
  peaks[[name]] = readGeneric(file, meta.col=list(score=5))
}
peaks = GRangesList(peaks)
peaks = peaks[seqnames(peaks) == 'chr21' & width(peaks) < 1000  & width(peaks) > 100]

## ----findFeatureComb, eval=TRUE, tidy=TRUE------------------------------------
  tf.comb = findFeatureComb(peaks, width=1000)

## ----visualizeFeatureComb, eval=FALSE, tidy=TRUE, fig.width=5, fig.height=3, dpi=300----
#   tf.comb = tf.comb[order(tf.comb$class)]
#   bam.files = list.files(genomationDataPath, full.names=TRUE, pattern='bam$')
#   bam.files = bam.files[!grepl('Cage', bam.files)]
#   sml = ScoreMatrixList(bam.files, tf.comb, bin.num=20, type='bam')
#   names(sml) = sampleInfo$sampleName[match(names(sml),sampleInfo$fileName)]
#   sml.scaled = scaleScoreMatrixList(sml)
#   multiHeatMatrix(sml.scaled, xcoords=c(-500, 500), col=c('lightgray', 'blue'))

## ----echo=FALSE,out.width="350px"---------------------------------------------
knitr::include_graphics(c("https://raw.githubusercontent.com/BIMSBbioinfo/genomation/master/vignettes/Figures/visualizeFeatureComb-1.png"     
                          ))

## ----annotationHubExample1, eval=FALSE, tidy=TRUE-----------------------------
# library(AnnotationHub)
# ah = AnnotationHub()
# # retrieve CpG island data from annotationHub
# cpgi_query <- query(ah, c("CpG Islands", "UCSC", "hg19"))
# cpgi <- ah[[names(cpgi_query)]]
# dnase_query <- query(ah, c("wgEncodeOpenChromDnase8988tPk.narrowPeak.gz", "UCSC", "hg19"))
# dnaseP <- ah[[names(dnase_query)]]
# sm = ScoreMatrixBin(target=dnaseP, windows=cpgi, strand.aware=FALSE)

## -----------------------------------------------------------------------------
sessionInfo()

# functions for plotting metaProfiles:


# default color scheme for heatmaps

#.jets <- colorRampPalette(c("#334b8e", "#456ca7", "#71c6cd", "#8fc56c", 
#                           "#f3e92b","#e6762b","#d9272a"))
.jets<-colorRampPalette(c("#00007F", "blue", "#007FFF", "cyan",
                          "#7FFF7F", "yellow", "#FF7F00", "red", "#7F0000"))

# winsorize a matrix using percentile ranges
.winsorize<-function(mat,rng){
  hi.th=quantile(mat,rng[2]/100,na.rm=TRUE)
  lo.th=quantile(mat,rng[1]/100,na.rm=TRUE)
  mat[mat>hi.th]=hi.th
  mat[mat<lo.th]=lo.th
  mat
}

# ---------------------------------------------------------------------------- #
#' Heatmap for meta-region profiles
#' 
#' Function calculates meta-profile(s) from a ScoreMatrix or a ScoreMatrixList, then
#' produces a heatmap or a set of stacked heatmaps for meta-region profiles
#'
#' @param mat \code{ScoreMatrix} or \code{ScoreMatrixList} to be plotted 
#' @param centralTend a character that determines central tendency of meta-profile(s). 
#'                     It takes "mean" (default) or "median".
#' @param profile.names a character vector for names of profiles. If NULL, 
#'                      the names
#'                      will be taken from names(mat) if mat is a 
#'                      \code{ScoreMatrixList} object.
#' @param xcoords a vector of numbers showing relative positions of the bases or 
#'                windows. It must match the number of columns in the \code{ScoreMatrix}
#'                For example: if there are 2001 elements in the matrices which
#'                are base-pair resolution 
#'                and they are centered around an anchor point like TSS, the xcoords
#'                argument should be -1000:1000. This argument is used to plot
#'                accurate x-axis labels for the plots.If NULL it will be equal
#'                to 1:ncol(mat).
#' @param meta.rescale if TRUE meta-region profiles are scaled to 0 to 1 range by
#'                     subracting the min from profiles and dividing them by 
#'                     max-min.
#' @param winsorize Numeric vector of two, defaults to c(0,100). This vector 
#'                  determines the upper and lower percentile values to limit the 
#'                  extreme values. For example, c(0,99) will limit the values to
#'                  only 99th percentile, everything above the 99 percentile will 
#'                  be equalized to the value of 99th percentile. This is useful 
#'                  for visualization of matrices that have outliers.             
#' @param col a vector of color pallete. 
#'        color scheme to be used. If NULL, a version of jet colors will be
#'            used.
#' @param legend.name a character label plotted next to the legend
#' @param cex.legend  A numerical value giving the amount by which 
#'                    legend axis marks should be magnified relative to the default
#' @param xlab label a character string for x-axis
#' @param main a character string for the plot title
#' @param cex.lab  A numerical value giving the amount by which 
#'                    axis labels (including 'legend.name') 
#'                    should be magnified relative to the default.
#' @param cex.axis  A numerical value giving the amount by which 
#'                    axis marks should be magnified relative to the default
#' 
#' @return returns meta-profile matrix invisibly.
#' 
#' 
#' @examples
#'  data(cage)
#'  data(promoters)
#'  scores1=ScoreMatrix(target=cage,windows=promoters,strand.aware=TRUE)
#'  data(cpgi)
#'  scores2=ScoreMatrix(target=cpgi,windows=promoters,strand.aware=TRUE)
#' 
#'  x=new("ScoreMatrixList",list(scores1,scores2))
#'  \donttest{
#'  heatMeta(mat=x,legend.name="fg",cex.legend=0.8,main="fdf",cex.lab=6,
#'           cex.axis=0.9)
#'  }
#' @export
heatMeta<-function(mat, centralTend="mean",
                   profile.names=NULL,xcoords=NULL,col=NULL,
                   meta.rescale=FALSE, winsorize=c(0,100),
                   legend.name=NULL,cex.legend=1,xlab=NULL,
                   main="",cex.lab=1,cex.axis=1){
  
  # check class
  if(! class(mat) %in% c("ScoreMatrix","ScoreMatrixList"))
    stop("mat is not ScoreMatrix or ScoreMatrixList\n")
  # check centralTend
  if(! centralTend %in% c("median","mean"))
    stop("centralTend is not mean or median\n")
  
  
  # get meta profiles by taking the mean
  if( class(mat)=="ScoreMatrix" ){
    if(centralTend=="mean"){
      metas=list(colMeans(mat,na.rm=TRUE))
    }else{
      metas=list(apply(mat, 2, function(x) median(x,na.rm=TRUE)))
    }
  }else if( class(mat)=="ScoreMatrixList" ){
    if(centralTend=="mean"){
    metas=lapply(mat,function(a) colMeans(a,na.rm=TRUE) )
    }else{
    metas=lapply(mat,function(a) apply(a, 2, function(x) median(x,na.rm=TRUE)) )
    }
  }  
  
  # if the ncols of matrices do not match do not plot anything
  if(length(unique(sapply(metas,ncol))) != 1){
    stop("ScoreMatrix number of columns do not match\n",
         "Try using binMatrix to make matrices with large number of columns",
         "equal to the smaller ones\n")
  }
  
  # get the default xcoordinates to plot
  if(!is.null(xcoords)){
    
    # if it is a two element vector of start and end coordinates 
    # of the first and the last column of the matrix
    if(length(xcoords)==2 & xcoords[1]<xcoords[2]){
      xcoords=seq(xcoords[1],xcoords[2],length.out=length(metas[[1]]) )
    }
    
    if(length(xcoords) != length(metas[[1]])) 
      stop("xcoords has wrong length: ",length(xcoords)," \n",
           "it should be equal to the number of columns of ScoreMatrices\n",
           "which is: ",length(metas[[1]]),"\n")    
  }else{
    xcoords=1:length(metas[[1]])
  }
  
  
  # try to get profile names from names of ScoreMatrixList
  if(is.null(profile.names) & !is.null(names(mat)) & class(mat)=="ScoreMatrixList" )
  {
    profile.names=names(mat)
  }
  # if user wants scaling
  if(meta.rescale){
    metas=lapply(metas,function(x) (x-min(x))/(max(x)-min(x)  )  )
  }
  
  img=do.call("rbind",metas)
  
  if(is.null(col)){
    col=.jets(100)
  }
  
  marOrg=par()$mar # get original parMar to be used later
  marNew=marOrg
  marNew[4]=6.1
  par(mar=marNew)
  image(x=xcoords,y=1:nrow(img),z=as.matrix(t(img[nrow(img):1,,drop=FALSE])),useRaster=TRUE,
        col=col,yaxt="n",ylab="",xlab=xlab,main=main,
        cex.lab=cex.lab,cex.axis=cex.axis)
  if(!is.null(profile.names)){
    axis(side=4,at=1:nrow(img),labels=rev(profile.names),
         las=2,cex.axis=cex.axis)
  }
  
  #plot.new()
  vps <- baseViewports()
  pushViewport(vps$figure) # get the plot viewport from base graphics
  # showViewport(current.viewport());current.vpTree()
  #grid.text(c("one"),
  #          x=unit(1, "native"), y=unit(2, "native"),
  #          just="right", rot=60)
  
  if(par()$mar[2]<4.1){
    warning("left margin of the plot (set by mar in par()) should not be less then 4.1 lines")
  }
  # make view port for the legend
  legendVp <- viewport(width=unit(1, "lines"), height=unit(0.4, "npc"),
                       x = unit(3, "lines"), y = unit(0.5, "npc"),just="left")
  pushViewport(legendVp) # push the legend VP
  #current.viewport()
  current.vpTree()
  rng=range(img)
  .heatLegendY(min=rng[1],max=rng[2],cols=col,
               legend.name=legend.name,main=TRUE,cex.legend=cex.legend,
               cex.lab=cex.lab)
  popViewport(2) # remove the legend VP
  current.vpTree()         
  par(mar=marOrg)              
  
  metas=do.call("rbind",metas) 
  invisible(metas)
}

# function based on plotrix::dispersion, 
# and additionally it takes into account NA values in data
# the problem is that when scoreMatrix is calculated then
# some of the columns have only one numerical value and rest of them is NA
# and then mean of such column is a numerical value, but variation e.g. sd is NA.
# border = NA omits borders.
# Args:
# ulim and llim: the extent of the dispersion measures (upper and lower band)           
# intervals: whether the limits are intervals (TRUE) or absolute values (FALSE)
# border: line type for drawing a border on the confidence band, e.g. 2
.dispersion2 <- function(x, y, ulim, llim=ulim, intervals=TRUE, 
                               border = NA, ...){ 
  if (intervals) {
    llim <- y - llim
    ulim <- y + ulim
  }
  ulim.na <- is.na(ulim)
  if(any(ulim.na)){
    #selecting 'segments' of numerical values that are located between NA's
    w <- which(ulim.na)
    previous.v <- 0 #location of NA to the left of segment
    for(i in 1:length(w)){ #for each segment
      if(w[i]==previous.v+1){
        #if there are some neighboring NA
        previous.v <- w[i]
        next
      }
      from <- previous.v+1
      to <- w[i]-1
      #running polygon separately for each segment
      polygon(c(x[from:to], rev(x[from:to])), 
              c(llim[from:to], rev(ulim[from:to])), 
              border = border, ...)      
      previous.v <- w[i]
    }
    from <- previous.v+1
    to <- length(x)
    polygon(c(x[from:to], rev(x[from:to])), 
            c(llim[from:to], rev(ulim[from:to])), 
            border = border, ...)
  }else{
    polygon(c(x, rev(x)), c(llim, rev(ulim)),
            border = border, ...)
  }
}
           
# ---------------------------------------------------------------------------- #
#' Line plot(s) for meta-region profiles
#' 
#' Function calculates meta-profile(s) from a ScoreMatrix or a ScoreMatrixList, then
#' produces a line plot or a set of line plots for meta-region profiles
#' 
#' @param mat \code{ScoreMatrix} or \code{ScoreMatrixList} object. If it is a 
#' \code{ScoreMatrixList} object, all matrices in the ScoreMatrixList should have 
#' the same number of 
#' columns.
#' @param centralTend a character that determines central tendency of meta-profile(s). 
#'                     It takes "mean" (default) or "median".
#' @param overlay If TRUE multiple profiles will be overlayed in the same plot
#'                (Default:TRUE). If FALSE, and mat is a ScoreMatrixList, consider
#'                using par(mfrow=c(1,length(mat))) to see the plots from all
#'                matrices at once.
#' @param winsorize Numeric vector of two, defaults to c(0,100). This vector 
#'                  determines the upper and lower percentile values to limit the 
#'                  extreme values. For example, c(0,99) will limit the values to
#'                  only 99th percentile, everything above the 99 percentile will 
#'                  be equalized to the value of 99th percentile.This is useful 
#'                  for visualization of matrices that have outliers.
#' @param profile.names a character vector for names of the profiles. The order
#'        should be same as the as the order of ScoreMatrixList.
#' @param xcoords a numeric vector which designates 
#'        relative base positions of the meta-region profiles.
#'        For example, for a 2001 column ScoreMatrix, xcoord=-1000:1000 indicates
#'        relative positions of each column in the score matrix. If NULL (Default),
#'        xcoords equals to 1:ncol(mat) 
#' @param meta.rescale if TRUE meta-region profiles are scaled to 0 to 1 range by
#'                     subtracting the min from profiles and dividing them by max-min.
#'                     If dispersion is not NULL, then dispersion will be scaled as well. 
#' @param smoothfun a function to smooth central tendency and dispersion bands (Default: NULL), e.g. 
#'                    stats::lowess.
#' @param line.col color of lines for \code{centralTend} of meta-region profiles. Defaults to colors from
#'        \code{rainbow()} function.
#' @param ylim same as \code{ylim} at \code{\link{plot}} function. 
#'             if NULL ylim is estimated from all meta-region profiles.
#' @param ylab same as \code{ylab} at \code{\link{plot}} function. 
#'             Default: "average score"
#' @param xlab same as \code{xlab} at \code{\link{plot}} function. 
#'             Default: "bases"
#' @param dispersion shows dispersion interval bands around \code{centralTend} (default:NULL). It takes 
#'        one of the character:
#' \itemize{
#'  \item{"se"}{shows standard error of the mean and 95 percent confidence interval for the mean}
#'  \item{"sd"}{shows standard deviation and 2*(standard deviation)}
#'  \item{"IQR"}{shows 1st and 3rd quartile and 
#'               confidence interval around the median based on the median +/- 1.57 * IQR/sqrt(n) (notches)}
#' }
#' @param dispersion.col color of bands of \code{dispersion}.
#'        Defaults to colors from \code{rainbow()} and transparency is set to 0.5
#'        (rainbow(length(mat), alpha = 0.5)).
#' @param ... other options to \code{\link{plot}}
#' 
#' @return returns the meta-region profiles invisibly as a matrix.
#' 
#' @note
#' Score matrices are plotted according to ScoreMatrixList order. 
#' If ScoreMatrixList contains more than one matrix then they will
#' overlap each other on a plot, i.e.
#' the first one is plotted first and every next one overlays previous one(s) and 
#' the last one is the topmost.
#' 
#' Missing values in data slow down plotting dispersion around central tendency.
#' The reason is that dispersion is plotted only for non-missing values,
#' for each segment that
#' contains numerical values \code{graphics::polygon} function is used to plot dispersion bands.
#' There might be a situation, when in a column of ScoreMatrix is only one
#' numeric number and the rest are NAs, then at corresponding position 
#' only central tendency will be plotted.
#' 
#' Notches show the 95 percent confidence interval for the median 
#' according to an approximation based on the normal distribution.
#' They are used to compare groups - if notches corresponding to adjacent base pairs
#' on the plot do not overlap, this is strong evidence that medians differ.
#' Small sample sizes (5-10) can cause notches to extend beyond the interquartile range (IQR) 
#' (Martin Krzywinski \emph{et al}. \emph{Nature Methods 11}, 119-120 (2014))
#' @examples
#'  data(cage)
#'  data(promoters)
#'  scores1=ScoreMatrix(target=cage,windows=promoters,strand.aware=TRUE)
#'
#'  data(cpgi)
#'  scores2=ScoreMatrix(target=cpgi,windows=promoters,strand.aware=TRUE)
#' 
#'  # create a new ScoreMatrixList
#'  x=new("ScoreMatrixList",list(scores1,scores2))
#'  \donttest{
#'  plotMeta(mat=x,overlay=TRUE,main="my plotowski")
#' 
#'  # plot dispersion nd smooth central tendency and variation interval bands
#'  plotMeta(mat=x, centralTend="mean", dispersion="se", winsorize=c(0,99), 
#'          main="Dispersion as interquartile band", lwd=4, 
#'          smoothfun=function(x) stats::lowess(x, f = 1/5))
#'  }
#' @export
#' @docType methods
#' @rdname plotMeta
#' 
plotMeta<-function(mat, centralTend="mean",
                   overlay=TRUE,winsorize=c(0,100),
                   profile.names=NULL,xcoords=NULL,
                   meta.rescale=FALSE,
                   smoothfun=NULL,
                   line.col=NULL,
                   dispersion=NULL,dispersion.col=NULL,
                   ylim=NULL,ylab="average score",xlab="bases", ...){
  
  
  # check class
  if(! class(mat) %in% c("ScoreMatrix","ScoreMatrixList"))
    stop("mat is not ScoreMatrix or ScoreMatrixList\n")
  # check centralTend args
  if(! centralTend %in% c("median","mean"))
    stop("centralTend is not mean or median\n")
  # check dispersion args
  disp.args <- c("se","sd","IQR") #dispersion arguments
  if(!is.null(dispersion) && !dispersion %in% c(disp.args)){
      stop("dispersion is not FALSE, 'se', 'sd' or 'IQR'\n")
  }

  # check smoothfun
  if(!is.null(smoothfun) & !is.function(smoothfun)){stop("'smoothfun' has to be a function or NULL\n")}
  
  
  if(is.null(line.col) & is.null(dispersion))
    line.col=ifelse(is.list(mat),
                    list(rainbow(length(mat))),
                    "black")[[1]]
  if(is.null(line.col) & is.null(dispersion.col)){
    dispersion.col=ifelse(is.list(mat),
                          list(rainbow(length(mat), alpha = 0.4)),
                          rainbow(1, alpha=0.4))[[1]]
    line.col=ifelse(is.list(mat),
                    list(rainbow(length(mat))),
                    rainbow(1))[[1]]
  }
  
  #mat is always a list/ScoreMatrixList
  if( class(mat)=="ScoreMatrix" ){
    mat <- list(mat)
  }
  
  # if the ncols of matrices do not match do not plot anything
  if(length(unique(sapply(mat,ncol))) != 1){
    stop("ScoreMatrix number of columns do not match\n",
         "Try using binMatrix to make matrices with high number of columns",
         "equal\n")
  }
  
  #init of some variables before for loop
  if(!is.null(dispersion) && dispersion %in% disp.args){
    bound1<-list()
    if(dispersion=="IQR"){q1<-list(); q3<-list();
    }else{bound2 <- list()}
  }
  metas<-list()
  
  for(i in 1:length(mat)){ #for every score matrix
    
    # this can set extreme values to given percentile
    if(winsorize[2]<100 | winsorize[1]>0){
      mat[[i]]=.winsorize(mat[[i]]@.Data, winsorize)
    }
    
    # get meta profiles by taking the mean/median
    if(centralTend=="mean"){
      if(!is.null(dispersion) && dispersion=="IQR"){
           warning("dispersion is set to show 1st and 3rd quartile and 
                confidence interval around the median, 
                but centralTend is 'mean'. Setting centralTend to 'median'..\n")
        metas[[i]]=colMedians(mat[[i]], na.rm=TRUE)
      }else{
        metas[[i]]=colMeans(mat[[i]], na.rm=TRUE) 
      }
    }else if(centralTend=="median"){
      if(!is.null(dispersion) && dispersion=="se"){
        warning("dispersion is set to standard error of the mean and 95% confidence interval for the mean, but
                centralTend is 'median'. Setting centralTend to 'mean'..\n")
        metas[[i]]=colMeans(mat[[i]],na.rm=TRUE) 
      }else{
        metas[[i]]=colMedians(mat[[i]], na.rm=TRUE)
      }
    }

    # calculate dispersion around the mean/median
    if(!is.null(dispersion) && dispersion %in% disp.args){      
      if(dispersion=="se"){
        bound1[[i]] <- std.error(mat[[i]], na.rm = TRUE)
        bound2[[i]] <- bound1[[i]] * 1.96
      }else if(dispersion=="sd"){
        bound1[[i]] <- colSds(mat[[i]], na.rm=TRUE)
        bound2[[i]] <- bound1[[i]] * 2
      }else if(dispersion=="IQR"){
        q <- colQuantiles(mat[[i]], probs=c(0.25, 0.75), na.rm=TRUE)
        q1[[i]] <- q[,1] #1st quartile
        q3[[i]] <- q[,2] #3rd quartile
        n<-nrow(mat[[i]])
        bound1[[i]] <- (1.57*(q3[[i]] - q1[[i]])) / sqrt(n) #notch
      }
    }
    
    if(meta.rescale){
      val2unit <- function(x){(x-min(x, na.rm = TRUE))/(max(x, na.rm = TRUE)-min(x, na.rm = TRUE))} 
      metas[[i]]=val2unit(metas[[i]])
      if(!is.null(dispersion) && dispersion %in% disp.args){
        bound1[[i]]=val2unit(bound1[[i]])
        if(dispersion=="IQR"){
          q1[[i]]=val2unit(q1[[i]])
          q3[[i]]=val2unit(q3[[i]]) 
        }else{
          bound2[[i]]=val2unit(bound2[[i]])
        }
      }
    }

    #smoothing
    if(!is.null(smoothfun)){
      metas[[i]] <- smoothfun(metas[[i]])$y
      if(!is.null(dispersion) && dispersion %in% disp.args){
        bound1[[i]] <- smoothfun(bound1[[i]])$y
        if(dispersion=="IQR"){
          q1[[i]] <- smoothfun(q1[[i]])$y
          q3[[i]] <- smoothfun(q3[[i]])$y
        }else{
          bound2[[i]] <- smoothfun(bound2[[i]])$y
        }
      }
    }

  }#end of for loop
  

  # get the default xcoordinates to plot
  if(!is.null(xcoords)){
    
    # if it is a two element vector of start and end coordinates 
    # of the first and the last column of the matrix
    if(length(xcoords)==2 & xcoords[1]<xcoords[2]){
      xcoords=seq(xcoords[1],xcoords[2],length.out=length(metas[[1]]) )
    }
    
    if(length(xcoords) != length(metas[[1]])) 
      stop("xcoords has wrong length: ",length(xcoords)," \n",
           "it should be equal to the number of columns of ScoreMatrices\n",
           "which is: ",length(metas[[1]]),"\n")    
  }else{
    xcoords=1:length(metas[[1]])
  }
  
  
  # if ylim is not NULL, change the ranges to plot to ylim
  if(!is.null(ylim)){
    myrange=ylim
  }else{
    myrange=range(unlist(metas), na.rm = TRUE)
    if(!is.null(dispersion) && dispersion %in% disp.args){
      if(dispersion!="IQR"){
        myrange[2] <- max(unlist(metas) + unlist(bound2), na.rm = TRUE)
        myrange[1] <- min(unlist(metas) - unlist(bound2), na.rm = TRUE)
      }else{
        # can be a situation when q1 or q2 are equal to median
	max.q3 <- max(unlist(metas) + unlist(q3), na.rm = TRUE)
	min.q1 <- min(unlist(metas) - unlist(q1), na.rm = TRUE)
	max.notch.upper <- max(unlist(metas) + unlist(bound1), na.rm = TRUE) 
	min.notch.lower <- min(unlist(metas) - unlist(bound1), na.rm = TRUE)
	if(min.notch.lower < min.q1){
	    myrange[1] <- min.notch.lower
	}else{
	    myrange[1] <- min.q1
	}
	if(max.notch.upper > max.q3){
	    myrange[2] <- max.notch.upper
	}else{
	    myrange[2] <- max.q3
        }

      }
    }
  }
  
  marOrg=par()$mar # get original parMar to be used later
  marNew=marOrg
  marNew[4]=8
  par(mar=marNew) # extend right margin for the legend
  par(xpd=TRUE) # do this so that you can plot legend out of the plotting box
    
  if(overlay & length(metas)>1){
    # plot overlayed lines

    if(!is.null(dispersion) && dispersion %in% disp.args){
    
      plot(xcoords,metas[[1]],type="l",col=dispersion.col[1],
           ylim=myrange,ylab=ylab,xlab=xlab, ...)
      for(i in 1:length(metas)){
        # "IQR"
        if(dispersion=="IQR"){ 
          # plotting notches             
          .dispersion2(xcoords, metas[[i]], bound1[[i]],
                     col=dispersion.col[i], ...)
          # plotting 1st and 3rd quartile
          .dispersion2(xcoords, metas[[i]], 
		       llim=q1[[i]], ulim=q3[[i]], intervals=FALSE,
                       col=dispersion.col[i], ...)
          # if 1st or 3rd quartile is equal to median, then
          # colour notch(es) again
          if(all(metas[[i]]==q1[[i]])){
             # then colour 'lower' notch again
             .dispersion2(xcoords, metas[[i]], 
                       llim=bound1[[i]], ulim=metas[[i]],
                       col=dispersion.col[i], ...)
          }
          if(all(metas[[i]]==q3[[i]])){
	     # then colour 'upper' notch again
	     .dispersion2(xcoords, metas[[i]], 
	                  ulim=bound1[[i]], llim=metas[[i]],
                          col=dispersion.col[i], ...)
          }
        
        # "sd" or "se"
        }else{        
	  .dispersion2(xcoords, metas[[i]], bound2[[i]],
                   col=dispersion.col[i], ...)
          .dispersion2(xcoords, metas[[i]], bound1[[i]],
                     col=dispersion.col[i], ...)
        }  
      }
      for(j in 1:length(metas) ){ #drawing central tendency line(s) on top
        lines(xcoords,metas[[j]],col=line.col[j],...)
      }
      
    }else{ #without plotting dispersion
      plot(xcoords,metas[[1]],type="l",col=line.col[1],
           ylim=myrange,ylab=ylab,xlab=xlab,...)
      for(i in 2:length(metas) ){ #drawing central tendency line(s) on top
        lines(xcoords,metas[[i]],col=line.col[i],...)
      }
    }
    
    # if profile names are given, plot them as legend
    if(!is.null(profile.names))
      if(!is.null(dispersion) && dispersion %in% disp.args){
        legend(max(xcoords)+0.05*max(xcoords),myrange[2],legend=profile.names
               ,fill=dispersion.col,bty="n", border=line.col)
      }else{
        legend(max(xcoords)+0.05*max(xcoords),myrange[2],legend=profile.names
               ,fill=line.col,bty="n")
      }
  }else{ # plot things one by one, in this case user must use par
    
    for(j in 1:length(metas)){
      plot(xcoords,metas[[j]],type="l",col=line.col[j],
           ylim=myrange,ylab=ylab,xlab=xlab,...)
      if(!is.null(dispersion) && dispersion %in% disp.args){
        if(dispersion=="IQR"){
          .dispersion2(xcoords, metas[[j]], bound1[[j]],
                       col=dispersion.col[j], ...)
          .dispersion2(xcoords, metas[[j]], 
                       llim=metas[[j]]-q1[[j]], ulim=q3[[j]]-metas[[j]],
                       col=dispersion.col[j], ...)
                       
          # if 1st or 3rd quartile is equal to median, then
          # colour notch(es) again             
          if(all(metas[[i]]==q1[[i]])){
             # then colour 'lower' notch again
             .dispersion2(xcoords, metas[[i]], 
                       llim=bound1[[i]], ulim=metas[[i]],
                       col=dispersion.col[i], ...)
          }
          if(all(metas[[i]]==q3[[i]])){
	     # then colour 'upper' notch again
	     .dispersion2(xcoords, metas[[i]], 
	                  ulim=bound1[[i]], llim=metas[[i]],
                          col=dispersion.col[i], ...)
          }             
                       
                       
        }else{
          .dispersion2(xcoords, metas[[j]], bound1[[j]],
                     col=dispersion.col[j], ...)
          .dispersion2(xcoords, metas[[j]], bound2[[j]],
                     col=dispersion.col[j], ...)
        }
      }
      lines(xcoords,metas[[j]],col=line.col[j],...)
    }
  }
  # revert par shit to its original state
  par(xpd=FALSE)
  par(mar=marOrg)
  
  invisible(do.call("rbind",metas))
}


# put a y axis legend
.heatLegendY<-function(min,max,cols,legend.name,main=TRUE,cex.legend=1,
                       cex.lab=1){
  
  # get value range as a vector of 100
  vals=seq(min,max,length.out=100)
  rng <- range(vals, na.rm = TRUE) # get min/mqx
  m <- (vals - min)/(max-min) # get normalized range
  rasta= rgb(colorRamp(cols)(m), maxColorValue= 255) # get color for each element of range
  
  grid.raster( rev(rasta), interpolate=FALSE,height = unit(1, "npc"),
               width=unit(1, "npc")) # make the legend
  # make legend ticks
  at = seq(0,1,length.out=5); label = seq(min,max,length.out=5)
  
  #make the axis of the legend
  grid.yaxis(at=at,label=formatC(label,digits=2,format="g"),main=main,
             edits=gEdit("labels", rot=90,hjust=0.5),
             gp=gpar(cex=cex.legend)) # make axis for legend
  my.x = -2
  if(main==FALSE) 
    my.x=3.4
  grid.text(legend.name,rot=90,x=unit(my.x, "npc"),gp=gpar(cex=cex.lab))
  
}

.heatLegendX<-function(min,max,cols,legend.name,main=TRUE,cex.legend=1,
                       cex.lab=1,hjust=0,vjust=0){
  
  # get value range as a vector of 100
  vals=seq(min,max,length.out=100)
  rng <- range(vals, na.rm = TRUE) # get min/mqx
  m <- (vals - min)/(max-min) # get normalized range
  rasta= rgb(colorRamp(cols)(m), maxColorValue = 255) # get color for each element of range
  
  grid.raster( matrix((rasta),nrow=1), interpolate=FALSE,height = unit(1, "npc"),
               width=unit(1, "npc")) # make the legend
  # make legend ticks
  label = pretty(c(min,max),n=5);at = seq(0,1,length.out=length(label)); 
  
  #make the axis of the legend
  grid.xaxis(at=at,label=label,main=main,
             edits=gEdit("labels",hjust=hjust,vjust=vjust),
             gp=gpar(cex=cex.legend)) # make axis for legend
  my.y = -3
  grid.text(legend.name,y=unit(my.y, "npc"),gp=gpar(cex=cex.lab))
  
}

# convert a matrix or vector to color matrix
# to be used in grid.raster()
.convertToColors <- function(mat,cols,rng=NULL) {
  
  if(is.null(rng)){
    # Produce 'normalized' version of matrix, with values ranging from 0 to 1
    rng <- range(mat, na.rm = TRUE)
  }
  
  # get the color number
  nc = length(cols)
  
  # arrange min and max of the matrix in different situations
  if (diff(rng) == 0) 
    rng <- if (rng[1L] == 0){ 
      c(-1, 1)
    }else{rng[1L] + c(-0.4, 0.4) * abs(rng[1L])}
  
  # normalize matrix
  mat <- (mat - rng[1L])/diff(rng)
  
  # assign color numbers to normalized matrix for each cell
  zi <- floor((nc - 1e-05) * mat + 1e-07)
  zi[zi < 0 | zi >= nc] <- NA
  
  # construct the final matrix
  zc <- cols[zi + 1L]
  dim(zc) <- dim(mat)
  return(zc)
}

# make a heatmap from a given matrix using grid.raster()
.gridHeat<-function(mat,col,xcoords,xlab,cex.lab,cex.axis,angle=0,
                    hjust=0,vjust=0,rng=NULL){
  
  mat2=.convertToColors(mat,col,rng)
  ras=grid.raster(mat2,interpolate = FALSE, width= unit(1, "npc"),
                  height=unit(1, "npc"))
  
  if(any(is.character(xcoords))) {
    # take legend labels 
    at = seq(0,1,length.out=length(xcoords)); label = xcoords
  } else {
  # make legend ticks
  at = seq(0,1,length.out=5); label = seq(min(xcoords),max(xcoords)
                                          ,length.out=5)
  }
  
  ax=grid.xaxis(at=at,label=formatC(label,digits=4,format="g"),
                edits=gEdit("labels", rot=angle,hjust=hjust,vjust=vjust),
                gp=gpar(cex=cex.axis)) # make axis for legend
  
  grid.text(xlab,y=unit(-2.5, "lines"),gp=gpar(cex=cex.lab)) 
  #grid.draw(ax)
}


.rowSideCol<-function(group.vector,group.names=NULL,group.col=NULL,cex.lab=1){
  
  if( is.null(group.col) ){
    cols=rainbow(length(unique(group.vector)))
    img=cols[factor(group.vector,levels=unique(group.vector))]
  }else{
    img=group.col[factor(group.vector,levels=unique(group.vector))]    
  }
  grid.raster(img,interpolate = FALSE, width= unit(1, "npc"),
              height=unit(1, "npc"))
  
  # segment heights calculated from group.vector
  # will be used to put group names in the middle of the segment
  segh=as.vector(table(factor(group.vector,levels=unique(group.vector))))
  name.coord=1-((cumsum(segh)-(segh/2))/sum(segh)) # NPC coord
  
  if( is.null(group.names)){
    grid.text(unique(group.vector), y=unit(name.coord,"npc"), 
              x = unit(-0.5, "lines"),
              gp=gpar(cex=cex.lab),just="right")
  }else{
    grid.text(group.names, y=unit(name.coord,"npc"),
              x = unit(-0.5, "lines"),
              gp=gpar(cex=cex.lab),just="right")    
  }
  
}

# ---------------------------------------------------------------------------- #
#' Draw a heatmap of a given ScoreMatrix object
#' 
#' 
#' The function makes a heatmap out of given \code{ScoreMatrix} object. If desired
#' it can use clustering using given clustering function 
#' (e.g. k-means) and plot cluster color codes as a sidebar. 
#' In addition, user can define groups of rows using 'group' argument.
#' 
#' @param mat a \code{ScoreMatrix} object
#' @param grid  if TRUE, grid graphics will be used. if FALSE, base graphics
#'               will be used on the top level, so users can use par(mfrow)
#'               or par(mfcol) prior to calling the function. Default:FALSE              
#' @param col   a vector of colors, such as the ones created by heat.colors(10).
#'              If NULL (which is default), jet color scheme (common in matlab
#'              plots) will be used.
#' @param xcoords a vector of numbers showing relative positions of the bases or 
#'                windows. It must match the number of columns in the \code{ScoreMatrix}. 
#'                Alternatively, it could be a numeric vector of two elements. Such
#'                as c(0,100) showing the relative start and end coordinates of the first
#'                and last column of the \code{ScoreMatrix} object.
#'
#' @param group a list of vectors of row numbers or a factor. This grouping is
#'              used for rowside colors of the heatmap. If it is a list,
#'              each element of the list must be a vector of row numbers. Names
#'              of the elements of the list will be used as names of groups. 
#'              If \code{group} is a factor
#'              , it's length must match the number of rows of the matrix, and 
#'              factor levels will be used as the names of the groups in the plot.
#
#' @param group.col a vector of color names to be used at the rowside colors if
#'                  \code{group} argument is given or \code{clustfun} function is given.
#' @param order    Logical indicating if the rows should be ordered or not 
#'                 (Default:FALSE). If \code{order=TRUE} the matrix will be ordered
#'                 with rowSums(mat) values in descending order. 
#'                 If \code{group} argument is provided, first the groups
#'                 will be ordered in descending order of sums of rows then, everything
#'                 within the clusters will be ordered by sums of rows.
#'                 If \code{clustfun} is given then rows within clusters
#'                 will be order in descending order of sums of rows.
#'                 
#' @param user.order a numerical vector indicating the order of groups/clusters (it works only
#'                   when \code{group} or \code{clustfun} argument is given). 
#'                   
#' @param winsorize Numeric vector of two, defaults to c(0,100). This vector 
#'                  determines the upper and lower percentile values to limit the 
#'                  extreme values. For example, c(0,99) will limit the values to
#'                  only 99th percentile, everything above the 99 percentile will 
#'                  be equalized to the value of 99th percentile.This is useful 
#'                  for visualization of matrices that have outliers.
#' @param clustfun  a function for clustering
#'                  rows of \code{mat} that returns 
#'                  a vector of integers indicating the cluster to which 
#'                  each point is allocated (a vector of cluster membership),
#'                  e.g. k-means algorithm with 3 centers: 
#'                  function(x) kmeans(x, centers=3)$cluster. 
#'                  By default FALSE.
#' @param main a character string for the plot title
#' @param legend.name a character label plotted next to the legend
#' @param cex.legend  A numerical value giving the amount by which 
#'                    legend axis marks should be magnified relative to the default
#' @param xlab label a character string for x-axis of the heatmap
#' @param cex.lab  A numerical value giving the amount by which 
#'                    axis labels (including 'legend.name') 
#'                    should be magnified relative to the default.
#' @param cex.main  A numerical value giving the amount by which 
#'                    plot title should be magnified  
#' @param cex.axis  A numerical value giving the amount by which 
#'                    axis marks should be magnified relative to the default
#' @param newpage logical indicating if \code{grid.newpage()} function should be
#'                invoked if \code{grid=TRUE}.
#'                
#' @return
#' returns clustering result invisibly, if clustfun is definied
#'                                    
#' @examples
#' 
#' data(cage)
#' data(promoters)
#' scores1=ScoreMatrix(target=cage,windows=promoters,strand.aware=TRUE,
#'                    weight.col="tpm")
#' 
#' set.seed(1000)
#' \donttest{
#' heatMatrix(mat=scores1,legend.name="tpm",winsorize=c(0,99),xlab="region around TSS",
#'            xcoords=-1000:1000,
#'            cex.legend=0.8,main="CAGE clusters on promoters",cex.lab=1,
#'            cex.axis=0.9,grid=FALSE)
#'            
#' ## examples using clustering functions
#' ## k-means
#' cl1 <- function(x) kmeans(x, centers=3)$cluster
#' set.seed(1000)
#' heatMatrix(mat=scores1,legend.name="tpm",winsorize=c(0,99),xlab="region around TSS",
#'          xcoords=-1000:1000,clustfun=cl1,
#'          cex.legend=0.8,main="CAGE clusters on promoters",cex.lab=1,
#'          cex.axis=0.9,grid=FALSE,
#'          user.order=c(1,3,2))
#' 
#' ## hierarchical clustering
#' cl2 <- function(x) cutree(hclust(dist(x), method="complete"), k=3)
#' set.seed(1000)
#' heatMatrix(mat=scores1,legend.name="tpm",winsorize=c(0,99),xlab="region around TSS",
#'          xcoords=-1000:1000,clustfun=cl2,
#'          cex.legend=0.8,main="CAGE clusters on promoters",cex.lab=1,
#'          cex.axis=0.9,grid=FALSE)
#' }
#' 
#' 
#' @export
#' @rdname heatMatrix
heatMatrix<-function(mat,grid=FALSE,col=NULL,xcoords=NULL,
                     group=NULL,group.col=NULL,
                     order=FALSE,user.order=FALSE,
                     winsorize=c(0,100),
                     clustfun=NULL,
                     main="",legend.name=NULL,cex.legend=1,xlab=NULL,cex.main=1,
                     cex.lab=1,cex.axis=1,newpage=TRUE
){
  
  if( class(mat) !="ScoreMatrix" ){stop("'mat' is not a ScoreMatrix object\n")}
  if(!is.null(clustfun) & !is.function(clustfun)){stop("'clustfun' has to be a function or NULL\n")}
  
  mat2=mat@.Data # get the matrix class, some operations are not transitive
  
  # if this is changed, a rowSide color map will be drawn
  # setting clustering function or group.list will populate this vector
  # and this function will check on its value later on
  # to decide to plot rowside colors or not
  group.vector=NULL
  group.names=NULL # will be plotted as annotation if filled in later
  
  
  # this can set extreme values to given percentile
  # better for visualization
  # alternative is to take log or sth
  # but that should be done prior to heatMatrix
  if(winsorize[2]<100 | winsorize[1]>0){
    mat2=.winsorize(mat2, winsorize)
  }
  
  # do clustfun if requested
  if(!is.null(clustfun)){
    
    # impute values if there are NAs
    if(any(is.na(mat2)) ) {
      mat3=impute.knn(mat2, k = 10, 
                      rowmax = 0.5, colmax = 0.8, 
                      maxp = 1500)$data
      clu=clustfun(mat3)
    }
    else{
      clu=clustfun(mat2)
    }
    
    group.vector=clu
    # order things by clusters only
    mat2=mat2[order(group.vector),] 
    group.vector=group.vector[order(group.vector)]
    
    # if user wants to order
    if(order){ 
       
       # ordering within clusters
       my.order=order(group.vector,-rowSums(mat2,na.rm=TRUE))
       
       # commence the new order: Novus Ordo Seclorum
       mat2=mat2[my.order,]
       group.vector=group.vector[my.order]
     }
    
    group.names=unique(group.vector) # to be used for the rowSide colors
    
  }
  
  # check conditions of group.list
  # group must not have duplicated numbers
  # warn if total number group elements is below nrow(mat)
  if(!is.null(group) & is.null(clustfun)){ #if grouping then without clustering
    
    # if group is a list of rowids, row numbers for original windows argument
    if(is.list(group)){
      # group must not have duplicated numbers
      win.numbs=(lapply(group, function(x) unique(x) ))
      win.vec=unlist(win.numbs)
      if(any(table(unlist(win.numbs))>1)){
        stop("'group' containing a list must not have duplicated numbers\n")
      }
      row.ids=rownames(mat2) # get row ids from the matrix
      group.vector=rep(0,length(row.ids)) # make a starting vector of zeros
      for(i in 1:length(win.numbs)){
        group.vector[row.ids %in% win.numbs[[i]] ]=i # make group vector
      }
      
      # if list has names, take it as group.names
      # group.names will be plotted with rowSide colors
      if(!is.null(names(group))){
        group.names=names(group)
      }
      
      # stop if total number group elements is below nrow(mat)
      if(all(group.vector==0)){
        stop("None of the elements in 'group' are a part of rownames(mat) \n")
      }
      
      # warn if total number group elements is below nrow(mat)
      if(any(group.vector==0)){
        warning("Number of elements in 'group' argument is less then nrow(mat) \n",
                "Dropping rows from 'mat' that are not contained in 'group'\n")
        mat2=mat2[group.vector>0,]
        group.vector=group.vector[group.vector>0]
      }       
      
    }
    else if(is.factor(group)){
      
      # check if the length is same as nrow(mat)
      if( length(group) != nrow(mat)){
        stop("'group' is a factor, and its length should be equal to nrow(mat)\n")
      }
      # arrange factor levels by the order of appeareance
      group=factor(as.character(group),levels=as.character(unique(group)))
      group.names=levels(group)
      levels(group)=1:length(levels(group))
      group.vector=as.numeric(group)
    }else{
      stop("'group' must be a factor or a list\n")
    }
    
    
    
    # order things by clusters only
    mat2=mat2[order(group.vector),]
    group.vector=group.vector[order(group.vector)]

    if(order){
      my.order=order(group.vector,-rowSums(mat2,na.rm=TRUE))
      
      # commence the new order: Novus Ordo Seclorum
      mat2=mat2[my.order,]
      group.vector=group.vector[my.order]
      names(group.vector)=rownames(mat2)
    }
    
 }else if(order & is.null(clustfun) ){ # if only ordering is needed, no grouping or clustering
    #czy napewno clustfun FALSE? chyba tak?
    order.vector       =rep(1,nrow(mat2))
    names(order.vector)=rownames(mat2)
    order.vector       = order.vector[order(-rowSums(mat2,na.rm=TRUE))]
    mat2               =mat2[order(-rowSums(mat2,na.rm=TRUE)),]
  }
  
 
 
  #if user.order is provided
  if(!identical(user.order, FALSE) & !is.null(group.names)){
    if(length(user.order)!=length(group.names)){
      warning(paste0("length of 'user.order' vector (", length(user.order) ,") should be the the same as number of clusters (",length(group.names),"). Skipping it..\n"))
    }else{
      gv.fac.ord <- sort(factor(group.vector, levels = user.order))
      group.vector <- group.vector[names(gv.fac.ord)] #convert factor to vector
      #group.names <- user.order
    }
  }else if(!identical(user.order, FALSE) & is.null(group.names)){
    warning("There are no groups or clusters to order. Skipping it..")
  }
  
  
  # THE PLOTTING STARTS HERE with ordered mat2
  if(!grid){
    plot.new()
    vps <- baseViewports()
    pushViewport(vps$figure) # get the plot viewport from base graphics
    
  }else{
    if(newpage)
      grid.newpage()
  }
  
  # get the default/given xcoordinates to plot
  if(!is.null(xcoords) & is.vector(xcoords)){
    
    ## if it is a numeric vector
    if(all(is.numeric(xcoords))) { 
      # if it is a two element vector of start and end coordinates 
      # of the first and the last column of the matrix
      if(length(xcoords)==2 & xcoords[1]<xcoords[2]){
        xcoords=seq(xcoords[1],xcoords[2],length.out=ncol(mat2) )
      }
    }
    
    if( all(is.numeric(xcoords)) & ( length(xcoords) != ncol(mat2) ) )
      stop("xcoords has wrong length: ",length(xcoords)," \n",
           " it should be equal to the number of columns of ScoreMatrix\n",
           " which is",ncol(mat2),"\n")    
  }else{
    xcoords=1:ncol(mat2)
  }
  
  # get heatcolor scale
  if(is.null(col)){
    
    col=.jets(100)
  }
  
  
  
  
  # make legend viewport
  legendVp <- viewport(width=unit(0.7, "lines"), height=unit(0.4, "npc"),
                       x = unit(0.71, "npc"), y = unit(0.5, "npc"),just="left")
  pushViewport(legendVp)
  #grid.rect()
  rng=range(mat2,na.rm=TRUE)
  # make the legend 
  .heatLegendY(min=rng[1],max=rng[2],
               col,legend.name,main=FALSE,cex.legend,
               cex.lab)
  popViewport()
  
  
  
  # make heatmap viewport
  heatHeightNPC=0.7
  heatVp <- viewport(width=unit(0.5, "npc"), height=unit(heatHeightNPC, "npc"),
                     x = unit(0.2, "npc"), y = unit(0.5, "npc"),just="left")
  pushViewport(heatVp) # push the heatmap VP
  .gridHeat(mat2,col,xcoords,xlab,cex.lab,cex.axis,hjust=0.5) # make the heat  
  #upViewport(1) # up one level current.vpTree()
  popViewport()
  
  
  # make side colors
  if(!is.null(group.vector)){
    sideVp <- viewport(width=unit(0.05, "npc"), height=unit(heatHeightNPC, "npc"),
                       x = unit(0.145, "npc"), y = unit(0.5, "npc"),just="left")
    pushViewport(sideVp) # push the viewport
    
    grid.rect()
    
    # make the rowside colors and labels
    .rowSideCol(group.vector,group.names=group.names,
                group.col=group.col,cex.lab=cex.lab)
    
    popViewport()
  }
  
  # make the title
  #title.y=convertY(unit(0.85, "npc"), "lines")
  title.y=unit(0.9, "npc")
  grid.text(main, y=title.y, 
            x = unit(0.45, "npc"),
            gp=gpar(cex=cex.main))
  
  #return groups if clustfun is given
  if(!grid)popViewport()
  
  if(identical(clustfun, FALSE) | !is.null(group.vector)){
    return(invisible(group.vector)) 
  }else if(order & is.null(group.vector) ){
    return(invisible(order.vector)) 
  }
  
}


# ---------------------------------------------------------------------------- #
#' Draw multiple heatmaps from a ScoreMatrixList object
#' 
#' The function plots multiple heatmaps for a ScoreMatrixList object side by side.
#' Each matrix can have different color schemes but it is essential that each matrix
#' is obtained from same regions or neighbouring regions. 
#' 
#' @param sml a \code{ScoreMatrixList} object
#' @param grid  if TRUE, grid graphics will be used. if FALSE, base graphics
#'               will be used on the top level, so users can use par(mfrow)
#'               or par(mfcol) prior to calling the function. Default:FALSE   
#' @param col    a color palette or list of color palettes, such as
#'               list(heat.colors(10),topo.colors(10)). If it is a list,
#'               it is length must match the number of matrices to be plotted.
#'               If it is a single palette
#'               every heatmap will have the same colors.
#' @param xcoords a vector of numbers showing relative positions of the bases or 
#'                windows or a list of vectors. 
#'                The elements of the list must match the number of columns in the
#'                corresponding \code{ScoreMatrix}. 
#'                Alternatively, the elements could be a numeric vector of two elements. Such
#'                as c(0,100) showing the relative start and end coordinates of the first
#'                and last column of the \code{ScoreMatrix} object. The remaining
#'                coordinates will be automatically matched in this case. If the
#'                argument is not a list but a single vector, then all heatmaps
#'                will have the same coordinate on their x-axis.
#'                
#' @param group a list of vectors of row numbers or a factor. The rows will be 
#'              reordered to match their grouping. The grouping is
#'              used for rowside colors of the heatmap. If it is a list,
#'              each element of the list must be a vector of row numbers. Names
#'              of the elements of the list will be used as names of groups. 
#'              If \code{group} is a factor
#'              , it's length must match the number of rows of the matrix, and 
#'              factor levels will be used as the names of the groups in the plot.
#
#'               
#' @param group.col a vector of color names to be used at the rowside colors if
#'                  \code{group} and \code{clustfun} arguments are given
#' @param order    Logical indicating if the rows should be ordered or not 
#'                 (Default:FALSE). If \code{order=TRUE} the matrix will be ordered
#'                 with rowSums(mat) values in descending order. 
#'                 If \code{group} argument is provided, first the groups
#'                 will be ordered in descending order of sums of rows then, everything
#'                 within the clusters will be ordered by sums of rows.
#'                 If \code{clustfun} is given then rows within clusters
#'                 will be order in descending order by sums of rows.
#'                 
#' @param user.order a numerical vector indicating the order of groups/clusters (it works only
#'                   when \code{group} or \code{clustfun} argument is given). 
#'
#' @param winsorize Numeric vector of two, defaults to c(0,100). This vector 
#'                  determines the upper and lower percentile values to limit the 
#'                  extreme values. For example, c(0,99) will limit the values to
#'                  only 99th percentile for a matrix, 
#'                  everything above the 99 percentile will 
#'                  be equalized to the value of 99th percentile.This is useful 
#'                  for visualization of matrices that have outliers.
#' @param clustfun  a function for clustering
#'                  rows of \code{mat} that returns 
#'                  a vector of integers indicating the cluster to which 
#'                  each point is allocated (a vector of cluster membership),
#'                  e.g. k-means algorithm with 3 centers: 
#'                  function(x) kmeans(x, centers=3)$cluster. 
#'                  By default FALSE.
#' @param clust.matrix a numerical vector of indexes or a character vector of names
#'                     of the \code{ScoreMatrix} objects in 'sml' 
#'                     to be used in clustering (if \code{clustfun} argument is provided).
#'                     By default all matrices are clustered. Matrices that are
#'                     not indicated in clust.matrix are ordered according to
#'                     result of clustering algorithm.
#' @param column.scale Logical indicating if matrices should be scaled or not,
#'                     prior to clustering or ordering. Setting this
#'                     to TRUE scales the columns of the 
#'                     matrices using,
#'                     \code{scale()} function. scaled columns are only used for
#'                     clustering or ordering. Original scores are displayed for heatmaps.
#' @param matrix.main a vector of strings for the titles of the heatmaps. If NULL
#'                    titles will be obtained from names of the \code{ScoreMatrix}
#'                    objects in the \code{ScoreMatrixList} objects.
#' @param common.scale if TRUE (Default:FALSE) all the heatmap colors will be 
#'                    coming from the same score
#'                    scale, although each heatmap color scale can be different.
#'                    The color intensities will be coming from the same scale. 
#'                    The scale will be
#'                    determined by minimum of all matrices and maximum of all 
#'                    matrices. This is useful when all matrices are on the same
#'                    score scale. If FALSE, the color scale will be determined
#'                    by minimum and maximum of each matrix individually.
#' @param legend      if TRUE and color legend for the heatmap is drawn.                   
#' @param legend.name a vector of legend labels to be plotted with legends
#'                    of each heatmap. If it is a length 1 vector, all heatmaps
#'                    will have the same legend label.
#' @param cex.legend  A numerical value giving the amount by which 
#'                    legend axis marks should be magnified relative to the default
#' 
#' @param xlab  a vector of character strings for x-axis labels of the heatmaps. 
#'              if it is length 1, all heatmaps will have the same label.
#' @param cex.lab  A numerical value giving the amount by which 
#'                    axis labels (including 'legend.name') 
#'                    should be magnified relative to the default.
#' @param cex.main  A numerical value giving the amount by which 
#'                    plot title should be magnified  
#' @param cex.axis  A numerical value giving the amount by which 
#'                    axis marks should be magnified relative to the default
#' @param newpage logical indicating if \code{grid.newpage()} function should be
#'                invoked if \code{grid=TRUE}.
#' 
#' @return
#'  invisibly returns the order of rows, if clustfun is provided and/or order=TRUE
#'  
#' @examples
#' data(cage)
#' data(promoters)
#' scores1=ScoreMatrix(target=cage,windows=promoters,strand.aware=TRUE)
#' 
#' data(cpgi)
#' scores2=ScoreMatrix(target=cpgi,windows=promoters,strand.aware=TRUE)
#' 
#' sml=new("ScoreMatrixList",list(a=scores1,b=scores2))
#'
#' # use with k-means
#' \donttest{multiHeatMatrix(sml,
#'                  clustfun=function(x) kmeans(x, centers=2)$cluster,
#'                  cex.axis=0.8,xcoords=c(-1000,1000),
#'                  winsorize=c(0,99),
#'                  legend.name=c("tpm","coverage"),xlab="region around TSS")
#'                  
#' # use with hierarchical clustering
#' cl2 <- function(x) cutree(hclust(dist(x), method="complete"), k=2)
#' multiHeatMatrix(sml,legend.name="tpm",winsorize=c(0,99),xlab="region around TSS",
#'          xcoords=-1000:1000,clustfun=cl2,
#'          cex.legend=0.8,cex.lab=1,
#'          cex.axis=0.9,grid=FALSE)
#' 
#' # use different colors
#' require(RColorBrewer)
#' col.cage= brewer.pal(9,"Blues")
#' col.cpgi= brewer.pal(9,"YlGn")
#' multiHeatMatrix(sml,
#'                  clustfun=function(x) kmeans(x, centers=2)$cluster,
#'                  cex.axis=0.8,xcoords=c(-1000,1000),
#'                  winsorize=c(0,99),col=list(col.cage,col.cpgi),
#'                  legend.name=c("tpm","coverage"),xlab="region around TSS")
#' }
#' 
#' 
#' 
#' @export
multiHeatMatrix<-function(sml,grid=TRUE,col=NULL,xcoords=NULL,
                          group=NULL,group.col=NULL,
                          order=FALSE,user.order=FALSE,
                          winsorize=c(0,100),
                          clustfun=FALSE,
                          clust.matrix=NULL,
                          column.scale=TRUE,
                          matrix.main=NULL,
                          common.scale=FALSE,legend=TRUE,
                          legend.name=NULL,cex.legend=0.8,
                          xlab=NULL,
                          cex.lab=1, cex.main=1,cex.axis=0.8,newpage=TRUE)
{
  
  if( class(sml) !="ScoreMatrixList" ){stop("'sml' is not a ScoreMatrix object\n")}
  
  # check if col and xcoords are lists
  # if so their number should be equal to the number of matrices
  if(is.list(col) & length(col) != length(sml)){
    stop("'col' is a list and its length does not match the length of ScoreMatrixList\n")
    col=NULL
  }
  
  if( is.list(xcoords) & length(xcoords) != length(sml) ){
    stop("'xcoords' is a list and its length does not match the length of ScoreMatrixList\n")
    xcoords=NULL
  }
  
  # check legend.name argument if , it is a vector
  if(!is.null(legend.name) & length(legend.name)>1 & length(legend.name) != length(sml)){
    stop("'legend.name' should match the length of the 'sml' ",
         "if it is not a length 1 vector\n")
  }else if (length(legend.name)==1){
    legend.name=rep(legend.name,length(sml))
  }
  
  # check xlab argument and set it
  if(!is.null(xlab) & length(xlab)>1 & length(xlab) != length(sml)){
    stop("'xlab' should match the length of the 'sml' ",
         "if it is not a length 1 vector\n")
  }else if (length(xlab)==1){
    xlab=rep(xlab,length(sml))
  } 
  
  #check number of rows if they match or not
  # if not, warn and run unionScoreMatrixList
  if( length(unique(sapply(sml,nrow)))>1  ){
    warning("\nThe row numbers are different\n",
            "attempting to get common rows and to reorder rows\n",
            "using 'intersectScoreMatrixList()'\n")
    sml=intersectScoreMatrixList(sml,reorder=TRUE)
  }
  
  # check clust.matrix arg
  if(!is.null(clust.matrix)){
    clust.matrix = unique(clust.matrix)
    if(is.numeric(clust.matrix)){
      if(length(clust.matrix)>length(sml) | max(clust.matrix) > length(sml) | 0 %in% clust.matrix){
	warning("\n'clust.matrix' vector shouldn't be longer than 'sml'\n",
             "and shouldn't have greater values that length of 'sml'\n",
              "clustering all matrices\n")
	clust.matrix = NULL 
      }
    }else{
      if(length(clust.matrix)>length(sml) | sum(clust.matrix %in% names(sml)) == length(clust.matrix)){
	warning("\n'clust.matrix' vector shouldn't be longer than 'sml'\n", 
              "and should contain names of matrices of 'sml'\n",
              "clustering all matrices\n")
	clust.matrix = NULL
      }
    }
  }
    
  mat.list=lapply(sml,function(x) x@.Data) # get the matrix class, some operations are not transitive
  
  
  # if this is changed, a rowSide color map will be drawn
  # setting clustfun or group.list will populate this vector
  # and this function will check on its value later on
  # to decide to plot rowside colors or not
  group.vector=NULL
  group.names=NULL # will be plotted as annotation if filled in later
  
  
  # this can set extreme values to given percentile
  # better for visualization
  # alternative is to take log or sth
  # but that should be done prior to heatMatrix
  if(winsorize[2]<100 | winsorize[1]>0){
    mat.list=lapply(mat.list,function(x) .winsorize(x@.Data, winsorize) )
  }
  
  
  # if order is true | clustfun is provided
  # make a one large matrix by cbind
  if(!identical(clustfun, FALSE) | order){
    if(!is.null(clust.matrix)){
       mat2=do.call("cbind",mat.list[clust.matrix])
    }else{
       mat2=do.call("cbind",mat.list)
    }
    if(column.scale){
      mat2=scale(mat2)
      mat2[is.nan(mat2)]=0
    }
  }
  
  # do clustfun if requested
  if(!identical(clustfun, FALSE)){
    
    # impute values if there are NAs
    if(any(is.na(mat2)) ) {
      mat3=impute.knn(mat2 ,k = 10, 
                      rowmax = 0.5, colmax = 0.8, 
                      maxp = 1500)$data
      clu=clustfun(mat3)
    }
    else{
      # cluster 
      clu=clustfun(mat2)
    }
    
    # get group.vector centers, will be used at ordering later
    group.vector=clu
    
    # order things by clusters only
    mat.list=lapply(mat.list,function(x) x[order(group.vector),])
    group.vector=group.vector[order(group.vector)]
    
    # if user wants to order
    if(order){
      
      # replicate center value for each cluster
      g.factor=factor(group.vector,levels=unique(group.vector))
      
      # do ordering within clusters
      my.order=order(group.vector,-rowSums(mat2,na.rm=TRUE))
      
      # commence the new order: Novus Ordo Seclorum
      mat.list=lapply(mat.list,function(x) x[my.order,])
      group.vector=group.vector[my.order]
    }
    
    group.names=unique(group.vector) # to be used for the rowSide colors
  }
  
  
  # check conditions of group.list
  # group must not have duplicated numbers
  # warn if total number group elements is below nrow(mat)
  if(!is.null(group)  & identical(clustfun, FALSE)){
    
    # if group is a list of rowids, row numbers for original windows argument
    if(is.list(group)){
      # group must not have duplicated numbers
      win.numbs=(lapply(group, function(x) unique(x) ))
      win.vec=unlist(win.numbs)
      if(any(table(unlist(win.numbs))>1)){
        stop("'group' containing a list must not have duplicated numbers\n")
      }
      row.ids=rownames(mat.list[[1]]) # get row ids from the matrix
      group.vector=rep(0,length(row.ids)) # make a starting vector of zeros
      for(i in 1:length(win.numbs)){
        group.vector[row.ids %in% win.numbs[[i]] ]=i # make group vector
      }
      
      # if list has names, take it as group.names
      # group.names will be plotted with rowSide colors
      if(!is.null(names(group))){
        group.names=names(group)
      }
      
      # stop if total number group elements is below nrow(mat)
      if(all(group.vector==0)){
        stop("None of the elements in 'group' are a part of rownames(mat) \n")
      }
      
      # warn if total number group elements is below nrow(mat)
      if(any(group.vector==0)){
        warning("Number of elements in 'group' argument is less then nrow(mat) \n",
                "Dropping rows from 'mat' that are not contained in 'group'\n")
        
        mat.list=lapply(mat.list,function(x) x[group.vector>0,])
        group.vector=group.vector[group.vector>0]
      }       
      
      
    }
    else if(is.factor(group)){
      
      # check if the length is same as nrow(mat)
      if(length(group) != nrow(mat.list[[1]])){
        stop("'group' is a factor, and its length should be equal to nrow(mat)\n")
      }
      # arrange factor levels by the order of appeareance
      group=factor(as.character(group),levels=as.character(unique(group)))
      group.names=levels(group)
      levels(group)=1:length(levels(group))
      group.vector=as.numeric(group)
    }else{
      stop("'group' must be a factor or a list\n")
    }
    
    
    
    # order things by clusters only
    mat.list=lapply(mat.list,function(x) x[order(group.vector),])
    group.vector=group.vector[order(group.vector)]
    
    
    if(order){
      
      # get cbound matrix for ordering
      mat2=do.call("cbind",mat.list)
      if(column.scale){
        mat2=scale(mat2)
        mat2[is.nan(mat2)]=0
      }
      my.order=order(group.vector,-rowSums(mat2,na.rm=TRUE))
      
      # commence the new order: Novus Ordo Seclorum
      mat.list=lapply(mat.list,function(x) x[my.order,])
      group.vector=group.vector[my.order]
    }
    
  }else if(order & identical(clustfun, FALSE) ){ # if only ordering is needed no group or clustering
    
    # get cbound matrix for ordering
    mat2=do.call("cbind",mat.list)
    if(column.scale){
      mat2=scale(mat2)
      mat2[is.nan(mat2)]=0
    }
    order.vector       =rep(1,nrow(mat2))
    names(order.vector)=rownames(mat2)
    my.order=order(-rowSums(mat2,na.rm=TRUE))
    mat.list=lapply(mat.list,function(x) x[my.order,])
    order.vector       = order.vector[my.order]
    
  }
  
  #if user.order is provided
  if(!identical(user.order, FALSE) & !is.null(group.names)){
    if(length(user.order)!=length(group.names)){
      warning(paste0("length of 'user.order' vector (", length(user.order) ,") should be the the same as number of clusters (",length(group.names),"). Skipping it..\n"))
    }else{
      gv.fac.ord <- sort(factor(group.vector, levels = user.order))
      group.vector <- group.vector[names(gv.fac.ord)] #convert factor to vector
      group.names <- user.order
    }
  }else if(!identical(user.order, FALSE) & is.null(group.names)){
    warning("There are no groups or clusters to order. Skipping it..")
  }
  
  
  
  # THE PLOTTING STARTS HERE with ordered mat.list
  if(!grid){
    plot.new()
    vps <- baseViewports()
    pushViewport(vps$figure) # get the plot viewport from base graphics
    
  }else{
    if(newpage)
      grid.newpage()
  }
  
  # try to get matrix names from names of ScoreMatrixList
  if(is.null(matrix.main) & !is.null(names(sml)) & class(sml)=="ScoreMatrixList" )
  {
    matrix.main=  names(sml)
  }else if(!is.null(matrix.main) & length(matrix.main) != length(sml)){
    warning("'matrix.main' length does not match to the 'sml' length\n",
            "setting it to NULL")
    matrix.main=NULL
    
  }
  
  # calculate the width of heatmaps and everything else relative to that
  l.sml=length(mat.list) # get length of matrix
  # get the npc width of the heatmap
  # this will be our rudder when plotting
  # every coordinate and dimension will be relative to this value
  hw=40*(1-(convertX( unit(4,"lines"),"npc",valueOnly=TRUE)) )/(l.sml*44+7)
  hh=0.7 # this heatmap height in NPC
  hy=0.6 # heatmap y coordinate on the plot
  
  # if group.vector is not NULL, plot the rowside colors
  # make side colors
  if(!is.null(group.vector)){
    sideVp <- viewport(width=unit(hw/8, "npc"), height=unit(hh, "npc"),
                       x = convertX( unit(4,"lines"),"npc"), 
                       y = unit(hy, "npc"),just="left")
    pushViewport(sideVp) # push the viewport
    
    grid.rect()
    
    # make the rowside colors and labels
    .rowSideCol(group.vector,group.names=group.names,
                group.col=group.col,cex.lab=cex.lab)
    
    popViewport()
  }
  
  # heatmap start coordinates
  heat.startCoord=convertX( unit(4,"lines"),"npc",valueOnly=TRUE) + (hw/8) + (hw/20)
  
  # if the same scale to be used for all plots
  common.range=NULL
  if(common.scale){
    common.range=range(mat.list,na.rm=TRUE)
  }
  
  for(i in 1:length(mat.list)){
    
    
    # cxcoords stands for current xcoords
    if(is.list(xcoords)){
      cxcoords=xcoords[[i]]
    }else if( is.vector(xcoords) | is.null(xcoords) ){
      cxcoords=xcoords   
    }
    else if(!is.null(xcoords)){
      warning("xcoords should be a vector or a list or NULL,nothing else!!\n",
              "setting xcoords to NULL")
      cxcoords=NULL
    }
    
    ## if it is a numeric vector
    if(all(is.numeric(cxcoords))) {    
      # if it is a two element vector of start and end coordinates 
      # of the first and the last column of the matrix
      if(length(cxcoords)==2){
        cxcoords=seq(cxcoords[1],cxcoords[2],length.out=ncol(mat.list[[i]]))
      }
    }
    
    # ccol: stands for current.color
    if(is.list(col)){
      ccol=col[[i]]
    }else if(is.vector(col) | is.null(col) ){
      ccol=col
    }else if(!is.null(col)){
      warning("col should be a vector or a list or NULL,nothing else!!\n",
              "setting colors to default")
      ccol=NULL
    }
    
    
    # get the default/given xcoordinates to plot
    if(!is.null(cxcoords)){
      if( all(is.numeric(cxcoords)) & ( length(cxcoords) != ncol( mat.list[[i]] ) ) ) 
        stop("xcoords has wrong length: ",length(cxcoords)," \n",
             " it should be equal to the number of columns of ScoreMatrix\n",
             " which is",ncol(mat.list[[i]]),"\n")    
    }else{
      cxcoords=1:ncol( mat.list[[i]] )
    }
    
    # get heatcolor scale
    if(is.null(ccol)) 
      ccol=.jets(100)
    
    
    # make heatmap viewport
    heatVp <- viewport(width=unit(hw, "npc"), height=unit(hh, "npc"),
                       x =unit(heat.startCoord,"npc"), 
                       y = unit(hy, "npc"),just="left")
    
    
    pushViewport(heatVp) # push the heatmap VP
    grid.rect()
    .gridHeat(mat.list[[i]],ccol,cxcoords,xlab[i],cex.lab,cex.axis,
              angle=60,hjust=0.6,vjust=-0.5,rng=common.range) # make the heat  
    #upViewport(1) # up one level current.vpTree()
    popViewport()
    
    if(legend){
      # make the legend viewport
      legendVp <- viewport(width=unit(hw*0.7, "npc"), 
                           height=unit(0.5, "lines"),
                           x =unit(heat.startCoord+hw*0.15,"npc"), 
                           y = unit(0.1, "npc"),
                           just="left")
      pushViewport(legendVp)
      grid.rect()
      
      if(common.scale){
        rng=common.range
      }else{  
        rng=range(mat.list[[i]],na.rm=TRUE)
      }
      # make the legend 
      .heatLegendX(min=rng[1],max=rng[2],
                   ccol,legend.name[i],main=TRUE,cex.legend,
                   cex.lab,vjust=-0.5,hjust=0.5)
      popViewport()
    }
    
    if(!is.null(matrix.main)){
      title.y=unit(0.96, "npc")
      grid.text(matrix.main[i], y=title.y, 
                x = unit(heat.startCoord+hw*0.5,"npc"),
                gp=gpar(cex=cex.main),just="bottom")
    }
    
    # update start coord for next one
    heat.startCoord=heat.startCoord+ hw+ (hw/20)
    
  }
  
  if(!grid){
    popViewport()
  }
  
  if(!identical(clustfun, FALSE) | !is.null(group.vector)){
    return(invisible(group.vector)) 
  }else if(order & is.null(group.vector) ){
    return(invisible(order.vector)) 
  }
  
}

# PART THAT DEALS with annotation of genomic features

##############################################################################
# SECTION 1:
# reading annotation to GRanges
# makes GRanges object from a given bed6 or bed12 file to granges object
##############################################################################

#######################################
# SECTION 1: S3 functions
#######################################

# ---------------------------------------------------------------------------- #

#' bed12ToExons function
#' 
#' extracts exons from a bed12 file and puts them into GRanges object
#'
#' @param ref data.frame object
#' @keywords internal
bed12ToExons<-function(ref){

    ref=unique(ref)

    # apply strsplit on columns 11 and 12 to extract exon start positions and exon sizes
    b.start.size = cbind(as.integer(unlist(strsplit(as.character(ref$V12),"," ))),
                       as.integer(unlist(strsplit(as.character(ref$V11),"," ))))
    rep.ref = ref[rep(1:nrow(ref),ref[,10]),] # replicate rows occurs as many as its exon number


    exon.id = unlist( mapply( function(x,y)
                              if(x=="+"){return(1:y)}
                              else{return(y:1)}, 
                              ref[,6],ref[,10]))
    rep.ref$V5=exon.id

    rep.ref$V3 = rep.ref$V2+b.start.size[,1]+b.start.size[,2] # calculate exon start and ends
    rep.ref$V2 = rep.ref$V2+b.start.size[,1]

    strands = as.character(rep.ref$V6)
    strands[strands=="."]="*"
    grange.exons = GenomicRanges::GRanges(seqnames=as.character(rep.ref$V1),
    ranges = IRanges(start=rep.ref$V2+1, end=rep.ref$V3),
    strand = strands, 
    score=rep.ref$V5,
    name=rep.ref$V4)
    return(grange.exons)
} 

# ---------------------------------------------------------------------------- #
#' bed12ToIntrons function
#' 
#' extracts introns from a bed12 file and puts them into GRanges object
#'
#' @param ref data.frame object
#' @keywords internal
bed12ToIntrons<-function(ref){

    #remove the genes with one exon only (they won't have any introns)
    ref=ref[ref[,10]>1,]
    ids=paste(ref[,1],ref[,2],ref[,3],ref[,4],sep="")
    ref=cbind(ref,id=ids)
    ref=unique(ref)

    # apply strsplit on columns 11 and 12 to extract exon start positions and exon sizes
    b.start.size=cbind(as.integer(unlist(strsplit(as.character(ref$V12),"," ))),
                     as.integer(unlist(strsplit(as.character(ref$V11),"," ))))

    # replicate rows occurs as many as its exon number
    rep.ref = ref[rep(1:nrow(ref),ref[,10]),] 

    exon.id = unlist( mapply( function(x,y) 
                                if(x=="+"){return(1:y)}
                                else{return(y:1)} ,
                                ref[,6],ref[,10]  ) )
    rep.ref$V5 = exon.id

    # calculate exon start and ends
    rep.ref$V3 = rep.ref$V2+b.start.size[,1]+b.start.size[,2] 
    rep.ref$V2 = rep.ref$V2+b.start.size[,1]
    rep.ref = rep.ref[,c(1:6,13)]

    # now try to get the exons by cbinding consecutive exons
    temp.ref        = cbind(rep.ref[1:(nrow(rep.ref)-1),],rep.ref[2:nrow(rep.ref),])
    temp.ref        = temp.ref[temp.ref[,7]==temp.ref[,14],]
    temp.ref[,2]    = temp.ref[,3]
    temp.ref[,3]    = temp.ref[,9]
    rep.ref            = temp.ref[,1:6]

    # subtract 1 from - strand exon numbers
    rep.ref[rep.ref[,6]=="-",5]=rep.ref[rep.ref[,6]=="-",5]-1 

    strands=as.character(rep.ref$V6)
    strands[strands=="."]="*"
    grange.exons=GenomicRanges::GRanges(seqnames=as.character(rep.ref$V1),
    ranges=IRanges(start=rep.ref$V2+1, end=rep.ref$V3),
    strand=strands, score=rep.ref$V5,name=rep.ref$V4)
    return(grange.exons)
} 

# ---------------------------------------------------------------------------- #
#' checkBedValidity function
#' 
#' checks the validity of the bed data.frame if it is a legitimate bed columns
#'
#' @param bed.df data.frame object
#' @param type type
#' @keywords internal
checkBedValidity<-function(bed.df,type="none"){

    # does it have at least 3 columns
    num.col=(ncol(bed.df)>=3)

    # does it have 1st and 2nd column as numbers
    col1.2= (is.integer(bed.df[,2]) & is.integer(bed.df[,3]) )

    # does it have chr string in 1st column
    #chr= sum(grepl("chr",bed.df[,1]) )

    if(type=="exons" | type=="introns") {
        #does it have 12>= columns
        ex=(ncol(bed.df)>=12) 

        return(num.col & col1.2 & ex)
    }
    return(num.col & col1.2)
}




#######################################
# SECTION 1: S4 functions
#######################################


# ---------------------------------------------------------------------------- #
#' convert a data frame read-in from a bed file to a GRanges object
#'  
#' @param bed  a data.frame where column order and content 
#'             resembles a bed file with 12 columns
#' @usage convertBedDf(bed)
#' @return \code{\link{GRanges}} object
#'
#' @note one bed track per file is only accepted, the bed 
#'       files with multiple tracks will cause en error
#' @note bed files are expected not to have header lines
#'
#' @export
#' @docType methods
#' @rdname convertBedDf-methods
setGeneric("convertBedDf",function(bed) standardGeneric("convertBedDf"))

#' @aliases convertBedDf,data.frame-method
#' @rdname convertBedDf-methods
setMethod("convertBedDf" ,
          signature(bed = "data.frame" ),
          function(bed){

            if(! checkBedValidity(bed))
                stop("this is not a valid bed file")
                
            if(ncol(bed)>=6){
            
                # check if 6th column is really strand
                if( sum( unique(bed[,6]) %in% c("+","-",".") ) != length(unique(bed[,6])) )
                    stop("Strand column of the bed file or data.frame is wrong")
                        
                #convert to granges
                strands=as.character(bed$V6)
                strands[strands=="."]="*"
                grange=GRanges(seqnames=as.character(bed$V1),
                               ranges=IRanges(start=bed$V2+1, end=bed$V3),
                               strand=strands, 
                               score=bed$V5,
                               name=bed$V4)
            }

            if(ncol(bed)==5){
                grange = GRanges(seqnames=bed$V1,
                         ranges=IRanges(start=bed$V2+1, end=bed$V3),
                         strand=rep("*",nrow(bed)), score=bed$V5,name=bed$V4 )
            }
            if(ncol(bed)==4){
                grange = GRanges(seqnames=bed$V1,
                         ranges=IRanges(start=bed$V2+1, end=bed$V3),
                         strand=rep("*",nrow(bed)),name=bed$V4)
            }    
            if(ncol(bed)==3){
                grange = GRanges(seqnames=bed$V1,
                         ranges=IRanges(start=bed$V2+1, end=bed$V3),
                         strand=rep("*",nrow(bed)))
            }                           
            return(grange)
})

# ---------------------------------------------------------------------------- #
#' convert a data frame read-in from a bed file to a GRanges object for exons
#'  
#' @param bed.df a data.frame where column order and content resembles a bed file with 12 columns
#' @usage convertBed2Exons(bed.df)
#' @return \code{\link{GRanges}} object
#'
#' @note one bed track per file is only accepted, the bed files with multiple tracks will cause en error
#'
#'
#' @examples
#' file = system.file('extdata/chr21.refseq.hg19.bed', package='genomation')
#' bed12 = read.table(file)
#' exons = convertBed2Exons(bed12)
#' head(exons)
#' 
#' @export
#' @docType methods
#' @rdname convertBed2Exons-methods
setGeneric("convertBed2Exons",
                function(bed.df) standardGeneric("convertBed2Exons"))

#' @aliases convertBed2Exons,data.frame-method
#' @rdname convertBed2Exons-methods
setMethod("convertBed2Exons" ,
            signature(bed.df = "data.frame" ),
            function(bed.df){

            if(! checkBedValidity(bed.df,"exon"))
                stop("this is not a valid bed file")
                
            bed12ToExons(bed.df)
})


# ---------------------------------------------------------------------------- #
#' convert a data frame read-in from a bed file to a GRanges object for introns
#'  
#' @param bed.df  a data.frame where column order and content 
#'                resembles a bed file with 12 columns
#' @usage convertBed2Introns(bed.df)
#' @return \code{\link{GRanges}} object
#'
#' @note one bed track per file is only accepted, the bed files with 
#'       multiple tracks will cause en error
#'
#' @examples
#' file = system.file('extdata/chr21.refseq.hg19.bed', package='genomation')
#' bed12 = read.table(file)
#' introns = convertBed2Introns(bed12)
#' head(introns)
#'
#' @export
#' @docType methods
#' @rdname convertBed2Introns-methods
setGeneric("convertBed2Introns",
                  function(bed.df) standardGeneric("convertBed2Introns"))

#' @aliases convertBed2Introns,data.frame-method
#' @rdname convertBed2Introns-methods
setMethod("convertBed2Introns",
            signature(bed.df = "data.frame" ),
            function(bed.df){

            if(! checkBedValidity(bed.df,"exon"))
                stop("this is not a valid bed file")
                
            bed12ToIntrons(bed.df)
})



# ---------------------------------------------------------------------------- #
#' Function to get upstream and downstream adjecent regions to a genomic feature such as CpG islands
#' 
#' @param grange GRanges object for the feature
#' @param flank  number of basepairs for the flanking regions
#' @param clean  If set to TRUE, flanks overlapping with other main features 
#'               will be trimmed, and overlapping flanks will be removed. 
#'               This will remove multiple counts when other features overlap 
#'               with flanks
#'
#' @usage getFlanks(grange,flank=2000,clean=TRUE)
#' 
#' @examples
#' data(cpgi)
#' cpgi.flanks = getFlanks(cpgi)
#' head(cpgi.flanks)
#' 
#' @return GRanges object for flanking regions
#' @export
#' @docType methods
#' @rdname getFlanks-methods
setGeneric("getFlanks", 
                function(grange,flank=2000,clean=TRUE) 
                          standardGeneric("getFlanks"))

#' @aliases getFlanks,GRanges-method
#' @rdname getFlanks-methods
setMethod("getFlanks", signature(grange= "GRanges"),
            function(grange,flank=2000,clean=TRUE){

            shores = c( IRanges::flank(grange,flank),
                        IRanges::flank(grange,flank, FALSE) )
            
            # merge overlapping shores remove CpG coordinates from all shores, das ist so cool!!
            if(clean)
                shores=IRanges::reduce(IRanges::setdiff(shores, grange)) 
            
            shores
})




##############################################################################
# SECTION 2:
# annotate granges objects with annotations that read-in and converted to GRanges objects
##############################################################################

#' @rdname show-methods
#' @aliases show,AnnotationByGeneParts-method
setMethod("show", "AnnotationByGeneParts", 
            function(object) {

                message("Summary of target set annotation with genic parts");
                message("Rows in target set: ", nrow(object@members))
                message("-----------------------")
                
                message("percentage of target features overlapping with annotation:")
                print(round(object@annotation,2))
                message()
                
                message("percentage of target features overlapping with annotation:")
                message("(with promoter > exon > intron precedence):"); 
                print(round(object@precedence,2))
                message()
                
                message("percentage of annotation boundaries with feature overlap:")
                print(round(object@perc.of.OlapFeat,2))
                message()  
                
                message("summary of distances to the nearest TSS:")
                print(summary(abs(object@dist.to.TSS[,2])))
                message()
})


#' @rdname show-methods
#' @aliases show,AnnotationByFeature-method
setMethod("show", "AnnotationByFeature",
          function(object) {
            
            message("summary of target set annotation with feature annotation:")
            message("Rows in target set: ",nrow(object@members))
            message("----------------------------")
            
            message("percentage of target elements overlapping with features:")
            print(round(object@annotation,2))
            message()
            
            message("percentage of feature elements overlapping with target:")
            print(round(object@perc.of.OlapFeat,2))
            message()
})



#######################################
# SECTION 2: S3 FUNCTIONS 
# these shouldn't be exported
#######################################

# ---------------------------------------------------------------------------- #
#' annotatGrWithGeneParts function
#'
#' @param gr target object
#' @param prom promotors 
#' @param exon exons
#' @param intron introns
#' @param strand logical
#' @keywords internal
annotatGrWithGeneParts <- function(gr, prom, exon, intron, strand=FALSE){

    if( ! strand){strand(gr)="*"}
    memb = data.frame(matrix(rep(0,length(gr)*3),ncol=3) )
    colnames(memb)=c("prom","exon","intron")
    memb[countOverlaps(gr,prom) > 0,1] = 1
    memb[countOverlaps(gr,exon) > 0,2] = 1
    memb[countOverlaps(gr,intron) > 0,3] = 1

    # percentage of overlap for each region
    annotation = c( promoter     = 100*sum(memb$prom>0)/nrow(memb),
                    exon        = 100*sum(memb$exon>0)/nrow(memb),
                    intron        = 100*sum(memb$intron>0)/nrow(memb),
                    intergenic    = 100*sum(rowSums(memb)==0)/nrow(memb) )

    
    num.annotation = c( promoter    = sum(memb$prom>0),
                        exon        = sum(memb$exon>0),
                        intron        = sum(memb$intron>0),
                        intergenic    = sum(rowSums(memb)==0) )

    precedence = c( promoter  = 100*sum(memb$prom>0)/nrow(memb),
                    exon      = 100*sum(memb$exon>0 & memb$prom==0)/nrow(memb),
                    intron    = 100*sum(memb$intron>0 & memb$exon==0 & memb$prom==0)/nrow(memb),
                    intergenic= 100*sum(rowSums(memb)==0)/nrow(memb) )

    num.precedence = c( promoter  = sum(memb$prom>0),
                        exon      = sum(memb$exon>0 & memb$prom==0),
                        intron    = sum(memb$intron>0 & memb$exon==0 & memb$prom==0),
                        intergenic= sum(rowSums(memb)==0) )  

    numberOfOlapFeat = c( promoter=sum(countOverlaps(prom,gr)>0),
                          exon=sum(countOverlaps(exon,gr)>0),
                          intron=sum(countOverlaps(intron,gr)>0) )
    percOfOlapFeat =100*numberOfOlapFeat/c(length(prom),length(exon),length(intron))
    
    return(list(members = memb,
                annotation=annotation,
                precedence=precedence,
                num.annotation=num.annotation,
                num.precedence=num.precedence,
                numberOfOlapFeat=numberOfOlapFeat,percOfOlapFeat=percOfOlapFeat))
}

# ---------------------------------------------------------------------------- #
#' distance2NearestFeature function
#'
#' @param g.idh target object
#' @param tss TSSes
#' @keywords internal
distance2NearestFeature<-function(g.idh,tss){

    elementMetadata(g.idh) = data.frame(elementMetadata(g.idh),orig.row=1:length(g.idh))
    # get the row number column
    id.col = ncol( elementMetadata(g.idh))+5 

    # get the id column for tss in the merged data.frame below
    tss.id.col = ncol( elementMetadata(g.idh))+5+7 
    
    # get the id column for tss
    strnd.col=ncol( elementMetadata(g.idh))+5+7-2 

    # a merged data.frame is returned by this function
    met.tss = .nearest.2bed(g.idh, tss) 

    dist.col=ncol(met.tss)
    met.tss[met.tss$end < met.tss$start.y & met.tss$strand.y=="+", dist.col] = 
    -1* met.tss[met.tss$end<met.tss$start.y & met.tss$strand.y=="+",dist.col]

    met.tss[met.tss$end > met.tss$start.y & met.tss$strand.y=="-",dist.col] = 
    -1* met.tss[met.tss$end>met.tss$start.y & met.tss$strand.y=="-",dist.col]

    res = met.tss[order(met.tss[,id.col]),c(id.col,dist.col,tss.id.col,strnd.col)]
    names(res)=c("target.row", "dist.to.feature",  "feature.name", "feature.strand")

    return(res)
}


# ---------------------------------------------------------------------------- #
# get the nearest features between a subject and query bed file
# get grange objects in bed zormat
# g.bed is GenomicRanges object, biatch!!!
# subject is also GenomicRanges object
# I think this function can be easily simplified
.nearest.2bed<-function(g.bed,subject){
    
    chrs1 = levels(seqnames(g.bed))
    chrs2 = levels(seqnames(subject))
    chrs  = intersect(chrs1, chrs2)
    res.df=NULL
    
    for(i in 1:length(chrs)){
        
        # find the nearest for this range
        ind = nearest(ranges(g.bed[seqnames(g.bed)==chrs[i],]),
                  ranges(subject[seqnames(subject)==chrs[i],]))
        sbdf1 = IRanges::as.data.frame(g.bed[seqnames(g.bed)==chrs[i],] )
        sbdf2 = IRanges::as.data.frame(subject[seqnames(subject)==chrs[i],] )
        sbdf2 = sbdf2[ind,]
        names(sbdf2)=paste(names(sbdf2),".y",sep="")
        temp  = cbind(sbdf1,sbdf2)
        res.df = rbind(res.df,temp)
    }

    res.dist = rep(0,nrow(res.df))
    dist1 = abs(res.df$start-res.df$end.y)+1
    dist2 = abs(res.df$start.y-res.df$end)+1
    
    #total length
    totti = res.df$width + res.df$width.y 
    res.dist[pmax(dist1,dist2)>totti]=pmin(dist1,dist2)[pmax(dist1, dist2) > totti] 
    res.df = cbind(res.df,dist=res.dist)
    return(res.df)
}





#######################################
# SECTION 2: S4 FUNCTIONS 
#######################################


# ---------------------------------------------------------------------------- #
#' Annotate given  object with promoter, exon, intron and intergenic regions
#' 
#' The function annotates \code{GRangesList} or \code{GRanges} object as 
#' overlapping with promoter,exon,intron or intergenic regions.
#'
#' @param target  \code{\link{GRanges}} or \code{\link{GRangesList}}
#'                object storing chromosome locations
#'                to be annotated (e.g. chipseq peaks)
#' @param feature GRangesList object containing GRanges object for
#'                promoter, exons, introns and transcription start sites, 
#'                or simply output of readTranscriptFeatures function
#' @param strand If set to TRUE, annotation features and target features will be 
#'        overlapped based on strand  (def:FALSE)
#' @param intersect.chr boolean, whether to select only chromosomes that are 
#'        common to feature and target. FALSE by default
#' 
#' @return \code{AnnotationByGeneParts} object or a list of 
#'         \code{AnnotationByGeneParts}
#'         objects if target is a  \code{\link{GRangesList}} object.
#' 
#' @examples
#' data(cage)
#' bed.file = system.file("extdata/chr21.refseq.hg19.bed", package = "genomation")
#' gene.parts = readTranscriptFeatures(bed.file)
#' cage.annot = annotateWithGeneParts(cage, gene.parts, intersect.chr=TRUE)
#' cage.annot
#' @export
#' @docType methods
#' @rdname annotateWithGeneParts-methods
setGeneric("annotateWithGeneParts", 
                function(target,feature,strand=FALSE, intersect.chr=FALSE)
                    standardGeneric("annotateWithGeneParts") )

#' @aliases annotateWithGeneParts,GRanges,GRangesList-method
#' @rdname annotateWithGeneParts-methods
setMethod("annotateWithGeneParts", 
          signature(target = "GRanges", feature = "GRangesList"),
          function(target, feature, strand, intersect.chr){

          if(intersect.chr){
            message('intersecting chromosomes...')
            chrs = intersect(unique(as.character(seqnames(target))), 
                         unique(unlist(
                           lapply(feature, 
                                  function(x)as.character(seqnames(x))))))
        
            if(length(chrs) == 0)
              stop('target and feature do not have intersecting chromosomes')
            target=target[seqnames(target) %in% chrs]
            feature = lapply(feature, function(x)x[seqnames(x) %in% chrs])
          }  
        
            a.list = annotatGrWithGeneParts(target, 
                                            feature$promoters,
                                            feature$exons,
                                            feature$introns,
                                            strand=strand)
            dist2TSS = distance2NearestFeature(target,feature$TSSes)

            new("AnnotationByGeneParts",
                members         = as.matrix(a.list$members),
                annotation      = a.list$annotation,
                precedence        = a.list$precedence,
                num.annotation  = a.list$num.annotation,
                num.precedence    = a.list$num.precedence,
                no.of.OlapFeat  = a.list$numberOfOlapFeat,
                perc.of.OlapFeat= a.list$percOfOlapFeat,
                dist.to.TSS     = dist2TSS )
})



#' @aliases annotateWithGeneParts,GRangesList,GRangesList-method
#' @rdname annotateWithGeneParts-methods
setMethod("annotateWithGeneParts",
          signature(target = "GRangesList", feature= "GRangesList"),
          function(target, feature, strand, intersect.chr){
          
        
      l = list()
      if(is.null(names(target)))
          names(target) = 1:length(target)
      for(i in 1:length(target)){
      
        name = names(target)[i]
        message(paste('Working on:',name))
        x = target[[name]]
        l[[name]] = annotateWithGeneParts(x, feature, strand, intersect.chr)
        
      }                          
            return(l)
})




# ---------------------------------------------------------------------------- #
#' Function to annotate a given GRanges object with promoter,exon,intron & intergenic values
#'  
#' @param target    a granges object storing chromosome locations to be annotated
#' @param feature   a granges object storing chromosome locations of a feature 
#'                  (can be CpG islands, ChIP-seq peaks, etc)
#' @param flank     a granges object storing chromosome locations of 
#'                  the flanks of the feature
#' @param feature.name     string for the name of the feature
#' @param flank.name     string for the name of the flanks
#' @param strand   If set to TRUE, annotation features and target features 
#'                 will be overlapped based on strand  (def:FAULT)
#' @param intersect.chr boolean, whether to select only chromosomes that are 
#'                      common to feature and target. FALSE by default
#'                      
#'                      
#' @return returns an \code{AnnotationByFeature} object
#' 
#' 
#' @examples
#' data(cpgi)
#' data(cage)
#' cpgi.flanks = getFlanks(cpgi)
#' flank.annot = annotateWithFeatureFlank(cage, cpgi, cpgi.flanks)
#' 
#' @export
#' @docType methods
#' @rdname annotateWithFeatureFlank-methods
setGeneric("annotateWithFeatureFlank", 
                          function(target,
                                   feature,
                                   flank,
                                   feature.name=NULL,
                                   flank.name="flank",
                                   strand=FALSE, 
                                   intersect.chr=FALSE) 
                            standardGeneric("annotateWithFeatureFlank") )

#' @aliases annotateWithFeatureFlank,GRanges,GRanges,GRanges-method
#' @rdname annotateWithFeatureFlank-methods
setMethod( "annotateWithFeatureFlank", 
            signature(target = "GRanges",feature="GRanges",flank="GRanges"),
            function(target, feature, flank, feature.name, flank.name, strand, 
               intersect.chr){

              if(is.null(feature.name))
                feature.name= deparse(substitute(feature))
        
              if(intersect.chr){
                message('intersecting chromosomes...')
                chrs = intersect(unique(as.character(seqnames(target))), 
                                 unique(as.character(seqnames(feature))))
                if(length(chrs) == 0)
                  stop('target and feature do not have intersecting chromosomes')
                target  = target[seqnames(target) %in% chrs]
                feature = feature[seqnames(feature) %in% chrs]
              }
        
                if( ! strand )
                    strand(target) = "*"
                    memb=data.frame(matrix(rep(0,length(target)*2),ncol=2) )
                    colnames(memb) = c(feature.name,flank.name)
                    memb[countOverlaps(target,feature)>0,1]=1
                    memb[countOverlaps(target,flank)>0,2]=1

                    annotation=c(100*sum(memb[,1]>0)/nrow(memb),
                                 100*sum(memb[,2]>0)/nrow(memb),
                                 100*sum(rowSums(memb)==0)/nrow(memb) )
                    names(annotation)=c(feature.name, flank.name, "other")

                    num.annotation=c( sum(memb[,1]>0), 
                                      sum(memb[,2]>0), 
                                      sum(rowSums(memb)==0) )
                    names(num.annotation)=c(feature.name,flank.name,"other")                      

                    precedence=c(100*sum(memb[,1]>0)/nrow(memb) ,
                                 100*sum(memb[,2]>0 & memb[,1]==0)/nrow(memb) ,
                                 100*sum(rowSums(memb)==0)/nrow(memb) )
                    names(precedence)=c(feature.name,flank.name,"other")

                    num.precedence=c( sum(memb[,1]>0), 
                                      sum(memb[,2]>0 & memb[,1]==0), 
                                      sum(rowSums(memb)==0)  )
                    names(num.precedence)=c(feature.name,flank.name,"other")                      

                    numberOfOlapFeat = c(sum(countOverlaps(feature,target)>0),
                    sum(countOverlaps(flank,target)>0) )
                    names(numberOfOlapFeat) = c(feature.name,flank.name)
                    percOfOlapFeat = 100*numberOfOlapFeat/c(length(feature),length(flank) )

                    new("AnnotationByFeature",
                        members         = as.matrix(memb),
                        annotation      = annotation,
                        precedence        = precedence,
                        num.annotation  = num.annotation,
                        num.precedence    = num.precedence,
                        no.of.OlapFeat  = numberOfOlapFeat,
                        perc.of.OlapFeat= percOfOlapFeat)

})


# ---------------------------------------------------------------------------- #
#' Function to annotate given GRanges object with a given genomic feature
#' 
#' @param target   a GRanges object storing chromosome locations to be annotated
#' @param feature  a GRanges object storing chromosome locations of a feature 
#'                 (can be CpG islands, ChIP-seq peaks, etc)
#' @param strand   If set to TRUE, annotation features and target features will 
#'                 be overlapped based on strand  (def:FAULT)
#' @param extend   specifiying a positive value will extend the feature on both 
#'                 sides as much as \code{extend}
#' @param feature.name name of the annotation feature. 
#'                     For example: H3K4me1,CpGisland etc.
#'                     by default the name is taken from the given variable
#' @param intersect.chr boolean, whether to select only chromosomes that are 
#'                      common to feature and target. FALSE by default
#'                      
#' @return returns an \code{AnnotationByFeature} object
#' 
#' 
#' @examples
#' data(cpgi)
#' data(promoters)
#' annot = annotateWithFeature(cpgi, promoters)
#' 
#' 
#' @export
#' @docType methods
#' @rdname annotateWithFeature-methods
setGeneric("annotateWithFeature", 
                    function(target,
                             feature,
                             strand=FALSE,
                             extend=0,
                             feature.name=NULL, 
                             intersect.chr=FALSE) 
                      standardGeneric("annotateWithFeature") )

#' @aliases annotateWithFeature,GRanges,GRanges-method
#' @rdname annotateWithFeature-methods
setMethod("annotateWithFeature", 
           signature(target = "GRanges",feature="GRanges"),
           function(target, feature, strand,extend,feature.name,intersect.chr){

                if(is.null(feature.name))
          feature.name= deparse(substitute(feature))

                if(extend>0){
                  message('extending features...')
          feature = resize(feature, width=(width(feature)+2*extend),
                           fix='center')
                }
        
        # selects common chromosomes for target and feature
        if(intersect.chr){
          message('intersecting chromosomes...')
          chrs = intersect(unique(as.character(seqnames(target))), 
                           unique(as.character(seqnames(feature))))
          if(length(chrs) == 0)
            stop('target and feature do not have intersecting chromosomes')
          target=target[seqnames(target) %in% chrs]
          feature = feature[seqnames(feature) %in% chrs]
        }
        
                if( ! strand)
          strand(target)="*"
                memb=rep(0,length(target))
        
                memb[countOverlaps(target,feature) > 0] = 1

                annotation = c( 100*sum(memb >  0)/length(memb) ,
                                        100*sum(memb == 0)/length(memb) )
                
                num.annotation = c( sum( memb >  0),
                                              sum( memb == 0) )
                names(annotation) = c(feature.name, "other")

                numberOfOlapFeat = c(sum(countOverlaps(feature,target)>0))
                percOfOlapFeat = 100*numberOfOlapFeat/c(length(feature))

                new("AnnotationByFeature",
                    members         = as.matrix(memb),
                    annotation      = annotation,
                    precedence          = annotation,
                    num.annotation  = num.annotation,
                    num.precedence    = num.annotation,
                    no.of.OlapFeat  = numberOfOlapFeat,
                    perc.of.OlapFeat= percOfOlapFeat)
})



#' Annotate given ranges with genomic features
#' 
#' The function annotates a target GRangesList or GRanges object as overlapping
#' or not with 
#' the elements of a named GRangesList. This is useful to annotate your regions
#' of interest with genomic features with arbitrary categories such as repeat
#' classes or families, or output from genome segmentation alogorithms such
#' as chromHMM.
#' 
#'
#' @param target GRanges or GRangesList object storing chromosome locations to be 
#'        annotated (e.g. chipseq peaks)
#' @param features a named GRangesList object containing GRanges objects different set
#'        of features. The function calculates percent overlaps with and without
#'        precendence at the same time. The order of objects in GRangesList
#'        defines their precedence. If a range in \code{target} overlaps with
#'        a more precedent range in an element of \code{features}, the other 
#'        overlaps from other less precedent elments will be discarded. This is
#'        useful for getting piecharts where percentages should add up to 100.
#'        
#' @param strand.aware	if set to TRUE, annotation features and target features will 
#' be overlapped based on strand (def:FALSE)
#' @param intersect.chr	logical value, whether to select only chromosomes 
#'        that are common to feature and target. FALSE by default
#' @return returns an \code{AnnotationByFeature} object or if target is 
#'         a GRangesList, a list of \code{AnnotationByFeature} objects.
#'      
#' @export
#' @examples
#' library(GenomicRanges)
#' data(cage)
#' data(cpgi)
#' cage$tpm=NULL
#' gl = GRangesList(cage=cage, cpgi=cpgi)
#' 
#' bed.file = system.file("extdata/chr21.refseq.hg19.bed", package = "genomation")
#' gene.parts = readTranscriptFeatures(bed.file)
#' annot = annotateWithFeatures(gl, gene.parts, intersect.chr=TRUE)
#' 
#' @seealso 
#' see  \code{\link{getMembers}}, 
#'            \code{\link{heatTargetAnnotation}}, 
#'            \code{\link{plotTargetAnnotation}}
#' 
#' 
#' @docType methods
#' @rdname annotateWithFeatures-methods
setGeneric("annotateWithFeatures", 
           function(target,features,strand.aware=FALSE, intersect.chr=FALSE)
             standardGeneric("annotateWithFeatures") )

#' @aliases annotateWithFeatures,GRanges,GRangesList-method
#' @rdname annotateWithFeatures-methods
setMethod("annotateWithFeatures", 
          signature(target = "GRanges", features = "GRangesList"),
          function(target, features, strand.aware, intersect.chr){
            
            if(intersect.chr){
              message('intersecting chromosomes...')
              chrs = intersect(unique(as.character(seqnames(target))), 
                               unique(unlist(
                                 lapply(features, 
                                        function(x)as.character(seqnames(x))))))
              
              if(length(chrs) == 0)
                stop('target and feature do not have intersecting chromosomes')
              target=target[seqnames(target) %in% chrs]
              features = endoapply(features, function(x)x[seqnames(x) %in% chrs])
            }  
            
            
            x=unlist(features)
            
            # get overlap matrix
            overlaps <- as.matrix(GenomicRanges::findOverlaps(target  ,x,
                                                              ignore.strand= !strand.aware ))
            
            # get a datable from overlaps and the annotation category names
            dt=data.table(cbind(overlaps,annot=names(x)[overlaps[,2]]))
            
            # order dt by order of feature gategories in features GRangesList
            dt=dt[order(match(dt$annot,names(features))),]
            
            # get the number of target ranges overlapping with feature categories
            # and the number of features with target ranges
            annot=queryHits=subjectHits=NULL
            hits=dt[, list(targetHits=length(unique(queryHits )),
                           featureHits=length(unique(subjectHits ))) ,by=annot]
            
            # get the number of target ranges overlapping with feature categories
            # and the number of features with target ranges
            # with precendence
            
            p.hits=dt[!duplicated(dt$queryHits), 
                      list(targetHits=length(unique(queryHits )),
                           featureHits=length(unique(subjectHits ))) ,by=annot]
            
            
            # get a matrix of overlaps for targets
            # this matrix shows overlap status for each element in target
            dmemb=dcast(dt,queryHits~annot, 
                        fun.aggregate=function(x) ifelse(length(x)>0,1,0),
                        value.var="annot")
            
            # order dmemb by order of feature categories in features GRangesList
            col.order=c(colnames(dmemb)[1],
                        colnames(dmemb)[-1][order(match(colnames(dmemb)[-1],
                                                        names(features)))])
            dmemb=dmemb[,col.order,with=FALSE]
            
            # get an empty matrix of zeros, not all target ranges will be overlapping
            # with an annotation
            memb=matrix(0,nrow=length(target),ncol=ncol(dmemb)-1)
            
            # fill in the matrix of 0s with values from  the overlap matrix
            memb[as.numeric(dmemb$queryHits),]=as.matrix(dmemb[,2:ncol(dmemb),
                                                               with=FALSE])
            colnames(memb)=colnames(dmemb)[-1]
            
            # calculate number of overlaps etc before hand
            # populate stats for features that didn't overlap with the target
            no.of.OlapFeat=num.annotation=num.precedence=rep(0,length(features))
            
            names(no.of.OlapFeat)=names(num.precedence)=names(num.annotation)=names(features)
            
            no.of.OlapFeat[match( hits$annot,names(features))]=hits$featureHits
            num.annotation[match( hits$annot,names(features))]=hits$targetHits
            num.precedence[match( p.hits$annot,names(features))]=p.hits$targetHits
            
            # output the object
            new("AnnotationByFeature",
                members         = memb,
                annotation      = 100*num.annotation/length(target),
                precedence        = 100*num.precedence/length(target),
                num.annotation  =  num.annotation,
                num.precedence    =  num.precedence,
                no.of.OlapFeat  = no.of.OlapFeat ,
                perc.of.OlapFeat= 100*no.of.OlapFeat/sapply(features,length) 
            )
          })



#' @aliases annotateWithFeatures,GRangesList,GRangesList-method
#' @rdname annotateWithFeatures-methods
setMethod("annotateWithFeatures",
          signature(target = "GRangesList", features= "GRangesList"),
          function(target, features, strand.aware, intersect.chr){
            
            
            l = list()
            if(is.null(names(target)))
              names(target) = 1:length(target)
            for(i in 1:length(target)){
              
              name = names(target)[i]
              message(paste('Working on:',name))
              x = target[[name]]
              l[[name]] = annotateWithFeatures(x, features, strand.aware, intersect.chr)
              
            }                          
            return(l)
          })





# ACCESSOR FUNCTIONS
#AnnotationByFeature
#AnnotationByGeneParts


# ---------------------------------------------------------------------------- #
#' Get the membership slot of AnnotationByFeature
#'
#' Membership slot defines the overlap of target features with annotation features
#'  For example, if a target feature overlaps with an exon
#' 
#' @param x a \code{AnnotationByFeature}  object
#' 
#' @return matrix showing overlap of target features with annotation features. 
#'         1 for overlap, 0 for non-overlap
#' 
#' @usage getMembers(x)
#'
#' @export
#' @docType methods
#' @rdname getMembers-methods                      
setGeneric("getMembers", def=function(x) standardGeneric("getMembers"))

#' @aliases getMembers,AnnotationByFeature-method
#' @rdname getMembers-methods
setMethod("getMembers", 
            signature(x = "AnnotationByFeature"),
            function(x){
                return(x@members)
})

# ---------------------------------------------------------------------------- #
#' Get the percentage of target features overlapping with annotation from AnnotationByFeature
#'
#' This function retrieves percentage/number of target features 
#' overlapping with annotation
#'  
#' @param x a \code{AnnotationByFeature}  object
#' @param percentage TRUE|FALSE. If TRUE percentage of target 
#'                   features will be returned. If FALSE, 
#'                   number of target features will be returned
#' @param precedence TRUE|FALSE. If TRUE there will be a hierachy of annotation 
#'                   features when calculating numbers 
#'                   (with promoter>exon>intron precedence)
#'                   
#' That means if a feature overlaps with a promoter it will be counted as 
#' promoter overlapping only, or if it is overlapping with a an exon 
#' but not a promoter, #' it will be counted as exon overlapping only whether or
#'  not it overlaps with an intron.
#'
#' @usage getTargetAnnotationStats(x,percentage=TRUE,precedence=TRUE)
#' 
#' @examples
#' data(cage)
#' bed.file=system.file("extdata/chr21.refseq.hg19.bed", package = "genomation")
#' gene.parts = readTranscriptFeatures(bed.file)
#' cage.annot=annotateWithGeneParts(cage, gene.parts, intersect.chr=TRUE)
#' getTargetAnnotationStats(cage.annot)
#'
#' @return a vector of percentages or counts showing quantity of target features
#'         overlapping with annotation
#' 
#' @export
#' @docType methods
#' @rdname getTargetAnnotationStats-methods
setGeneric("getTargetAnnotationStats", 
                  function(x,percentage=TRUE,precedence=TRUE) 
                    standardGeneric("getTargetAnnotationStats"))

#' @rdname getTargetAnnotationStats-methods
#' @aliases getTargetAnnotationStats,AnnotationByFeature-method
setMethod("getTargetAnnotationStats", 
            signature(x = "AnnotationByFeature"),
            function(x, percentage ,precedence ){                      
            
            if(percentage){
                if(precedence){
                    return(round(x@precedence,2))
                }else{
                    return(round(x@annotation,2))
                }
            }else{
                if(precedence){
                    return(round(x@num.precedence,2))
                }else{
                    return(round(x@num.annotation,2))
                }
            }
})


# ---------------------------------------------------------------------------- #
#' Get the percentage/count of annotation features overlapping with target 
#' features from AnnotationByFeature
#'
#' This function retrieves percentage/number of 
#' annotation features overlapping with targets. 
#' For example, if \code{AnnotationByFeature}  object is containing 
#' statistics of differentially methylated 
#' regions overlapping with gene annotation. This function will return 
#' number/percentage of introns,exons and promoters
#' overlapping with differentially methylated regions.
#'  
#' @param x a \code{AnnotationByFeature}  object
#' @param percentage TRUE|FALSE. If TRUE percentage of annotation features will 
#'        be returned. If FALSE, number of annotation features will be returned
#'
#' @return RETURNS  a vector of percentages or counts showing quantity 
#'         of annotation features overlapping with target features
#'      
#'         
#' @examples
#' data(cage)
#' bed.file=system.file("extdata/chr21.refseq.hg19.bed", package = "genomation")
#' gene.parts = readTranscriptFeatures(bed.file)
#' cage.annot = annotateWithGeneParts(cage, gene.parts, intersect.chr=TRUE)
#' getFeatsWithTargetsStats(cage.annot)
#' 
#' @usage getFeatsWithTargetsStats(x,percentage=TRUE)
#' @export
#' @docType methods
#' @rdname getFeatsWithTargetsStats-methods
setGeneric("getFeatsWithTargetsStats", 
                  function(x,percentage=TRUE) 
                    standardGeneric("getFeatsWithTargetsStats"))


#' @rdname getFeatsWithTargetsStats-methods
#' @aliases getFeatsWithTargetsStats,AnnotationByFeature-method
setMethod("getFeatsWithTargetsStats", 
            signature(x = "AnnotationByFeature" ),
            function( x,percentage ){                      
            
                if(percentage){
                    return(round(x@perc.of.OlapFeat,2))
                }else{
                    return(round(x@no.of.OlapFeat,2))                        
                }
})


# ---------------------------------------------------------------------------- #
#' Get distance to nearest TSS and gene id from AnnotationByGeneParts
#'
#' This accessor function gets the nearest TSS, its distance to target feature,
#' strand and name of TSS/gene from AnnotationByGeneParts object
#' 
#' @param x a \code{AnnotationByGeneParts}  object
#' 
#' @return RETURNS a data.frame containing row number of the target features,
#'         distance of target to nearest TSS, TSS/Gene name, TSS strand
#' @usage getAssociationWithTSS(x)
#' @aliases getAssociationWithTSS,-methods getAssociationWithTSS,
#'          AnnotationByGeneParts-method
#' 
#' @examples
#' data(cage)
#' bed.file = system.file("extdata/chr21.refseq.hg19.bed", package = "genomation")
#' gene.parts = readTranscriptFeatures(bed.file)
#' cage.annot = annotateWithGeneParts(cage, gene.parts, intersect.chr=TRUE)        
#' head(getAssociationWithTSS(cage.annot))
#'  
#' @export
#' @docType methods
#' @rdname AnnotationByGeneParts-methods
setGeneric("getAssociationWithTSS", 
                  function(x) 
                    standardGeneric("getAssociationWithTSS"))

#' @rdname AnnotationByGeneParts-methods
#' @docType methods
#' @aliases getAssociationWithTSS,AnnotationByGeneParts-method
setMethod("getAssociationWithTSS", 
            signature(x = "AnnotationByGeneParts"),
            function(x){
                return(x@dist.to.TSS)
})



# PLOTTING FUNCTIONS

# ---------------------------------------------------------------------------- #
#' Plot annotation categories from AnnotationByGeneParts or AnnotationByFeature
#'
#' This function plots a pie or bar chart for showing percentages of targets 
#' annotated by genic parts or other query features
#' 
#' @param x a \code{AnnotationByFeature} or  
#'        \code{AnnotationByGeneParts} object
#' @param precedence TRUE|FALSE. If TRUE there will be a hierachy of annotation 
#'        features when calculating numbers 
#'        (with promoter>exon>intron precedence). 
#'        This option is only valid when x is a 
#'        \code{AnnotationByGeneParts} object
#' @param col a vector of colors for piechart or the bar plot
#' @param cex.legend a numeric value of length 1 to specify the size of the legend. By default 1.
#' @param ... graphical parameters to be passed to \code{pie} 
#'            or \code{barplot} functions
#'
#'
#' @examples
#' data(cage)
#' 
#' bed.file = system.file("extdata/chr21.refseq.hg19.bed", package = "genomation")
#' gene.parts = readTranscriptFeatures(bed.file)
#' annot = annotateWithGeneParts(cage, gene.parts, intersect.chr=TRUE)
#' \donttest{
#' plotTargetAnnotation(annot)
#' }
#'
#' @return plots a piechart or a barplot for percentage of 
#'         the target features overlapping with annotation
#' 
#' @export
#' @docType methods
#' @rdname plotTargetAnnotation-methods
setGeneric("plotTargetAnnotation", 
              function(x,
                       precedence=TRUE,
                       col=getColors(length(x@annotation)),
                       cex.legend=1,...) 
                       standardGeneric("plotTargetAnnotation"))

#' @rdname plotTargetAnnotation-methods
#' @docType methods
#' @aliases plotTargetAnnotation,AnnotationByFeature-method
setMethod("plotTargetAnnotation", 
            signature(x = "AnnotationByFeature"),
            function(x,precedence,col,cex.legend,...){
                
                props=getTargetAnnotationStats(x,precedence)

            if(precedence | ( is(x,"AnnotationByFeature") & 
                       !is(x,"AnnotationByGeneParts")) ){
                slice.names=names(props)
                names(props)=paste( paste(round(props),"%"),sep=" ")

                pie(props,cex=0.9,col=col,...)
                legend("topright",legend=slice.names,fill=col, cex=cex.legend)

            }else{

                mids = barplot(props,col=col,...)  
                text(mids, y = props,labels=paste(paste(round(props),"%",sep="")),pos=1) 
            }
})



# ---------------------------------------------------------------------------- #
#' Plots the percentage of overlapping intervals with genomic features in a heatmap
#'
#' This function plots a heatmap for percentage of overlapping ranges 
#' with provided genomic features. The input object is a list of 
#' \code{AnnotationByFeature} objects, which contains necessary information
#' about overlap statistics to make the plot.
#' 
#' @param l a \code{list} of  \code{AnnotationByFeature} objects. This 
#'        could be returned by \code{\link{annotateWithFeatures}} function.
#' @param cluster TRUE/FALSE. If TRUE the heatmap is going to be clustered 
#'        using a default hierarchical clustering scheme.
#' @param col a vector of two colors that will be used for interpolation. 
#'        The first color is the lowest one, the second is the highest one
#' @param precedence TRUE|FALSE. If TRUE the precedence of annotation 
#'        features will be used when plotting. The precedence will be taken
#'        from the GRangesList used as annotation. The first GRanges will be 
#'        treated as most important, and the second as the second most important
#'        and so on. Such that, if an interval overlaps with features on that 
#'        is part of the first GRanges object in the annotation GRangesList,
#'        the rest of its overlaps with other elements in the annotation 
#'        GRangesList will be ignored. This feature is important to have if 
#'        the user desires that percentage of overlaps adds up to 100. This is
#'        only possible when the annotation features are non-overlaping with
#'        eachother or there is a hierarchy/precedence among them such as 
#'        (with promoter>exon>intron precedence). In this case, anything that
#'        overlaps with a promoter annotation will only be counted as promoter
#'        even if it overlaps with exons or introns.
#'    
#'        
#' @param plot If FALSE, does not plot the heatmap just returns the matrix
#'        used to make the heatmap
#' 
#' 
#' @return returns the matrix used to make the heatmap when plot FALSE, 
#' otherwise returns ggplot2 object which can be modified further.
#'          
#' 
#' @examples
#' 
#' library(GenomicRanges)
#' data(cage)
#' data(cpgi)
#' cage$tpm=NULL
#' gl = GRangesList(cage=cage, cpgi=cpgi)
#' 
#' bed.file = system.file("extdata/chr21.refseq.hg19.bed", package = "genomation")
#' gene.parts = readTranscriptFeatures(bed.file)
#' annot = annotateWithFeatures(gl, gene.parts, intersect.chr=TRUE)
#' 
#'             
#' \donttest{  
#' heatTargetAnnotation(annot)
#' }
#' 
#' @seealso 
#' see  \code{\link{getMembers}}, 
#'            \code{\link{annotateWithFeatures}} 
#'          
#'            
#' @export
#' @docType methods
#' @rdname heatTargetAnnotation-methods
#' @aliases plotGeneAnnotation
#' @docType methods
heatTargetAnnotation<-function(l, cluster=FALSE, col=c("white","blue"),
                               precedence=FALSE,plot=TRUE){
  
  if(!all(unlist(lapply(l, class)) == "AnnotationByFeature"))
    stop("All elements of the input list need to be AnnotationByFeature-class")
  if(precedence){
    d = data.frame(do.call(rbind, lapply(l, function(x)x@precedence)))
  }else{
    d = data.frame(do.call(rbind, lapply(l, function(x)x@annotation)))
  }
  
  if(!plot){
    return(as.matrix(d))    
  }
  d = data.frame('SampleName'=rownames(d), d)
  
  ind = 1:nrow(d)
  if(cluster == TRUE){
    h = hclust(dist(d[,-1]))
    ind = h$order
  }
  
  m = reshape2::melt(d, id.vars='SampleName')
  colnames(m)[2] = 'Feature'

  Feature=SampleName=value=NULL
  p = ggplot(m, aes(x=Feature, y=SampleName, fill=value)) +
    theme(
      axis.title.x = element_text(colour='white'), 
      axis.text.x  = element_text(colour='black', face='bold'), 
      axis.text.y  = element_text(colour='black'), 
      axis.title.y = element_text(colour='white', face='bold')) 
  p = p + geom_tile(aes(fill = value)) +
    geom_text(aes(fill = m$value, label = round(m$value, 1))) +
    scale_fill_gradient(low =col[1], high = col[2]) +
    theme(axis.text.x = element_text(angle = 90, hjust = 1)) 
  p
}

#' @export
plotGeneAnnotation  <- function(l, 
                                cluster=FALSE, 
                                col=c('white','cornflowerblue'))
{
  .Deprecated("heatTargetAnnotation ")
  ## use new function, or remainder of myOldFunc
  heatTargetAnnotation(l,cluster=cluster, col=col,
                         precedence=FALSE,plot=TRUE)
}

# ---------------------------------------------------------------------------- #
#' read.zip function
#' 
#' @param file zip file
#' @param ... additional objects and parameters
#' @keywords internal
read.zip <- function(file, ...) {
  zipFileInfo <- unzip(file, list=TRUE)
  if(nrow(zipFileInfo) > 1)
    stop("More than one data file inside zip")
  else
    read.table(unz(file, as.character(zipFileInfo$Name)), ...)
}

#' compressedAndUrl2temp function
#' 
#' @param filename file name
#' @keywords internal
compressedAndUrl2temp <- function(filename){ 
  if(grepl('^(http://|https://|ftp://|ftps://).*(.gz|.bz2|.xz|.zip)$',filename)){
    temp <- tempfile()
    download.file(filename, destfile=temp, method="auto", quiet = TRUE)
    ext <- tail(unlist(strsplit(x=filename, split="\\.")), n=1)
    filename <- paste0(temp, ".", ext)
    file.rename(temp, filename)
  }
  filename
}

#' detectUCSCheader function
#' 
#' detects UCSC header (and first track)
#' 
#' @param filename file name
#' @keywords internal
detectUCSCheader <- function(filename){
  skip=0
  if(grepl("^.*(.zip)[[:space:]]*$", filename)){
    zipFileInfo <- unzip(filename, list=TRUE)
    if(length(zipFileInfo$Name)>1) stop("More than one data file inside zip")
    con <- unz(filename, filename=zipFileInfo$Name); open(con)
  }else{
    con <- file(filename, open = "r")
  }
  while (length(oneLine <- readLines(con, n = 1, warn = FALSE)) > 0) {
    if(grepl("^ *browser", oneLine) | 
         grepl("^ *track", oneLine) |
         grepl("^ *#", oneLine)){
      skip = skip + 1
    }else{
      break
    }
  }  
  close(con)
  return(skip)
}

#' readTableFast function
#' 
#' fast reading of big tables
#' chr indicates index of column of chromosomes
#' @param filename file name
#' @param header logical
#' @param skip numeric 
#' @param sep character
#' @param chr numeric
#' @keywords internal
readTableFast<-function(filename,header=TRUE,skip=0,sep="\t",chr=1){
  
  filename <- compressedAndUrl2temp(filename)
  if(skip==FALSE){
    skip = 0
  }else if(skip=="auto"){
    skip = detectUCSCheader(filename)
  }
 
  if(grepl("^.*(.zip)[[:space:]]*$", filename)){
	tab30rows <- read.zip(filename,
			      sep=sep, 
			      skip=skip,
			      nrows=30,
			      header=header,
			      stringsAsFactors=FALSE)
  }else{
	tab30rows <- read.table(file=filename, 
				sep=sep, 
				skip=skip,
				nrows=30,
				header=header,
				stringsAsFactors=FALSE)
  }
  classes  <- sapply(tab30rows, class)
  cl=""
  for(cla in classes){
    if(cla=="character"){
      cl=paste0(cl, "c")
    }else if(cla=="numeric" | cla=="double"){
      cl=paste0(cl, "d")
    }else if(cla=="integer"){
      cl=paste0(cl, "i")
    }else if(cla=="logical")
      cl=paste0(cl, "l")
  }
  # set column with chrs as character
  # when chromosomes are numeric in first 30 rows
  # and in >=31st row is e.g. "X" chromosome 
  # then it causes issues
   if(is.numeric(chr)){
     cl_split <- unlist(strsplit(cl, "")[[1]])
     cl_split[chr]="c"
     cl=paste0(cl_split, collapse = "")
   }
  
  df <- read_delim(file=filename, 
                   delim=sep, 
                   skip=skip,
                   col_names=header,
                   col_types=cl,
                   locale=locale(grouping_mark = "_")
                   )
  #changing default variables names from read_delim (X[0-9]+) to data.frame (V[0-9]+)
  colnames(df) <- gsub("^X(\\d+)$", "V\\1", colnames(df)) 
  return(as.data.frame(df))
}   

#---------------------------------------------------------------------------- #
#' Read a tabular file and convert it to GRanges. 
#' 
#' The function reads a tabular  text file that contains location and other information
#' on genomic features and returns a \code{\link{GRanges}} object. 
#' The minimal information that the file has to have is chromosome, 
#' start and end columns. Strand information is not compulsory.
#` 
#'  
#' @param file location of the file, a character string such as: "/home/user/my.bed"
#'             or the input itself as a string (containing at least one \\n).
#'             
#' @param chr  number of the column that has chromsomes information in the table (Def:1)
#' @param start number of the column that has start coordinates in the table (Def:2)
#' @param end  number of the column that has end coordinates in the table (Def:3)
#' @param strand number of the column that has strand information, only -/+
#'               is accepted (Default:NULL)
#'               
#' @param meta.cols  named \code{list} that maps column numbers to 
#'                   meta data columns. 
#'                   e.g. list(name=5, score=10), which means 5th column will be
#'                   named "name", and 10th column will be named "score" and their
#'                   contents will be a part of the returned GRanges object. 
#'                   If header = TRUE, meta.cols parameter will over-write the 
#'                   column names given by the header line of the data frame.
#'                   
#' @param keep.all.metadata \code{logical} determining if the extra columns (
#'        the ones that are not designated by chr,start,end,strand and meta.cols
#'        arguments )
#'        should be kept or not. (Default:FALSE)
#'        
#'        
#' @param zero.based a boolean which tells whether the ranges in 
#'        the bed file are 0 or 1 base encoded. (Default: FALSE)
#'        
#' @param remove.unusual if TRUE(default) remove the chromosomes with unsual 
#'        names, such as chrX_random (Default:FALSE)
#'        
#' @param header whether the original file contains a header line
#'  which designates the column names. If \code{TRUE} header will be used to 
#'  construct column names. These names can be over written by meta.cols argument.
#' @param skip number of lines to skip. If there is a header line(s) you do not
#'        wish to include you can use skip argument to skip that line.
#' @param sep a single character which designates the separator in the file. 
#'        The default value is tab. 
#'        
#' @return \code{\link{GRanges}} object
#'
#' @examples
#' my.file=system.file("extdata","chr21.refseq.hg19.bed",package="genomation")
#' refseq = readGeneric(my.file,chr=1,start=2,end=3,strand=NULL,
#'                       meta.cols=list(score=5,name=4),
#'                       keep.all.metadata=FALSE, zero.based=TRUE)
#' head(refseq)
#' @export
#' @docType methods
#' @rdname readGeneric
readGeneric<-function(file, chr=1,start=2,end=3,strand=NULL,meta.cols=NULL, 
                      keep.all.metadata=FALSE, zero.based=FALSE, 
                      remove.unusual=FALSE, header=FALSE, 
                      skip=0, sep="\t"){
  
  # reads the bed files
  df=readTableFast(file, header=header, skip=skip, sep=sep, chr=chr)                    

  # make a list of new column names, and their column numbers
  col.names1=list(chr=chr,start=start,end=end,strand=strand)
  col.names=c(col.names1,meta.cols) # put the meta colums if any
  
  # check if col number exceeds dimensions of the original df.
  if( max(unlist(col.names)) > ncol(df) ) 
    stop("Number of columns is lower than designated number of columns by ",
         "meta.cols,chr,start,end or strand arguments\n")
  
  # change the col names to the ones given by meta.cols and chr,str,end,strand
  colnames(df)[unlist(col.names)] = names(unlist(col.names))
  
  # converts the . strand character to *
  sind = grepl('strand',colnames(df))
  if(any(sind) & !is.null(strand))
    df[, sind] = sub('\\.', '*', df[,sind])
  
  # removes nonstandard chromosome names
  if(remove.unusual)
    df = df[grep("_", as.character(df$chr),invert=TRUE),]
  
  g = makeGRangesFromDataFrame(
    df, 
    keep.extra.columns=FALSE, 
    starts.in.df.are.0based=zero.based,
    ignore.strand=is.null(strand))
  
  # this names can not be column names in meta-data
  black.names=c("seqnames", "ranges", "strand", "seqlevels", "seqlengths",
                "isCircular", "start", "end", "width", "element")
  
  
  if(keep.all.metadata){
    my.mcols = df[,-unlist(col.names1),drop=FALSE]
    mcols(g) = my.mcols[, !colnames(my.mcols) %in% black.names]
  }else if(!is.null(meta.cols)){
    my.mcols=df[,unlist(meta.cols),drop=FALSE]
    values(g) = my.mcols[, !colnames(my.mcols) %in% black.names, drop=FALSE]
  }
  
  return(g)
}

#' Read a BED file and convert it to GRanges. 
#' 
#' The function reads a BED file that contains location and other information
#' on genomic features and returns a \code{\link{GRanges}} object. 
#' The minimal information that the BED file has to have is chromosome, 
#' start and end columns. it can handle all BED formats up to 12 columns. 
#' 
#'  
#' @param file location of the file, a character string such as: "/home/user/my.bed"
#'             or the input itself as a string (containing at least one \\n).
#'             The file can end in \code{.gz}, \code{.bz2}, \code{.xz}, or \code{.zip}
#'             and/or start with \code{http://} or \code{ftp://}. If the file is not 
#'             compressed it can also start with \code{https://} or \code{ftps://}.
#'
#' @param track.line the number of track lines to skip, "auto" to detect them 
#'                   automatically or FALSE(default) if the bed file doesn't 
#'                   have track lines
#'                   
#' @param remove.unusual if TRUE remove the chromosomes with unsual 
#'        names, such as chrX_random (Default:FALSE)        
#' @param zero.based a boolean which tells whether the ranges in 
#'        the bed file are 0 or 1 base encoded. (Default: TRUE)
#'        
#' @return \code{\link{GRanges}} object
#'
#' @examples
#' my.file=system.file("extdata","chr21.refseq.hg19.bed",package="genomation")
#' refseq = readBed(my.file,track.line=FALSE,remove.unusual=FALSE)
#' head(refseq)
#' 
#' @export
#' @docType methods
#' @rdname readBed
readBed<-function(file,track.line=FALSE,remove.unusual=FALSE,zero.based=TRUE)
{
  
  meta.cols=list(score=5,name=4,thickStart=7,  
                 thickEnd=8, 
                 itemRgb=9,  
                 blockCount=10, 
                 blockSizes=11,  
                 blockStarts=12 )
  
  file <- compressedAndUrl2temp(file)
  if(is.numeric(track.line)){
    skip = track.line
  }else if(track.line=="auto"){
    skip = detectUCSCheader(file)
  }else{
    skip = 0
  }
  
  df=read_delim(file,skip=skip,n_max=2,col_names=FALSE, delim="\t")
  numcol=ncol(df)
  
  if(numcol==3){
    df=readGeneric(file, chr = 1, start = 2, end = 3, strand = NULL,
                   meta.cols = NULL,   zero.based = zero.based,
                   remove.unusual =remove.unusual, header = FALSE, skip = skip, sep = "\t")
  }else if(numcol==4){
    df=readGeneric(file, chr = 1, start = 2, end = 3, strand = NULL,
                   meta.cols = meta.cols[1],   zero.based = zero.based,
                   remove.unusual =remove.unusual, header = FALSE, skip = skip, sep = "\t")    
  }else if(numcol==5){
    df=readGeneric(file, chr = 1, start = 2, end = 3, strand = NULL,
                   meta.cols = meta.cols[1:2],   zero.based = zero.based,
                   remove.unusual =remove.unusual, header = FALSE, skip = skip, sep = "\t")    
  }else if(numcol == 6){
    df=readGeneric(file, chr = 1, start = 2, end = 3, strand = 6,
                   meta.cols = meta.cols[1:2],   zero.based = zero.based,
                   remove.unusual =remove.unusual, header = FALSE, skip = skip, sep = "\t") 
  }else if(numcol > 6){
    df=readGeneric(file, chr = 1, start = 2, end = 3, strand = 6,
                   meta.cols = meta.cols[c(1:2,(3:8)[1:(numcol-6)])],   zero.based = zero.based,
                   remove.unusual =remove.unusual, header = FALSE, skip = skip, sep = "\t") 
  }
  df
  
} 

# ---------------------------------------------------------------------------- #
#' A function to read the Encode formatted broad peak file into a GRanges object
#'
#' @param file an absolute or relative path to a bed file formatted by the Encode broadPeak standard.
#'             The file can end in \code{.gz}, \code{.bz2}, \code{.xz}, or \code{.zip}
#'             and/or start with \code{http://} or \code{ftp://}. If the file is not compressed
#'             it can also start with \code{https://} or \code{ftps://}.
#' @param track.line the number of track lines to skip, "auto" to detect them automatically
#'                   or FALSE(default) if the bed file doesn't have track lines
#' @param zero.based a boolean which tells whether the ranges in 
#'        the bed file are 0 or 1 base encoded. (Default: TRUE)
#' @usage readBroadPeak(file, track.line=FALSE, zero.based=TRUE)
#' @return a GRanges object
#'
#' @examples
#' broad.peak.file = system.file('extdata',"ex.broadPeak", package='genomation')
#'                            
#' broad.peak = readBroadPeak(broad.peak.file)
#' head(broad.peak)
#' 
#' @docType methods
#' @rdname readBroadPeak
#' @export
readBroadPeak<-function(file, track.line=FALSE, zero.based=TRUE){
  
  g = readGeneric(file,
                  strand=6,
                  meta.cols=list(name=4,
                                 score=5,
                                 signalValue=7,
                                 pvalue=8,
                                 qvalue=9),
                  header=FALSE,
                  skip=track.line,
                  zero.based=zero.based)
  return(g)
}

# ---------------------------------------------------------------------------- #
#' A function to read the Encode formatted narrowPeak file into a GRanges object
#' @param file an absolute or relative path to a bed file formatted by the Encode 
#'             narrowPeak standard. The file can end in \code{.gz}, \code{.bz2}, 
#'             \code{.xz}, or \code{.zip} and/or start with \code{http://} or 
#'             \code{ftp://}. If the file is not compressed
#'             it can also start with \code{https://} or \code{ftps://}.
#' @param track.line the number of track lines to skip, "auto" to detect them automatically
#'                   or FALSE(default) if the bed file doesn't have track lines
#' @param zero.based a boolean which tells whether the ranges in 
#'        the bed file are 0 or 1 base encoded. (Default: TRUE)
#' @usage readNarrowPeak(file, track.line=FALSE, zero.based=TRUE)
#' @return a GRanges object
#'
#' @examples
#' narrow.peak.file = system.file('extdata',"ex.narrowPeak", package='genomation')
#'                  
#' narrow.peak = readBroadPeak(narrow.peak.file)
#' head(narrow.peak)
#' @docType methods
#' @rdname readNarrowPeak
#' @export
readNarrowPeak<-function(file, track.line=FALSE, zero.based=TRUE){
  
  g = readGeneric(file,
                  strand=6,
                  meta.cols=list(name=4,
                                 score=5,
                                 signalValue=7,
                                 pvalue=8,
                                 qvalue=9,
                                 peak=10),
                  header=FALSE,
                  skip=track.line,
                  zero.based=zero.based)
  return(g)
}



# ---------------------------------------------------------------------------- #
#' A function to read-in genomic features and their upstream and downstream adjecent regions
#' such as CpG islands and their shores
#'
#' @param location for the bed file of the feature.
#' @param flank number of basepairs for the flanking regions
#' @param clean If set to TRUE, flanks overlapping with other main features will be trimmed
#' @param remove.unusual  remove chromsomes with unsual names random, Un and antyhing with "_" character
#' @param feature.flank.name the names for feature and flank ranges, it should be a character 
#'                           vector of length 2. example: c("CpGi","shores")
#' @usage readFeatureFlank(location,remove.unusual=TRUE,flank=2000,
#'                         clean=TRUE,feature.flank.name=NULL)
#' @return a GRangesList object containing one GRanges object for flanks and one for GRanges object
#'         for the main feature.
#'   NOTE: This can not return a CompressedGRangesList at the moment because flanking regions do not
#'   have to have the same column name as the feature. CompressedGRangesList elements should resemble
#'   each other in the column content. We can not satisfy that criteria for the flanks
#'
#' @examples
#' cgi.path = system.file('extdata/chr21.CpGi.hg19.bed', package='genomation')
#' cgi.shores = readFeatureFlank(cgi.path)
#' cgi.shores
#' @export
#' @docType methods
#' @rdname readFeatureFlank-methods
setGeneric("readFeatureFlank", 
           function(location,remove.unusual=TRUE,
                    flank=2000,
                    clean=TRUE,
                    feature.flank.name=NULL) 
             standardGeneric("readFeatureFlank") )

#' @aliases readFeatureFlank,character-method
#' @rdname readFeatureFlank-methods
setMethod("readFeatureFlank", 
          signature(location = "character"),
          function(location,remove.unusual,flank ,clean,feature.flank.name){
            
            feat = readGeneric(location)
            flanks = getFlanks(feat,flank=flank,clean=clean)
            x = GRangesList(features=feat,flanks=flanks,compress=FALSE)
            if(!is.null(feature.flank.name) & length(feature.flank.name)==2)
              names(x)=feature.flank.name
            
            return(x)
          })

# ---------------------------------------------------------------------------- #
#' Function for reading exon intron and promoter structure from a given bed file
#'
#' @param location location of the bed file with 12 or more columns. 
#'                 The file can end in \code{.gz}, \code{.bz2}, \code{.xz}, or \code{.zip}
#'                 and/or start with \code{http://} or \code{ftp://}. If the file is not compressed
#'                 it can also start with \code{https://} or \code{ftps://}.
#' @param remove.unusual remove the chromomesomes with unsual names, mainly random chromsomes etc
#' @param up.flank  up-stream from TSS to detect promoter boundaries
#' @param down.flank down-stream from TSS to detect promoter boundaries
#' @param unique.prom get only the unique promoters, promoter boundaries will not have 
#'                    a gene name if you set this option to be TRUE
#' @usage readTranscriptFeatures(location,remove.unusual=TRUE,
#'                               up.flank=1000,down.flank=1000,unique.prom=TRUE)
#' @return a \code{\link{GRangesList}} containing locations of exon/intron/promoter/TSS
#' @note  one bed track per file is only accepted, the bed files with multiple tracks will cause en error
#' 
#' @examples
#' my.bed12.file = system.file("extdata/chr21.refseq.hg19.bed", package = "genomation")
#' my.bed12.file
#' feats = readTranscriptFeatures(my.bed12.file) 
#' names(feats)
#' sapply(feats, head)
#' @export
#' @docType methods
#' @rdname readTranscriptFeatures-methods
setGeneric("readTranscriptFeatures", 
           function(location,remove.unusual=TRUE,
                    up.flank=1000,
                    down.flank=1000,
                    unique.prom=TRUE) 
             standardGeneric("readTranscriptFeatures"))

#' @aliases readTranscriptFeatures,character-method
#' @rdname readTranscriptFeatures-methods
setMethod("readTranscriptFeatures", 
          signature(location = "character"),
          function(location,remove.unusual,up.flank ,down.flank ,unique.prom){
            
            # readBed6
            message('Reading the table...\r')
            bed=readTableFast(location,header=FALSE,skip="auto")                    
            if(remove.unusual)
              bed=bed[grep("_", as.character(bed[,1]),invert=TRUE),]
            
            # introns
            message('Calculating intron coordinates...\r')
            introns    = convertBed2Introns(bed)
            # exons
            message('Calculating exon coordinates...\r')
            exons    = convertBed2Exons(bed)
            
            # get the locations of TSSes
            message('Calculating TSS coordinates...\r')
            tss=bed
            #  + strand
            tss[tss$V6=="+",3] = tss[tss$V6=="+",2]
            #  - strand
            tss[tss$V6=="-",2]=tss[tss$V6=="-",3]
            
            tssg = GRanges(seqnames=as.character(tss$V1),
                           ranges=IRanges(start=tss$V2, end=tss$V3),
                           strand=as.character(tss$V6),
                           score=rep(0,nrow(tss)),
                           name=tss$V4)
            
            message('Calculating promoter coordinates...\r')
            # get the locations of promoters
            # + strand
            bed[bed$V6=="+",3]=bed[bed$V6=="+",2]+down.flank
            bed[bed$V6=="+",2]=bed[bed$V6=="+",2]-up.flank
            
            #  - strand
            bed[bed$V6=="-",2]=bed[bed$V6=="-",3]-down.flank
            bed[bed$V6=="-",3]=bed[bed$V6=="-",3]+up.flank
            
            
            if(! unique.prom){
              prom.df = (bed[,c(1,2,3,4,6)])
              prom = GRanges(seqnames=as.character(prom.df$V1),
                             ranges = IRanges(start=prom.df$V2, end=prom.df$V3),
                             strand = as.character(prom.df$V6),
                             score=rep(0,nrow(prom.df)),
                             name=prom.df$V4)
            }else{
              prom.df = unique(bed[,c(1,2,3,6)])
              prom = GRanges(seqnames=as.character(prom.df$V1),
                             ranges=IRanges(start=prom.df$V2, end=prom.df$V3),
                             strand=as.character(prom.df$V6),
                             score=rep(0,nrow(prom.df)),
                             name=rep(".",nrow(prom.df)) )
            }
            
            message('Outputting the final GRangesList...\r\n')
            GRangesList(exons=exons,introns=introns,promoters=prom,TSSes=tssg)
          })


# ---------------------------------------------------------------------------- #
#' Converts a gff formated data.frame into a GenomicRanges object. 
#' The GenomicRanges object needs to be properly formated for the function to work.
#' @param gff.file path to a gff formatted file. 
#'                 The file can end in \code{.gz}, \code{.bz2}, \code{.xz}, or \code{.zip}
#'                 and/or start with \code{http://} or \code{ftp://}. If the file is not compressed
#'                 it can also start with \code{https://} or \code{ftps://}.
#' @param filter a character designating which elements to retain from the gff file (e.g. exon, CDS, ...)
#' @param zero.based \code{boolean} whether the coordinates are 0 or 1 based. 0 is the default
#' @param ensembl \code{boolean} if TRUE, add the chr prefix to seqlevels. FALSE by default
#' @return returns a \code{GenomicRanges} object
#' 
#' @examples
#' gff.file = system.file('extdata/chr21.refseq.hg19.gtf', package='genomation')
#' gff = gffToGRanges(gff.file)
#' 
#' @docType methods
#' @export
gffToGRanges = function(gff.file, filter=NULL, zero.based=FALSE, ensembl=FALSE){
  
  gff = rtracklayer::import(gff.file)
  if(zero.based)
    gff$start = gff$start + 1
  
  if(ensembl)
    seqlevels(gff) = paste('chr',seqlevels(gff),sep='')
  
  if(!is.null(filter)){
    if(!any(gff$type == filter))
      stop(paste(filter, 'category does not exist in the gff file'))
    gff = gff[grepl(filter, gff$type)]
  }
  
  return(gff)
}

# ---------------------------------------------------------------------------- #
# test for ScoreMatrix function
test_ScoreMatrixBin_RleList_GRanges = function()
{
  
  # -----------------------------------------------#
  # usage
  # input RleList
  rl = RleList(chr1 = Rle(rep(c(1,2,3), each=4)), chr2=Rle(rep(c(4,5,6), each=4)))
  
  #1. test for proper workings
  gr1 = GRanges(rep(c('chr1','chr2'), each=2), IRanges(c(1,6,1,6),c(4,9,4,9)), 
                strand=c('+','-','+','-'))
  m1 = matrix(c(1,1,1,1,2,2,2,3,4,4,4,4,5,5,5,6), ncol=4, byrow=T)
  m1 = as(cbind(rowMeans(m1[,1:2]), rowMeans(m1[,3:4])),'ScoreMatrix')
  rownames(m1) = 1:4
  s1 = ScoreMatrixBin(rl, gr1, bin.num=2)
  checkEquals(s1, m1)
  
  #1.1 check for name dropping
  s1.1 = ScoreMatrixBin(rl, gr1, bin.num=1)
  checkEquals(rownames(s1.1),as.character(1:4))
  
  
  #2. test for different bin.op
  for(fun in c('min', 'max', 'median')){
    message('testing:', fun)
    m2 = matrix(c(1,1,1,1,2,2,2,3,4,4,4,4,5,5,5,6), ncol=4, byrow=T)
    m2 = as(cbind(apply(m2[,1:2],1,match.fun(fun)), 
                  apply(m2[,3:4],1,match.fun(fun))),'ScoreMatrix')
    rownames(m2) = 1:4
    s2 = ScoreMatrixBin(rl, gr1, bin.num=2, bin.op=fun)
    checkEquals(s2, m2)  
  }
  
  #3. test strand aware
  m3 = matrix(c(1,1,1,1,2,2,2,3,4,4,4,4,5,5,5,6), ncol=4, byrow=T)
  m3 = as(cbind(rowMeans(m3[,1:2]), rowMeans(m3[,3:4])),'ScoreMatrix')
  rownames(m3) = 1:4
  m3[c(2,4),] = m3[c(2,4),2:1]
  s3 = ScoreMatrixBin(rl, gr1, bin.num=2, strand.aware=T)
  checkEquals(s3, m3)
  
  #4.A bin.num > 1 and dropping chromosomes
  t4 = GRanges(rep(c('chr1','chr2'), times=c(3,2)), IRanges(c(seq(1,20,4)), width=3))
  t4$score = 1
  w4 = GRanges(rep(c('chr3','chr2'), each=2), IRanges(rep(c(13,17), times=2), width=3))
  s4 = suppressWarnings(ScoreMatrixBin(t4, w4, bin.num = 2,type = "bigWig"))
  checkEquals(as.numeric(rownames(as.data.frame(s4))), c(3,4))
  
  #4.B bin.num = 1 and dropping chromosomes
  t5 = GRanges(rep(c('chr1','chr2'), times=c(3,2)), IRanges(c(seq(1,20,4)), width=3))
  t5$score = 1
  w5 = GRanges(rep(c('chr3','chr2'), each=2), IRanges(rep(c(13,17), times=2), width=3))
  s5 = suppressWarnings(ScoreMatrixBin(t5, w5,bin.num = 1,type = "bigWig"))
  checkEquals(as.numeric(rownames(as.data.frame(s5))), c(3,4))
  
  # -----------------------------------------------#
  # errors
  # error for removing all bins
  checkException(SCoreMatrixBin(rl, gr1, bin.num=2), silent=TRUE)
  
  #gr5 = GRanges(rep(c('chr1','chr2'), each=2), IRanges(c(1,5,1,5),c(5,6,5,6)), 
  #              strand=c('+','-','+','-'))
  #checkException(ScoreMatrixBin(rl, gr5, bin.num=3), silent=FALSE) 
}

# ---------------------------------------------------------------------------- #
test_ScoreMatrixBin_GRanges_GRanges = function()
{
  target = GRanges(rep(c(1,2),each=6), 
                   IRanges(rep(c(1,2,3,7,8,9), times=2), width=5),
                   weight = rep(c(1,2),each=6))
  windows = GRanges(rep(c(1,2),each=2), IRanges(rep(c(1,2), times=2), width=5), 
                    strand=c('-','+','-','+'))
  
  # -----------------------------------------------#
  # usage
  # normal function works
  s1 = ScoreMatrixBin(target=target, windows=windows, bin.num=2)
  m1 = matrix(rep(c(1,2,3,3,3,2,3,3,3,2),times=2), ncol=5, byrow=T)
  m1 = as(cbind(rowMeans(m1[,1:3]), rowMeans(m1[,4:5])), 'ScoreMatrix')
  rownames(m1) = 1:4
  checkEquals(s1,m1)
  
  # function with weight col works
  s2 = ScoreMatrixBin(target=target, windows=windows,bin.num=2, weight.col='weight')
  m2 = matrix(rep(c(1,2,3,3,3,2,3,3,3,2),times=2), ncol=5, byrow=T)
  m2[3:4,] = m2[3:4,]*2
  m2 = as(cbind(rowMeans(m2[,1:3]), rowMeans(m2[,4:5])), 'ScoreMatrix')
  rownames(m2) = 1:4
  checkEquals(s2,m2)  
  
  #strand aware
  s3 = ScoreMatrixBin(target=target, windows=windows, bin.num=2, strand.aware=T)
  m3 = matrix(rep(c(1,2,3,3,3,2,3,3,3,2),times=2), ncol=5, byrow=T)
  rownames(m3) = 1:4
  m3 = cbind(rowMeans(m3[,1:3]), rowMeans(m3[,4:5]))
  m3[c(1,3),] = rev(m3[c(1,3),])
  m3 = as(m3, 'ScoreMatrix')
  rownames(m3) = 1:4
  checkEquals(s3,m3)
  
  # -----------------------------------------------#
  # errors
  checkException(ScoreMatrixBin(target, windows, weight.col=''), silent=TRUE)
  
  # number of bins > ncol 
  #expect_warning(ScoreMatrixBin(target, windows, strand.aware = FALSE, bin.num=10))
               
}


# ---------------------------------------------------------------------------- #
test_ScoreMatrixBin_character_GRanges = function()
{
  target = GRanges(rep(c(1,2),each=7), 
                   IRanges(rep(c(1,1,2,3,7,8,9), times=2), width=5),
                   weight = rep(c(1,2),each=7), 
                   strand=c('-', '-', '-', '-', '+', '-', '+', '-', '-', '-', '-', '-', '-', '+'))
  windows = GRanges(rep(c(1,2),each=2), IRanges(rep(c(1,2), times=2), width=5), 
                    strand=c('-','+','-','+'))
  target.paired.end = GRanges(rep(1,each=7), 
                              IRanges(c(1,1,2,3,7,8,9), width=c(19, 19, 19, 19, 16, 16, 16)),
                              strand=rep("+", times=7))
  windows.paired.end = GRanges(rep(c(1),each=4), IRanges(c(7,8,20, 21), width=10), 
                               strand=c('+','+','+','+'))
  # -----------------------------------------------#
  # usage
  # bam file
  bam.file = system.file('unitTests/test.bam', package='genomation')
  s1 = ScoreMatrixBin(bam.file, windows, type='bam', bin.num=2)
  m1 = ScoreMatrixBin(target, windows, bin.num=2)
  checkEquals(s1,m1)
  
  # bam file, rpm=TRUE
  s2 = ScoreMatrixBin(bam.file, windows,bin.num=2, type='bam', rpm=TRUE)
  tot = 1e6/sum(idxstatsBam(normalizePath(bam.file))$mapped)
  m2 = m1*tot
  checkEquals(s2, m2)
  
  #bam file, rpm=FALSE, unique=TRUE
  s3 = ScoreMatrixBin(bam.file, windows,bin.num=2, type='bam', unique=T)
  m3 = ScoreMatrixBin(unique(target), windows, bin.num=2)
  checkEquals(s3,m3)
  
  #bam file, rpm=FALSE, unique=TRUE, extend=1
  s4 = ScoreMatrixBin(bam.file, windows, type='bam', bin.num=2, unique=T, extend=1)
  m4 = ScoreMatrixBin(resize(unique(target), width=1), windows, bin.num=2)
  checkEquals(s4,m4)
  
  # bam file with paired-end reads, rpm=FALSE, unique=TRUE, extend=16
  bam.pe.file = system.file('unitTests/test_pairedend.bam', package='genomation')
  s5 = ScoreMatrixBin(bam.pe.file, windows.paired.end, type='bam', bam.paired.end=TRUE, unique=TRUE, extend=16)
  m5 = ScoreMatrixBin(resize(unique(target.paired.end), width=16), windows.paired.end)
  checkEquals(s5,m5)
  

  if (.Platform$OS.type == "windows")
       return()
  

  # bigWig file with is.noCovNA=TRUE and weight.col="score"
  test_bw <- system.file("unitTests/test.bw", package = "genomation")
  g = GRanges(seqnames=c("chr2","chr19", "chr19", "chr19"),
              IRanges(start=c(1201, 1501, 2401, 2800), width=6),
              strand='*')
  s1 = ScoreMatrixBin(test_bw, g, strand.aware=FALSE, 
                   weight.col="score",is.noCovNA=TRUE,
                   type="bigWig", bin.num=2)
  m1 = matrix(c(rep(0, 2), rep(0.25,2), rep(1,2), rep(NA,2)), 
              nrow=4,  byrow = TRUE)
  rownames(m1) = 1:4
  m1 = as(m1, 'ScoreMatrix')
  checkEquals(s1, m1)
  
  # -----------------------------------------------#
  # errors
  # error upon not specifying the file
  checkException(ScoreMatrixBin('',windows), silent=TRUE)
}

# ---------------------------------------------------------------------------- #
test_that_ScoreMatrix_character_GRange_bigWig = function()
{
  if (.Platform$OS.type == "windows")
    return()
  
  test_bw <- system.file("unitTests/test.bw", package = "genomation")
  
  st = seq(200, 300, 20)
  g = GRanges(rep('chr2', length(st)), IRanges(st, width=5))
  s = ScoreMatrixBin(test_bw, g, type='bigWig', bin.num=2)
  
  m = matrix(-1, ncol=2, nrow=6)
  m[6,1] = -0.833333333333
  m[6,2] = -0.75
  rownames(m) = 1:6
  m = as(m, 'ScoreMatrix')
  checkEquals(s, m)
}

# # ---------------------------------------------------------------------------- #
# test for ScoreMatrixBin function
test_ScoreMatrixBin_RleList_GRangesList = function()
{
  # usage
  # target RleList, windows GRangesList
  target = RleList('c1'= Rle(c(1,1,1,2,2,2,3,3,3)),
                   'c2'= Rle(c(10,10,10,20,20,20,30,30,30)),
                   'c3'= Rle(c(1,2,3,4,5,6,7,8,9)))
  
  gr1 = GRanges(rep('c1',each=3), IRanges(c(1, 4, 7),width=3), strand="-")
  gr2 = GRanges(rep('c2',each=3), IRanges(c(1, 4, 7),width=3), strand="+")
  gr3 = GRanges(rep('c3',each=3), IRanges(c(1, 4, 7),width=3), strand="-")
  grl = GRangesList("transcript1" = gr1, "transcript2" = gr2, 'tr3'=gr3)
  
  s6 = ScoreMatrixBin(target, grl, bin.num=3)
  m6 = matrix(c(1,2,3,10,20,30,2,5,8), ncol=3, byrow=T)
  checkEquals(s6, as(m6, 'ScoreMatrix'))
  
  #2. test for different bin.op
  s7 = ScoreMatrixBin(target, grl, bin.num=1, bin.op='min')
  m7 = matrix(c(1,10,1), ncol=1, byrow=T)
  checkEquals(s7, as(m7, 'ScoreMatrix'))
  
  s8 = ScoreMatrixBin(target, grl, bin.num=1, bin.op='max')
  m8 = matrix(c(3,30,9), ncol=1, byrow=T)
  checkEquals(s8, as(m8, 'ScoreMatrix'))
  
  #3. test strand aware
  m9 = matrix(c(3,2,1,10,20,30,8,5,2), ncol=3, byrow=T)
  s9 = ScoreMatrixBin(target, grl, bin.num=3, strand.aware=T)
  checkEquals(s9, as(m9, "ScoreMatrix"))
  
  #4. test bin.num exception
  checkException(ScoreMatrixBin(target, grl, bin.num=10, strand.aware=T),
                 'Please remove transcripts that are shorter than bin.num',
                 silent=TRUE)

}

# # ---------------------------------------------------------------------------- #
# test for ScoreMatrixBin function
test_ScoreMatrixBin_GRanges_GRangesList = function()
{
  target = RleList('c1'= Rle(c(1,1,1,2,2,2,3,3,3)),
                   'c2'= Rle(c(10,10,10,20,20,20,30,30,30)))
  target = as(target, 'GRanges')
  
  gr1 = GRanges(rep('c1',each=3), IRanges(c(1, 4, 7),width=3), strand="-")
  gr2 = GRanges(rep('c2',each=3), IRanges(c(1, 4, 7),width=3), strand="+")
  grl = GRangesList("transcript1" = gr1, "transcript2" = gr2)
  
  s11 = ScoreMatrixBin(target, grl, bin.num=3, weight.col='score')
  m11 = matrix(c(1,2,3,10,20,30), ncol=3, byrow=T)
  checkEquals(s11, as(m11, 'ScoreMatrix'))
}

# # ---------------------------------------------------------------------------- #
# test for ScoreMatrixBin function
test_ScoreMatrixBin_character_GRangesList = function()
{
  require(GenomicAlignments)
  bam.file = system.file('unitTests/test.bam', package='genomation')
  ga = as(readGAlignments(bam.file),'GRanges')
  
  gr1 = GRanges(rep('1',each=3), IRanges(c(1, 4, 7),width=3), strand="-")
  gr2 = GRanges(rep('2',each=3), IRanges(c(1, 4, 7),width=3), strand="+")
  grl = GRangesList("transcript1" = gr1, "transcript2" = gr2)
  
  m12 = ScoreMatrixBin(ga, grl, bin.num=3)
  s12 = ScoreMatrixBin(bam.file, grl, type='bam', bin.num=3)
  
  checkEquals(s12, as(m12, "ScoreMatrix"))
}

# ---------------------------------------------------------------------------- #
# tests for PatternMatrix function

test_PatternMatrix_matrix_DNAStringSet = function()
{
  # one pwm matrix
  patterns <- DNAStringSet(c("AAA", "AAA", "AAA"))
  pwm <- PWM(patterns)
  windows = as(list(DNAString("AAAGCTAAAGGTAAAGCAAAA"),
		    DNAString("AAAGCTAAAGGTAAAGCAAAA"),
                    DNAString("AAAGCTAAAGGTAAAGCAAAA")), "DNAStringSet")
  p1 = patternMatrix(pattern=pwm, windows=windows, min.score=1)
  mat1=as((as.matrix(windows)=="A")*1, "ScoreMatrix")
  checkEquals(p1, mat1)
  
  #more than one pwm matrix
  patterns <- DNAStringSet(c("AAA", "AAA", "AAA"))
  pwmlist <- list(PWM(patterns),PWM(patterns))
  windows = as(list(DNAString("AAAGCTAAAGGTAAAGCAAAA"),
                    DNAString("AAAGCTAAAGGTAAAGCAAAA"),
                    DNAString("AAAGCTAAAGGTAAAGCAAAA")), "DNAStringSet")
  p1.1 = patternMatrix(pattern=pwmlist, windows=windows, min.score=1)
  sm1.1=as((as.matrix(windows)=="A")*1, "ScoreMatrix")
  sml1.1 <- as(list(sm1.1, sm1.1), "ScoreMatrixList") 
  checkEquals(p1.1, sml1.1)

}

test_PatternMatrix_character_DNAStringSet = function()
{
  # one pattern
  pattern <- "AAA"
  windows = as(list(DNAString("AAAGCTAAAGGTAAAGCAAAA"),
		    DNAString("AAAGCTAAAGGTAAAGCAAAA"),
                    DNAString("AAAGCTAAAGGTAAAGCAAAA")), "DNAStringSet")
  p2 = patternMatrix(pattern=pattern, windows=windows)
  mat1=as((as.matrix(windows)=="A")*1, "ScoreMatrix")
  checkEquals(p2, mat1) 

  # when windows have only "N"
  pattern <- "AAA"
  windows = as(list(DNAString("NNNNNNNNNNNNNN"),
  		    DNAString("NNNNNNNNNNNNNN")), "DNAStringSet")
  p3 = patternMatrix(pattern=pattern, windows=windows)
  mat2=as((as.matrix(windows)=="A")*0, "ScoreMatrix")
  checkEquals(p3, mat2) 
  
  
  # more than one pattern
  pattern <- c("AAA","AAA")
  windows = as(list(DNAString("AAAGCTAAAGGTAAAGCAAAA"),
                    DNAString("AAAGCTAAAGGTAAAGCAAAA"),
                    DNAString("AAAGCTAAAGGTAAAGCAAAA")), "DNAStringSet")
  p4 = patternMatrix(pattern=pattern, windows=windows)
  sml1=as(list((as.matrix(windows)=="A")*1, (as.matrix(windows)=="A")*1), 
          "ScoreMatrixList")
  checkEquals(p4, sml1) 
  
}

---
title: "genomation: a toolkit for annotation and visualization of genomic data"
date: "`r Sys.Date()`"
output: 
      html_document:
        toc: true
        toc_float: true
        number_sections: true
        toc_depth: 2

      
vignette: >
  %\VignetteIndexEntry{genomation}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup2, include=FALSE, cache=FALSE, eval=TRUE}
library(knitr)
opts_chunk$set(root.dir=getwd(),fig.align='center',
               fig.path='Figures', 
               dev='png',
               fig.show='hold', 
               cache=FALSE)
library(genomation)
library(genomationData)
library(GenomicRanges)
set.seed=10
```



# Introduction

```{r,echo=FALSE,warnings=FALSE,out.width=500}

knitr::include_graphics("https://raw.githubusercontent.com/BIMSBbioinfo/genomation/master/vignettes/Figures/genomationFlowChart1.png")
```
`genomation` is a  toolkit for annotation 
and in bulk visualization of genomic intervals (scored or unscored)
over predefined regions.
The genomic intervals which the package can handle can 
be anything with a minimal information of chromosome, start and end. 
The intervals could have any length and most of the time they are 
associated with a score. Typical examples of such data sets include aligned  
reads from high-throughput sequencing (HTS) experiments, percent methylation 
values for CpGs (or other cytosines), locations of transcription factor binding, 
and so on. On the other hand, throughout the vignette we use the phrase 
"genomic annotation" to refer to the regions of the genome associated with a 
potential function which does not necessarily have a score 
(examples: CpG islands, genes, enhancers, promoter, exons, etc. ). 
These genomic annotations are usually the regions of interest, and distribution 
of genomic intervals over/around the annotations are can make the way for 
biological interpretation of the data.
The pipeline for computational knowledge extraction consists of three steps: 
data filtering, integration of data from multiple sources or generation of 
predictive models and biological interpretation of produced models, which leads 
to novel hypotheses that can be tested in the wetlab.`genomation` aims
to facilitate the integration of multiple sources of genomic intervals with 
genomic annotation or already published experimental results.

# Access the data


High-throughput data which will be used to show the functonality of the 
`genomation` are located in two places. The annotation and cap analysis
of gene expression (CAGE) data comes prepared with the `genomation` package, while
the raw HTS data can be found in the sister package `genomationData`.
To install the `genomation` and the complementary data package the from
the Bioconductor repository, copy and paste the following lines into your R interpreter:

```{r eval=FALSE}
BiocManager::install('genomationData')
BiocManager::install('genomation')
```


The r Biocpkg("genomationData") vignette contains a verbose description of 
included files. 
To list the available data, type:
```{r listgenomationData, eval=TRUE}
list.files(system.file('extdata',package='genomationData'))
```

To see the descriptions of the files:
```{r genomationDataInfo, eval=TRUE, tidy=TRUE}
sampleInfo = read.table(system.file('extdata/SamplesInfo.txt',
                                    package='genomationData'),header=TRUE, sep='\t')
sampleInfo[1:5,1:5]
```

Basic annotation data and processed experimental data can be found within the 
`genomation` package.
The data can be accesed throught the data command or located 
in the extdata folder. 
```{r genomationInfo, eval=FALSE, tidy=TRUE}
library(genomation)
data(cage)
data(cpgi)

list.files(system.file('extdata', package='genomation'))
```

# Data input

One of larger hindrances in computational genomics stems from the myriad of 
formats that are used to store the data. Although some formats have been 
selected as de facto standards for specific kind of biological data (e.g. BAM, VCF), 
almost all publications come with supplementary tables that do not have the 
same structure, but hold similar information. The tables usually have a tabular 
format, contain the locationof elements in genomic coordinates and various 
metadata colums. `genomation` contais functions to read genomic 
intervals and genomic annotation provided they are in a tabular format. 
These functions will read the data from flat files into GRanges or GRangesList 
objects.

readGeneric is the workhorse of the `genomation` package. It is a function
developed specifically for input of genomic data in tabular formats, and their 
conversion to a GRanges object. 
By default, the function persumes that the file is a standard .bed file 
containing columns chr, start, end.
```{r readGeneric1, eval=TRUE, tidy=TRUE}
library(genomation)
tab.file1=system.file("extdata/tab1.bed", package = "genomation")
readGeneric(tab.file1, header=TRUE)
```

If the file contains meta data columns (as in extended bed format), 
it is possible to read all or some of the additional columns.
To select columns which you want to read in, use the meta.col argument 
```{r readGeneric2, eval=TRUE, tidy=TRUE}
readGeneric(tab.file1, header=TRUE, keep.all.metadata=TRUE)

readGeneric(tab.file1, header=TRUE, meta.col = list(CpGnum=4))
```

If the file contains header, the function will automatically recognize the 
columns using the header names.
```{r readGeneric3, eval=TRUE, tidy=TRUE}
readGeneric(tab.file1, header=TRUE, keep.all.metadata=TRUE)
```

If the files have permutted columns, such that the first three do not represent
chromosome, start and end, you can select an arbitrary set of columns using the
chr, start and end arguments.
```{r readGeneric4, eval=TRUE, tidy=TRUE}
tab.file2=system.file("extdata/tab2.bed", package = "genomation")
readGeneric(tab.file2, chr=3, start=2, end=1)
```

readGeneric function can easily be extended to read almost any kind of 
biological data. As an example we have provided convenience functions to read 
the Encode narrowPeak and broadPeak formats, and gtf formatted files.
```{r readGff1, eval=TRUE, tidy=TRUE}
gff.file=system.file("extdata/chr21.refseq.hg19.gtf", package = "genomation")
gff = gffToGRanges(gff.file)
head(gff)
```

In order to split the last column of the gff file, use the 
split.group=TRUE argument.
```{r readGff2, eval=FALSE, tidy=TRUE}
gff = gffToGRanges(gff.file, split.group=TRUE)
head(gff)
```

There are specific functions to read genomic annotation from flat bed files.
readFeatureFlank is a convenience function used to get the ranges flanking 
the set of interest. As an example, it could be used to get the CpG island shores, 
which have been shown to harbour condition specific differential methylation.
```{r readFeatureFlank, eval=TRUE, tidy=TRUE}
# reading genes stored as a BED files
cpgi.file = system.file("extdata/chr21.CpGi.hg19.bed", package = "genomation")
cpgi.flanks = readFeatureFlank(cpgi.file)
head(cpgi.flanks$flanks)
```


# Extraction and visualization of genomic data 

A standard step in a computational genomics experiment is visualization of 
average enrichment over a certain predefined set of ranges, such as mean coverage
of a certain histone modification around a transcription factor binding site,
or visualization of histone positions around transcription start sites.

## Extraction of data over genomic windows

*ScoreMatrix* and *ScoreMatrixBin* are functions used to extract 
data over predefined windows.

*ScoreMatrix* is used when all of the windows
have the same width, such as a designated area around the transcription start
site, while the *ScoreMatrixBin* is designed for use with windows of 
unequal width (e.g. enrichment of methylation over exons).

Both functions have 2 main arguments: *target* and 
*windows*. *target* is the data that we want to extract, while the 
*windows* represents the regions over which we want to see the enrichment.
The target data can be in 3 forms: a GRanges, a RLeList or a path to an indexed 
.bam file. The *windows* can be GRanges or GRangesList object.

As an example we will extract the density of cage tags around the promoters on 
the human chromosome 21.
```{r ScoreMatrixExample, eval=TRUE, tidy=TRUE}
data(cage)
data(promoters)
sm = ScoreMatrix(target=cage, windows=promoters)
sm
```
  
*ScoreMatrixBin* function has an additional *bin.num* argument which
specifies the number of bins that will represent each window  
(ie. it converts windows of unequal width into ones of equal width.). 
By default, the binning function is set to *mean*.
```{r ScoreMatrixBinExample, eval=FALSE, tidy=TRUE}
data(cage)
gff.file = system.file('extdata/chr21.refseq.hg19.gtf', package="genomation")
exons = gffToGRanges(gff.file, filter='exon')
sm = ScoreMatrixBin(target=cage, windows=exons, bin.num=50)
sm
```
  
Running *ScoreMatrixBin* with *bin.num=50* on 
a set of exons warned us that some of the exons are shorter than 50 bp and
were thus removed from the set before binning. 
The rownames of the resulting *ScoreMatrix* object correspond
to the ranges that were used to construct the windows (e.g. row name 10 means 
that the 10th element in the target GRanges object was used to extract the data). 
If a certain rowname is not present in the ScoreMatrix object, that means that 
the corresponding range was filtered out (e.g. the range could have been on a 
chromosome that was not present in the target).

The *windows* of *ScoreMatrixBin* might be not only GRanges, but also GRangesList object. 
For instance, *ScoreMatrixBin* can be used to calculate transcript coverage of a set of exons. 
In the example below, *ScoreMatrixBin* function calculates coverage of concatenated
exons within each transcript together, bins them into 50 bins and 
calculate average coverage within bins.
```{r ScoreMatrixBinExample_GRangesList, eval=FALSE, tidy=TRUE}
data(cage)
library(GenomicRanges)
gff.file = system.file('extdata/chr21.refseq.hg19.gtf', package="genomation")
exons = gffToGRanges(gff.file, filter="exon")
transcripts = split(exons, exons$transcript_id)
sm = ScoreMatrixBin(target=cage, windows=transcripts, bin.num=10)
sm
```

To simultaneously work on multiple files you can use the *ScoreMatrixList*
function. The function also has 2 obligatory arguments *targets* and 
*windows*. While the *windows* is the same as in *ScoreMatrix*, 
the targets argument contains results from multiple experiments. 
It can be in one of the three formats: a list of RleLists, a list of GRanges 
(or a GRangesList object), or a character vector designating a set of 
.bam or .bigWig files.
```{r ScoreMatrixList, eval=TRUE, tidy=TRUE}
data(promoters)
data(cpgi)
data(cage)

cage$tpm = NULL
targets = list(cage=cage, cpgi=cpgi)
sm = ScoreMatrixList(targets=targets, windows=promoters, bin.num=50)
sm
```
  
If all of the windows have the same width *ScoreMatrixList* will use the
*ScoreMatrix* function. That can be overridden by explicitly specifying the 
*bin.num* argument, as we did in the example.


## Visualization of multiple genomic experiments

There are 2 basic modes of visualization of enrichment over windows: either 
as a heatmap, or as a histogram. *heatMatrix*, *plotMeta* and 
*multiHeatMatrix* are functions for visualization of *ScoreMatrix* 
and *ScoreMatrixList* objects.

We will plot the distribution of CAGE tags around promoters on human chr21.

```{r heatMatrix1, eval=FALSE, tidy=TRUE,  fig.keep='all'}
data(cage)
data(promoters)
sm = ScoreMatrix(target=cage, windows=promoters)

heatMatrix(sm, xcoords=c(-1000, 1000))
plotMeta(sm, xcoords=c(-1000, 1000))
```

```{r , echo=FALSE,out.width="300px",fig.align='default' }
knitr::include_graphics(c("https://raw.githubusercontent.com/BIMSBbioinfo/genomation/master/vignettes/Figures/heatMatrix1-1.png",
                          "https://raw.githubusercontent.com/BIMSBbioinfo/genomation/master/vignettes/Figures/heatMatrix1-2.png"),dpi=30)
```


The *heatMatrix* function can also take a list of numeric vectors 
designating row names, or a factor variable that represent our 
annotation over the windows.
```{r heatMatrix2, eval=FALSE, tidy=TRUE, fig.width=4, fig.height=3}
library(GenomicRanges) 
data(cage)
data(promoters)
data(cpgi)

sm = ScoreMatrix(target=cage, windows=promoters, strand.aware=TRUE)
cpg.ind = which(countOverlaps(promoters, cpgi)>0)
nocpg.ind = which(countOverlaps(promoters, cpgi)==0)
heatMatrix(sm, xcoords=c(-1000, 1000), group=list(CpGi=cpg.ind, noCpGi=nocpg.ind))
```

```{r , echo=FALSE,out.width="400px"}
knitr::include_graphics("https://raw.githubusercontent.com/BIMSBbioinfo/genomation/master/vignettes/Figures/heatMatrix2-1.png")
```

Because the enrichment in windows can have a high dynamic range, it is sometimes
convenient to scale the matrix before plotting.
```{r heatMatrixScales, eval=FALSE, tidy=TRUE, echo=TRUE, fig.width=4, fig.height=3, dpi=300}
sm.scaled = scaleScoreMatrix(sm)
heatMatrix(sm.scaled, xcoords=c(-1000, 1000), group=list(CpGi=cpg.ind, noCpGi=nocpg.ind))
```

```{r , echo=FALSE,out.width="400px"}
knitr::include_graphics("https://raw.githubusercontent.com/BIMSBbioinfo/genomation/master/vignettes/Figures/heatMatrixScales-1.png")
```

Several experiments can be plotted in a side by side fashion using a combination
of ScoreMatrixList and multiHeatMatrix.
```{r multiHeatMatrix1, eval=FALSE, tidy=TRUE, fig.width=4.5, fig.height=3, dpi=300}
cage$tpm = NULL
targets = list(cage=cage, cpgi=cpgi)
sml = ScoreMatrixList(targets=targets, windows=promoters, bin.num=50, strand.aware=TRUE)
multiHeatMatrix(sml, xcoords=c(-1000, 1000))
```


```{r , echo=FALSE,out.width="400px"}
knitr::include_graphics("https://raw.githubusercontent.com/BIMSBbioinfo/genomation/master/vignettes/Figures/multiHeatMatrix1-1.png")
```

We can put the clustering function clustfun to see whether there are any patterns present in the data.
```{r multiHeatMatrix2, eval=FALSE, tidy=TRUE, fig.width=4.5, fig.height=3, dpi=300}
multiHeatMatrix(sml, xcoords=c(-1000, 1000), clustfun=function(x) kmeans(x, centers=2)$cluster)
```


```{r , echo=FALSE,out.width="400px"}
knitr::include_graphics("https://raw.githubusercontent.com/BIMSBbioinfo/genomation/master/vignettes/Figures/multiHeatMatrix2-1.png")
```

More advance usage of the ScoreMatrix family of functions and their
visualtization can be found in the specific use-cases at the end of the vignette.

# Annotation of genomic intervals
Searching for correlation between sets of genomic intervals is a standard exploratory
method in computational genomics. It is usually done by looking at the overlap between 
2 or more sets of ranges and calculating various overlap statistics. 
`genomation` contains two sets of functions for annotation of ranges:
the first one is used to facilitate the general annotation of any sets of ranges,
while the second one is used to annotate a given feature with gene structures (promoter, 
exon, intron).

## Annotation by generic features

Firstly, we will select the broadPeak files from the `genomatonData` package,
and read in the peaks for the Ctcf transcription factor

```{r getFiles_annotation, eval=TRUE, tidy=TRUE}
library(genomationData)
genomationDataPath = system.file('extdata',package='genomationData')
sampleInfo = read.table(file.path(genomationDataPath, 'SamplesInfo.txt'), 
                        header=TRUE, sep='\t', stringsAsFactors=FALSE)

peak.files=list.files(genomationDataPath, full.names=TRUE, pattern='broadPeak')
names(peak.files) = sampleInfo$sampleName[match(basename(peak.files),sampleInfo$fileName)]

ctcf.peaks = readBroadPeak(peak.files['Ctcf'])
```

Now we will annotate the human the Ctcf binding sites using the CpG islands.
Because the CpG islands are restricted to chromosomes 21 and 22, we will set the
*intersect.chr = TRUE*, which will limit the analysis only to the 
chromosomes that are present in both data sets.
```{r readAllPeaks_annotation, eval=TRUE, tidy=TRUE}
data(cpgi)
peak.annot = annotateWithFeature(ctcf.peaks, cpgi, intersect.chr=TRUE)
peak.annot
```
  
The output of the annotateWithFeature function shows three types of information:
The total number of elements in the target dataset, the percentage of target 
dataset that overlaps with the feature dataset. And the percentage of the feature
elements that overlap the target.

## Annotation of genomic intervals by gene structures

To find the distribution of our designated intervals around gene structures, 
we will first read the transcript features from a file using the 
*readTranscriptFeatures* function. *readTranscriptFeatures* reads
a bed12 formatted file and parses the coordinates into a *GRangesList* containing
four elements: exons, introns. promoters and transcription start sites (TSSes).
```{r readTranscriptFeatures, eval=TRUE, tidy=TRUE}
bed.file=system.file("extdata/chr21.refseq.hg19.bed", package = "genomation")
gene.parts = readTranscriptFeatures(bed.file)
```

*annotateWithGeneParts* will give us the  overlap statistics between our 
CTCF peaks and gene structures. We will again use the *intersect.chr=TRUE*
to limit the analysis.
```{r annotateCtcf, eval=TRUE, tidy=TRUE}
ctcf.annot=annotateWithGeneParts(ctcf.peaks, gene.parts, intersect.chr=TRUE)
ctcf.annot
```
  
annotateWithGeneParts can also take a set of feature ranges as an argument.
We will use the readGeneric function to load all of the broadPeak files in the
`genomationData`, which we will then annotate.
```{r annotateTargetList, eval=TRUE, tidy=TRUE}
peaks = GRangesList(lapply(peak.files, readGeneric))
names(peaks) = names(peak.files)
annot.list = annotateWithGeneParts(peaks, gene.parts, intersect.chr=TRUE)
```
  
Gene annotation of multiple feature objects can be visualized in a form of a heatmap, 
where rows represent samples, columns the gene structure, and the value is the 
percentage of overlap given by priority. If *cluster=TRUE*, then the function
will use hierarhical clustering to order the heatmap.
```{r plotGeneAnnotation, eval=FALSE, tidy=TRUE, dpi=300}
plotGeneAnnotation(annot.list)
plotGeneAnnotation(annot.list, cluster=TRUE)
```
 

```{r , echo=FALSE,out.width="300px",fig.align='default'}
knitr::include_graphics(c("https://raw.githubusercontent.com/BIMSBbioinfo/genomation/master/vignettes/Figures/plotGeneAnnotation-1.png",
                     "https://raw.githubusercontent.com/BIMSBbioinfo/genomation/master/vignettes/Figures/plotGeneAnnotation-2.png"     
                          ))
```


# Use cases for `genomation` package

The `genomation` package provides generalizable functions for genomic data analysis
and visualization. Below we will demonstrate the functionality on specific use cases

## Visualization of ChiP sequencing data

We will visualize the binding profiles of 6 transcription factors around the 
Ctcf binding sites. 

In the fist step we will select the *.bam files containing mapped reads.
```{r selectBamChipseq, eval=TRUE, tidy=TRUE}
genomationDataPath = system.file('extdata',package='genomationData')
bam.files = list.files(genomationDataPath, full.names=TRUE, pattern='bam$')
bam.files = bam.files[!grepl('Cage', bam.files)]
```
  
Firstly, we will read in the Ctcf peaks, filter regions from human chromosome 21,
and order them by their signal values. In the end we will resize all ranges to 
have a uniform width of 500 bases, fixed on the center of the peak.
```{r readCtcfPeaks, eval=TRUE, tidy=TRUE}
ctcf.peaks = readBroadPeak(file.path(genomationDataPath, 
                                     'wgEncodeBroadHistoneH1hescCtcfStdPk.broadPeak.gz'))
ctcf.peaks = ctcf.peaks[seqnames(ctcf.peaks) == 'chr21']
ctcf.peaks = ctcf.peaks[order(-ctcf.peaks$signalValue)]
ctcf.peaks = resize(ctcf.peaks, width=1000, fix='center')
```
  
In order to extract the coverage values of all transcription factors around 
chipseq peaks, we will use the *ScoreMatrixList* function. 
*ScoreMatrixList* assign names to each element of the list based on the 
names of the bam files. We will use the names of the files to find the 
corresponding names of each sample in the SamplesInfo.txt.
Using the *heatmapProfile* on our *ScoreMatrixList*, 
we can plot the underlying signal side by side.


```{r ctcfScoreMatrixList, eval=FALSE, tidy=TRUE, fig.width=5, fig.height=3.5, dpi=300}
sml = ScoreMatrixList(bam.files, ctcf.peaks, bin.num=50, type='bam')
sampleInfo = read.table(system.file('extdata/SamplesInfo.txt',
                                    package='genomationData'),header=TRUE, sep='\t')
names(sml) = sampleInfo$sampleName[match(names(sml),sampleInfo$fileName)]
multiHeatMatrix(sml, xcoords=c(-500, 500), col=c('lightgray', 'blue'))
```

```{r , echo=FALSE,out.width="350px",fig.cap="Heatmap profile of unscaled coverage shows a slight colocalization of Ctcf, Rad21 and Znf143."}
knitr::include_graphics(c("https://raw.githubusercontent.com/BIMSBbioinfo/genomation/master/vignettes/Figures/ctcfScoreMatrixList-1.png"     
                          ))
```

Because of the large range of signal values in chipseq peaks, the *heatmapProfile* 
will not show the true extent of colocalization. To get around this, it is advisable
to independently scale the rows of each element in the ScoreMatrixList.

```{r plotScaledProfile, eval=FALSE, tidy=TRUE, fig.width=5, fig.height=3.5, dpi=300}
sml.scaled = scaleScoreMatrixList(sml)
multiHeatMatrix(sml.scaled, xcoords=c(-500, 500), col=c('lightgray', 'blue'))
```



```{r , echo=FALSE,out.width="350px",fig.cap="Heatmap profile of scaled coverage shows much stronger colocalization   of the transcription factors; nevertheless, it is evident that some of the CTCF   peaks have a very weak enrichment"}
knitr::include_graphics(c("https://raw.githubusercontent.com/BIMSBbioinfo/genomation/master/vignettes/Figures/plotScaledProfile-1.png"     
                          ))
```

## Combinatorial binding of transcription factors

In the first step we will read all peak files into a GRanges list. We will use 
the SamplesInfo file from the `genomationData` to get he names of the 
samples. Four of the peak files are in the Encode broadPeak format, while one is
in the narrowPeak. To read the files, we will use the readGeneric 
function. It enables us to select from the files only columns of interest. 
As the last step, we will restrict ourselves to peaks that are located on 
chromosome 21 and have width 100 and 1000 bp
```{r eadAllPeaks, eval=TRUE, tidy=TRUE}
genomationDataPath = system.file('extdata',package='genomationData')
sampleInfo = read.table(file.path(genomationDataPath, 'SamplesInfo.txt'), 
                        header=TRUE, sep='\t', stringsAsFactors=FALSE)


peak.files = list.files(genomationDataPath, full.names=TRUE, pattern='Peak.gz$')
peaks = list()
for(i in 1:length(peak.files)){
  file = peak.files[i]
  name=sampleInfo$sampleName[match(basename(file),sampleInfo$fileName)]
  message(name)
  peaks[[name]] = readGeneric(file, meta.col=list(score=5))
}
peaks = GRangesList(peaks)
peaks = peaks[seqnames(peaks) == 'chr21' & width(peaks) < 1000  & width(peaks) > 100]
```

To find the combination of binding sites we will use the *findFeatureComb*
function. It takes a granges list object, finds the union of the 
ranges and designates each range by the combination of overlaps from the original set.
By default, the returned ranges will have a numeric *class*
meta data column, which designates the correponding combination. 
If you are interested in the names of the TF which make 
the combinations, put the *use.names=TRUE*.
```{r findFeatureComb, eval=TRUE, tidy=TRUE}
  tf.comb = findFeatureComb(peaks, width=1000)
```
  
To visualize the results, we will plot the enricment of resulting regions. 
Before doing that we will order the regions by their *class* argument.
```{r visualizeFeatureComb, eval=FALSE, tidy=TRUE, fig.width=5, fig.height=3, dpi=300}
  tf.comb = tf.comb[order(tf.comb$class)]
  bam.files = list.files(genomationDataPath, full.names=TRUE, pattern='bam$')
  bam.files = bam.files[!grepl('Cage', bam.files)]
  sml = ScoreMatrixList(bam.files, tf.comb, bin.num=20, type='bam')
  names(sml) = sampleInfo$sampleName[match(names(sml),sampleInfo$fileName)]
  sml.scaled = scaleScoreMatrixList(sml)
  multiHeatMatrix(sml.scaled, xcoords=c(-500, 500), col=c('lightgray', 'blue'))
```


```{r , echo=FALSE,out.width="350px"}
knitr::include_graphics(c("https://raw.githubusercontent.com/BIMSBbioinfo/genomation/master/vignettes/Figures/visualizeFeatureComb-1.png"     
                          ))
```

The plot shows perfectly how misleading the peak calling process can be. 
Although the plots show that CTFC, Rad21 and Znf143 have almost perfect 
colozalization, peak callers have trouble identifying peaks in regions with 
lower enrichments and as a  result, we get different statistics 
when using overlaps.

### Using data from AnnotationHub

We can also use data from AnnotationHub since it can return data as GRanges object. Below we download CpG island and DNAse peak locations from AnnotationHub and get a scoreMatrix on the CpG islands.

```{r annotationHubExample1, eval=FALSE, tidy=TRUE}
library(AnnotationHub)
ah = AnnotationHub()
# retrieve CpG island data from annotationHub
cpgi_query <- query(ah, c("CpG Islands", "UCSC", "hg19"))
cpgi <- ah[[names(cpgi_query)]]
dnase_query <- query(ah, c("wgEncodeOpenChromDnase8988tPk.narrowPeak.gz", "UCSC", "hg19"))
dnaseP <- ah[[names(dnase_query)]]
sm = ScoreMatrixBin(target=dnaseP, windows=cpgi, strand.aware=FALSE)
```

# sessionInfo
```{r }
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readAnnotate.R
\name{bed12ToExons}
\alias{bed12ToExons}
\title{bed12ToExons function}
\usage{
bed12ToExons(ref)
}
\arguments{
\item{ref}{data.frame object}
}
\description{
extracts exons from a bed12 file and puts them into GRanges object
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readAnnotate.R
\docType{methods}
\name{plotTargetAnnotation}
\alias{plotTargetAnnotation}
\alias{plotTargetAnnotation,AnnotationByFeature-method}
\title{Plot annotation categories from AnnotationByGeneParts or AnnotationByFeature}
\usage{
plotTargetAnnotation(x, precedence = TRUE,
  col = getColors(length(x@annotation)), cex.legend = 1, ...)

\S4method{plotTargetAnnotation}{AnnotationByFeature}(x, precedence = TRUE,
  col = getColors(length(x@annotation)), cex.legend = 1, ...)
}
\arguments{
\item{x}{a \code{AnnotationByFeature} or
\code{AnnotationByGeneParts} object}

\item{precedence}{TRUE|FALSE. If TRUE there will be a hierachy of annotation 
features when calculating numbers 
(with promoter>exon>intron precedence). 
This option is only valid when x is a 
\code{AnnotationByGeneParts} object}

\item{col}{a vector of colors for piechart or the bar plot}

\item{cex.legend}{a numeric value of length 1 to specify the size of the legend. By default 1.}

\item{...}{graphical parameters to be passed to \code{pie} 
or \code{barplot} functions}
}
\value{
plots a piechart or a barplot for percentage of 
        the target features overlapping with annotation
}
\description{
This function plots a pie or bar chart for showing percentages of targets 
annotated by genic parts or other query features
}
\examples{
data(cage)

bed.file = system.file("extdata/chr21.refseq.hg19.bed", package = "genomation")
gene.parts = readTranscriptFeatures(bed.file)
annot = annotateWithGeneParts(cage, gene.parts, intersect.chr=TRUE)
\donttest{
plotTargetAnnotation(annot)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readAnnotate.R
\name{distance2NearestFeature}
\alias{distance2NearestFeature}
\title{distance2NearestFeature function}
\usage{
distance2NearestFeature(g.idh, tss)
}
\arguments{
\item{g.idh}{target object}

\item{tss}{TSSes}
}
\description{
distance2NearestFeature function
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{listSliceMin}
\alias{listSliceMin}
\title{Function creates a matrix storing data with desirable number of bins for each window}
\usage{
listSliceMin(xlist, n)
}
\arguments{
\item{xlist}{List of vectors storing values of a bin}

\item{n}{intiger - number of bins}
}
\description{
listSliceMin() function calls the binMin() function
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/genomation-classes.R
\docType{class}
\name{AnnotationByGeneParts-class}
\alias{AnnotationByGeneParts-class}
\title{An S4 class that information on overlap of target features with annotation features}
\description{
This object is desgined to hold statistics and information about genomic feature overlaps
}
\section{Slots}{
\describe{
                 \item{\code{members}}{a matrix showing overlap of target features with annotation genomic features}

                 \item{\code{annotation}}{a named vector of percentages}

                 \item{\code{precedence}}{a named vector of percentages}

                 \item{\code{num.annotation}}{vector}
                 \item{\code{num.precedence}}{vector}

                 \item{\code{no.of.OlapFeat}}{vector}

                 \item{\code{perc.of.OlapFeat}}{vector}

                 \item{dist.to.TSS}{a data frame showing distances to TSS and gene/TSS names and strand}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readAnnotate.R
\name{checkBedValidity}
\alias{checkBedValidity}
\title{checkBedValidity function}
\usage{
checkBedValidity(bed.df, type = "none")
}
\arguments{
\item{bed.df}{data.frame object}

\item{type}{type}
}
\description{
checks the validity of the bed data.frame if it is a legitimate bed columns
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readAnnotate.R
\docType{methods}
\name{getMembers}
\alias{getMembers}
\alias{getMembers,AnnotationByFeature-method}
\title{Get the membership slot of AnnotationByFeature}
\usage{
getMembers(x)

\S4method{getMembers}{AnnotationByFeature}(x)
}
\arguments{
\item{x}{a \code{AnnotationByFeature}  object}
}
\value{
matrix showing overlap of target features with annotation features. 
        1 for overlap, 0 for non-overlap
}
\description{
Membership slot defines the overlap of target features with annotation features
 For example, if a target feature overlaps with an exon
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scoreMatrix.R
\name{getColors}
\alias{getColors}
\title{getColors function}
\usage{
getColors(n)
}
\arguments{
\item{n}{n}
}
\description{
gets colors for a factor variable
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readData.R
\docType{methods}
\name{readTranscriptFeatures}
\alias{readTranscriptFeatures}
\alias{readTranscriptFeatures,character-method}
\title{Function for reading exon intron and promoter structure from a given bed file}
\usage{
readTranscriptFeatures(location,remove.unusual=TRUE,
                              up.flank=1000,down.flank=1000,unique.prom=TRUE)

\S4method{readTranscriptFeatures}{character}(location, remove.unusual = TRUE,
  up.flank = 1000, down.flank = 1000, unique.prom = TRUE)
}
\arguments{
\item{location}{location of the bed file with 12 or more columns. 
The file can end in \code{.gz}, \code{.bz2}, \code{.xz}, or \code{.zip}
and/or start with \code{http://} or \code{ftp://}. If the file is not compressed
it can also start with \code{https://} or \code{ftps://}.}

\item{remove.unusual}{remove the chromomesomes with unsual names, mainly random chromsomes etc}

\item{up.flank}{up-stream from TSS to detect promoter boundaries}

\item{down.flank}{down-stream from TSS to detect promoter boundaries}

\item{unique.prom}{get only the unique promoters, promoter boundaries will not have 
a gene name if you set this option to be TRUE}
}
\value{
a \code{\link{GRangesList}} containing locations of exon/intron/promoter/TSS
}
\description{
Function for reading exon intron and promoter structure from a given bed file
}
\note{
one bed track per file is only accepted, the bed files with multiple tracks will cause en error
}
\examples{
my.bed12.file = system.file("extdata/chr21.refseq.hg19.bed", package = "genomation")
my.bed12.file
feats = readTranscriptFeatures(my.bed12.file) 
names(feats)
sapply(feats, head)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readAnnotate.R
\docType{methods}
\name{convertBedDf}
\alias{convertBedDf}
\alias{convertBedDf,data.frame-method}
\title{convert a data frame read-in from a bed file to a GRanges object}
\usage{
convertBedDf(bed)

\S4method{convertBedDf}{data.frame}(bed)
}
\arguments{
\item{bed}{a data.frame where column order and content 
resembles a bed file with 12 columns}
}
\value{
\code{\link{GRanges}} object
}
\description{
convert a data frame read-in from a bed file to a GRanges object
}
\note{
one bed track per file is only accepted, the bed 
      files with multiple tracks will cause en error

bed files are expected not to have header lines
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentData.R
\docType{data}
\name{cage}
\alias{cage}
\title{Example CAGE data set.}
\format{\code{\link{GRanges}} object}
\description{
Location and tag per million values for CAGE TSS clusters on chr21 and chr22
of human genome (hg19 assembly). The clusters are dowloaded from ENCODE project
downloads for NHEK cells.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scoreMatrix.R
\name{galpTo2Ranges}
\alias{galpTo2Ranges}
\title{galpTo2Ranges function}
\usage{
galpTo2Ranges(x)
}
\arguments{
\item{x}{object}
}
\description{
galpTo2Ranges function
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{binMin}
\alias{binMin}
\title{Function that computes a minimum value for each bin}
\usage{
binMin(x, n)
}
\arguments{
\item{x}{NumericVector}

\item{n}{intiger - number of bins}
}
\description{
Function that computes a minimum value for each bin
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getRandomEnrichment.R
\docType{methods}
\name{getRandomEnrichment}
\alias{getRandomEnrichment}
\alias{getRandomEnrichment,GRanges,GRanges-method}
\title{get enrichment based on randomized feature overlap}
\usage{
getRandomEnrichment(target, query, randomizations = 1000, rand.set = NULL,
  ...)

\S4method{getRandomEnrichment}{GRanges,GRanges}(target, query,
  randomizations = 1000, rand.set = NULL, ...)
}
\arguments{
\item{target}{a \code{GRanges} object to be overlapped with \code{query}}

\item{query}{a \code{GRanges} object that will be randomly placed across the genome and overlap of these random regions with \code{target} will be the background distribution of association between \code{target} and \code{query}.}

\item{randomizations}{number of times the features to be shuffled}

\item{rand.set}{instead of randomly placing features in \code{query} one can supply an already shuffled set of \code{query} genomic features.}

\item{...}{other parameters to be passed to \code{randomizeFeature} function. These parameters ccontrol how randomization is done.}
}
\value{
returns a \code{RandomEnrichment} object
}
\description{
This function measures the association between two genomic features by randomizing one feature and counting the overlaps in randomized sets.
That is to say, \code{query} feature will be randomly distributed over the genome (constrained by provided options), and the overlap of \code{target} with these randomized features will be measured.
}
\examples{
data(cage)
data(cpgi)
\donttest{ 
enr = getRandomEnrichment(cage, cpgi, randomizations=50)
}
}
\seealso{
\code{\link{randomizeFeature}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{Sum_c}
\alias{Sum_c}
\title{Function that computes a sum value}
\usage{
Sum_c(x)
}
\arguments{
\item{x}{NumericVector}
}
\description{
Function that computes a sum value
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotMatrix.R
\name{multiHeatMatrix}
\alias{multiHeatMatrix}
\title{Draw multiple heatmaps from a ScoreMatrixList object}
\usage{
multiHeatMatrix(sml, grid = TRUE, col = NULL, xcoords = NULL,
  group = NULL, group.col = NULL, order = FALSE, user.order = FALSE,
  winsorize = c(0, 100), clustfun = FALSE, clust.matrix = NULL,
  column.scale = TRUE, matrix.main = NULL, common.scale = FALSE,
  legend = TRUE, legend.name = NULL, cex.legend = 0.8, xlab = NULL,
  cex.lab = 1, cex.main = 1, cex.axis = 0.8, newpage = TRUE)
}
\arguments{
\item{sml}{a \code{ScoreMatrixList} object}

\item{grid}{if TRUE, grid graphics will be used. if FALSE, base graphics
will be used on the top level, so users can use par(mfrow)
or par(mfcol) prior to calling the function. Default:FALSE}

\item{col}{a color palette or list of color palettes, such as
list(heat.colors(10),topo.colors(10)). If it is a list,
it is length must match the number of matrices to be plotted.
If it is a single palette
every heatmap will have the same colors.}

\item{xcoords}{a vector of numbers showing relative positions of the bases or 
windows or a list of vectors. 
The elements of the list must match the number of columns in the
corresponding \code{ScoreMatrix}. 
Alternatively, the elements could be a numeric vector of two elements. Such
as c(0,100) showing the relative start and end coordinates of the first
and last column of the \code{ScoreMatrix} object. The remaining
coordinates will be automatically matched in this case. If the
argument is not a list but a single vector, then all heatmaps
will have the same coordinate on their x-axis.}

\item{group}{a list of vectors of row numbers or a factor. The rows will be 
reordered to match their grouping. The grouping is
used for rowside colors of the heatmap. If it is a list,
each element of the list must be a vector of row numbers. Names
of the elements of the list will be used as names of groups. 
If \code{group} is a factor
, it's length must match the number of rows of the matrix, and 
factor levels will be used as the names of the groups in the plot.}

\item{group.col}{a vector of color names to be used at the rowside colors if
\code{group} and \code{clustfun} arguments are given}

\item{order}{Logical indicating if the rows should be ordered or not 
(Default:FALSE). If \code{order=TRUE} the matrix will be ordered
with rowSums(mat) values in descending order. 
If \code{group} argument is provided, first the groups
will be ordered in descending order of sums of rows then, everything
within the clusters will be ordered by sums of rows.
If \code{clustfun} is given then rows within clusters
will be order in descending order by sums of rows.}

\item{user.order}{a numerical vector indicating the order of groups/clusters (it works only
when \code{group} or \code{clustfun} argument is given).}

\item{winsorize}{Numeric vector of two, defaults to c(0,100). This vector 
determines the upper and lower percentile values to limit the 
extreme values. For example, c(0,99) will limit the values to
only 99th percentile for a matrix, 
everything above the 99 percentile will 
be equalized to the value of 99th percentile.This is useful 
for visualization of matrices that have outliers.}

\item{clustfun}{a function for clustering
rows of \code{mat} that returns 
a vector of integers indicating the cluster to which 
each point is allocated (a vector of cluster membership),
e.g. k-means algorithm with 3 centers: 
function(x) kmeans(x, centers=3)$cluster. 
By default FALSE.}

\item{clust.matrix}{a numerical vector of indexes or a character vector of names
of the \code{ScoreMatrix} objects in 'sml' 
to be used in clustering (if \code{clustfun} argument is provided).
By default all matrices are clustered. Matrices that are
not indicated in clust.matrix are ordered according to
result of clustering algorithm.}

\item{column.scale}{Logical indicating if matrices should be scaled or not,
prior to clustering or ordering. Setting this
to TRUE scales the columns of the 
matrices using,
\code{scale()} function. scaled columns are only used for
clustering or ordering. Original scores are displayed for heatmaps.}

\item{matrix.main}{a vector of strings for the titles of the heatmaps. If NULL
titles will be obtained from names of the \code{ScoreMatrix}
objects in the \code{ScoreMatrixList} objects.}

\item{common.scale}{if TRUE (Default:FALSE) all the heatmap colors will be 
coming from the same score
scale, although each heatmap color scale can be different.
The color intensities will be coming from the same scale. 
The scale will be
determined by minimum of all matrices and maximum of all 
matrices. This is useful when all matrices are on the same
score scale. If FALSE, the color scale will be determined
by minimum and maximum of each matrix individually.}

\item{legend}{if TRUE and color legend for the heatmap is drawn.}

\item{legend.name}{a vector of legend labels to be plotted with legends
of each heatmap. If it is a length 1 vector, all heatmaps
will have the same legend label.}

\item{cex.legend}{A numerical value giving the amount by which 
legend axis marks should be magnified relative to the default}

\item{xlab}{a vector of character strings for x-axis labels of the heatmaps. 
if it is length 1, all heatmaps will have the same label.}

\item{cex.lab}{A numerical value giving the amount by which 
axis labels (including 'legend.name') 
should be magnified relative to the default.}

\item{cex.main}{A numerical value giving the amount by which 
plot title should be magnified}

\item{cex.axis}{A numerical value giving the amount by which 
axis marks should be magnified relative to the default}

\item{newpage}{logical indicating if \code{grid.newpage()} function should be
invoked if \code{grid=TRUE}.}
}
\value{
invisibly returns the order of rows, if clustfun is provided and/or order=TRUE
}
\description{
The function plots multiple heatmaps for a ScoreMatrixList object side by side.
Each matrix can have different color schemes but it is essential that each matrix
is obtained from same regions or neighbouring regions.
}
\examples{
data(cage)
data(promoters)
scores1=ScoreMatrix(target=cage,windows=promoters,strand.aware=TRUE)

data(cpgi)
scores2=ScoreMatrix(target=cpgi,windows=promoters,strand.aware=TRUE)

sml=new("ScoreMatrixList",list(a=scores1,b=scores2))

# use with k-means
\donttest{multiHeatMatrix(sml,
                 clustfun=function(x) kmeans(x, centers=2)$cluster,
                 cex.axis=0.8,xcoords=c(-1000,1000),
                 winsorize=c(0,99),
                 legend.name=c("tpm","coverage"),xlab="region around TSS")
                 
# use with hierarchical clustering
cl2 <- function(x) cutree(hclust(dist(x), method="complete"), k=2)
multiHeatMatrix(sml,legend.name="tpm",winsorize=c(0,99),xlab="region around TSS",
         xcoords=-1000:1000,clustfun=cl2,
         cex.legend=0.8,cex.lab=1,
         cex.axis=0.9,grid=FALSE)

# use different colors
require(RColorBrewer)
col.cage= brewer.pal(9,"Blues")
col.cpgi= brewer.pal(9,"YlGn")
multiHeatMatrix(sml,
                 clustfun=function(x) kmeans(x, centers=2)$cluster,
                 cex.axis=0.8,xcoords=c(-1000,1000),
                 winsorize=c(0,99),col=list(col.cage,col.cpgi),
                 legend.name=c("tpm","coverage"),xlab="region around TSS")
}



}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/genomation-classes.R
\docType{class}
\name{RandomEnrichment-class}
\alias{RandomEnrichment-class}
\title{An S4 class for storing \code{getRandomEnrichment} function results}
\description{
The resulting object stores the results of \code{getRandomEnrichment} function
}
\section{Slots}{
\describe{
                 \item{\code{orig.cnt}:}{number of features overlapping with query at \code{getRandomEnrichment} }
                 \item{\code{rand.olap.dist}:}{set of number of features overlapping with randomized queries at \code{getRandomEnrichment}}
                 \item{\code{log2fc}:}{ log2 fold change calculated by dividing \code{orig.cnt} by mean(\code{rand.olap.dist}) and taking log2 of that result}
                 \item{\code{p.value}:}{P-value assuming \code{rand.olap.dist} has a normal distribution and comparing \code{orig.cnt} with that distribution }
                 \item{\code{rand.p.value}:}{ p-value from randomization by calculation the proportion of how many times a random number of overlap exceeds the original number of overlap}
                }
}

\seealso{
\code{\link{getRandomEnrichment}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scoreMatrixList.R
\docType{methods}
\name{intersectScoreMatrixList}
\alias{intersectScoreMatrixList}
\alias{intersectScoreMatrixList,ScoreMatrixList-method}
\title{Get common rows from all matrices in a ScoreMatrixList object}
\usage{
intersectScoreMatrixList(sml, reorder = FALSE)

\S4method{intersectScoreMatrixList}{ScoreMatrixList}(sml, reorder = FALSE)
}
\arguments{
\item{sml}{a \code{ScoreMatrixList} object}

\item{reorder}{if TRUE \code{ScoreMatrix} objects in the list are sorted
based on their common row ids.}
}
\value{
\code{ScoreMatrixList} object
}
\description{
Returns a intersection of rows for each matrix in a ScoreMatrixList object. 
This is done using the rownames of each element in the list.
}
\examples{
library(GenomicRanges)
target = GRanges(rep(c(1,2),each=7), 
                  IRanges(rep(c(1,1,2,3,7,8,9), times=2), width=5), 
                  weight = rep(c(1,2),each=7))
                 
windows1 = GRanges(rep(c(1,2),each=2), 
                    IRanges(rep(c(1,2), times=2), width=5), 
                    strand=c('-','+','-','+'))
windows2 = windows1[c(1,3)]
sml = as(list(ScoreMatrix(target, windows1),
               ScoreMatrix(target, windows2)), 'ScoreMatrixList')
sml
intersectScoreMatrixList(sml)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/genomation-classes.R, R/readAnnotate.R,
%   R/scoreMatrix.R, R/scoreMatrixList.R
\docType{methods}
\name{show,RandomEnrichment-method}
\alias{show,RandomEnrichment-method}
\alias{show,AnnotationByGeneParts-method}
\alias{show,AnnotationByFeature-method}
\alias{show,ScoreMatrix-method}
\alias{show,ScoreMatrixList-method}
\title{show method for some of the genomation classes}
\usage{
\S4method{show}{RandomEnrichment}(object)

\S4method{show}{AnnotationByGeneParts}(object)

\S4method{show}{AnnotationByFeature}(object)

\S4method{show}{ScoreMatrix}(object)

\S4method{show}{ScoreMatrixList}(object)
}
\arguments{
\item{object}{object of class RandomEnrichment}
}
\value{
Shows the dimension of the ScoreMatrix

Shows the number of matrices and their sizes
}
\description{
show method for some of the genomation classes
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readData.R
\docType{methods}
\name{readBed}
\alias{readBed}
\title{Read a BED file and convert it to GRanges.}
\usage{
readBed(file, track.line = FALSE, remove.unusual = FALSE,
  zero.based = TRUE)
}
\arguments{
\item{file}{location of the file, a character string such as: "/home/user/my.bed"
or the input itself as a string (containing at least one \\n).
The file can end in \code{.gz}, \code{.bz2}, \code{.xz}, or \code{.zip}
and/or start with \code{http://} or \code{ftp://}. If the file is not 
compressed it can also start with \code{https://} or \code{ftps://}.}

\item{track.line}{the number of track lines to skip, "auto" to detect them 
automatically or FALSE(default) if the bed file doesn't 
have track lines}

\item{remove.unusual}{if TRUE remove the chromosomes with unsual 
names, such as chrX_random (Default:FALSE)}

\item{zero.based}{a boolean which tells whether the ranges in 
the bed file are 0 or 1 base encoded. (Default: TRUE)}
}
\value{
\code{\link{GRanges}} object
}
\description{
The function reads a BED file that contains location and other information
on genomic features and returns a \code{\link{GRanges}} object. 
The minimal information that the BED file has to have is chromosome, 
start and end columns. it can handle all BED formats up to 12 columns.
}
\examples{
my.file=system.file("extdata","chr21.refseq.hg19.bed",package="genomation")
refseq = readBed(my.file,track.line=FALSE,remove.unusual=FALSE)
head(refseq)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/genomation-classes.R
\docType{class}
\name{ScoreMatrix-class}
\alias{ScoreMatrix-class}
\title{An S4 class for storing \code{ScoreMatrix} function results}
\description{
The resulting object is an extension of a \code{matrix} object, and stores values (typically genome-wide scores) for a predefined set of regions
Each row on the ScoreMatrix is a predefined region (Ex: CpG islands, promoters) and columns are values across those regions.
}
\section{Constructors}{

see \code{\link{ScoreMatrix}}
}

\section{Coercion}{

as(from, "matrix"): Creates a matrix from \code{ScoreMatrix} object. You can also use S3Part() function to extract the matrix from \code{ScoreMatrix} object.
}

\section{Subsetting}{

In the code snippets below, x is a ScoreMatrix object.
\code{'x[i,j]'}: Get or set elements from row \code{i} and column \code{j} and return a subset ScoreMatrix object.
}

\seealso{
\code{\link{ScoreMatrix}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readAnnotate.R
\docType{methods}
\name{annotateWithFeature}
\alias{annotateWithFeature}
\alias{annotateWithFeature,GRanges,GRanges-method}
\title{Function to annotate given GRanges object with a given genomic feature}
\usage{
annotateWithFeature(target, feature, strand = FALSE, extend = 0,
  feature.name = NULL, intersect.chr = FALSE)

\S4method{annotateWithFeature}{GRanges,GRanges}(target, feature,
  strand = FALSE, extend = 0, feature.name = NULL,
  intersect.chr = FALSE)
}
\arguments{
\item{target}{a GRanges object storing chromosome locations to be annotated}

\item{feature}{a GRanges object storing chromosome locations of a feature 
(can be CpG islands, ChIP-seq peaks, etc)}

\item{strand}{If set to TRUE, annotation features and target features will 
be overlapped based on strand  (def:FAULT)}

\item{extend}{specifiying a positive value will extend the feature on both 
sides as much as \code{extend}}

\item{feature.name}{name of the annotation feature. 
For example: H3K4me1,CpGisland etc.
by default the name is taken from the given variable}

\item{intersect.chr}{boolean, whether to select only chromosomes that are 
common to feature and target. FALSE by default}
}
\value{
returns an \code{AnnotationByFeature} object
}
\description{
Function to annotate given GRanges object with a given genomic feature
}
\examples{
data(cpgi)
data(promoters)
annot = annotateWithFeature(cpgi, promoters)


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scoreMatrix.R
\name{file.ext}
\alias{file.ext}
\title{file.ext function}
\usage{
file.ext(x)
}
\arguments{
\item{x}{file}
}
\description{
Extract file extension from file path
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readData.R
\docType{methods}
\name{gffToGRanges}
\alias{gffToGRanges}
\title{Converts a gff formated data.frame into a GenomicRanges object. 
The GenomicRanges object needs to be properly formated for the function to work.}
\usage{
gffToGRanges(gff.file, filter = NULL, zero.based = FALSE, ensembl = FALSE)
}
\arguments{
\item{gff.file}{path to a gff formatted file. 
The file can end in \code{.gz}, \code{.bz2}, \code{.xz}, or \code{.zip}
and/or start with \code{http://} or \code{ftp://}. If the file is not compressed
it can also start with \code{https://} or \code{ftps://}.}

\item{filter}{a character designating which elements to retain from the gff file (e.g. exon, CDS, ...)}

\item{zero.based}{\code{boolean} whether the coordinates are 0 or 1 based. 0 is the default}

\item{ensembl}{\code{boolean} if TRUE, add the chr prefix to seqlevels. FALSE by default}
}
\value{
returns a \code{GenomicRanges} object
}
\description{
Converts a gff formated data.frame into a GenomicRanges object. 
The GenomicRanges object needs to be properly formated for the function to work.
}
\examples{
gff.file = system.file('extdata/chr21.refseq.hg19.gtf', package='genomation')
gff = gffToGRanges(gff.file)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/enrichmentMatrix.R
\docType{methods}
\name{enrichmentMatrix,ScoreMatrixList,ScoreMatrix-method}
\alias{enrichmentMatrix,ScoreMatrixList,ScoreMatrix-method}
\title{Compute an enrichment of IP (stored in ScoreMatrixList object) over control (stored in ScoreMatrix object)}
\usage{
\\S4method{enrichmentMatrix}{ScoreMatrixList,ScoreMatrix}(IP, control)
}
\arguments{
\item{IP}{\code{\link{ScoreMatrixList}} object storing IP samples}

\item{control}{\code{\link{ScoreMatrix}} storing control sample}
}
\value{
\code{ScoreMatrixList} object
}
\description{
This is an \code{enrichmentMatrix} function for IP \code{ScoreMatrixList} object and control \code{ScoreMatrix} object, 
that enables to normalize ChIP-seq signals with respect to IgG or input DNA control.
}
\note{
The function computes an enrichment of IP over control as follow:
Suppose both IP and control are ScoreMatrix objects that have same dimensions. 
Then, the enrichment is calculated usign a formula: log2((IP + 1) / (control + 1)).
}
\examples{
#load IP and control BAM files and create ScoreMatrix objects
library('genomationData')
data(promoters)
bam.file_IP_1 <- system.file("extdata", 
"wgEncodeSydhTfbsH1hescZnf143IggrabAlnRep1.chr21.bam", package = "genomationData")
IP_1 <- ScoreMatrix(target = bam.file_IP_1, windows = promoters, type = 'bam')

bam.file_IP_2 <- system.file("extdata", 
"wgEncodeBroadHistoneH1hescSuz12051317AlnRep1.chr21.bam", package = "genomationData")
IP_2 <- ScoreMatrix(target=bam.file_IP_2, windows = promoters, type = 'bam')

bam.file_c <- system.file("extdata", 
"wgEncodeBroadHistoneH1hescCtcfStdAlnRep1.chr21.bam", package = "genomationData")
control <- ScoreMatrix(target = bam.file_c, windows = promoters, type = 'bam')

# create a ScoreMatrixList object storing IP ScoreMatrix objects
sml_IP <- ScoreMatrixList(list(IP1 = IP_1, IP2 = IP_2))
 
# compute an enrichment of IP over control
enrichmentMatrix(sml_IP, control)

}
\seealso{
\code{\link{ScoreMatrixList}}, \code{\link{ScoreMatrix}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/patternMatrix.R
\docType{methods}
\name{patternMatrix}
\alias{patternMatrix}
\alias{patternMatrix,character,DNAStringSet,ANY-method}
\alias{patternMatrix,character,DNAStringSet-method}
\alias{patternMatrix,character,GRanges,BSgenome-method}
\alias{patternMatrix,matrix,DNAStringSet,ANY-method}
\alias{patternMatrix,matrix,DNAStringSet-method}
\alias{patternMatrix,matrix,GRanges,BSgenome-method}
\alias{patternMatrix,list,DNAStringSet,ANY-method}
\alias{patternMatrix,list,DNAStringSet-method}
\alias{patternMatrix,list,GRanges,BSgenome-method}
\title{Get scores that correspond to k-mer or PWM matrix occurrence for bases in each window}
\usage{
patternMatrix(pattern, windows, genome = NULL, min.score = 0.8,
  asPercentage = FALSE, cores = 1)

\\S4method{patternMatrix}{character,DNAStringSet}(pattern, windows,
                                                          asPercentage, cores)

\\S4method{patternMatrix}{character,GRanges,BSgenome}(pattern, windows, genome,
                                                              cores)

\\S4method{patternMatrix}{matrix,DNAStringSet}(pattern, windows,
                                                       min.score, asPercentage, 
                                                       cores)

\\S4method{patternMatrix}{matrix,GRanges,BSgenome}(pattern, windows, genome,
                                                           min.score, asPercentage, 
                                                           cores)

\\S4method{patternMatrix}{list,DNAStringSet}(pattern, windows,
                                                     min.score, asPercentage, 
                                                     cores)

\\S4method{patternMatrix}{list,GRanges,BSgenome}(pattern, windows, genome, 
                                                         min.score, asPercentage, 
                                                         cores)
}
\arguments{
\item{pattern}{matrix (a PWM matrix), list of matrices or a character vector of length 1 or more.
A matrix is a PWM matrix that needs to have one row for each nucleotide 
("A","C","G" and "T" respectively).
IUPAC ambiguity codes can be used and it will match any letter 
in the subject that is associated with the code.}

\item{windows}{\code{\link{GRanges}} object or \code{\link{DNAStringSet}} object 
that have equal width of ranges or sequences.}

\item{genome}{\code{\link{BSgenome}} object}

\item{min.score}{numeric or character indicating minimum score to count a match. 
It can be given as a character string containing
a percentage of the highest possible score or a single number
(by default "80\%" or 0.8). If min.score is set to NULL
then \code{patternMatrix} returns scores themselves (default).}

\item{asPercentage}{boolean telling whether scores represent percentage of the maximal 
motif PWM score (default: TRUE) or raw scores (FALSE).}

\item{cores}{the number of cores to use (default: 1). It is supported only on Unix-like platforms.}
}
\value{
returns a \code{scoreMatrix} object or a \code{scoreMatrixList} object
}
\description{
The function produces a base-pair resolution matrix or matrices of scores that correspond to 
k-mer or PWM matrix occurrence over predefined windows that  have equal width.
It finds either positions of pattern hits above a specified threshold and 
creates score matrix filled with 1 (presence of pattern) and 0 (its absence) or
matrix with scores themselves.
If pattern is a character of length 1 or PWM matrix then the function returns 
a ScoreMatrix object, if character of length more than 1 or list of PWMs 
then ScoreMatrixList.
}
\details{
\code{patternMatrix} is based on functions from the seqPattern package:
getPatternOccurrenceList function to find position of pattern that is a character vector in 
a list of sequences (a DNAStringSet object)
and adapted function motifScanHits to find pattern that is a PWM matrix
in sequences (a DNAStringSet object).

If cores > 1 is provided then for every window occurrence of pattern is counted in paralallel.
}
\examples{
library(Biostrings)

# consensus sequence of the ctcf motif
motif = "CCGCGNGGNGGCAG"
# Creates 10 random DNA sequences
seqs = sapply(1:10, 
       function(x) paste(sample(c("A","T","G","C"), 180, replace=TRUE), collapse=""))
windows = DNAStringSet(seqs)
p = patternMatrix(pattern=motif, windows=windows, min.score=0.8)
p

}
\seealso{
\code{\link{ScoreMatrix}}, \code{\link{ScoreMatrixList}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotMatrix.R
\name{heatMatrix}
\alias{heatMatrix}
\title{Draw a heatmap of a given ScoreMatrix object}
\usage{
heatMatrix(mat, grid = FALSE, col = NULL, xcoords = NULL, group = NULL,
  group.col = NULL, order = FALSE, user.order = FALSE, winsorize = c(0,
  100), clustfun = NULL, main = "", legend.name = NULL, cex.legend = 1,
  xlab = NULL, cex.main = 1, cex.lab = 1, cex.axis = 1,
  newpage = TRUE)
}
\arguments{
\item{mat}{a \code{ScoreMatrix} object}

\item{grid}{if TRUE, grid graphics will be used. if FALSE, base graphics
will be used on the top level, so users can use par(mfrow)
or par(mfcol) prior to calling the function. Default:FALSE}

\item{col}{a vector of colors, such as the ones created by heat.colors(10).
If NULL (which is default), jet color scheme (common in matlab
plots) will be used.}

\item{xcoords}{a vector of numbers showing relative positions of the bases or 
windows. It must match the number of columns in the \code{ScoreMatrix}. 
Alternatively, it could be a numeric vector of two elements. Such
as c(0,100) showing the relative start and end coordinates of the first
and last column of the \code{ScoreMatrix} object.}

\item{group}{a list of vectors of row numbers or a factor. This grouping is
used for rowside colors of the heatmap. If it is a list,
each element of the list must be a vector of row numbers. Names
of the elements of the list will be used as names of groups. 
If \code{group} is a factor
, it's length must match the number of rows of the matrix, and 
factor levels will be used as the names of the groups in the plot.}

\item{group.col}{a vector of color names to be used at the rowside colors if
\code{group} argument is given or \code{clustfun} function is given.}

\item{order}{Logical indicating if the rows should be ordered or not 
(Default:FALSE). If \code{order=TRUE} the matrix will be ordered
with rowSums(mat) values in descending order. 
If \code{group} argument is provided, first the groups
will be ordered in descending order of sums of rows then, everything
within the clusters will be ordered by sums of rows.
If \code{clustfun} is given then rows within clusters
will be order in descending order of sums of rows.}

\item{user.order}{a numerical vector indicating the order of groups/clusters (it works only
when \code{group} or \code{clustfun} argument is given).}

\item{winsorize}{Numeric vector of two, defaults to c(0,100). This vector 
determines the upper and lower percentile values to limit the 
extreme values. For example, c(0,99) will limit the values to
only 99th percentile, everything above the 99 percentile will 
be equalized to the value of 99th percentile.This is useful 
for visualization of matrices that have outliers.}

\item{clustfun}{a function for clustering
rows of \code{mat} that returns 
a vector of integers indicating the cluster to which 
each point is allocated (a vector of cluster membership),
e.g. k-means algorithm with 3 centers: 
function(x) kmeans(x, centers=3)$cluster. 
By default FALSE.}

\item{main}{a character string for the plot title}

\item{legend.name}{a character label plotted next to the legend}

\item{cex.legend}{A numerical value giving the amount by which 
legend axis marks should be magnified relative to the default}

\item{xlab}{label a character string for x-axis of the heatmap}

\item{cex.main}{A numerical value giving the amount by which 
plot title should be magnified}

\item{cex.lab}{A numerical value giving the amount by which 
axis labels (including 'legend.name') 
should be magnified relative to the default.}

\item{cex.axis}{A numerical value giving the amount by which 
axis marks should be magnified relative to the default}

\item{newpage}{logical indicating if \code{grid.newpage()} function should be
invoked if \code{grid=TRUE}.}
}
\value{
returns clustering result invisibly, if clustfun is definied
}
\description{
The function makes a heatmap out of given \code{ScoreMatrix} object. If desired
it can use clustering using given clustering function 
(e.g. k-means) and plot cluster color codes as a sidebar. 
In addition, user can define groups of rows using 'group' argument.
}
\examples{

data(cage)
data(promoters)
scores1=ScoreMatrix(target=cage,windows=promoters,strand.aware=TRUE,
                   weight.col="tpm")

set.seed(1000)
\donttest{
heatMatrix(mat=scores1,legend.name="tpm",winsorize=c(0,99),xlab="region around TSS",
           xcoords=-1000:1000,
           cex.legend=0.8,main="CAGE clusters on promoters",cex.lab=1,
           cex.axis=0.9,grid=FALSE)
           
## examples using clustering functions
## k-means
cl1 <- function(x) kmeans(x, centers=3)$cluster
set.seed(1000)
heatMatrix(mat=scores1,legend.name="tpm",winsorize=c(0,99),xlab="region around TSS",
         xcoords=-1000:1000,clustfun=cl1,
         cex.legend=0.8,main="CAGE clusters on promoters",cex.lab=1,
         cex.axis=0.9,grid=FALSE,
         user.order=c(1,3,2))

## hierarchical clustering
cl2 <- function(x) cutree(hclust(dist(x), method="complete"), k=3)
set.seed(1000)
heatMatrix(mat=scores1,legend.name="tpm",winsorize=c(0,99),xlab="region around TSS",
         xcoords=-1000:1000,clustfun=cl2,
         cex.legend=0.8,main="CAGE clusters on promoters",cex.lab=1,
         cex.axis=0.9,grid=FALSE)
}


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readAnnotate.R
\docType{methods}
\name{getTargetAnnotationStats}
\alias{getTargetAnnotationStats}
\alias{getTargetAnnotationStats,AnnotationByFeature-method}
\title{Get the percentage of target features overlapping with annotation from AnnotationByFeature}
\usage{
getTargetAnnotationStats(x,percentage=TRUE,precedence=TRUE)

\S4method{getTargetAnnotationStats}{AnnotationByFeature}(x, percentage = TRUE,
  precedence = TRUE)
}
\arguments{
\item{x}{a \code{AnnotationByFeature}  object}

\item{percentage}{TRUE|FALSE. If TRUE percentage of target 
features will be returned. If FALSE, 
number of target features will be returned}

\item{precedence}{TRUE|FALSE. If TRUE there will be a hierachy of annotation 
                  features when calculating numbers 
                  (with promoter>exon>intron precedence)
                  
That means if a feature overlaps with a promoter it will be counted as 
promoter overlapping only, or if it is overlapping with a an exon 
but not a promoter, #' it will be counted as exon overlapping only whether or
 not it overlaps with an intron.}
}
\value{
a vector of percentages or counts showing quantity of target features
        overlapping with annotation
}
\description{
This function retrieves percentage/number of target features 
overlapping with annotation
}
\examples{
data(cage)
bed.file=system.file("extdata/chr21.refseq.hg19.bed", package = "genomation")
gene.parts = readTranscriptFeatures(bed.file)
cage.annot=annotateWithGeneParts(cage, gene.parts, intersect.chr=TRUE)
getTargetAnnotationStats(cage.annot)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readAnnotate.R
\name{bed12ToIntrons}
\alias{bed12ToIntrons}
\title{bed12ToIntrons function}
\usage{
bed12ToIntrons(ref)
}
\arguments{
\item{ref}{data.frame object}
}
\description{
extracts introns from a bed12 file and puts them into GRanges object
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scoreMatrix.R
\name{binner}
\alias{binner}
\title{binner function}
\usage{
binner(start, end, nbins)
}
\arguments{
\item{start}{start position}

\item{end}{end position}

\item{nbins}{number of bins}
}
\description{
given a vector and length smooths the vector to a given size
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{Max_c}
\alias{Max_c}
\title{Function that computes a max value}
\usage{
Max_c(x)
}
\arguments{
\item{x}{NumericVector}
}
\description{
Function that computes a max value
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scoreMatrixBin.R
\docType{methods}
\name{ScoreMatrixBin}
\alias{ScoreMatrixBin}
\alias{ScoreMatrixBin,RleList,GRanges-method}
\alias{ScoreMatrixBin,GRanges,GRanges-method}
\alias{ScoreMatrixBin,character,GRanges-method}
\alias{ScoreMatrixBin,RleList,GRangesList-method}
\alias{ScoreMatrixBin,GRanges,GRangesList-method}
\alias{ScoreMatrixBin,character,GRangesList-method}
\title{Get bin score for bins on each window}
\usage{
ScoreMatrixBin(target, windows, bin.num = 10, bin.op = "mean",
  strand.aware = FALSE, weight.col = NULL, is.noCovNA = FALSE,
  type = "auto", rpm = FALSE, unique = FALSE, extend = 0,
  param = NULL, bam.paired.end = FALSE, library.size = NULL)

\\S4method{ScoreMatrixBin}{RleList,GRanges}(target, windows, bin.num, bin.op,
                                                   strand.aware)

\\S4method{ScoreMatrixBin}{GRanges,GRanges}(target,windows,
                                                   bin.num,bin.op,
                                                   strand.aware,weight.col,
                                                   is.noCovNA)

\\S4method{ScoreMatrixBin}{character,GRanges}(target, windows, bin.num=10,
                                                     bin.op='mean',strand.aware, 
                                                     is.noCovNA=FALSE, type='auto',
                                                     rpm, unique, extend, param,
                                                     bam.paired.end=FALSE, 
                                                     library.size=NULL)

\\S4method{ScoreMatrixBin}{RleList,GRangesList}(target,windows,
                                                        bin.num, bin.op,
                                                        strand.aware)

\\S4method{ScoreMatrixBin}{GRanges,GRangesList}(target,windows,
                                                   bin.num,bin.op,
                                                   strand.aware,weight.col,
                                                   is.noCovNA)

\\S4method{ScoreMatrixBin}{character,GRangesList}(target, windows, bin.num=10,
                                                     bin.op='mean',strand.aware, 
                                                     weight.col=NULL,
                                                     is.noCovNA=FALSE, type='auto',
                                                     rpm, unique, extend, param,
                                                     bam.paired.end=FALSE, 
                                                     library.size=NULL)
}
\arguments{
\item{target}{\code{RleList}, \code{GRanges}, a BAM file or a bigWig file 
object to be overlapped with ranges in \code{windows}}

\item{windows}{\code{GRanges} or \code{GRangesList} object that contains
the windows of interest. It could be promoters, CpG islands, 
exons, introns as GRanges object or GrangesList object representing
exons of each transcript. Exons must be ordered by ascending rank
by their position in transcript. The sizes of windows 
does NOT have to be equal.}

\item{bin.num}{single \code{integer} value denoting how many bins there 
should be for each window}

\item{bin.op}{bin operation that is either one of the following strings: 
"max","min","mean","median","sum". The operation is applied on the 
values in the bin. Defaults to "mean"}

\item{strand.aware}{If TRUE (default: FALSE), the strands of the windows will 
be taken into account in the resulting \code{scoreMatrix}. 
If the strand of a window is -, the values of the bins 
for that window will be reversed}

\item{weight.col}{if the object is \code{GRanges} object a numeric column
in meta data part can be used as weights. This is particularly
useful when genomic regions have scores other than their
coverage values, such as percent methylation, conservation
scores, GC content, etc.}

\item{is.noCovNA}{(Default:FALSE)
if TRUE,and if 'target' is a GRanges object with 'weight.col'
 provided, the bases that are uncovered will be preserved as
 NA in the returned object. This useful for situations where
 you can not have coverage all over the genome, such as CpG
 methylation values.}

\item{type}{(Default:"auto")
if target is a character vector of file paths, then type designates 
the type of the corresponding files (bam or bigWig)}

\item{rpm}{boolean telling whether to normalize the coverage to per milion reads. 
FALSE by default. See \code{library.size}.}

\item{unique}{boolean which tells the function to remove duplicated reads 
based on chr, start, end and strand}

\item{extend}{numeric which tells the function to extend the reads to width=extend}

\item{param}{ScanBamParam object}

\item{bam.paired.end}{boolean indicating whether given BAM file contains paired-end 
reads (default:FALSE). Paired-reads will be treated as 
fragments.}

\item{library.size}{numeric indicating total number of mapped reads in a BAM file
(\code{rpm} has to be set to TRUE).
If is not given (default: NULL) then library size 
is calculated using the Rsamtools idxstatsBam function:
sum(idxstatsBam(target)$mapped).}
}
\value{
returns a \code{scoreMatrix} object
}
\description{
The function first bins each window to equal number of bins, and calculates
the a summary matrix for scores of each bin (currently, mean, max and min supported)
A scoreMatrix object can be used to draw average profiles or heatmap of read coverage
or wig track-like data. \code{windows} can be a predefined region such as CpG islands,
gene bodies, transcripts or CDS (coding sequences) that are not necessarily equi-width. 
Each window will be chopped to equal number of bins based on \code{bin.num} option.
}
\examples{
data(cage)
data(cpgi)
data(promoters)
myMat=ScoreMatrixBin(target=cage,
                      windows=cpgi,bin.num=10,bin.op="mean",weight.col="tpm")
\donttest{
plot(colMeans(myMat,na.rm=TRUE),type="l")
}
  
myMat2=ScoreMatrixBin(target=cage,
                       windows=promoters,bin.num=10,bin.op="mean",
                       weight.col="tpm",strand.aware=TRUE)
\donttest{
plot(colMeans(myMat2,na.rm=TRUE),type="l")
}

# Compute transcript coverage of a set of exons.
library(GenomicRanges)
bed.file = system.file("extdata/chr21.refseq.hg19.bed", 
                       package = "genomation")
gene.parts = readTranscriptFeatures(bed.file)
transcripts = split(gene.parts$exons, gene.parts$exons$name)
transcripts = transcripts[]
myMat3 = ScoreMatrixBin(target=cage, windows=transcripts[1:250], 
                    bin.num=10)
myMat3                     
}
\seealso{
\code{\link{ScoreMatrix}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scoreMatrixList.R
\docType{methods}
\name{orderBy}
\alias{orderBy}
\alias{orderBy,ScoreMatrixList-method}
\title{Reorder all elements of a ScoreMatrixList to a given ordering vector}
\usage{
orderBy(sml, ord.vec)

\S4method{orderBy}{ScoreMatrixList}(sml, ord.vec)
}
\arguments{
\item{sml}{\code{ScoreMatrixList} object}

\item{ord.vec}{an integer vector}
}
\value{
\code{ScoreMatrixList} object
}
\description{
Reorder all elements of a ScoreMatrixList to a given ordering vector
}
\examples{
library(GenomicRanges)
data(cage)
data(cpgi)
data(promoters)

cage$tpm = NULL
targets = GRangesList(cage=cage, cpgi=cpgi)
sml = ScoreMatrixList(targets, promoters, bin.num=10)
kmeans.clust = kmeans(sml$cage,3)
 
sml.ordered = orderBy(sml, kmeans.clust$cluster)
\donttest{
multiHeatMatrix(sml.ordered)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readAnnotate.R
\docType{methods}
\name{annotateWithFeatures}
\alias{annotateWithFeatures}
\alias{annotateWithFeatures,GRanges,GRangesList-method}
\alias{annotateWithFeatures,GRangesList,GRangesList-method}
\title{Annotate given ranges with genomic features}
\usage{
annotateWithFeatures(target, features, strand.aware = FALSE,
  intersect.chr = FALSE)

\S4method{annotateWithFeatures}{GRanges,GRangesList}(target, features,
  strand.aware = FALSE, intersect.chr = FALSE)

\S4method{annotateWithFeatures}{GRangesList,GRangesList}(target, features,
  strand.aware = FALSE, intersect.chr = FALSE)
}
\arguments{
\item{target}{GRanges or GRangesList object storing chromosome locations to be 
annotated (e.g. chipseq peaks)}

\item{features}{a named GRangesList object containing GRanges objects different set
of features. The function calculates percent overlaps with and without
precendence at the same time. The order of objects in GRangesList
defines their precedence. If a range in \code{target} overlaps with
a more precedent range in an element of \code{features}, the other 
overlaps from other less precedent elments will be discarded. This is
useful for getting piecharts where percentages should add up to 100.}

\item{strand.aware}{if set to TRUE, annotation features and target features will 
be overlapped based on strand (def:FALSE)}

\item{intersect.chr}{logical value, whether to select only chromosomes 
that are common to feature and target. FALSE by default}
}
\value{
returns an \code{AnnotationByFeature} object or if target is 
        a GRangesList, a list of \code{AnnotationByFeature} objects.
}
\description{
The function annotates a target GRangesList or GRanges object as overlapping
or not with 
the elements of a named GRangesList. This is useful to annotate your regions
of interest with genomic features with arbitrary categories such as repeat
classes or families, or output from genome segmentation alogorithms such
as chromHMM.
}
\examples{
library(GenomicRanges)
data(cage)
data(cpgi)
cage$tpm=NULL
gl = GRangesList(cage=cage, cpgi=cpgi)

bed.file = system.file("extdata/chr21.refseq.hg19.bed", package = "genomation")
gene.parts = readTranscriptFeatures(bed.file)
annot = annotateWithFeatures(gl, gene.parts, intersect.chr=TRUE)

}
\seealso{
see  \code{\link{getMembers}}, 
           \code{\link{heatTargetAnnotation}}, 
           \code{\link{plotTargetAnnotation}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/genomation-classes.R
\docType{class}
\name{ScoreMatrixList-class}
\alias{ScoreMatrixList-class}
\title{An S4 class for storing a set of \code{ScoreMatrixList}}
\description{
The resulting object is an extension of a \code{list} object, where each element corresponds to a score matrix object
}
\section{Constructors}{

see \code{\link{ScoreMatrixList}}
}

\section{Coercion}{

as(from, "ScoreMatrixList"): Creates a \code{ScoreMatrixList} object from a list containing \code{\link{ScoreMatrix}} or \code{\link{ScoreMatrixBin}} objects.
}

\section{Subsetting}{

In the code snippets below, x is a ScoreMatrixList object.

\code{x[[i]]},\code{x[[i]]}: Get or set elements \code{i}, where \code{i} is a numeric or character vector of length 1.

\code{x$name}, \code{x$name}: value: Get or set element \code{name}, where \code{name} is a name or character vector of length 1.
}

\seealso{
\code{\link{ScoreMatrixList}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readData.R
\docType{methods}
\name{readFeatureFlank}
\alias{readFeatureFlank}
\alias{readFeatureFlank,character-method}
\title{A function to read-in genomic features and their upstream and downstream adjecent regions
such as CpG islands and their shores}
\usage{
readFeatureFlank(location,remove.unusual=TRUE,flank=2000,
                        clean=TRUE,feature.flank.name=NULL)

\S4method{readFeatureFlank}{character}(location, remove.unusual = TRUE,
  flank = 2000, clean = TRUE, feature.flank.name = NULL)
}
\arguments{
\item{location}{for the bed file of the feature.}

\item{remove.unusual}{remove chromsomes with unsual names random, Un and antyhing with "_" character}

\item{flank}{number of basepairs for the flanking regions}

\item{clean}{If set to TRUE, flanks overlapping with other main features will be trimmed}

\item{feature.flank.name}{the names for feature and flank ranges, it should be a character 
vector of length 2. example: c("CpGi","shores")}
}
\value{
a GRangesList object containing one GRanges object for flanks and one for GRanges object
        for the main feature.
  NOTE: This can not return a CompressedGRangesList at the moment because flanking regions do not
  have to have the same column name as the feature. CompressedGRangesList elements should resemble
  each other in the column content. We can not satisfy that criteria for the flanks
}
\description{
A function to read-in genomic features and their upstream and downstream adjecent regions
such as CpG islands and their shores
}
\examples{
cgi.path = system.file('extdata/chr21.CpGi.hg19.bed', package='genomation')
cgi.shores = readFeatureFlank(cgi.path)
cgi.shores
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/randomizeFeature.R
\docType{methods}
\name{randomizeFeature}
\alias{randomizeFeature}
\alias{randomizeFeature,GRanges-method}
\title{function that randomizes the genomic coordinates}
\usage{
randomizeFeature(feature, chrom.sizes = NULL, stranded = TRUE,
  keep.strand.prop = TRUE, keep.chrom = TRUE, exclude = NULL,
  include = NULL, seed = NULL, nrand = 1)

\S4method{randomizeFeature}{GRanges}(feature, chrom.sizes = NULL,
  stranded = TRUE, keep.strand.prop = TRUE, keep.chrom = TRUE,
  exclude = NULL, include = NULL, seed = NULL, nrand = 1)
}
\arguments{
\item{feature}{a GRanges object to be randomized}

\item{chrom.sizes}{sizes of chromosomes as a named vector (names are
chromsomes names and elements of the vectors are lengths). , if not
given sizes in GRanges object will be used if no sizes there the end
of each chr will be the end last feature on each chr}

\item{stranded}{if FALSE, all of the returned features will be strandless
(will have "*" in the strand slot)}

\item{keep.strand.prop}{If TRUE strands will have the same proportion as the
features}

\item{keep.chrom}{If TRUE, number of features and randomized features for a
chromosome will match. Currently seeting this to FALSE is not supported.}

\item{exclude}{A GRanges object where no randomized feature should overlap,
can be gaps or unmappable regions in the genome as an example.}

\item{include}{A GRanges object which defines the boundaries of randomized
features. If not provided the whole genome is used, as defined using 
the chrom.sizes parameter.}

\item{seed}{random number generator seed}

\item{nrand}{number of randomizations (default:1)}
}
\value{
returns a GRanges object which is randomized version of the feature, 
along with a "set" column in the metadata which designates to which iteration 
of the randomization the range belong.
}
\description{
This function randomly distributes the coordinates of genomic features which
is stored in a \code{GRanges} object. The randomization can be constrained by
 supplied arguments.
The function is still in Beta mode - the regions can overlap excluded regions, and the randomized regions are not disjoint. Please take care that the excluded and included regions are not too strict when compared to the total width of the ranges.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scoreMatrix.R, R/scoreMatrixList.R
\docType{methods}
\name{binMatrix}
\alias{binMatrix}
\alias{binMatrix,ScoreMatrix-method}
\alias{binMatrix,ScoreMatrixList-method}
\title{Bins the columns of a matrix using a user provided function}
\usage{
binMatrix(x, bin.num = NULL, fun = "mean")

\S4method{binMatrix}{ScoreMatrix}(x, bin.num = NULL, fun = "mean")

\S4method{binMatrix}{ScoreMatrixList}(x, bin.num = NULL, fun = "mean")
}
\arguments{
\item{x}{\code{ScoreMatrix} or a \code{ScoreMatrixList} object}

\item{bin.num}{\code{integer} number of bins in the final matrix}

\item{fun}{\code{character} vector or an anonymous function that will be used for binning}
}
\value{
\code{ScoreMatrix} or \code{ScoreMatrixList} object
}
\description{
Bins the columns of a matrix using a user provided function
}
\examples{

# binning the columns in a ScoreMatrix object
library(GenomicRanges)
target = GRanges(rep(c(1,2),each=7), IRanges(rep(c(1,1,2,3,7,8,9), times=2), width=5), 
weight = rep(c(1,2),each=7), 
strand=c('-', '-', '-', '-', '+', '-', '+', '-', '-', '-', '-', '-', '-', '+'))
windows = GRanges(rep(c(1,2),each=2), IRanges(rep(c(1,2), times=2), width=5), 
strand=c('-','+','-','+'))
sm = ScoreMatrix(target, windows)
bin = binMatrix(sm, bin.num=2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scoreMatrixList.R
\docType{methods}
\name{scaleScoreMatrixList}
\alias{scaleScoreMatrixList}
\alias{scaleScoreMatrixList,ScoreMatrixList-method}
\title{Scale the ScoreMatrixList}
\usage{
scaleScoreMatrixList(sml, columns, rows, scalefun)

\S4method{scaleScoreMatrixList}{ScoreMatrixList}(sml, columns = FALSE,
  rows = TRUE, scalefun = NULL)
}
\arguments{
\item{sml}{a \code{ScoreMatrixList} object}

\item{columns}{a \code{columns} whether to scale the matrix by columns. 
Set by default to FALSE}

\item{rows}{a \code{rows} Whether to scale the matrix by rows. Set by default 
to TRUE}

\item{scalefun}{a function object that takes as input a matrix and returns a 
      matrix.
By default  the argument is set to the R scale function with center=TRUE and 
scale=TRUE}
}
\value{
\code{ScoreMatrixList} object
}
\description{
Scales each ScoreMatrix in the ScoreMatrixList object, by rows and/or columns
}
\examples{
library(GenomicRanges)
data(cage)
data(cpgi)
data(promoters)
 
cage$tpm = NULL
targets = GRangesList(cage=cage, cpgi=cpgi)
sml = ScoreMatrixList(targets, promoters, bin.num=10, strand.aware=TRUE)
sml.scaled = scaleScoreMatrixList(sml, rows=TRUE)
sml.scaled
\donttest{
multiHeatMatrix(sml) 
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readData.R
\name{read.zip}
\alias{read.zip}
\title{read.zip function}
\usage{
read.zip(file, ...)
}
\arguments{
\item{file}{zip file}

\item{...}{additional objects and parameters}
}
\description{
read.zip function
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Ops.R
\docType{methods}
\name{Ops,ScoreMatrixList,ScoreMatrixList-method}
\alias{Ops,ScoreMatrixList,ScoreMatrixList-method}
\title{Ops method for a ScoreMatrixList object. It enables to use arithmetic, indicator and logic operations on ScoreMatrixList objects.}
\usage{
\S4method{Ops}{ScoreMatrixList,ScoreMatrixList}(e1, e2)
}
\arguments{
\item{e1}{the \code{\link{ScoreMatrixList}} object}

\item{e2}{the \code{\link{ScoreMatrixList}} object}
}
\value{
\code{ScoreMatrixList}
}
\description{
Arithmetic methods for ScoreMatrixList
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotMatrix.R
\docType{methods}
\name{plotMeta}
\alias{plotMeta}
\title{Line plot(s) for meta-region profiles}
\usage{
plotMeta(mat, centralTend = "mean", overlay = TRUE, winsorize = c(0, 100),
  profile.names = NULL, xcoords = NULL, meta.rescale = FALSE,
  smoothfun = NULL, line.col = NULL, dispersion = NULL,
  dispersion.col = NULL, ylim = NULL, ylab = "average score",
  xlab = "bases", ...)
}
\arguments{
\item{mat}{\code{ScoreMatrix} or \code{ScoreMatrixList} object. If it is a 
\code{ScoreMatrixList} object, all matrices in the ScoreMatrixList should have 
the same number of 
columns.}

\item{centralTend}{a character that determines central tendency of meta-profile(s). 
It takes "mean" (default) or "median".}

\item{overlay}{If TRUE multiple profiles will be overlayed in the same plot
(Default:TRUE). If FALSE, and mat is a ScoreMatrixList, consider
using par(mfrow=c(1,length(mat))) to see the plots from all
matrices at once.}

\item{winsorize}{Numeric vector of two, defaults to c(0,100). This vector 
determines the upper and lower percentile values to limit the 
extreme values. For example, c(0,99) will limit the values to
only 99th percentile, everything above the 99 percentile will 
be equalized to the value of 99th percentile.This is useful 
for visualization of matrices that have outliers.}

\item{profile.names}{a character vector for names of the profiles. The order
should be same as the as the order of ScoreMatrixList.}

\item{xcoords}{a numeric vector which designates 
relative base positions of the meta-region profiles.
For example, for a 2001 column ScoreMatrix, xcoord=-1000:1000 indicates
relative positions of each column in the score matrix. If NULL (Default),
xcoords equals to 1:ncol(mat)}

\item{meta.rescale}{if TRUE meta-region profiles are scaled to 0 to 1 range by
subtracting the min from profiles and dividing them by max-min.
If dispersion is not NULL, then dispersion will be scaled as well.}

\item{smoothfun}{a function to smooth central tendency and dispersion bands (Default: NULL), e.g. 
stats::lowess.}

\item{line.col}{color of lines for \code{centralTend} of meta-region profiles. Defaults to colors from
\code{rainbow()} function.}

\item{dispersion}{shows dispersion interval bands around \code{centralTend} (default:NULL). It takes 
       one of the character:
\itemize{
 \item{"se"}{shows standard error of the mean and 95 percent confidence interval for the mean}
 \item{"sd"}{shows standard deviation and 2*(standard deviation)}
 \item{"IQR"}{shows 1st and 3rd quartile and 
              confidence interval around the median based on the median +/- 1.57 * IQR/sqrt(n) (notches)}
}}

\item{dispersion.col}{color of bands of \code{dispersion}.
Defaults to colors from \code{rainbow()} and transparency is set to 0.5
(rainbow(length(mat), alpha = 0.5)).}

\item{ylim}{same as \code{ylim} at \code{\link{plot}} function. 
if NULL ylim is estimated from all meta-region profiles.}

\item{ylab}{same as \code{ylab} at \code{\link{plot}} function. 
Default: "average score"}

\item{xlab}{same as \code{xlab} at \code{\link{plot}} function. 
Default: "bases"}

\item{...}{other options to \code{\link{plot}}}
}
\value{
returns the meta-region profiles invisibly as a matrix.
}
\description{
Function calculates meta-profile(s) from a ScoreMatrix or a ScoreMatrixList, then
produces a line plot or a set of line plots for meta-region profiles
}
\note{
Score matrices are plotted according to ScoreMatrixList order. 
If ScoreMatrixList contains more than one matrix then they will
overlap each other on a plot, i.e.
the first one is plotted first and every next one overlays previous one(s) and 
the last one is the topmost.

Missing values in data slow down plotting dispersion around central tendency.
The reason is that dispersion is plotted only for non-missing values,
for each segment that
contains numerical values \code{graphics::polygon} function is used to plot dispersion bands.
There might be a situation, when in a column of ScoreMatrix is only one
numeric number and the rest are NAs, then at corresponding position 
only central tendency will be plotted.

Notches show the 95 percent confidence interval for the median 
according to an approximation based on the normal distribution.
They are used to compare groups - if notches corresponding to adjacent base pairs
on the plot do not overlap, this is strong evidence that medians differ.
Small sample sizes (5-10) can cause notches to extend beyond the interquartile range (IQR) 
(Martin Krzywinski \emph{et al}. \emph{Nature Methods 11}, 119-120 (2014))
}
\examples{
 data(cage)
 data(promoters)
 scores1=ScoreMatrix(target=cage,windows=promoters,strand.aware=TRUE)

 data(cpgi)
 scores2=ScoreMatrix(target=cpgi,windows=promoters,strand.aware=TRUE)

 # create a new ScoreMatrixList
 x=new("ScoreMatrixList",list(scores1,scores2))
 \donttest{
 plotMeta(mat=x,overlay=TRUE,main="my plotowski")

 # plot dispersion nd smooth central tendency and variation interval bands
 plotMeta(mat=x, centralTend="mean", dispersion="se", winsorize=c(0,99), 
         main="Dispersion as interquartile band", lwd=4, 
         smoothfun=function(x) stats::lowess(x, f = 1/5))
 }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scoreMatrix.R
\name{readBigWig}
\alias{readBigWig}
\title{readBigWig function}
\usage{
readBigWig(target, windows = NULL, ...)
}
\arguments{
\item{target}{target object}

\item{windows}{windows}

\item{...}{additional parameters}
}
\description{
given a big wig path reads the big wig file into a RleList
to be used by ScoreMatrix:char,GRanges
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scoreMatrix.R
\docType{methods}
\name{scaleScoreMatrix}
\alias{scaleScoreMatrix}
\alias{scaleScoreMatrix,ScoreMatrix-method}
\title{Scales the values in the matrix by rows and/or columns}
\usage{
scaleScoreMatrix(mat, columns = FALSE, rows = TRUE, scalefun = NULL)

\S4method{scaleScoreMatrix}{ScoreMatrix}(mat, columns = FALSE, rows = TRUE,
  scalefun = NULL)
}
\arguments{
\item{mat}{\code{ScoreMatrix} object}

\item{columns}{\code{columns} whether to scale the matrix by columns. Set by default to FALSE.}

\item{rows}{\code{rows} Whether to scale the matrix by rows. Set by default to TRUE}

\item{scalefun}{function object that takes as input a matrix and returns a matrix. 
By default  the argument is set to (x - mean(x))/(max(x)-min(x)+1)}
}
\value{
\code{ScoreMatrix} object
}
\description{
Scales the values in the matrix by rows and/or columns
}
\examples{

# scale the rows of a scoreMatrix object
library(GenomicRanges)
target = GRanges(rep(c(1,2),each=7), IRanges(rep(c(1,1,2,3,7,8,9), times=2), width=5), 
         weight = rep(c(1,2),each=7), 
         strand=c('-', '-', '-', '-', '+', '-', '+', '-', '-', '-', '-', '-', '-', '+'))
windows = GRanges(rep(c(1,2),each=2), IRanges(rep(c(1,2), times=2), width=5), 
           strand=c('-','+','-','+'))
sm = ScoreMatrix(target, windows)
ssm = scaleScoreMatrix(sm, rows=TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readAnnotate.R
\docType{methods}
\name{getFeatsWithTargetsStats}
\alias{getFeatsWithTargetsStats}
\alias{getFeatsWithTargetsStats,AnnotationByFeature-method}
\title{Get the percentage/count of annotation features overlapping with target 
features from AnnotationByFeature}
\usage{
getFeatsWithTargetsStats(x,percentage=TRUE)

\S4method{getFeatsWithTargetsStats}{AnnotationByFeature}(x, percentage = TRUE)
}
\arguments{
\item{x}{a \code{AnnotationByFeature}  object}

\item{percentage}{TRUE|FALSE. If TRUE percentage of annotation features will 
be returned. If FALSE, number of annotation features will be returned}
}
\value{
RETURNS  a vector of percentages or counts showing quantity 
        of annotation features overlapping with target features
}
\description{
This function retrieves percentage/number of 
annotation features overlapping with targets. 
For example, if \code{AnnotationByFeature}  object is containing 
statistics of differentially methylated 
regions overlapping with gene annotation. This function will return 
number/percentage of introns,exons and promoters
overlapping with differentially methylated regions.
}
\examples{
data(cage)
bed.file=system.file("extdata/chr21.refseq.hg19.bed", package = "genomation")
gene.parts = readTranscriptFeatures(bed.file)
cage.annot = annotateWithGeneParts(cage, gene.parts, intersect.chr=TRUE)
getFeatsWithTargetsStats(cage.annot)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readAnnotate.R
\docType{methods}
\name{annotateWithGeneParts}
\alias{annotateWithGeneParts}
\alias{annotateWithGeneParts,GRanges,GRangesList-method}
\alias{annotateWithGeneParts,GRangesList,GRangesList-method}
\title{Annotate given  object with promoter, exon, intron and intergenic regions}
\usage{
annotateWithGeneParts(target, feature, strand = FALSE,
  intersect.chr = FALSE)

\S4method{annotateWithGeneParts}{GRanges,GRangesList}(target, feature,
  strand = FALSE, intersect.chr = FALSE)

\S4method{annotateWithGeneParts}{GRangesList,GRangesList}(target, feature,
  strand = FALSE, intersect.chr = FALSE)
}
\arguments{
\item{target}{\code{\link{GRanges}} or \code{\link{GRangesList}}
object storing chromosome locations
to be annotated (e.g. chipseq peaks)}

\item{feature}{GRangesList object containing GRanges object for
promoter, exons, introns and transcription start sites, 
or simply output of readTranscriptFeatures function}

\item{strand}{If set to TRUE, annotation features and target features will be 
overlapped based on strand  (def:FALSE)}

\item{intersect.chr}{boolean, whether to select only chromosomes that are 
common to feature and target. FALSE by default}
}
\value{
\code{AnnotationByGeneParts} object or a list of 
        \code{AnnotationByGeneParts}
        objects if target is a  \code{\link{GRangesList}} object.
}
\description{
The function annotates \code{GRangesList} or \code{GRanges} object as 
overlapping with promoter,exon,intron or intergenic regions.
}
\examples{
data(cage)
bed.file = system.file("extdata/chr21.refseq.hg19.bed", package = "genomation")
gene.parts = readTranscriptFeatures(bed.file)
cage.annot = annotateWithGeneParts(cage, gene.parts, intersect.chr=TRUE)
cage.annot
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{binMean}
\alias{binMean}
\title{Function that computes a mean value for each bin}
\usage{
binMean(x, n)
}
\arguments{
\item{x}{NumericVector}

\item{n}{intiger - number of bins}
}
\description{
Function that computes a mean value for each bin
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/enrichmentMatrix.R
\docType{methods}
\name{enrichmentMatrix}
\alias{enrichmentMatrix}
\alias{enrichmentMatrix,ScoreMatrix,ScoreMatrix-method}
\title{Compute an enrichment of IP over control both stored in ScoreMatrix objects}
\usage{
\\S4method{enrichmentMatrix}{ScoreMatrix,ScoreMatrix}(IP, control)
}
\arguments{
\item{IP}{\code{\link{ScoreMatrix}} object storing an IP sample}

\item{control}{\code{\link{ScoreMatrix}} object storing a control sample}
}
\value{
\code{ScoreMatrix} object
}
\description{
This is an \code{enrichmentMatrix} function for \code{ScoreMatrix} objects, 
that enables to normalize ChIP-seq signals with respect to IgG or input DNA control.
}
\note{
The function computes an enrichment of IP over control as follow:
Suppose both IP and control are ScoreMatrix objects that have same dimensions. 
Then, the enrichment is calculated usign a formula: log2((IP + 1) / (control + 1)).
}
\examples{
 
#load IP and control BAM files and create ScoreMatrix objects
library('genomationData')
bam.file_IP <- system.file("extdata", 
"wgEncodeBroadHistoneH1hescSuz12051317AlnRep1.chr21.bam", package = "genomationData")
bam.file_c <- system.file("extdata", 
"wgEncodeBroadHistoneH1hescCtcfStdAlnRep1.chr21.bam", package = "genomationData")
data(promoters)
IP <- ScoreMatrix(target = bam.file_IP, windows = promoters, type = 'bam')
control <- ScoreMatrix(target = bam.file_c, windows = promoters, type = 'bam')

# compute an enrichment of IP over control
enrichmentMatrix(IP, control)

}
\seealso{
\code{\link{ScoreMatrix}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scoreMatrix.R
\docType{methods}
\name{ScoreMatrix}
\alias{ScoreMatrix}
\alias{ScoreMatrix,RleList,GRanges-method}
\alias{ScoreMatrix,GRanges,GRanges-method}
\alias{ScoreMatrix,character,GRanges-method}
\title{Get base-pair score for bases in each window}
\usage{
ScoreMatrix(target, windows, strand.aware = FALSE, weight.col = NULL,
  is.noCovNA = FALSE, type = "auto", rpm = FALSE, unique = FALSE,
  extend = 0, param = NULL, bam.paired.end = FALSE, library.size = NULL)

\\S4method{ScoreMatrix}{RleList,GRanges}(target,windows,strand.aware)

\\S4method{ScoreMatrix}{GRanges,GRanges}(target, windows, strand.aware,
                                                weight.col, is.noCovNA)

\\S4method{ScoreMatrix}{character,GRanges}(target, windows, strand.aware,
                                                  weight.col=NULL,is.noCovNA=FALSE, 
                                                  type='auto', rpm=FALSE,
                                                  unique=FALSE, extend=0, param=NULL, 
                                                  bam.paired.end=FALSE,
                                                  library.size=NULL)
}
\arguments{
\item{target}{\code{RleList} , \code{GRanges}, a BAM file or a BigWig
to be overlapped with ranges in \code{windows}}

\item{windows}{\code{GRanges} object that contains the windows of interest. 
It could be promoters, CpG islands, exons, introns. 
However the sizes of windows have to be equal.}

\item{strand.aware}{If TRUE (default: FALSE), the strands of the
windows will be taken into account in the resulting
 \code{ScoreMatrix}.
  If the strand of a window is -, the values of the bins 
  for that window will be reversed}

\item{weight.col}{if the object is \code{GRanges} object a numeric column
 in meta data part can be used as weights. This is particularly
useful when genomic regions have scores other than their
coverage values, such as percent methylation, conservation
scores, GC content, etc.}

\item{is.noCovNA}{(Default:FALSE)
if TRUE,and if 'target' is a GRanges object with 'weight.col'
 provided, the bases that are uncovered will be preserved as
 NA in the returned object. This useful for situations where
 you can not have coverage all over the genome, such as CpG 
 methylation values.}

\item{type}{(Default:"auto")
if target is a character vector of file paths, then type designates
the type of the corresponding files (bam or bigWig).}

\item{rpm}{boolean telling whether to normalize the coverage to per milion 
reads. FALSE by default. See \code{library.size}.}

\item{unique}{boolean which tells the function to remove duplicated reads 
based on chr, start, end and strand}

\item{extend}{numeric which tells the function to extend the reads to width=extend}

\item{param}{ScanBamParam object}

\item{bam.paired.end}{boolean indicating whether given BAM file contains 
paired-end reads (default:FALSE).
Paired-reads will be treated as fragments.}

\item{library.size}{numeric indicating total number of mapped reads in a BAM file
(\code{rpm} has to be set to TRUE).
If is not given (default: NULL) then library size 
is calculated using the Rsamtools idxstatsBam function:
sum(idxstatsBam(target)$mapped).}
}
\value{
returns a \code{ScoreMatrix} object
}
\description{
The funcion produces a base-pair resolution matrix of scores for given equal
width windows of interest. The returned matrix  can be used to 
draw meta profiles or heatmap of read coverage or wig track-like data.
The \code{windows} argument can be a predefined region around transcription start sites 
or other regions of interest that have equal lengths
The function removes all window that fall off the Rle object - 
have the start coordinate < 1 or end coordinate > length(Rle)
The function takes the intersection of names in the Rle and GRanges objects.
On Windows OS the function will give an error if the target is a file in .bigWig format.
}
\note{
We assume that a paired-end BAM file contains reads with unique ids and we remove 
both mates of reads if they are repeated. Due to the fact that \code{ScoreMatrix} 
uses the GenomicAlignments:readGAlignmentPairs function to read paired-end BAM files
a duplication of reads occurs when mates of one pair map into two different windows.

Strands of reads in a paired-end BAM are inferred depending on strand of 
first alignment from the pair. This is a default setting in the 
GenomicAlignments:readGAlignmentPairs function (see a strandMode argument). 
This mode should be used when the paired-end data was generated using 
one of the following stranded protocols: 
Directional Illumina (Ligation), Standard SOLiD.
}
\examples{
# When target is GRanges
data(cage)
data(promoters)
scores1=ScoreMatrix(target=cage,windows=promoters,strand.aware=TRUE,
                                weight.col="tpm")                                
                                 

# When target is RleList
library(GenomicRanges)
covs = coverage(cage)
scores2 = ScoreMatrix(target=covs,windows=promoters,strand.aware=TRUE)   
scores2 

# When target is a bam file
bam.file = system.file('unitTests/test.bam', package='genomation')
windows = GRanges(rep(c(1,2),each=2), IRanges(rep(c(1,2), times=2), width=5))
scores3 = ScoreMatrix(target=bam.file,windows=windows, type='bam') 
scores3

}
\seealso{
\code{\link{ScoreMatrixBin}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scoreMatrix.R
\name{constrainRanges}
\alias{constrainRanges}
\title{constrainRanges function}
\usage{
constrainRanges(target, windows)
}
\arguments{
\item{target}{target file}

\item{windows}{windows}
}
\description{
removes ranges that fell of the rle object
does not check for the correspondence of the chromosome names - always 
check before using this function
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scoreMatrix.R
\name{target.type}
\alias{target.type}
\title{target.type function}
\usage{
target.type(target, type = "")
}
\arguments{
\item{target}{target file}

\item{type}{type name}
}
\description{
Check if a target file is in bam and bigWig formats
by looking at the file extension
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readData.R
\docType{methods}
\name{readNarrowPeak}
\alias{readNarrowPeak}
\title{A function to read the Encode formatted narrowPeak file into a GRanges object}
\usage{
readNarrowPeak(file, track.line=FALSE, zero.based=TRUE)
}
\arguments{
\item{file}{an absolute or relative path to a bed file formatted by the Encode 
narrowPeak standard. The file can end in \code{.gz}, \code{.bz2}, 
\code{.xz}, or \code{.zip} and/or start with \code{http://} or 
\code{ftp://}. If the file is not compressed
it can also start with \code{https://} or \code{ftps://}.}

\item{track.line}{the number of track lines to skip, "auto" to detect them automatically
or FALSE(default) if the bed file doesn't have track lines}

\item{zero.based}{a boolean which tells whether the ranges in 
the bed file are 0 or 1 base encoded. (Default: TRUE)}
}
\value{
a GRanges object
}
\description{
A function to read the Encode formatted narrowPeak file into a GRanges object
}
\examples{
narrow.peak.file = system.file('extdata',"ex.narrowPeak", package='genomation')
                 
narrow.peak = readBroadPeak(narrow.peak.file)
head(narrow.peak)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readAnnotate.R
\docType{methods}
\name{convertBed2Exons}
\alias{convertBed2Exons}
\alias{convertBed2Exons,data.frame-method}
\title{convert a data frame read-in from a bed file to a GRanges object for exons}
\usage{
convertBed2Exons(bed.df)

\S4method{convertBed2Exons}{data.frame}(bed.df)
}
\arguments{
\item{bed.df}{a data.frame where column order and content resembles a bed file with 12 columns}
}
\value{
\code{\link{GRanges}} object
}
\description{
convert a data frame read-in from a bed file to a GRanges object for exons
}
\note{
one bed track per file is only accepted, the bed files with multiple tracks will cause en error
}
\examples{
file = system.file('extdata/chr21.refseq.hg19.bed', package='genomation')
bed12 = read.table(file)
exons = convertBed2Exons(bed12)
head(exons)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readData.R
\name{compressedAndUrl2temp}
\alias{compressedAndUrl2temp}
\title{compressedAndUrl2temp function}
\usage{
compressedAndUrl2temp(filename)
}
\arguments{
\item{filename}{file name}
}
\description{
compressedAndUrl2temp function
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readAnnotate.R
\docType{methods}
\name{heatTargetAnnotation}
\alias{heatTargetAnnotation}
\alias{plotGeneAnnotation}
\title{Plots the percentage of overlapping intervals with genomic features in a heatmap}
\usage{
heatTargetAnnotation(l, cluster = FALSE, col = c("white", "blue"),
  precedence = FALSE, plot = TRUE)
}
\arguments{
\item{l}{a \code{list} of  \code{AnnotationByFeature} objects. This 
could be returned by \code{\link{annotateWithFeatures}} function.}

\item{cluster}{TRUE/FALSE. If TRUE the heatmap is going to be clustered 
using a default hierarchical clustering scheme.}

\item{col}{a vector of two colors that will be used for interpolation. 
The first color is the lowest one, the second is the highest one}

\item{precedence}{TRUE|FALSE. If TRUE the precedence of annotation 
features will be used when plotting. The precedence will be taken
from the GRangesList used as annotation. The first GRanges will be 
treated as most important, and the second as the second most important
and so on. Such that, if an interval overlaps with features on that 
is part of the first GRanges object in the annotation GRangesList,
the rest of its overlaps with other elements in the annotation 
GRangesList will be ignored. This feature is important to have if 
the user desires that percentage of overlaps adds up to 100. This is
only possible when the annotation features are non-overlaping with
eachother or there is a hierarchy/precedence among them such as 
(with promoter>exon>intron precedence). In this case, anything that
overlaps with a promoter annotation will only be counted as promoter
even if it overlaps with exons or introns.}

\item{plot}{If FALSE, does not plot the heatmap just returns the matrix
used to make the heatmap}
}
\value{
returns the matrix used to make the heatmap when plot FALSE, 
otherwise returns ggplot2 object which can be modified further.
}
\description{
This function plots a heatmap for percentage of overlapping ranges 
with provided genomic features. The input object is a list of 
\code{AnnotationByFeature} objects, which contains necessary information
about overlap statistics to make the plot.
}
\examples{

library(GenomicRanges)
data(cage)
data(cpgi)
cage$tpm=NULL
gl = GRangesList(cage=cage, cpgi=cpgi)

bed.file = system.file("extdata/chr21.refseq.hg19.bed", package = "genomation")
gene.parts = readTranscriptFeatures(bed.file)
annot = annotateWithFeatures(gl, gene.parts, intersect.chr=TRUE)

            
\donttest{  
heatTargetAnnotation(annot)
}

}
\seealso{
see  \code{\link{getMembers}}, 
           \code{\link{annotateWithFeatures}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scoreMatrix.R
\name{readBam}
\alias{readBam}
\title{readBam function}
\usage{
readBam(target, windows, rpm = FALSE, unique = FALSE, extend = 0,
  param = NULL, paired.end = FALSE, library.size = NULL, ...)
}
\arguments{
\item{target}{target object}

\item{windows}{windows}

\item{rpm}{logical}

\item{unique}{logical}

\item{extend}{numeric}

\item{param}{ScanBamParam object}

\item{paired.end}{llogical}

\item{library.size}{numeric}

\item{...}{additional parameters}
}
\description{
given a big bam path reads the big wig file into a RleList
to be used by ScoreMatrix:char,GRanges
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentData.R
\docType{data}
\name{genes}
\alias{genes}
\title{Example RefSeq genes data set.}
\format{\code{\link{GRanges}} object}
\description{
RefSeq genes of hg19 assembly of human genome on chr21 and chr22. Downloaded
from UCSC genome browser.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scoreMatrixList.R
\docType{methods}
\name{ScoreMatrixList}
\alias{ScoreMatrixList}
\title{Make ScoreMatrixList from multiple targets}
\usage{
ScoreMatrixList(targets, windows = NULL, bin.num = NULL, bin.op = "mean",
  strand.aware = FALSE, weight.col = NULL, is.noCovNA = FALSE,
  type = "auto", rpm = FALSE, unique = FALSE, extend = 0,
  param = NULL, library.size = NULL, cores = 1)
}
\arguments{
\item{targets}{can be a list of \code{scoreMatrix} objects, that are coerced 
to the \code{ScoreMatrixList}, a list of \code{RleList} objects, or a 
character vector specifying the locations of mulitple bam files  or
bigWig files that 
are used to construct the \code{scoreMatrixList}. If it is either a 
RleList object or a character vector of files, it is obligatory to 
give a windows argument.}

\item{windows}{\code{GenomicRanges} containing viewpoints for the scoreMatrix 
or ScoreMatrixList functions}

\item{bin.num}{an integer telling the number of bins to bin the score matrix}

\item{bin.op}{an name of the function that will be used for smoothing windows of ranges}

\item{strand.aware}{a boolean telling the function whether to reverse the 
coverage of ranges that come from - strand (e.g. when plotting 
enrichment around transcription start sites)}

\item{weight.col}{if the object is \code{GRanges} object a numeric column
 in meta data part can be used as weights. This is particularly
useful when genomic regions have scores other than their
coverage values, such as percent methylation, conservation
scores, GC content, etc.}

\item{is.noCovNA}{(Default:FALSE)
if TRUE,and if 'targets' is a GRanges object with 'weight.col'
 provided, the bases that are uncovered will be preserved as
 NA in the returned object. This useful for situations where
 you can not have coverage all over the genome, such as CpG
  methylation values.}

\item{type}{(Default:"auto")
if \code{targets} is a character vector of file paths, then type 
designates the type of the corresponding files (bam or bigWig)}

\item{rpm}{boolean telling whether to normalize the coverage to per milion reads. 
FALSE by default. See \code{library.size}.}

\item{unique}{boolean which tells the function to remove duplicated reads 
based on chr, start, end and strand}

\item{extend}{numeric which tells the function to extend the features
( i.e aligned reads) to total
length ofwidth+extend}

\item{param}{ScanBamParam object}

\item{library.size}{a numeric vector of the same length as \code{targets} 
indicating total number of mapped reads in BAM files (\code{targets}).
If is not given (default: NULL) then library sizes for every target
is calculated using the Rsamtools idxstatsBam function:
sum(idxstatsBam(target)$mapped).
\code{rpm} argument has to be set to TRUE.}

\item{cores}{the number of cores to use (default: 1)}
}
\value{
returns a \code{ScoreMatrixList} object
}
\description{
The function constructs a list of \code{ScoreMatrix} objects in the form
of \code{ScoreMatrixList} object. This object can be visualized using 
\code{multiHeatMatrix}, \code{heatMeta} or \code{plotMeta}
}
\examples{

# visualize the distribution of cage clusters and cpg islands around promoters
library(GenomicRanges)
data(cage)
data(cpgi)
data(promoters)
 
cage$tpm = NULL
targets = GRangesList(cage=cage, cpgi=cpgi)
sml = ScoreMatrixList(targets, promoters, bin.num=10, strand.aware=TRUE)
sml
\donttest{
multiHeatMatrix(sml)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readAnnotate.R
\docType{methods}
\name{getAssociationWithTSS}
\alias{getAssociationWithTSS}
\alias{getAssociationWithTSS,-methods}
\alias{getAssociationWithTSS,}
\alias{AnnotationByGeneParts-method}
\alias{getAssociationWithTSS,AnnotationByGeneParts-method}
\title{Get distance to nearest TSS and gene id from AnnotationByGeneParts}
\usage{
getAssociationWithTSS(x)

\S4method{getAssociationWithTSS}{AnnotationByGeneParts}(x)
}
\arguments{
\item{x}{a \code{AnnotationByGeneParts}  object}
}
\value{
RETURNS a data.frame containing row number of the target features,
        distance of target to nearest TSS, TSS/Gene name, TSS strand
}
\description{
This accessor function gets the nearest TSS, its distance to target feature,
strand and name of TSS/gene from AnnotationByGeneParts object
}
\examples{
data(cage)
bed.file = system.file("extdata/chr21.refseq.hg19.bed", package = "genomation")
gene.parts = readTranscriptFeatures(bed.file)
cage.annot = annotateWithGeneParts(cage, gene.parts, intersect.chr=TRUE)        
head(getAssociationWithTSS(cage.annot))
 
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scoreMatrix.R
\name{checkClass}
\alias{checkClass}
\title{checkClass function}
\usage{
checkClass(x, class.name, var.name = deparse(substitute(x)))
}
\arguments{
\item{x}{object}

\item{class.name}{class name}

\item{var.name}{uses x object}
}
\description{
check whether the x object corresponds to the given class
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readData.R
\name{readTableFast}
\alias{readTableFast}
\title{readTableFast function}
\usage{
readTableFast(filename, header = TRUE, skip = 0, sep = "\\t", chr = 1)
}
\arguments{
\item{filename}{file name}

\item{header}{logical}

\item{skip}{numeric}

\item{sep}{character}

\item{chr}{numeric}
}
\description{
fast reading of big tables
chr indicates index of column of chromosomes
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readData.R
\docType{methods}
\name{readBroadPeak}
\alias{readBroadPeak}
\title{A function to read the Encode formatted broad peak file into a GRanges object}
\usage{
readBroadPeak(file, track.line=FALSE, zero.based=TRUE)
}
\arguments{
\item{file}{an absolute or relative path to a bed file formatted by the Encode broadPeak standard.
The file can end in \code{.gz}, \code{.bz2}, \code{.xz}, or \code{.zip}
and/or start with \code{http://} or \code{ftp://}. If the file is not compressed
it can also start with \code{https://} or \code{ftps://}.}

\item{track.line}{the number of track lines to skip, "auto" to detect them automatically
or FALSE(default) if the bed file doesn't have track lines}

\item{zero.based}{a boolean which tells whether the ranges in 
the bed file are 0 or 1 base encoded. (Default: TRUE)}
}
\value{
a GRanges object
}
\description{
A function to read the Encode formatted broad peak file into a GRanges object
}
\examples{
broad.peak.file = system.file('extdata',"ex.broadPeak", package='genomation')
                           
broad.peak = readBroadPeak(broad.peak.file)
head(broad.peak)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readData.R
\docType{methods}
\name{readGeneric}
\alias{readGeneric}
\title{Read a tabular file and convert it to GRanges.}
\usage{
readGeneric(file, chr = 1, start = 2, end = 3, strand = NULL,
  meta.cols = NULL, keep.all.metadata = FALSE, zero.based = FALSE,
  remove.unusual = FALSE, header = FALSE, skip = 0, sep = "\\t")
}
\arguments{
\item{file}{location of the file, a character string such as: "/home/user/my.bed"
or the input itself as a string (containing at least one \\n).}

\item{chr}{number of the column that has chromsomes information in the table (Def:1)}

\item{start}{number of the column that has start coordinates in the table (Def:2)}

\item{end}{number of the column that has end coordinates in the table (Def:3)}

\item{strand}{number of the column that has strand information, only -/+
is accepted (Default:NULL)}

\item{meta.cols}{named \code{list} that maps column numbers to 
meta data columns. 
e.g. list(name=5, score=10), which means 5th column will be
named "name", and 10th column will be named "score" and their
contents will be a part of the returned GRanges object. 
If header = TRUE, meta.cols parameter will over-write the 
column names given by the header line of the data frame.}

\item{keep.all.metadata}{\code{logical} determining if the extra columns (
the ones that are not designated by chr,start,end,strand and meta.cols
arguments )
should be kept or not. (Default:FALSE)}

\item{zero.based}{a boolean which tells whether the ranges in 
the bed file are 0 or 1 base encoded. (Default: FALSE)}

\item{remove.unusual}{if TRUE(default) remove the chromosomes with unsual 
names, such as chrX_random (Default:FALSE)}

\item{header}{whether the original file contains a header line
which designates the column names. If \code{TRUE} header will be used to 
construct column names. These names can be over written by meta.cols argument.}

\item{skip}{number of lines to skip. If there is a header line(s) you do not
wish to include you can use skip argument to skip that line.}

\item{sep}{a single character which designates the separator in the file. 
The default value is tab.}
}
\value{
\code{\link{GRanges}} object
}
\description{
The function reads a tabular  text file that contains location and other information
on genomic features and returns a \code{\link{GRanges}} object. 
The minimal information that the file has to have is chromosome, 
start and end columns. Strand information is not compulsory.
}
\examples{
my.file=system.file("extdata","chr21.refseq.hg19.bed",package="genomation")
refseq = readGeneric(my.file,chr=1,start=2,end=3,strand=NULL,
                      meta.cols=list(score=5,name=4),
                      keep.all.metadata=FALSE, zero.based=TRUE)
head(refseq)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readAnnotate.R
\docType{methods}
\name{annotateWithFeatureFlank}
\alias{annotateWithFeatureFlank}
\alias{annotateWithFeatureFlank,GRanges,GRanges,GRanges-method}
\title{Function to annotate a given GRanges object with promoter,exon,intron & intergenic values}
\usage{
annotateWithFeatureFlank(target, feature, flank, feature.name = NULL,
  flank.name = "flank", strand = FALSE, intersect.chr = FALSE)

\S4method{annotateWithFeatureFlank}{GRanges,GRanges,GRanges}(target, feature,
  flank, feature.name = NULL, flank.name = "flank", strand = FALSE,
  intersect.chr = FALSE)
}
\arguments{
\item{target}{a granges object storing chromosome locations to be annotated}

\item{feature}{a granges object storing chromosome locations of a feature 
(can be CpG islands, ChIP-seq peaks, etc)}

\item{flank}{a granges object storing chromosome locations of 
the flanks of the feature}

\item{feature.name}{string for the name of the feature}

\item{flank.name}{string for the name of the flanks}

\item{strand}{If set to TRUE, annotation features and target features 
will be overlapped based on strand  (def:FAULT)}

\item{intersect.chr}{boolean, whether to select only chromosomes that are 
common to feature and target. FALSE by default}
}
\value{
returns an \code{AnnotationByFeature} object
}
\description{
Function to annotate a given GRanges object with promoter,exon,intron & intergenic values
}
\examples{
data(cpgi)
data(cage)
cpgi.flanks = getFlanks(cpgi)
flank.annot = annotateWithFeatureFlank(cage, cpgi, cpgi.flanks)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readAnnotate.R
\name{annotatGrWithGeneParts}
\alias{annotatGrWithGeneParts}
\title{annotatGrWithGeneParts function}
\usage{
annotatGrWithGeneParts(gr, prom, exon, intron, strand = FALSE)
}
\arguments{
\item{gr}{target object}

\item{prom}{promotors}

\item{exon}{exons}

\item{intron}{introns}

\item{strand}{logical}
}
\description{
annotatGrWithGeneParts function
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentData.R
\docType{data}
\name{promoters}
\alias{promoters}
\title{Example promoter data set.}
\format{\code{\link{GRanges}} object}
\description{
promoters of hg19 assembly of human genome on chr21 and chr22. Promoter set is 
derived from refseq TSS.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readData.R
\name{detectUCSCheader}
\alias{detectUCSCheader}
\title{detectUCSCheader function}
\usage{
detectUCSCheader(filename)
}
\arguments{
\item{filename}{file name}
}
\description{
detects UCSC header (and first track)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Ops.R
\docType{methods}
\name{[,ScoreMatrixList,ANY,ANY,ANY-method}
\alias{[,ScoreMatrixList,ANY,ANY,ANY-method}
\alias{[,ScoreMatrixList-method}
\title{Extract method for a ScoreMatrixList object.}
\usage{
\S4method{[}{ScoreMatrixList,ANY,ANY,ANY}(x, i)
}
\arguments{
\item{x}{the \code{\link{ScoreMatrixList}} object}

\item{i}{numeric value}
}
\description{
Extract method for a ScoreMatrixList object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{binSum}
\alias{binSum}
\title{Function that computes a sum of values in a bin}
\usage{
binSum(x, n)
}
\arguments{
\item{x}{NumericVector - vector of values of a bin}

\item{n}{intiger - number of bins}
}
\description{
Function that computes a sum of values in a bin
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/findFeatureComb.R
\docType{methods}
\name{findFeatureComb}
\alias{findFeatureComb}
\alias{findFeatureComb,GRangesList-method}
\title{Find combitations of genomic features}
\usage{
findFeatureComb(gl, width=0, use.names=FALSE, collapse.char=':')

\S4method{findFeatureComb}{GRangesList}(gl, width = 0, use.names = FALSE,
  collapse.char = ":")
}
\arguments{
\item{gl}{a \code{GRangesList} object, containing ranges for which 
represent regions enriched for transcription factor binding}

\item{width}{\code{integer} is the requested width of each enriched region. If 0 the ranges are not resized, if a positive integer, the width of all ranges is set to that number. Ranges are resized relative to the center of original ranges.}

\item{use.names}{a boolean which tells the function whether to return the resulting ranges with a numeric vector which designates each class (the default), or to construct the names of each class using the names from the GRangesList}

\item{collapse.char}{a character which will be used to separate the class names if use.names=TRUE. The default is ':'}
}
\value{
a GRanges object
}
\description{
Provided a GRangesList, finds the combinations of sets of ranges. 
It is mostly used to look at the combinatorics of transcription factor 
binding. The function works by, firstly, constructing a union of all 
ranges in the list, which are then designated by the combinatorics of 
overlap with the original sets. A caveat of this approach is that the 
number of possible combinations increases exponentially, so we would 
advise you to use it with up to 6 data sets. If you wish to take a 
look at a greater number of factors, methods like self organizing 
maps or ChromHMM might be more appropriate.
}
\examples{
library(GenomicRanges)
g = GRanges(paste('chr',rep(1:2, each=3), sep=''), IRanges(rep(c(1,5,9), times=2), width=3))
gl = GRangesList(g1=g, g2=g[2:5], g3=g[3:4])
findFeatureComb(gl)
findFeatureComb(gl, use.names=TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotMatrix.R
\name{heatMeta}
\alias{heatMeta}
\title{Heatmap for meta-region profiles}
\usage{
heatMeta(mat, centralTend = "mean", profile.names = NULL, xcoords = NULL,
  col = NULL, meta.rescale = FALSE, winsorize = c(0, 100),
  legend.name = NULL, cex.legend = 1, xlab = NULL, main = "",
  cex.lab = 1, cex.axis = 1)
}
\arguments{
\item{mat}{\code{ScoreMatrix} or \code{ScoreMatrixList} to be plotted}

\item{centralTend}{a character that determines central tendency of meta-profile(s). 
It takes "mean" (default) or "median".}

\item{profile.names}{a character vector for names of profiles. If NULL, 
the names
will be taken from names(mat) if mat is a 
\code{ScoreMatrixList} object.}

\item{xcoords}{a vector of numbers showing relative positions of the bases or 
windows. It must match the number of columns in the \code{ScoreMatrix}
For example: if there are 2001 elements in the matrices which
are base-pair resolution 
and they are centered around an anchor point like TSS, the xcoords
argument should be -1000:1000. This argument is used to plot
accurate x-axis labels for the plots.If NULL it will be equal
to 1:ncol(mat).}

\item{col}{a vector of color pallete. 
color scheme to be used. If NULL, a version of jet colors will be
    used.}

\item{meta.rescale}{if TRUE meta-region profiles are scaled to 0 to 1 range by
subracting the min from profiles and dividing them by 
max-min.}

\item{winsorize}{Numeric vector of two, defaults to c(0,100). This vector 
determines the upper and lower percentile values to limit the 
extreme values. For example, c(0,99) will limit the values to
only 99th percentile, everything above the 99 percentile will 
be equalized to the value of 99th percentile. This is useful 
for visualization of matrices that have outliers.}

\item{legend.name}{a character label plotted next to the legend}

\item{cex.legend}{A numerical value giving the amount by which 
legend axis marks should be magnified relative to the default}

\item{xlab}{label a character string for x-axis}

\item{main}{a character string for the plot title}

\item{cex.lab}{A numerical value giving the amount by which 
axis labels (including 'legend.name') 
should be magnified relative to the default.}

\item{cex.axis}{A numerical value giving the amount by which 
axis marks should be magnified relative to the default}
}
\value{
returns meta-profile matrix invisibly.
}
\description{
Function calculates meta-profile(s) from a ScoreMatrix or a ScoreMatrixList, then
produces a heatmap or a set of stacked heatmaps for meta-region profiles
}
\examples{
 data(cage)
 data(promoters)
 scores1=ScoreMatrix(target=cage,windows=promoters,strand.aware=TRUE)
 data(cpgi)
 scores2=ScoreMatrix(target=cpgi,windows=promoters,strand.aware=TRUE)

 x=new("ScoreMatrixList",list(scores1,scores2))
 \donttest{
 heatMeta(mat=x,legend.name="fg",cex.legend=0.8,main="fdf",cex.lab=6,
          cex.axis=0.9)
 }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/combineScoreMatrixList.R
\name{c.ScoreMatrixList}
\alias{c.ScoreMatrixList}
\title{c.ScoreMatrixList}
\usage{
\method{c}{ScoreMatrixList}(..., recursive = FALSE, use.names = TRUE)
}
\arguments{
\item{...}{contais scoreMatrix and scoreMatrixList objects}

\item{recursive}{logical}

\item{use.names}{logical}
}
\value{
returns a \code{scoreMatrixList} object
}
\description{
Combine a scoreMatrix into a scoreMatrixList object - when a ScoreMatrixList is a first argument
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readAnnotate.R
\docType{methods}
\name{getFlanks}
\alias{getFlanks}
\alias{getFlanks,GRanges-method}
\title{Function to get upstream and downstream adjecent regions to a genomic feature such as CpG islands}
\usage{
getFlanks(grange,flank=2000,clean=TRUE)

\S4method{getFlanks}{GRanges}(grange, flank = 2000, clean = TRUE)
}
\arguments{
\item{grange}{GRanges object for the feature}

\item{flank}{number of basepairs for the flanking regions}

\item{clean}{If set to TRUE, flanks overlapping with other main features 
will be trimmed, and overlapping flanks will be removed. 
This will remove multiple counts when other features overlap 
with flanks}
}
\value{
GRanges object for flanking regions
}
\description{
Function to get upstream and downstream adjecent regions to a genomic feature such as CpG islands
}
\examples{
data(cpgi)
cpgi.flanks = getFlanks(cpgi)
head(cpgi.flanks)

}
